@@ -1,15 +1,14 @@
 PHASES = ["build", "test"]
 
 CUDA_VERSIONS = [
-    "92",
     "101",
     "102",
-    "110",
+    "111",
 ]
 
 ROCM_VERSIONS = [
-    "3.7",
-    "3.8",
+    "3.10",
+    "4.0.1",
 ]
 
 ROCM_VERSION_LABELS = ["rocm" + v for v in ROCM_VERSIONS]
