@@ -9,7 +9,7 @@ namespace at {
 
 enum class NameType: uint8_t { BASIC, WILDCARD };
 
-struct CAFFE2_API Dimname {
+struct TORCH_API Dimname {
   static Dimname fromSymbol(Symbol name);
   static Dimname wildcard();
   static bool isValidName(const std::string& name);
@@ -21,7 +21,7 @@ struct CAFFE2_API Dimname {
   bool isWildcard() const { return type_ == NameType::WILDCARD; }
 
   bool matches(Dimname other) const;
-  optional<Dimname> unify(Dimname other) const;
+  c10::optional<Dimname> unify(Dimname other) const;
 
  private:
   Dimname(Symbol name)
@@ -35,7 +35,7 @@ struct CAFFE2_API Dimname {
 
 using DimnameList = c10::ArrayRef<Dimname>;
 
-CAFFE2_API std::ostream& operator<<(std::ostream& out, const Dimname& dimname);
+TORCH_API std::ostream& operator<<(std::ostream& out, const Dimname& dimname);
 
 inline bool operator==(const Dimname& lhs, const Dimname& rhs) {
   return lhs.symbol() == rhs.symbol();
