@@ -27,7 +27,7 @@ std::ostream& operator<<(std::ostream & out, const Type & t) {
       out << "Tensor";
     }
     if (auto ndim = value->sizes().size()) {
-      bool has_valid_strides_info =
+      bool has_valid_strides_info = *ndim > 0 &&
           value->strides().isComplete() && value->strides().size() == ndim;
 
       out << "(";
@@ -41,10 +41,17 @@ std::ostream& operator<<(std::ostream & out, const Type & t) {
         } else {
           out << "*";
         }
-        if (has_valid_strides_info &&
-            type_verbosity() >= TypeVerbosity::TypeAndStride) {
-          out << ":" << *value->strides()[i];
+      }
+      if (has_valid_strides_info &&
+          type_verbosity() >= TypeVerbosity::TypeAndStride) {
+        out << ", strides=[";
+        for (size_t i = 0; i < *ndim; ++i) {
+          if (i > 0) {
+            out << ", ";
+          }
+          out << *value->strides()[i];
         }
+        out << "]";
       }
       if (type_verbosity() >= TypeVerbosity::Full) {
         if (value->requiresGrad()) {
@@ -61,22 +68,37 @@ std::ostream& operator<<(std::ostream & out, const Type & t) {
         }
       }
       out << ")";
+    } else {
+      if (type_verbosity() >= TypeVerbosity::Full) {
+        size_t i = 0;
+        if (value->requiresGrad()) {
+          out << "("
+              << "requires_grad=" << *value->requiresGrad();
+          i++;
+        }
+        if (value->device()) {
+          out << ((i++ > 0) ? ", " : "(") << "device=" << *value->device();
+        }
+        if (i > 0) {
+          out << ")";
+        }
+      }
     }
 
     if (value->undefined() && *value->undefined()) {
       out << "[Undefined]";
     }
   } else if(t.kind() == TypeKind::ListType) {
-    auto prim = t.cast<ListType>()->getElementType();
+    auto prim = t.castRaw<ListType>()->getElementType();
     out << *prim << "[]";
   } else if (t.kind() == TypeKind::OptionalType) {
-    auto prim = t.cast<OptionalType>()->getElementType();
+    auto prim = t.castRaw<OptionalType>()->getElementType();
     out << *prim << "?";
   } else if(t.kind() == TypeKind::FutureType) {
-    auto elem = t.cast<FutureType>()->getElementType();
+    auto elem = t.castRaw<FutureType>()->getElementType();
     out << "Future[" << *elem << "]";
   } else if(t.kind() == TypeKind::RRefType) {
-    auto elem = t.cast<RRefType>()->getElementType();
+    auto elem = t.castRaw<RRefType>()->getElementType();
     out << "RRef[" << *elem << "]";
   } else if(auto tup = t.cast<TupleType>()) {
     if (tup->schema()) {
@@ -123,10 +145,18 @@ FloatTypePtr FloatType::get() {
   static auto value = FloatType::create();
   return value;
 }
+ComplexTypePtr ComplexType::get() {
+  static auto value = ComplexType::create();
+  return value;
+}
 BoolTypePtr BoolType::get() {
   static auto value = BoolType::create();
   return value;
 }
+StorageTypePtr StorageType::get() {
+  static auto value = StorageType::create();
+  return value;
+}
 NoneTypePtr NoneType::get() {
   static auto value = NoneType::create();
   return value;
@@ -151,6 +181,10 @@ DeviceObjTypePtr DeviceObjType::get() {
   static auto value = DeviceObjType::create();
   return value;
 }
+StreamObjTypePtr StreamObjType::get() {
+  static auto value = StreamObjType::create();
+  return value;
+}
 ScalarTypeTypePtr ScalarTypeType::get() {
 static auto value = ScalarTypeType::create();
 return value;
@@ -179,6 +213,10 @@ ListTypePtr ListType::ofInts() {
   static auto value = ListType::create(IntType::get());
   return value;
 }
+ListTypePtr ListType::ofComplexDoubles() {
+  static auto value = ListType::create(ComplexType::get());
+  return value;
+}
 ListTypePtr ListType::ofFloats() {
   static auto value = ListType::create(FloatType::get());
   return value;
@@ -222,7 +260,7 @@ c10::optional<TypePtr> unifyTypesImpl(const TypePtr& t1, const TypePtr& t2) {
 
   // Handle non-container types which do not subtype each other and unify
   if (t1->kind() == TensorType::Kind && t2->kind() == TensorType::Kind) {
-    return t1->expect<TensorType>()->merge(t2->expect<TensorType>());
+    return t1->expectRef<TensorType>().merge(*t2->expect<TensorType>());
   }
 
   if (t1->isSubtypeOf(NoneType::get()) && !t2->isSubtypeOf(NoneType::get())) {
@@ -266,8 +304,8 @@ c10::optional<TypePtr> unifyTypesImpl(const TypePtr& t1, const TypePtr& t2) {
 
   if (t1->cast<FutureType>() && t2->cast<FutureType>()) {
     if (auto elem = unifyTypes(
-            t1->cast<FutureType>()->getElementType(),
-            t2->cast<FutureType>()->getElementType())) {
+            t1->castRaw<FutureType>()->getElementType(),
+            t2->castRaw<FutureType>()->getElementType())) {
       return FutureType::create(*elem);
     }
   }
@@ -425,7 +463,7 @@ MatchTypeReturn matchTypeVariables(
       // unknown type).
       return matchTypeVariables(opt_formal->getElementType(), actual, type_env);
     }
-    // note: if actual was non here we potentially did not fill in the type
+    // note: if actual was None here we potentially did not fill in the type
     // variables contained in the formal. It is still a valid match because None
     // matches Optional[T] later error checking on tryEvalTypeVariables will
     // report the problem if we never match variables in type T
@@ -454,7 +492,7 @@ MatchTypeReturn matchTypeVariables(
 }
 
 // change return types like List[List[t]] into List[List[int]]
-CAFFE2_API TypePtr tryEvalTypeVariables(TypePtr type, std::unordered_map<std::string, TypePtr>& type_env) {
+TORCH_API TypePtr tryEvalTypeVariables(TypePtr type, std::unordered_map<std::string, TypePtr>& type_env) {
   if (!type->hasFreeVariables()) {
     return type;
   }
@@ -479,7 +517,7 @@ CAFFE2_API TypePtr tryEvalTypeVariables(TypePtr type, std::unordered_map<std::st
   }
 }
 
-CAFFE2_API bool elementTypeCanBeInferredFromMembers(const TypePtr& elem_type) {
+TORCH_API bool elementTypeCanBeInferredFromMembers(const TypePtr& elem_type) {
   if (elem_type->kind() == OptionalType::Kind ||
       elem_type->kind() == NumberType::Kind) {
     // Builtin Union types
@@ -506,7 +544,7 @@ const char * typeKindToString(TypeKind kind) {
   return "";
 }
 
-bool Type::isSubtypeOfExt(const TypePtr rhs, std::ostream* why_not) const {
+bool Type::isSubtypeOfExt(const TypePtr& rhs, std::ostream* why_not) const {
   if (rhs->kind() == TypeKind::AnyType || *this == *rhs) {
     return true;
   }
@@ -549,16 +587,16 @@ VaryingShape<int64_t> TensorType::sizes() const {
       }));
 }
 
-TensorTypePtr TensorType::merge(TensorTypePtr other, bool merge_sizes) const {
-  auto scalar_type = merge_primitive(scalarType(), other->scalarType());
-  auto dev = merge_primitive(device(), other->device());
-  auto sprops = stride_properties().merge(other->stride_properties());
-  auto gr = merge_primitive(requiresGrad(), other->requiresGrad());
-  auto undef = merge_primitive(undefined(), other->undefined());
+TensorTypePtr TensorType::merge(const TensorType& other, bool merge_sizes) const {
+  auto scalar_type = merge_primitive(scalarType(), other.scalarType());
+  auto dev = merge_primitive(device(), other.device());
+  auto sprops = stride_properties().merge(other.stride_properties());
+  auto gr = merge_primitive(requiresGrad(), other.requiresGrad());
+  auto undef = merge_primitive(undefined(), other.undefined());
   return TensorType::create(
       scalar_type,
       dev,
-      merge_sizes ? symbolic_sizes().merge(other->symbolic_sizes())
+      merge_sizes ? symbolic_sizes().merge(other.symbolic_sizes())
                   : symbolic_sizes(),
       sprops,
       gr,
@@ -585,8 +623,10 @@ bool TensorType::matchTensor(const at::Tensor& t) {
   }
   // Here we know t.defined() == true and compare all other properties.
   bool rg = at::GradMode::is_enabled() && t.requires_grad();
-  bool matched_strides = (!t.has_storage() && !stride_properties().isComplete())
-    || stride_properties() == computeStrideProps(t.sizes(), t.strides(), t.is_contiguous());
+  bool matched_strides = (!stride_properties().size()) ||
+      (!t.has_storage() && !stride_properties().isComplete()) ||
+      stride_properties() ==
+          computeStrideProps(t.sizes(), t.strides(), t.is_contiguous());
   return scalarType().value_or(t.scalar_type()) == t.scalar_type()
     && device().value_or(t.device()) == t.device()
     && requiresGrad().value_or(rg) == rg
@@ -728,7 +768,7 @@ TupleType::TupleType(
   }
 }
 
-bool TupleType::isSubtypeOfExt(const TypePtr rhs_, std::ostream* why_not) const {
+bool TupleType::isSubtypeOfExt(const TypePtr& rhs_, std::ostream* why_not) const {
   if (Type::isSubtypeOfExt(rhs_, why_not)) {
     return true;
   }
@@ -763,7 +803,7 @@ bool TupleType::isSubtypeOfExt(const TypePtr rhs_, std::ostream* why_not) const
   });
 }
 
-bool ListType::isSubtypeOfExt(const TypePtr rhs_, std::ostream* why_not) const {
+bool ListType::isSubtypeOfExt(const TypePtr& rhs_, std::ostream* why_not) const {
   if (Type::isSubtypeOfExt(rhs_, why_not)) {
     return true;
   }
@@ -967,11 +1007,9 @@ TensorTypePtr TensorType::create(
     const SymbolicShape& sizes,
     const VaryingShape<Stride>& strides,
     c10::optional<bool> requires_grad,
-    c10::optional<bool> undefined,
-    bool is_inferred) {
-    auto pt = TensorTypePtr(new TensorType(
+    c10::optional<bool> undefined) {
+  auto pt = TensorTypePtr(new TensorType(
       scalar_type, device, sizes, strides, requires_grad, undefined));
-    pt->is_inferred_ = is_inferred;
   return pt;
 }
 
@@ -1006,13 +1044,13 @@ const SymbolicShape& TensorType::symbolic_sizes() const {
   return sizes_;
 }
 
-bool TensorType::isSubtypeOfExt(const TypePtr rhs, std::ostream* why_not) const {
+bool TensorType::isSubtypeOfExt(const TypePtr& rhs, std::ostream* why_not) const {
   if (auto rhs_p = rhs->cast<TensorType>()) {
     // if we have the same pointer, avoid computing the merge
     if (this == rhs_p.get()) {
       return true;
     }
-    return *merge(rhs_p) == *rhs_p;
+    return *merge(*rhs_p) == *rhs_p;
   }
   return Type::isSubtypeOfExt(rhs, why_not);
 }
@@ -1032,6 +1070,301 @@ void ClassType::addMethod(torch::jit::Function* method) {
   methods_.push_back(method);
 }
 
+const std::vector<torch::jit::Function*>& ClassType::getForwardHooks() const {
+    return forward_hooks_;
+}
+
+const std::vector<torch::jit::Function*>& ClassType::getForwardPreHooks() const {
+    return forward_pre_hooks_;
+}
+
+void ClassType::addForwardPreHook(torch::jit::Function* pre_hook_ptr) {
+    forward_pre_hooks_.emplace_back(pre_hook_ptr);
+}
+
+void ClassType::addForwardHook(torch::jit::Function* hook_ptr) {
+    forward_hooks_.emplace_back(hook_ptr);
+}
+
+torch::jit::Function* ClassType::findForwardPreHook(const std::string& name) const {
+  for (const auto& pre_hook : forward_pre_hooks_) {
+    if (name == pre_hook->name()) {
+      return pre_hook;
+    }
+  }
+  return nullptr;
+}
+
+torch::jit::Function* ClassType::findForwardHook(const std::string& name) const {
+  for (const auto& hook : forward_hooks_) {
+    if (name == hook->name()) {
+      return hook;
+    }
+  }
+  return nullptr;
+}
+
+std::string getSchemaInputTypesString(const FunctionSchema& schema) {
+  std::stringstream input_types;
+  const std::vector<Argument>& forward_args = schema.arguments();
+  for (int i = 1; i < forward_args.size(); ++i) {
+    input_types << forward_args[i].type()->annotation_str();
+    if (forward_args.size() - 1 != i) {
+      input_types << ", ";
+    }
+  }
+  if (forward_args.size() == 1) {
+    input_types << "()";
+  }
+  return input_types.str();
+}
+
+std::string ClassType::getForwardPreHookErrorMessage(int pre_hook_idx) const {
+  const std::string& pre_hook_name = forward_pre_hooks_[pre_hook_idx]->name();
+  const FunctionSchema& forward_schema = getMethod("forward").getSchema();
+  std::string input_types = getSchemaInputTypesString(forward_schema);
+  const std::vector<Argument>& forward_args = forward_schema.arguments();
+   
+  std::string single_output = "";
+  if (forward_args.size() == 2 &&
+      forward_args[1].type()->cast<TupleType>() == nullptr) {
+    // if the output type is a single tuple, it needs to be wrapped in an outer tuple
+    // to match eager's behavior 
+    single_output = ", '" + forward_args[1].type()->annotation_str() + "',";
+  }
+  std::string pre_hook_schema =
+      pre_hook_name + "(self, input: Tuple[" + input_types + "])";
+  std::string return_string =
+      "This error occured while scripting the forward pre-hook '" +
+      pre_hook_name + "' on module '" + name()->name() +
+      "'. If you did not want to script this pre-hook remove it from the "
+      "original NN module before scripting. Pre-hooks for module '" + 
+      name()->name() + "' are expected to have the following signature: " 
+      + pre_hook_schema + " with a return type of either 'None'" + 
+      single_output + " or 'Tuple[" + input_types + "]'.";
+  return return_string;
+}
+
+std::string ClassType::getForwardHookErrorMessage(int hook_idx) const {
+  const std::string& hook_name = forward_hooks_[hook_idx]->name();
+  const FunctionSchema& forward_schema = getMethod("forward").getSchema();
+  std::string input_types = getSchemaInputTypesString(forward_schema); 
+
+  // create expected output types string
+  const Argument& pre_output =
+      (hook_idx == 0)
+          ? forward_schema.returns()[0]
+          : forward_hooks_[hook_idx - 1]->getSchema().returns()[0];
+  std::string output_types = pre_output.type()->annotation_str();
+  // create error message
+  std::string hook_schema = hook_name + "(self, input: Tuple[" +
+                            input_types + "], output: " + output_types + ")";
+  std::string return_string =
+      "This error occured while scripting the forward hook '" 
+      + hook_name + "' on module " + name()->name() +
+      ". If you did not want to script this hook remove it from" +
+      " the original NN module before scripting. This hook was" +
+      " expected to have the following signature: " + hook_schema +
+      ". The type of the output arg is the returned type from" + 
+      " either the forward method or the previous hook if it exists. " + 
+      "Note that hooks can return anything, but if the hook is " + 
+      "on a submodule the outer module is expecting" +
+      " the same return type as the submodule's forward.";
+  return return_string;
+}
+
+void checkForwardHookInputArguments(
+    const FunctionSchema& forward_schema, 
+    const FunctionSchema& hook_schema, 
+    const std::string& hook_id, 
+    const std::string& hook_err_msg) {
+  // check for proper tuple input types
+  const std::vector<Argument>& forward_args = forward_schema.arguments();
+  const Argument input_arg = hook_schema.arguments()[1];
+  TORCH_CHECK(
+      input_arg.type()->cast<TupleType>() != nullptr, 
+      hook_id,
+      "expected the input argument to be typed as a Tuple but found type: '",
+      input_arg.type()->annotation_str(), 
+      "' instead.\n", 
+      hook_err_msg
+   );
+
+  const at::ArrayRef<TypePtr> input_tuple_types = input_arg.type()->castRaw<TupleType>()->elements();
+  if (forward_args.size() == 1) {
+    // check for empty forward case
+    TORCH_CHECK(
+        input_tuple_types.size() == 0,
+        hook_id,
+        "was expecting Tuple[()] as the input type. Received type: '",
+        input_arg.type()->annotation_str(),
+        "'.\n",
+        hook_err_msg
+      );
+  } else {
+    // check input tuple for correct size and correct contained types
+    TORCH_CHECK(
+        input_tuple_types.size() == forward_args.size() - 1,
+        hook_id,
+        "has the wrong number of contained types for the",
+        " input argument's Tuple. Received type: '",
+        input_arg.type()->annotation_str(),
+        "'.\n",
+        hook_err_msg
+    );
+
+    for (int i = 1; i < forward_args.size(); ++i) {
+      if (*forward_args[i].type() != *input_tuple_types[i - 1]) {
+        TORCH_CHECK(
+            false,
+            hook_id,
+            "has the wrong inner types for the input tuple argument. Received type: '",
+            input_arg.type()->annotation_str(),
+            "'.\n",
+            hook_err_msg
+        );
+      }
+    }
+  }
+}
+
+void ClassType::checkForwardPreHookSchema(
+    int pre_hook_idx, 
+    const FunctionSchema& pre_hook_schema) const {
+  const torch::jit::Function* pre_hook = forward_pre_hooks_[pre_hook_idx];
+  std::string hook_id =
+      "Pre-hook '" + pre_hook->name() + "' on module '" + name()->name() + "' ";
+  std::string pre_hook_err_msg = getForwardPreHookErrorMessage(pre_hook_idx) + "\n";
+
+  // Pre-hooks are expecting two inputs: self, and a Tuple containing the
+  // non-self arguments passed to Forward
+  TORCH_CHECK(
+      pre_hook_schema.arguments().size() == 2,
+      hook_id,
+      "was expected to only have exactly 2 inputs but it had ",
+      pre_hook_schema.arguments().size(),
+      " inputs. ",
+      pre_hook_err_msg
+   );
+
+  const FunctionSchema& forward_schema = getMethod("forward").getSchema();
+  const std::vector<Argument>& forward_args = forward_schema.arguments();
+  checkForwardHookInputArguments(forward_schema, pre_hook_schema, hook_id, pre_hook_err_msg);
+
+  // check return type, expected to be either None, the same type as the input,
+  // or the contained single type if the input was a tuple containing a single
+  // type.
+  TORCH_CHECK(
+            pre_hook_schema.returns().size() != 0,
+            hook_id,
+            "is missing a return annotation. Return annotations are required, please add one.\n",
+            pre_hook_err_msg
+  );
+  const Argument return_arg = pre_hook_schema.returns()[0];
+  std::string wrong_type_returned_err_msg = hook_id + 
+      "returned the wrong type of: '" +
+      return_arg.type()->annotation_str() + "'.";
+
+  if (return_arg.type()->kind() == NoneType::get()->kind()) {
+    return;
+  }
+  if (forward_args.size() == 2 && *forward_args[1].type() == *return_arg.type()) {
+    // TORCH_CHECK below is for the edge case where forward's input is a tuple and the 
+    // pre-hook returns a matching tuple. Eager doesn't support this- the working eager return
+    // for a tuple type is the forward's input tuple wrapped inside of another tuple. 
+    TORCH_CHECK(
+        return_arg.type()->cast<TupleType>() == nullptr,
+        wrong_type_returned_err_msg,
+        " When forward has a single tuple input argument, the return needs",
+        " to be 'None' or a nested tuple containing forward's input tuple",
+        " argument as in: 'Tuple[",
+        forward_args[1].type()->annotation_str(),
+        "]'.\n",
+        pre_hook_err_msg
+    );
+    return;
+  }
+  // return can only be tuple of nested types now
+  // check to make sure return is of tuple type
+  TORCH_CHECK(
+      return_arg.type()->cast<TupleType>() != nullptr,
+      wrong_type_returned_err_msg,
+      pre_hook_err_msg
+  );
+  const at::ArrayRef<TypePtr> return_tuple_types =
+      return_arg.type()->castRaw<TupleType>()->elements();
+  // check for edge case of Tuple[()] for when forward has no arguments
+  if (forward_args.size() == 1) {
+    TORCH_CHECK(
+        return_tuple_types.size() == 0,
+        wrong_type_returned_err_msg,
+        " Was expecting either 'None' or 'Tuple[()]' since forward had ",
+        "no arguments.\n",
+        pre_hook_err_msg
+    );
+    return;
+  }
+
+  // check that tuple has proper number of contained types
+  TORCH_CHECK(
+      return_tuple_types.size() == forward_args.size() - 1,
+      wrong_type_returned_err_msg,
+      " The returned tuple contains the wrong number of contained types.\n",
+      pre_hook_err_msg
+  );
+  // check that contained types match forward types
+  for (int i = 1; i < forward_args.size(); ++i) {
+    if (*forward_args[i].type() != *return_tuple_types[i - 1]) {
+      TORCH_CHECK(
+          false, 
+          wrong_type_returned_err_msg,
+          " The returned tuple contains the wrong inner types.\n",
+          pre_hook_err_msg);
+    }
+  }
+}
+
+void ClassType::checkForwardHookSchema(
+      int hook_idx, 
+      const FunctionSchema& hook_schema) const {
+  const torch::jit::Function* hook = forward_hooks_[hook_idx];
+  std::string hook_id =
+      "Hook '" + hook->name() + "' on module '" + name()->name() + "' ";
+  std::string hook_err_msg = getForwardHookErrorMessage(hook_idx) + "\n";
+  // Hooks are expecting three inputs: self, a Tuple containing the non-self
+  // arguments passed to Forward, and the output of either Forward or the
+  // previous hook
+  TORCH_CHECK(
+      hook_schema.arguments().size() == 3,
+      hook_id,
+      "was expected to only have exactly 3 inputs but it had ",
+      hook_schema.arguments().size(),
+      " inputs. ",
+      hook_err_msg
+  );
+
+  const FunctionSchema& forward_schema = getMethod("forward").getSchema();
+  checkForwardHookInputArguments(forward_schema, hook_schema, hook_id, hook_err_msg);
+
+  // check output tuple
+  const Argument& prev_output = (hook_idx == 0)
+            ? forward_schema.returns()[0]
+            : forward_hooks_[hook_idx - 1]->getSchema().returns()[0];
+  const Argument return_arg = hook_schema.arguments()[2];
+
+  // output tuple needs to match prev_output's return exactly
+  TORCH_CHECK(
+      *prev_output.type() == *return_arg.type(),
+      hook_id,
+      "has the wrong type for the output argument. Received type: '",
+      return_arg.type()->annotation_str(),
+      "'. Expected type: '",
+      prev_output.type()->annotation_str(),
+      "'.\n",
+      hook_err_msg
+  );
+}
+
 torch::jit::Function* ClassType::findMethod(const std::string& name) const {
   for (auto method : methods_) {
     if (name == method->name()) {
@@ -1051,6 +1384,27 @@ torch::jit::Function& ClassType::getMethod(const std::string& name) const {
       "'");
   return *method;
 }
+
+torch::jit::Function* ClassType::findHook(const std::string& name) const {
+  auto hook = findForwardHook(name);
+  if (hook == nullptr) {
+    hook = findForwardPreHook(name); 
+  } 
+  return hook;
+}
+
+torch::jit::Function& ClassType::getHook(const std::string& name) const {
+  torch::jit::Function* function = findHook(name);
+  TORCH_CHECK(
+      function != nullptr,
+      "Couldn't find: '",
+      name,
+      "' on class: '",
+      repr_str(),
+      "'as forward hook or forward pre_hook.");
+  return *function;
+}
+
 bool ClassType::hasMethod(const std::string& name) const {
   return findMethod(name) != nullptr;
 }
@@ -1087,7 +1441,7 @@ ClassTypePtr ClassType::refine(at::ArrayRef<TypePtr> refined_slots) const {
   return ptr;
 }
 
-bool ClassType::isSubtypeOfExt(const TypePtr rhs, std::ostream* why_not) const {
+bool ClassType::isSubtypeOfExt(const TypePtr& rhs, std::ostream* why_not) const {
   if (rhs->cast<AnyClassType>()) {
     return true;
   }
@@ -1171,7 +1525,7 @@ bool InterfaceType::isSubTypeImpl(
     return true;
 }
 
-bool InterfaceType::isSubtypeOfExt(const TypePtr rhs, std::ostream* why_not) const {
+bool InterfaceType::isSubtypeOfExt(const TypePtr& rhs, std::ostream* why_not) const {
   // to improve performance this check can be cached
   if (auto iface = rhs->cast<InterfaceType>()) {
     return isSubTypeImpl(*this, *iface, why_not);
@@ -1200,19 +1554,21 @@ InterfaceType::~InterfaceType() = default;
 ClassTypePtr ClassType::create(
     c10::optional<QualifiedName> qualifiedName,
     std::weak_ptr<CompilationUnit> cu,
-    bool is_module) {
+    bool is_module,
+    std::string doc_string) {
   return ClassTypePtr(
-      new ClassType(std::move(qualifiedName), std::move(cu), is_module));
+      new ClassType(std::move(qualifiedName), std::move(cu), is_module, std::move(doc_string)));
 }
 
 ClassType::ClassType(
     c10::optional<QualifiedName> name,
     std::weak_ptr<CompilationUnit> cu,
-    bool is_module = false)
+    bool is_module = false,
+    std::string doc_string = "")
     : NamedType(TypeKind::ClassType, std::move(name)),
       compilation_unit_(std::move(cu)),
-      isModule_(is_module) {
-}
+      isModule_(is_module),
+      doc_string_(std::move(doc_string)) {}
 
 const std::vector<torch::jit::Function*>& ClassType::methods() const {
   return methods_;
@@ -1285,7 +1641,7 @@ size_t ClassType::addAttribute(
     TORCH_CHECK(
         (type->kind() == TensorType::Kind) ||
             (type->kind() == OptionalType::Kind &&
-            type->expect<OptionalType>()->getElementType()->kind() ==
+            type->expectRef<OptionalType>().getElementType()->kind() ==
                 TensorType::Kind) ||
             (type->kind() == NoneType::Kind),
         "Expecting parameter or buffer to have either None, Tensor or Optional[Tensor] type, but got: ",
@@ -1302,6 +1658,14 @@ void ClassType::unsafeRemoveAttribute(const std::string& name) {
   AT_ASSERT(attributes_.size() == attributeTypes_.size());
 }
 
+void ClassType::unsafeChangeAttributeType(const std::string& name, TypePtr new_ty) {
+  auto slot = getAttributeSlot(name);
+  auto old_attr_info = attributes_[slot];
+  AT_ASSERT(old_attr_info.getKind() == AttributeKind::REGULAR_ATTRIBUTE);
+  attributes_[slot] = ClassAttribute(old_attr_info.getKind(), new_ty, old_attr_info.getName());
+  attributeTypes_[slot] = new_ty;
+}
+
 size_t ClassType::addConstant(const std::string& name, const IValue& value) {
   checkNotExist(name, "constant");
   size_t slot = constantNames_.size();
@@ -1419,7 +1783,7 @@ SymbolicShape SymbolicShape::merge(const SymbolicShape& other) const {
   return SymbolicShape(std::move(dims));
 }
 
-bool EnumType::isSubtypeOfExt(const TypePtr rhs, std::ostream* why_not) const {
+bool EnumType::isSubtypeOfExt(const TypePtr& rhs, std::ostream* why_not) const {
   return rhs->kind() == TypeKind::AnyType ||
       rhs->kind() == TypeKind::AnyEnumType || *this == *rhs;
 }
