@@ -259,12 +259,12 @@ public:
   // Only required because vec256_qint refers to this.
   // Once we specialize that implementation for ARM
   // this should be removed. TODO (kimishpatel)
-  const float operator[](int idx) const {
+  float operator[](int idx) const {
     __at_align32__ float tmp[size()];
     store(tmp);
     return tmp[idx];
-  };
-  const float operator[](int idx) {
+  }
+  float operator[](int idx) {
     __at_align32__ float tmp[size()];
     store(tmp);
     return tmp[idx];
@@ -362,6 +362,26 @@ public:
   Vec256<float> i0() const {
     return map(calc_i0);
   }
+  Vec256<float> igamma(const Vec256<float> &x) const {
+    __at_align32__ float tmp[size()];
+    __at_align32__ float tmp_x[size()];
+    store(tmp);
+    x.store(tmp_x);
+    for (int64_t i = 0; i < size(); i++) {
+      tmp[i] = calc_igamma(tmp[i], tmp_x[i]);
+    }
+    return loadu(tmp);
+  }
+  Vec256<float> igammac(const Vec256<float> &x) const {
+    __at_align32__ float tmp[size()];
+    __at_align32__ float tmp_x[size()];
+    store(tmp);
+    x.store(tmp_x);
+    for (int64_t i = 0; i < size(); i++) {
+      tmp[i] = calc_igammac(tmp[i], tmp_x[i]);
+    }
+    return loadu(tmp);
+  }
   Vec256<float> log() const {
     return map(std::log);
   }
@@ -432,14 +452,23 @@ public:
         vsqrtq_f32(values.val[1]));
   }
   Vec256<float> reciprocal() const {
-    return Vec256<float>(
-        vrecpeq_f32(values.val[0]),
-        vrecpeq_f32(values.val[1]));
+    float32x4_t r0 = vrecpeq_f32(values.val[0]);
+    float32x4_t r1 = vrecpeq_f32(values.val[1]);
+    // Run two more Netwon's method iterations to get more accurate results
+    r0 = vmulq_f32(vrecpsq_f32(values.val[0], r0), r0);
+    r0 = vmulq_f32(vrecpsq_f32(values.val[0], r0), r0);
+    r1 = vmulq_f32(vrecpsq_f32(values.val[1], r1), r1);
+    r1 = vmulq_f32(vrecpsq_f32(values.val[1], r1), r1);
+    return Vec256<float>(r0, r1);
   }
   Vec256<float> rsqrt() const {
-    return Vec256<float>(
-        vrsqrteq_f32(values.val[0]),
-        vrsqrteq_f32(values.val[1]));
+    float32x4_t r0 =  vrsqrteq_f32(values.val[0]);
+    float32x4_t r1 =  vrsqrteq_f32(values.val[1]);
+    r0 = vmulq_f32(vrsqrtsq_f32(vmulq_f32(values.val[0], r0), r0), r0);
+    r0 = vmulq_f32(vrsqrtsq_f32(vmulq_f32(values.val[0], r0), r0), r0);
+    r1 = vmulq_f32(vrsqrtsq_f32(vmulq_f32(values.val[1], r1), r1), r1);
+    r1 = vmulq_f32(vrsqrtsq_f32(vmulq_f32(values.val[1], r1), r1), r1);
+    return Vec256<float>(r0, r1);
   }
   Vec256<float> pow(const Vec256<float> &exp) const {
     __at_align32__ float tmp[size()];
@@ -665,6 +694,6 @@ Vec256<float> inline fmadd(const Vec256<float>& a, const Vec256<float>& b, const
   return Vec256<float>(r0, r1);
 }
 
-#endif
+#endif /* defined(aarch64) */
 
 }}}
