@@ -11,12 +11,18 @@ namespace at { namespace native {
 static CPUCapability compute_cpu_capability() {
   auto envar = std::getenv("ATEN_CPU_CAPABILITY");
   if (envar) {
+#ifdef HAVE_VSX_CPU_DEFINITION
+    if (strcmp(envar, "vsx") == 0) {
+      return CPUCapability::VSX;
+    }
+#else
     if (strcmp(envar, "avx2") == 0) {
       return CPUCapability::AVX2;
     }
     if (strcmp(envar, "avx") == 0) {
       return CPUCapability::AVX;
     }
+#endif     
     if (strcmp(envar, "default") == 0) {
       return CPUCapability::DEFAULT;
     }
@@ -33,7 +39,11 @@ static CPUCapability compute_cpu_capability() {
     }
   }
 #endif
+#ifdef HAVE_VSX_CPU_DEFINITION
+  return CPUCapability::VSX;
+#else
   return CPUCapability::DEFAULT;
+#endif
 }
 
 CPUCapability get_cpu_capability() {
