@@ -17,6 +17,7 @@
 #include <ATen/native/TensorIterator.h>
 #include <ATen/native/DistributionTemplates.h>
 #include <ATen/NamedTensorUtils.h>
+#include <ATen/native/cpu/Loops.h>
 
 #include <type_traits>
 #include <functional>
@@ -117,7 +118,7 @@ DEFINE_DISPATCH(bernoulli_tensor_stub);
 DEFINE_DISPATCH(bernoulli_scalar_stub);
 DEFINE_DISPATCH(cauchy_stub);
 DEFINE_DISPATCH(exponential_stub);
-DEFINE_DISPATCH(multinomial_stub);
+DEFINE_DISPATCH(multinomial_with_replacement_stub);
 DEFINE_DISPATCH(geometric_stub);
 DEFINE_DISPATCH(log_normal_stub);
 DEFINE_DISPATCH(uniform_stub);
@@ -300,24 +301,31 @@ Tensor& random_(Tensor& self, int64_t to, c10::optional<Generator> gen) {
 
 Tensor _standard_gamma_grad_cpu(const Tensor& self, const Tensor& output) {
   Tensor ret = at::empty(self.sizes(), self.options());
+  auto iter = TensorIteratorConfig()
+    .add_output(ret)
+    .add_input(self)
+    .add_input(output)
+    .build();
   AT_DISPATCH_FLOATING_TYPES(self.scalar_type(), "_standard_gamma_grad_cpu", [&] {
-    CPU_tensor_apply3<scalar_t, scalar_t, scalar_t>(ret, self, output,
-      [](scalar_t& ret_val, const scalar_t& self_val, const scalar_t &output_val) {
-        ret_val = standard_gamma_grad_one<scalar_t, double>(self_val, output_val);
-      }
-    );
+    cpu_serial_kernel(iter, [](scalar_t self_val, scalar_t output_val) -> scalar_t{
+      return standard_gamma_grad_one<scalar_t, double>(self_val, output_val);
+    });
   });
   return ret;
 }
 
 Tensor _dirichlet_grad_cpu(const Tensor& x, const Tensor& alpha, const Tensor& total) {
   Tensor ret = at::empty(x.sizes(), x.options());
+  auto iter = TensorIteratorConfig()
+    .add_output(ret)
+    .add_input(x)
+    .add_input(alpha)
+    .add_input(total)
+    .build();
   AT_DISPATCH_FLOATING_TYPES(x.scalar_type(), "_dirichlet_grad_cpu", [&] {
-    CPU_tensor_apply4<scalar_t, scalar_t, scalar_t, scalar_t>(ret, x, alpha, total,
-      [](scalar_t& ret_val, const scalar_t& x_val, const scalar_t& alpha_val, const scalar_t& total_val) {
-        ret_val = dirichlet_grad_one<scalar_t, double>(x_val, alpha_val, total_val);
-      }
-    );
+    cpu_serial_kernel(iter, [](scalar_t x_val, scalar_t alpha_val, scalar_t total_val) -> scalar_t{
+      return dirichlet_grad_one<scalar_t, double>(x_val, alpha_val, total_val);
+    });
   });
   return ret;
 }
@@ -328,67 +336,72 @@ Tensor _dirichlet_grad_cpu(const Tensor& x, const Tensor& alpha, const Tensor& t
 
 Tensor _s_binomial_cpu(const Tensor& count, const Tensor& prob, c10::optional<Generator> gen) {
   Tensor ret = at::zeros(count.sizes(), count.options());
+  auto iter = TensorIteratorConfig()
+    .add_output(ret)
+    .add_input(count)
+    .add_input(prob)
+    .build();
   AT_DISPATCH_FLOATING_TYPES(ret.scalar_type(), "binomial_cpu", [&] {
     CPUGeneratorImpl* generator = get_generator_or_default<CPUGeneratorImpl>(gen, detail::getDefaultCPUGenerator());
     // See Note [Acquire lock when using random generators]
     std::lock_guard<std::mutex> lock(generator->mutex_);
-    CPU_tensor_apply3<scalar_t, scalar_t, scalar_t>(ret, count, prob,
-      [generator](scalar_t& ret_val, const scalar_t& count, const scalar_t& prob){
+    cpu_serial_kernel(iter, [generator](scalar_t count_val, scalar_t prob_val) -> scalar_t{
+      auto uniform_lambda = [generator] () {
+        at::uniform_real_distribution<double> standard_uniform(0.0, 1.0);
+        return standard_uniform(generator);
+      };
+      BaseSampler<double, decltype(uniform_lambda)> standard_uniform(uniform_lambda);
 
-        auto uniform_lambda = [generator] () {
-          at::uniform_real_distribution<double> standard_uniform(0.0, 1.0);
-          return standard_uniform(generator);
-        };
-        BaseSampler<double, decltype(uniform_lambda)> standard_uniform(uniform_lambda);
-
-        auto sample = sample_binomial<scalar_t, double, decltype(uniform_lambda)>(count, prob, standard_uniform);
-        ret_val = static_cast<scalar_t>(sample);
-      }
-    );
+      auto sample = sample_binomial<scalar_t, double, decltype(uniform_lambda)>(count_val, prob_val, standard_uniform);
+      return static_cast<scalar_t>(sample);
     });
+  });
   return ret;
 }
 
 Tensor _s_poisson_cpu(const Tensor& lambda, c10::optional<Generator> gen) {
   Tensor ret = at::zeros(lambda.sizes(), lambda.options());
+  auto iter = TensorIteratorConfig()
+    .add_output(ret)
+    .add_input(lambda)
+    .build();
   AT_DISPATCH_FLOATING_TYPES(ret.scalar_type(), "poisson_cpu", [&] {
     CPUGeneratorImpl* generator = get_generator_or_default<CPUGeneratorImpl>(gen, detail::getDefaultCPUGenerator());
     // See Note [Acquire lock when using random generators]
     std::lock_guard<std::mutex> lock(generator->mutex_);
-    CPU_tensor_apply2<scalar_t, scalar_t>(ret, lambda,
-      [generator](scalar_t& ret_val, const scalar_t& lambda){
-        ret_val = static_cast<scalar_t>(sample_poisson(static_cast<double>(lambda), generator));
-      }
-    );
+    cpu_serial_kernel(iter, [generator](scalar_t lambda_val) -> scalar_t{
+      return static_cast<scalar_t>(sample_poisson(static_cast<double>(lambda_val), generator));
     });
+  });
   return ret;
 }
 
 Tensor _s_gamma_cpu(const Tensor& alpha, c10::optional<Generator> gen) {
   Tensor ret = at::zeros(alpha.sizes(), alpha.options());
+  auto iter = TensorIteratorConfig()
+    .add_output(ret)
+    .add_input(alpha)
+    .build();
   AT_DISPATCH_FLOATING_TYPES(ret.scalar_type(), "gamma_cpu", [&] {
     CPUGeneratorImpl* generator = get_generator_or_default<CPUGeneratorImpl>(gen, detail::getDefaultCPUGenerator());
     // See Note [Acquire lock when using random generators]
     std::lock_guard<std::mutex> lock(generator->mutex_);
-    CPU_tensor_apply2<scalar_t, scalar_t>(ret, alpha,
-      [generator](scalar_t& ret_val, const scalar_t& alpha){
+    cpu_serial_kernel(iter, [generator](scalar_t alpha_val) -> scalar_t{
+      auto uniform_lambda = [generator] () {
+        at::uniform_real_distribution<double> standard_uniform(0.0, 1.0);
+        return standard_uniform(generator);
+      };
+      BaseSampler<double, decltype(uniform_lambda)> standard_uniform(uniform_lambda);
 
-        auto uniform_lambda = [generator] () {
-          at::uniform_real_distribution<double> standard_uniform(0.0, 1.0);
-          return standard_uniform(generator);
-        };
-        BaseSampler<double, decltype(uniform_lambda)> standard_uniform(uniform_lambda);
-
-        auto normal_lambda = [generator] () {
-          at::normal_distribution<double> normal(0.0, 1.0);
-          return normal(generator);
-        };
-        BaseSampler<double, decltype(normal_lambda)> standard_normal(normal_lambda);
-        auto sample = sample_gamma<scalar_t, double, decltype(uniform_lambda), decltype(normal_lambda)>(alpha, standard_uniform, standard_normal);
-        ret_val = std::max(std::numeric_limits<scalar_t>::min(), (scalar_t) sample);
-      }
-    );
+      auto normal_lambda = [generator] () {
+        at::normal_distribution<double> normal(0.0, 1.0);
+        return normal(generator);
+      };
+      BaseSampler<double, decltype(normal_lambda)> standard_normal(normal_lambda);
+      auto sample = sample_gamma<scalar_t, double, decltype(uniform_lambda), decltype(normal_lambda)>(alpha_val, standard_uniform, standard_normal);
+      return std::max(std::numeric_limits<scalar_t>::min(), (scalar_t) sample);
     });
+  });
 
   return ret;
 }
@@ -401,35 +414,41 @@ Tensor _s_dirichlet_cpu(const Tensor& alpha, c10::optional<Generator> gen) {
     // See Note [Acquire lock when using random generators]
     std::lock_guard<std::mutex> lock(generator->mutex_);
     /* Generate gamma sample by casting alpha to double to prevent underflow. */
-    CPU_tensor_apply2<double, scalar_t>(gamma, alpha,
-      [generator](double& ret_val, const scalar_t& alpha){
-        auto uniform_lambda = [generator] () {
-          at::uniform_real_distribution<double> standard_uniform(0.0, 1.0);
-          return standard_uniform(generator);
-        };
-        BaseSampler<double, decltype(uniform_lambda)> standard_uniform(uniform_lambda);
+    auto iter1 = TensorIteratorConfig()
+      .add_output(gamma)
+      .add_input(alpha)
+      .check_all_same_dtype(false)
+      .build();
+    cpu_serial_kernel(iter1, [generator](scalar_t alpha_val) -> double{
+      auto uniform_lambda = [generator] () {
+        at::uniform_real_distribution<double> standard_uniform(0.0, 1.0);
+        return standard_uniform(generator);
+      };
+      BaseSampler<double, decltype(uniform_lambda)> standard_uniform(uniform_lambda);
 
-        auto normal_lambda = [generator] () {
-          at::normal_distribution<double> normal(0.0, 1.0);
-          return normal(generator);
-        };
-        BaseSampler<double, decltype(normal_lambda)> standard_normal(normal_lambda);
-        auto sample = sample_gamma<double, double, decltype(uniform_lambda), decltype(normal_lambda)>
-          (alpha, standard_uniform, standard_normal);
-        ret_val = std::max(std::numeric_limits<double>::min(), sample);
-      }
-    );
+      auto normal_lambda = [generator] () {
+        at::normal_distribution<double> normal(0.0, 1.0);
+        return normal(generator);
+      };
+      BaseSampler<double, decltype(normal_lambda)> standard_normal(normal_lambda);
+      auto sample = sample_gamma<double, double, decltype(uniform_lambda), decltype(normal_lambda)>
+        (alpha_val, standard_uniform, standard_normal);
+      return std::max(std::numeric_limits<double>::min(), sample);
+    });
     /* Normalize and cast back to scalar_t. */
     Tensor gamma_sum = gamma.sum(-1, true).expand(alpha.sizes());
-    CPU_tensor_apply3<scalar_t, double , double>(ret, gamma, gamma_sum,
-      [](scalar_t& ret_val, const double& gamma, const double& gamma_sum){
-        ret_val = gamma / gamma_sum;
-        auto min_val = std::numeric_limits<scalar_t>::min();
-        auto max_val = std::nexttoward(static_cast<scalar_t>(1.0f), 0.0f);
-        ret_val = std::min(max_val, std::max(min_val, ret_val));
-        ret_val = static_cast<scalar_t>(ret_val);
-      }
-    );
+    auto iter2 = TensorIteratorConfig()
+      .add_output(ret)
+      .add_input(gamma)
+      .add_input(gamma_sum)
+      .check_all_same_dtype(false)
+      .build();
+    cpu_serial_kernel(iter2, [](double gamma_val, double gamma_sum_val) -> scalar_t{
+      auto ret_val = gamma_val / gamma_sum_val;
+      auto min_val = std::numeric_limits<scalar_t>::min();
+      auto max_val = std::nexttoward(static_cast<scalar_t>(1.0f), 0.0f);
+      return std::min(max_val, std::max(min_val, static_cast<scalar_t>(ret_val)));
+    });
   });
   return ret;
 }
@@ -437,11 +456,21 @@ Tensor _s_dirichlet_cpu(const Tensor& alpha, c10::optional<Generator> gen) {
 /* The largest consecutive integer representable in float32 (2^24) */
 constexpr int64_t FLOAT32_MAX_CONSECUTIVE_INT = 1 << (FLT_MANT_DIG);
 
-Tensor& multinomial_out(Tensor& result, const Tensor& self, int64_t n_sample, bool with_replacement, c10::optional<Generator> gen) {
-  TORCH_CHECK(result.device() == self.device(), "multinomial arguments must have the same device");
-  TORCH_CHECK(self.dim() > 0 && self.dim() <= 2, "prob_dist must be 1 or 2 dim");
-  TORCH_CHECK(at::isFloatingType(self.scalar_type()),
-      "multinomial only supports floating-point dtypes for input, got: ", self.scalar_type());
+Tensor& multinomial_out(
+    Tensor& result,
+    const Tensor& self,
+    int64_t n_sample,
+    bool with_replacement,
+    c10::optional<Generator> gen) {
+  TORCH_CHECK(
+      result.device() == self.device(),
+      "multinomial arguments must have the same device");
+  TORCH_CHECK(
+      self.dim() > 0 && self.dim() <= 2, "prob_dist must be 1 or 2 dim");
+  TORCH_CHECK(
+      at::isFloatingType(self.scalar_type()),
+      "multinomial only supports floating-point dtypes for input, got: ",
+      self.scalar_type());
   TORCH_CHECK(result.scalar_type() == ScalarType::Long,
       "multinomial expects Long tensor out, got: ", result.scalar_type());
   TORCH_CHECK(n_sample > 0, "cannot sample n_sample <= 0 samples");
@@ -450,42 +479,76 @@ Tensor& multinomial_out(Tensor& result, const Tensor& self, int64_t n_sample, bo
       "cannot sample n_sample > prob_dist.size(-1) samples without replacement");
   // Since the index tensor is float, numCategories cannot exceed max
   // float integer precision
-  TORCH_CHECK(n_categories <= FLOAT32_MAX_CONSECUTIVE_INT, "number of categories cannot exceed 2^24");
-  if (self.dim() > 1) {
-    int64_t n_dist = self.size(-2);
-    result.resize_({n_dist, n_sample});
-    if (n_dist == 0) { return result; };
-  } else {
+  TORCH_CHECK(
+      n_categories <= FLOAT32_MAX_CONSECUTIVE_INT,
+      "number of categories cannot exceed 2^24");
+
+  if (self.dim() == 1) {
     result.resize_({n_sample});
+  } else {
+    const int64_t n_dist = self.size(0);
+    result.resize_({n_dist, n_sample});
   }
-  // Fast-path based on RobertoLat example.
+  if (result.numel() == 0) {
+    return result;
+  }
+
+  // Fast-path for no replacement.
   // Reference:
   // https://github.com/pytorch/pytorch/issues/11931#issuecomment-625882503
   // Half is not supported on CPU.
-  if (!with_replacement &&
-      !(self.device().is_cpu() && self.scalar_type() == ScalarType::Half)) {
-    if (result.numel()==0) return result;
+  TORCH_CHECK(
+      !(self.device().is_cpu() && self.scalar_type() == ScalarType::Half),
+      "multinomial is not implemented for half on CPU");
+  if (!with_replacement) {
     // Sanity checks on `self`.
     auto is_valid = ((self.max() < INFINITY) & (self.min() >= 0)).item();
-    TORCH_CHECK(is_valid.to<bool>(), "probability tensor contains either `inf`, `nan` or element < 0");
+    TORCH_CHECK(
+        is_valid.to<bool>(),
+        "probability tensor contains either `inf`, `nan` or element < 0");
     bool zero_prob_condition;
     if (self.dim() == 1){
       zero_prob_condition = (self.sum() == 0).item().to<bool>();
     } else {
       zero_prob_condition = (self.sum(1) == 0).sum().item().to<bool>();
     }
-    TORCH_CHECK(!zero_prob_condition, "invalid multinomial distribution (sum of probabilities <= 0)");
-    auto rand = at::empty_like(self).uniform_(0, 1, gen);
-    rand.log_().div_(self); //save memory with inplace operations
-    auto vals = at::empty(result.sizes(), self.options());
-    at::topk_out(vals, result, rand, n_sample);
+    TORCH_CHECK(
+        !zero_prob_condition,
+        "invalid multinomial distribution (sum of probabilities <= 0)");
+
+    // The algorithm is from gumbel softmax.
+    // s = argmax( logp - log(-log(eps)) ) where eps ~ U(0, 1)
+    // Here we can apply exp to the formula which will not affect result of
+    // argmax or topk. Then we have
+    // s = argmax( p / (-log(eps)) ) where eps ~ U(0, 1).
+    // We can also simplify the formula above by
+    // s = argmax( p / q ) where q ~ Exp(1)
+    Tensor q = at::empty_like(self).exponential_(1, gen);
+    // In theory the probability to generate 0 from exponential distribution is
+    // 0. However, on CUDA side there is a protection to avoid 0s, but on CPU
+    // side, there is a very low probability to generate 0 from
+    // exponential<double>. The probability is about 2^(-DBL_MANT_DIG). We just
+    // ignore it here, but there may be some risk to get invalid output on CPU.
+    at::div_out(q, self, q);
+    if (n_sample == 1) {
+      at::argmax_out(result, q, /*dim=*/-1, /*keepdim=*/true);
+    } else {
+      Tensor vals = at::empty(result.sizes(), self.options());
+      at::topk_out(vals, result, q, n_sample);
+    }
     return result;
   }
-  multinomial_stub(result.device().type(), result, self, n_sample, with_replacement, gen);
+
+  multinomial_with_replacement_stub(
+      result.device().type(), result, self, n_sample, gen);
   return result;
 }
 
-Tensor multinomial(const Tensor& self, int64_t n_sample, bool with_replacement, c10::optional<Generator> gen) {
+Tensor multinomial(
+    const Tensor& self,
+    int64_t n_sample,
+    bool with_replacement,
+    c10::optional<Generator> gen) {
   Tensor result = at::empty({0}, self.options().dtype(kLong));
   native::multinomial_out(result, self, n_sample, with_replacement, gen);
   return result;
