@@ -3,6 +3,7 @@
 #include <ATen/ATen.h>
 #include <ATen/ExpandUtils.h>
 #include <c10/macros/Macros.h>
+#include <c10/util/MathConstants.h>
 
 // ROCM hcc doesn't work well with using std:: in kernel functions
 #if defined(__CUDA_ARCH__)
@@ -276,8 +277,8 @@ C10_DEVICE static inline scalar_t digamma_one(scalar_t x) {
     }
     // it is more standard to write this as recursion, but
     // nvcc does not like that
-    additional_summand = -static_cast<accscalar_t>(M_PI) /
-        compat_tan(static_cast<accscalar_t>(M_PI) * x);
+    additional_summand = -c10::pi<scalar_t> /
+        compat_tan(c10::pi<scalar_t> * x);
     x = 1 - x;
   }
 
