@@ -11,7 +11,6 @@ DEFINE_DISPATCH(max_all_stub);
 DEFINE_DISPATCH(_aminmax_all_stub);
 
 Tensor min(const Tensor &self) {
-  TORCH_CHECK(!self.is_complex(), "min is not yet implemented for complex tensors.");
   TORCH_CHECK(self.numel() > 0, "operation does not have an identity.");
   Tensor result = at::empty({}, self.options());
   min_all_stub(self.device().type(), result, self.contiguous());
@@ -19,7 +18,6 @@ Tensor min(const Tensor &self) {
 }
 
 Tensor max(const Tensor &self) {
-  TORCH_CHECK(!self.is_complex(), "max is not yet implemented for complex tensors.");
   TORCH_CHECK(self.numel() > 0, "operation does not have an identity.");
   Tensor result = at::empty({}, self.options());
   max_all_stub(self.device().type(), result, self.contiguous());
@@ -27,7 +25,6 @@ Tensor max(const Tensor &self) {
 }
 
 std::tuple<Tensor, Tensor> _aminmax_all(const Tensor &self) {
-  TORCH_CHECK(!self.is_complex(), "max is not yet implemented for complex tensors.");
   TORCH_CHECK(self.numel() > 0, "operation does not have an identity.");
   Tensor min_result = at::empty({}, self.options());
   Tensor max_result = at::empty({}, self.options());
