@@ -5,6 +5,21 @@
 
 namespace at { namespace native {
 
+inline int64_t storage_size_for(IntArrayRef size, IntArrayRef stride) {
+  TORCH_INTERNAL_ASSERT_DEBUG_ONLY(size.size() == stride.size(),
+      "storage_size_for(size, stride) requires that size and stride ",
+      "have the same size as a precondition.");
+  int64_t storage_size = 1;
+  for (size_t dim = 0; dim < size.size(); ++dim) {
+    if (size[dim] == 0) {
+      storage_size = 0;
+      break;
+    }
+    storage_size += (size[dim] - 1) * stride[dim];
+  }
+  return storage_size;
+}
+
 inline Tensor& resize_named_tensor_(
     Tensor& self,
     IntArrayRef size,
