@@ -17,7 +17,12 @@
 namespace at {
 namespace native {
 
+// entrance of calculations of `inverse` using cusolver getrf + getrs, cublas getrfBatched + getriBatched
 Tensor _inverse_helper_cuda_lib(const Tensor& self);
+Tensor& _linalg_inv_out_helper_cuda_lib(Tensor& result, Tensor& infos_getrf, Tensor& infos_getrs);
+
+// entrance of calculations of `svd` using cusolver gesvdj and gesvdjBatched
+std::tuple<Tensor, Tensor, Tensor> _svd_helper_cuda_lib(const Tensor& self, bool some, bool compute_uv);
 
 }}  // namespace at::native
 
