@@ -19,7 +19,7 @@ namespace at { namespace native {
 Tensor _cudnn_rnn_flatten_weight(
     TensorList weight_arr, int64_t weight_stride0,
     int64_t input_size,
-    int64_t fn_mode, int64_t fn_hidden_size,
+    int64_t fn_mode, int64_t fn_hidden_size, int64_t fn_proj_size,
     int64_t fn_num_layers, bool batch_first,
     bool fn_bidirectional
     ) {
@@ -30,7 +30,7 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> _cudnn_rnn(
     const Tensor& input_r,
     TensorList weight, int64_t weight_stride0,
     const Tensor& weight_buf_r, const Tensor& hx, const Tensor& cx,
-    int64_t fn_mode, int64_t fn_hidden_size,
+    int64_t fn_mode, int64_t fn_hidden_size, int64_t fn_proj_size,
     int64_t fn_num_layers, bool batch_first, double fn_dropout,
     bool fn_train, bool fn_bidirectional, IntArrayRef fn_batch_sizes,
     const Tensor& fn_dropout_state
@@ -42,7 +42,7 @@ std::tuple<Tensor, Tensor, Tensor, std::vector<Tensor>> _cudnn_rnn_backward(
     const Tensor& input, TensorList weight, int64_t weight_stride0, const Tensor& weight_buf, const Tensor& hx, const Tensor& cx,
     const Tensor& output, const Tensor& grad_output_r, const Tensor& grad_hy_r,
     const Tensor& grad_cy_r,
-    int64_t mode, int64_t hidden_size,
+    int64_t mode, int64_t hidden_size, int64_t proj_size,
     int64_t num_layers, bool batch_first, double dropout,
     bool train, bool bidirectional, IntArrayRef batch_sizes,
     const Tensor& dropout_state, const Tensor& reserve,
@@ -92,6 +92,7 @@ namespace {
 
   struct RNNDescriptorParams {
     int64_t hidden_size;
+    int64_t proj_size;
     int64_t num_layers;
     cudnnDirectionMode_t bidirectional;
     cudnnRNNMode_t mode;
@@ -135,19 +136,19 @@ namespace {
       this->algo = algo;
     }
 
-    void set(int64_t mode, int64_t hidden_size, int64_t num_layers, bool bidirectional, cudnnDataType_t datatype, cudnnDataType_t input_datatype) {
+    void set(int64_t mode, int64_t hidden_size, int64_t proj_size, int64_t num_layers, bool bidirectional, cudnnDataType_t datatype, cudnnDataType_t input_datatype) {
       this->set_mode(mode);
       this->hidden_size = hidden_size;
+      this->proj_size = proj_size;
       this->num_layers = num_layers;
       this->set_bidirectional(bidirectional);
       this->datatype = datatype;
       this->input_datatype = input_datatype;
     }
 
-
     RNNDescriptor descriptor(cudnnHandle_t handle, DropoutDescriptor&& dropout_desc) const {
       RNNDescriptor rnn_desc;
-      rnn_desc.set(handle, hidden_size, num_layers, std::move(dropout_desc), input_mode, bidirectional, mode, datatype, input_datatype, algo, at::globalContext().allowTF32CuDNN());
+      rnn_desc.set(handle, hidden_size, proj_size, num_layers, std::move(dropout_desc), input_mode, bidirectional, mode, datatype, input_datatype, algo, at::globalContext().allowTF32CuDNN());
       return rnn_desc;
     }
 
@@ -359,7 +360,7 @@ namespace {
     size_t weight_size;
     AT_CUDNN_CHECK(cudnnGetRNNParamsSize(handle, rnn_desc.desc(), x_desc.desc(), &weight_size, datatype));
     auto elem_size = dataSize(datatype);
-    AT_ASSERTM(weight_size % elem_size == 0, "cudnnGetRNNParamsSize returned nonsensical weight_size");
+    TORCH_INTERNAL_ASSERT(weight_size % elem_size == 0, "cudnnGetRNNParamsSize returned nonsensical weight_size");
     return weight_size / elem_size;
   }
 
@@ -378,6 +379,58 @@ namespace {
     }
   }
 
+  void add_projection_weights(
+        cudnnHandle_t handle,
+        const RNNDescriptor& rnn_desc,
+        const TensorDescriptor& x_desc,
+        const FilterDescriptor& w_desc,
+        const Tensor& weight_buf,
+        int64_t layer,
+        std::vector<Tensor>& params
+  ) {
+    void* matrix_pointer = nullptr;
+    // assuming it's LSTM which has 8 "linear layers" (i.e. 4 weights and 4 biases)
+    int64_t linear_id = 8;
+    FilterDescriptor lin_layer_mat_desc;
+    AT_CUDNN_CHECK(cudnnGetRNNLinLayerMatrixParams(
+        /*handle=*/handle,
+        /*rnnDesc=*/rnn_desc.desc(),
+        /*layer=*/layer,
+        /*xDesc=*/x_desc.desc(),
+        /*wDesc=*/w_desc.desc(),
+        /*w=*/weight_buf.data_ptr(),
+        /*linLayerID=*/linear_id,
+        /*linLayerMatDesc=*/lin_layer_mat_desc.mut_desc(),
+        /*linLayerMat=*/&matrix_pointer));
+
+    cudnnDataType_t data_type;
+    cudnnTensorFormat_t format;
+    int nb_dims;
+    constexpr int min_dim = 3;
+    int filter_dim_a[min_dim];
+    AT_CUDNN_CHECK(cudnnGetFilterNdDescriptor(
+          lin_layer_mat_desc.desc(),
+          min_dim,
+          &data_type,
+          &format,
+          &nb_dims,
+          filter_dim_a
+          ));
+
+    TORCH_INTERNAL_ASSERT(nb_dims <= min_dim, "nb_dims = ", nb_dims, "; min_dim  = ", min_dim);
+    auto elem_size = dataSize(getCudnnDataType(weight_buf));
+    auto offset_bytes = (char*)matrix_pointer - (char*)weight_buf.data_ptr();
+    TORCH_INTERNAL_ASSERT(offset_bytes % elem_size == 0, "offset_bytes = ", offset_bytes, "; elem_size = ", elem_size);
+    size_t offset = offset_bytes / elem_size;
+
+    int mat_numel = prod_intlist(filter_dim_a, filter_dim_a + nb_dims);
+    // Generate a new parameter tensor which is a view into the weight_buf.
+    std::initializer_list<int64_t> size = {mat_numel, 1};
+    Tensor param = at::empty({0}, weight_buf.options()).set_(weight_buf.storage(), offset, size);
+    params.emplace_back(std::move(param));
+  }
+
+
   /*
     Returns weight and bias tensors for each layer of the RNN. These tensors
     are views on the underlying weight buffer allocated by CuDNN.
@@ -433,24 +486,20 @@ namespace {
           cudnnTensorFormat_t format;
           int nb_dims;
           constexpr int min_dim = 3;
-          // TODO: The use of CPU tensor here is a bit goofy in C++,
-          // some sort of alloca would be good enough except that it is
-          // kind of convenient to be able to prod() on it.
-          Tensor filter_dim_a = at::empty(min_dim, at::initialTensorOptions().dtype(kInt));
+          int filter_dim_a[min_dim];
           AT_CUDNN_CHECK(cudnnGetFilterNdDescriptor(
                 lin_layer_mat_desc.desc(),
                 min_dim,
                 &data_type,
                 &format,
                 &nb_dims,
-                filter_dim_a.data_ptr<int>()
+                filter_dim_a
                 ));
 
-          AT_ASSERTM(nb_dims <= min_dim, "nb_dims = ", nb_dims, "; min_dim  = ", min_dim);
-          filter_dim_a = filter_dim_a.slice(0, 0, nb_dims);
+          TORCH_INTERNAL_ASSERT(nb_dims <= min_dim, "nb_dims = ", nb_dims, "; min_dim  = ", min_dim);
           auto elem_size = dataSize(getCudnnDataType(weight_buf));
           auto offset_bytes = (char*)matrix_pointer - (char*)weight_buf.data_ptr();
-          AT_ASSERTM(offset_bytes % elem_size == 0, "offset_bytes = ", offset_bytes, "; elem_size = ", elem_size);
+          TORCH_INTERNAL_ASSERT(offset_bytes % elem_size == 0, "offset_bytes = ", offset_bytes, "; elem_size = ", elem_size);
           size_t offset = offset_bytes / elem_size;
 
           // for all the RNN types provided by CUDNN, all the ih weights
@@ -458,7 +507,7 @@ namespace {
           // (same for the hh weights, and the ih and hh biases).
           // Since we're storing all the weights in a single tensor anyway,
           // might as well merge the CUDNN ones into a single tensor as well
-          int mat_numel = *filter_dim_a.prod(at::ScalarType::Int).data_ptr<int>();
+          int mat_numel = prod_intlist(filter_dim_a, filter_dim_a + nb_dims);
           if (linear_id == 0 || linear_id == num_linear_layers / 2) {
             // We could also exclude bias params by restricting cudnn_methods to just { cudnnGetRNNLinLayerMatrixParams }
             // at the very top.  However, to do so would throw off the cur_offset account, which is currently a strict
@@ -474,15 +523,20 @@ namespace {
               layer_params_count++;
             }
           } else {
-            AT_ASSERTM(cur_offset == offset, "cur_offset = ", cur_offset, "; offset = ", offset);
+            TORCH_INTERNAL_ASSERT(cur_offset == offset, "cur_offset = ", cur_offset, "; offset = ", offset);
           }
           cur_offset = offset + mat_numel;
         }
       } // for cudnn_method
+      if (rnn.proj_size != 0) {
+        add_projection_weights(handle, rnn_desc, x_desc, w_desc, weight_buf, layer, params);
+        layer_params_count++;
+      }
+
       if (layer == 0) {
         global_layer_params_count = layer_params_count;
       } else {
-        AT_ASSERTM(global_layer_params_count == layer_params_count,
+        TORCH_INTERNAL_ASSERT(global_layer_params_count == layer_params_count,
                    "global_layer_params_count = ", global_layer_params_count,
                    "; layer_params_count = ", layer_params_count);
       }
@@ -502,7 +556,11 @@ namespace {
     int64_t num_dir_layers = rnn.num_directions() * rnn.num_layers;
     const auto cudnn_methods = { cudnnGetRNNLinLayerMatrixParams, cudnnGetRNNLinLayerBiasParams };
     std::vector<void*> data_ptrs;
-    data_ptrs.reserve(num_dir_layers * 2 * 2);
+    if (rnn.proj_size != 0) {
+      data_ptrs.reserve(num_dir_layers * (2 * 2 + 1));
+    } else {
+      data_ptrs.reserve(num_dir_layers * 2 * 2);
+    }
     for (int64_t layer = 0; layer < num_dir_layers; layer++) {
       for (auto cudnn_method : cudnn_methods) {
         // This API returns a separate pointer for weight of every gate,
@@ -526,34 +584,73 @@ namespace {
           data_ptrs.push_back(matrix_pointer);
         }
       }
+      if (rnn.proj_size != 0) {
+        // assuming it's LSTM which has 8 "linear layers" (i.e. 4 weights and 4 biases)
+        int64_t linear_id = 8;
+        FilterDescriptor lin_layer_mat_desc;
+        void* matrix_pointer;
+        AT_CUDNN_CHECK(cudnnGetRNNLinLayerMatrixParams(
+              handle,
+              rnn_desc.desc(),
+              layer,
+              x_desc.desc(),
+              w_desc.desc(),
+              weight_buf.data_ptr(),
+              linear_id,
+              lin_layer_mat_desc.mut_desc(),
+              &matrix_pointer
+              ));
+        data_ptrs.push_back(matrix_pointer);
+      }
     }
     return data_ptrs;
   }
 
+  void _viewOrCopyOneParam(const Tensor& param_from, const Tensor& param_to,
+                          bool copy, bool allow_type_change=false) {
+    // if copying, allow_type_change may be true or false.
+    // if viewing, allow_type_change must be false.
+    TORCH_INTERNAL_ASSERT(copy || !allow_type_change,
+                          "if viewing, type change is not allowed.");
+    TORCH_INTERNAL_ASSERT(allow_type_change || (param_from.scalar_type() == param_to.scalar_type()),
+                          "parameter types mismatch");
+    if (copy) {
+        param_to.copy_(param_from.view_as(param_to));
+    } else {
+        param_from.resize_as_(param_to);
+    }
+  }
+
   void _viewOrCopyParams(MatrixRef<Tensor> params_from, MatrixRef<Tensor> params_to,
                          bool copy, bool allow_type_change=false) {
-    AT_ASSERTM(params_from.size(0) == params_to.size(0), "number of layers mismatch");
+    TORCH_INTERNAL_ASSERT(params_from.size(0) == params_to.size(0), "number of layers mismatch");
     for (size_t i = 0; i < params_from.size(0); i++) {
       auto layer_params_from = params_from[i];
       auto layer_params_to = params_to[i];
       // NOTE: these lists have all weights before all biases, so if the layer
       // doesn't use biases, iteration will terminate once layer_params_from ends
       // and ignore them.
+
+      // NOTE: there is an exception from the above statement. If LSTMs with projections
+      // are used, weights layout will be w_ih, w_hh, b_ih, b_hh, w_hr. So need to handle no-bias
+      // case specially, because will need to copy 0->0, 1->1, 2->4. This case can be uniquely
+      // identified by checking if number of defined parameters for each layer is 3.
+      if (layer_params_from.size() == 3 && layer_params_to.size() != 3) {
+        _viewOrCopyOneParam(layer_params_from[0], layer_params_to[0], copy, allow_type_change);
+        _viewOrCopyOneParam(layer_params_from[1], layer_params_to[1], copy, allow_type_change);
+        _viewOrCopyOneParam(layer_params_from[2], layer_params_to[4], copy, allow_type_change);
+        continue;
+      }
+      if (layer_params_to.size() == 3 && layer_params_from.size() != 3) {
+        _viewOrCopyOneParam(layer_params_from[0], layer_params_to[0], copy, allow_type_change);
+        _viewOrCopyOneParam(layer_params_from[1], layer_params_to[1], copy, allow_type_change);
+        _viewOrCopyOneParam(layer_params_from[4], layer_params_to[2], copy, allow_type_change);
+        continue;
+      }
       for (auto a = layer_params_from.begin(), b = layer_params_to.begin();
-           a != layer_params_from.end() && b != layer_params_to.end();
-           ++a, ++b) {
-        auto param_from = *a, param_to = *b;
-        // if copying, allow_type_change may be true or false.
-        // if viewing, allow_type_change must be false.
-        TORCH_INTERNAL_ASSERT(copy || !allow_type_change,
-                              "if viewing, type change is not allowed.");
-        TORCH_INTERNAL_ASSERT(allow_type_change || (param_from.scalar_type() == param_to.scalar_type()),
-                              "parameter types mismatch");
-        if (copy) {
-            param_to.copy_(param_from.view_as(param_to));
-        } else {
-            param_from.resize_as_(param_to);
-        }
+            a != layer_params_from.end() && b != layer_params_to.end();
+            ++a, ++b) {
+        _viewOrCopyOneParam(*a, *b, copy, allow_type_change);
       }
     }
   }
@@ -576,14 +673,26 @@ namespace {
   }
 
   std::vector<int64_t> _hidden_size(const RNNDescriptorParams& rnn, const TensorDescriptorListParams& tensors) {
+    if (rnn.proj_size != 0) {
+      return {rnn.num_layers * rnn.num_directions(), tensors.mini_batch, rnn.proj_size};
+    } else {
+      return {rnn.num_layers * rnn.num_directions(), tensors.mini_batch, rnn.hidden_size};
+    }
+  }
+
+  std::vector<int64_t> _cell_size(const RNNDescriptorParams& rnn, const TensorDescriptorListParams& tensors) {
     return {rnn.num_layers * rnn.num_directions(), tensors.mini_batch, rnn.hidden_size};
   }
 
   std::vector<int64_t> _output_size(const RNNDescriptorParams& rnn, const TensorDescriptorListParams& tensors) {
+    auto out_size = rnn.hidden_size;
+    if (rnn.proj_size != 0) {
+      out_size = rnn.proj_size;
+    }
     if (tensors.is_input_packed()) {
-      return {tensors.batch_sizes_sum, rnn.hidden_size * rnn.num_directions()};
+      return {tensors.batch_sizes_sum, out_size * rnn.num_directions()};
     } else {
-      return {tensors.seq_length, tensors.mini_batch, rnn.hidden_size * rnn.num_directions()};
+      return {tensors.seq_length, tensors.mini_batch, out_size * rnn.num_directions()};
     }
   }
 
@@ -613,6 +722,11 @@ namespace {
                 (tensors.seq_length >=10 && bsize <=32));
       }
     } else if (prop->major >= 8) {
+      if (prop->minor == 6) {
+        // Excludes sm_86 GPU devices from using persistent rnn.
+        // This is because there are some edge cases that will throw exceptions with cudnn 8.0.5 on Nvidia A40 GPU.
+        return false;
+      }
       // Based on tests by Vasily Volkov and xwang233.  Vasily only tried bsize <= 128,
       // so conservatively enable persistence for bsize <= 128 only.
       // TODO:  Run more tests for bsize > 128.
@@ -622,10 +736,10 @@ namespace {
         return false;
       } else if (rnn.mode == CUDNN_LSTM) {
         // Persistent LSTMs are comparable to or better than non-persistent for bsize <= 128.
-        return bsize <= 128;
+        return (bsize % 8 == 0) && (bsize <= 128);
       } else {
         // Persistent RNN_RELU and TANH show poor performance when bsize >= 96 AND hidden size >= 896.
-        return (bsize <= 128) && (bsize < 96 || rnn.hidden_size < 896);
+        return (bsize % 8 == 0) && (bsize <= 128) && (bsize < 96 || rnn.hidden_size < 896);
       }
     } else {
       return false;
@@ -633,6 +747,11 @@ namespace {
   }
 
   cudnnRNNAlgo_t get_algo(const RNNDescriptorParams& rnn, const TensorDescriptorListParams& tensors, const Tensor input) {
+    // LSTM with projections only works with standard algorithm
+    if (rnn.proj_size != 0) {
+      return CUDNN_RNN_ALGO_STANDARD;
+    }
+
     if (getCudnnDataType(input) == CUDNN_DATA_HALF &&
         !tensors.is_input_packed()) {
       if (use_persist_common_heuristics(rnn, tensors) &&
@@ -655,73 +774,94 @@ namespace {
 // Utilities exposed in RNNUtils.h
 namespace cudnn_rnn {
 
-  TORCH_CUDA_API std::tuple<Tensor, std::vector<Tensor>> copy_weights_to_flat_buf_views(
-      TensorList weight_arr,
-      int64_t weight_stride0,
-      int64_t input_size,
-      int64_t mode,
-      int64_t hidden_size,
-      int64_t num_layers,
-      bool batch_first,
-      bool bidirectional,
-      const cudnnDataType_t flat_buf_datatype,
-      const TensorOptions& flat_buf_options,
-      bool set_orig_weights_to_flat_buf,
-      bool allow_type_change/*=false*/,
-      bool include_bias/*=true*/) {
-    // flat_buf_datatype is accepted as a separate argument (rather than extracted from flat_buf_options)
-    // because to extract flat_buf_datatype from flat_buf_options, we'd need to say
-    // auto flat_buf_datatype = getCudnnDataTypeFromScalarType(typeMetaToScalarType(options.dtype()));
-    // typeMetaToScalarType is a surprisingly nontrivial function.  We should avoid it if we can.
+TORCH_CUDA_CPP_API std::tuple<Tensor, std::vector<Tensor>>
+copy_weights_to_flat_buf_views(
+    TensorList weight_arr,
+    int64_t weight_stride0,
+    int64_t input_size,
+    int64_t mode,
+    int64_t hidden_size,
+    int64_t proj_size,
+    int64_t num_layers,
+    bool batch_first,
+    bool bidirectional,
+    const cudnnDataType_t flat_buf_datatype,
+    const TensorOptions& flat_buf_options,
+    bool set_orig_weights_to_flat_buf,
+    bool allow_type_change /*=false*/,
+    bool include_bias /*=true*/) {
+  // flat_buf_datatype is accepted as a separate argument (rather than extracted
+  // from flat_buf_options) because to extract flat_buf_datatype from
+  // flat_buf_options, we'd need to say auto flat_buf_datatype =
+  // getCudnnDataTypeFromScalarType(typeMetaToScalarType(options.dtype()));
+  // typeMetaToScalarType is a surprisingly nontrivial function.  We should
+  // avoid it if we can.
+  TORCH_CHECK(
+      weight_arr.size() > 0,
+      "copy_weights_to_flat_buf_views: cannot flatten empty weight list");
 
-    TORCH_CHECK(weight_arr.size() > 0,
-                "copy_weights_to_flat_buf_views: cannot flatten empty weight list");
+  RNNDescriptorParams rnn;
+  rnn.set(
+      mode,
+      hidden_size,
+      proj_size,
+      num_layers,
+      bidirectional,
+      promote_rnn_math_type(flat_buf_datatype),
+      flat_buf_datatype);
 
-    RNNDescriptorParams rnn;
-    rnn.set(mode, hidden_size, num_layers, bidirectional, promote_rnn_math_type(flat_buf_datatype), flat_buf_datatype);
+  auto handle = getCudnnHandle();
+  RNNDescriptor rnn_desc = rnn.descriptor(handle);
 
-    auto handle = getCudnnHandle();
-    RNNDescriptor rnn_desc = rnn.descriptor(handle);
+  TensorGeometry x_geom({1, input_size});
+  TensorDescriptor x_desc;
+  // Why do we pad to 5 dims here (and elsewhere)?
+  // https://docs.nvidia.com/deeplearning/sdk/cudnn-api/index.html#cudnnRNNForwardTraining
+  // expects descriptors padded to 3 dimensions.
+  x_desc.set(flat_buf_datatype, x_geom.sizes(), x_geom.strides(), 5);
 
-    TensorGeometry x_geom({1, input_size});
-    TensorDescriptor x_desc;
-    // Why do we pad to 5 dims here (and elsewhere)?
-    // https://docs.nvidia.com/deeplearning/sdk/cudnn-api/index.html#cudnnRNNForwardTraining
-    // expects descriptors padded to 3 dimensions.
-    x_desc.set(flat_buf_datatype, x_geom.sizes(), x_geom.strides(), 5);
+  auto num_weights =
+      get_num_weights(handle, rnn_desc, x_desc, flat_buf_datatype);
+  auto weight_buf = at::zeros(num_weights, flat_buf_options);
 
-    auto num_weights = get_num_weights(handle, rnn_desc, x_desc, flat_buf_datatype);
-    auto weight_buf = at::zeros(num_weights, flat_buf_options);
+  FilterDescriptor w_desc;
+  w_desc.set(weight_buf, 3);
 
-    FilterDescriptor w_desc;
-    w_desc.set(weight_buf, 3);
+  // Slice off views into weight_buf
+  std::vector<Tensor> params_arr;
+  size_t params_stride0;
+  std::tie(params_arr, params_stride0) = get_parameters(
+      handle, rnn, rnn_desc, x_desc, w_desc, weight_buf, include_bias);
+  MatrixRef<Tensor> weight{weight_arr, static_cast<size_t>(weight_stride0)},
+      params{params_arr, params_stride0};
 
-    // Slice off views into weight_buf
-    std::vector<Tensor> params_arr;
-    size_t params_stride0;
-    std::tie(params_arr, params_stride0) = get_parameters(handle, rnn, rnn_desc, x_desc, w_desc, weight_buf, include_bias);
-
-    MatrixRef<Tensor> weight{weight_arr, static_cast<size_t>(weight_stride0)},
-                      params{params_arr, params_stride0};
-
-    // Copy weights
-    _viewOrCopyParams(weight, params, /*copy=*/true, allow_type_change);
-
-    if (set_orig_weights_to_flat_buf) {
-      // Update the storage
-      for (size_t i = 0; i < weight.size(0); i++) {
-        for (auto orig_param_it = weight[i].begin(), new_param_it = params[i].begin();
-             orig_param_it != weight[i].end() && new_param_it != params[i].end();
+  // Copy weights
+  _viewOrCopyParams(weight, params, /*copy=*/true, allow_type_change);
+  if (set_orig_weights_to_flat_buf) {
+    // Update the storage
+    for (size_t i = 0; i < weight.size(0); i++) {
+      // There is a special case for LSTM with projections and no bias,
+      // where weight copy is done in 0->0, 1->1, 2->4 layout
+      if (weight[i].size() == 3 && params[i].size() == 5) {
+        weight[i][0].set_(params[i][0].view_as(weight[i][0]));
+        weight[i][1].set_(params[i][1].view_as(weight[i][1]));
+        weight[i][2].set_(params[i][4].view_as(weight[i][2]));
+      } else {
+        for (auto orig_param_it = weight[i].begin(),
+                  new_param_it = params[i].begin();
+             orig_param_it != weight[i].end() &&
+             new_param_it != params[i].end();
              orig_param_it++, new_param_it++) {
           auto orig_param = *orig_param_it, new_param = *new_param_it;
           orig_param.set_(new_param.view_as(orig_param));
         }
       }
     }
-
-    return std::make_tuple(weight_buf, params_arr);
   }
 
+  return std::make_tuple(weight_buf, params_arr);
+}
+
 } // namespace cudnn_rnn
 
 using namespace cudnn_rnn;
@@ -733,7 +873,7 @@ using namespace cudnn_rnn;
 Tensor _cudnn_rnn_flatten_weight(
     TensorList weight_arr, int64_t weight_stride0,
     int64_t input_size,
-    int64_t fn_mode, int64_t fn_hidden_size,
+    int64_t fn_mode, int64_t fn_hidden_size, int64_t fn_proj_size,
     int64_t fn_num_layers, bool batch_first,
     bool fn_bidirectional
     ) {
@@ -744,6 +884,7 @@ Tensor _cudnn_rnn_flatten_weight(
       input_size,
       fn_mode,
       fn_hidden_size,
+      fn_proj_size,
       fn_num_layers,
       batch_first,
       fn_bidirectional,
@@ -762,12 +903,11 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> _cudnn_rnn(
     const Tensor& input_r,
     TensorList weight, int64_t weight_stride0,
     const Tensor& weight_buf_r, const Tensor& hx, const Tensor& cx,
-    int64_t fn_mode, int64_t fn_hidden_size,
+    int64_t fn_mode, int64_t fn_hidden_size, int64_t fn_proj_size,
     int64_t fn_num_layers, bool batch_first, double fn_dropout,
     bool fn_train, bool fn_bidirectional, IntArrayRef fn_batch_sizes,
     const Tensor& fn_dropout_state
     ) {
-
   check_attributes(input_r, weight, {hx, cx}, /*check_dtype=*/true);
   auto input = input_r;
   auto weight_buf = weight_buf_r;
@@ -781,7 +921,7 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> _cudnn_rnn(
   }
   RNNParams fn;
   auto datatype = getCudnnDataType(input);
-  fn.rnn.set(fn_mode, fn_hidden_size, fn_num_layers, fn_bidirectional, promote_rnn_math_type(datatype), datatype);
+  fn.rnn.set(fn_mode, fn_hidden_size, fn_proj_size, fn_num_layers, fn_bidirectional, promote_rnn_math_type(datatype), datatype);
   fn.dropout.set(fn_train, fn_dropout, fn_dropout_state);
   fn.tensors.set(input.sizes(), fn_batch_sizes, batch_first);
 
@@ -799,6 +939,7 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> _cudnn_rnn(
   }
 
   auto hidden_size = _hidden_size(fn.rnn, fn.tensors);
+  auto cell_size = _cell_size(fn.rnn, fn.tensors);
   auto output_size = _output_size(fn.rnn, fn.tensors);
 
   TORCH_CHECK(hx.is_contiguous(),
@@ -811,7 +952,7 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> _cudnn_rnn(
   auto hy = at::empty(hidden_size, hx.options());
   Tensor cy;
   if (cx.defined()) {
-    cy = at::empty(hidden_size, cx.options());
+    cy = at::empty(cell_size, cx.options());
   } else {
     cy = at::empty({0}, hx.options()); // NB: Not allowed to return undefined tensors
   }
@@ -837,9 +978,8 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> _cudnn_rnn(
     w_desc.set(weight_buf, 3);
   }
 
-  TORCH_CHECK(!cx.defined() || cx.sizes().equals(hidden_size),
-           "Expected cell size ", IntArrayRef{hidden_size}, ", got ", cx.sizes());
-
+  TORCH_CHECK(!cx.defined() || cx.sizes().equals(cell_size),
+          "Expected cell size ", IntArrayRef{cell_size}, ", got ", cx.sizes());
   size_t workspace_size;
   auto x_descs_arr = descs.get_x_descs();
   auto y_descs_arr = descs.get_y_descs();
@@ -851,7 +991,6 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> _cudnn_rnn(
         &workspace_size
         ));
   Tensor workspace = at::empty(workspace_size, input.options().dtype(kByte));
-
   Tensor reserve;
   // NB: Previously, the test was for fn.requires_grad, but we don't have
   // this information.  Use 'train' as a proxy.
@@ -908,7 +1047,7 @@ std::tuple<Tensor, Tensor, Tensor> _cudnn_rnn_backward_input(
     const Tensor& input_r, const Tensor& weight_buf, const Tensor& hx, const Tensor& cx,
     const Tensor& output_r, const Tensor& grad_output_r, const Tensor& grad_hy,
     const Tensor& grad_cy,
-    int64_t fn_mode, int64_t fn_hidden_size,
+    int64_t fn_mode, int64_t fn_hidden_size, int64_t fn_proj_size,
     int64_t fn_num_layers, bool batch_first, double fn_dropout,
     bool fn_train, bool fn_bidirectional, IntArrayRef fn_batch_sizes,
     const Tensor& fn_dropout_state, const Tensor& fn_reserve,
@@ -921,7 +1060,7 @@ std::tuple<Tensor, Tensor, Tensor> _cudnn_rnn_backward_input(
 
   RNNParams fn;
   auto datatype = getCudnnDataType(input);
-  fn.rnn.set(fn_mode, fn_hidden_size, fn_num_layers, fn_bidirectional, promote_rnn_math_type(datatype), datatype);
+  fn.rnn.set(fn_mode, fn_hidden_size, fn_proj_size, fn_num_layers, fn_bidirectional, promote_rnn_math_type(datatype), datatype);
   fn.dropout.set(fn_train, fn_dropout, fn_dropout_state);
   fn.tensors.set(input.sizes(), fn_batch_sizes, batch_first);
 
@@ -942,6 +1081,7 @@ std::tuple<Tensor, Tensor, Tensor> _cudnn_rnn_backward_input(
 
   auto input_size = _input_size(fn.tensors);
   auto hidden_size = _hidden_size(fn.rnn, fn.tensors);
+  auto cell_size = _cell_size(fn.rnn, fn.tensors);
   auto output_size = _output_size(fn.rnn, fn.tensors);
 
   TORCH_CHECK(hx.is_contiguous(),
@@ -955,10 +1095,10 @@ std::tuple<Tensor, Tensor, Tensor> _cudnn_rnn_backward_input(
   auto w = weight_buf;
   auto dx = at::empty(input.sizes(), input.options()); // TODO: more compact way of saying this
   auto dhy = grad_hy.contiguous().view(hidden_size);
-  auto dcy = grad_cy.defined() ? grad_cy.contiguous().view(hidden_size) : Tensor();
+  auto dcy = grad_cy.defined() ? grad_cy.contiguous().view(cell_size) : Tensor();
   auto dhx = at::empty(hidden_size, hx.options());
-  AT_ASSERTM(cx.defined() || !output_mask[2], "illegally required grad of cx for non-LSTM RNN");
-  auto dcx = cx.defined() ? at::empty(hidden_size, cx.options()) : Tensor();
+  TORCH_INTERNAL_ASSERT(cx.defined() || !output_mask[2], "illegally required grad of cx for non-LSTM RNN");
+  auto dcx = cx.defined() ? at::empty(cell_size, cx.options()) : Tensor();
 
   TORCH_CHECK(fn_train,
            "cudnn RNN backward can only be called in training mode");
@@ -970,12 +1110,12 @@ std::tuple<Tensor, Tensor, Tensor> _cudnn_rnn_backward_input(
 
   TORCH_CHECK(!hx.defined() || hx.sizes().equals(hidden_size),
            "Expected hidden size ", IntArrayRef{hidden_size}, ", got ", hx.sizes());
-  TORCH_CHECK(!cx.defined() || cx.sizes().equals(hidden_size),
-           "Expected cell size ", IntArrayRef{hidden_size}, ", got ", cx.sizes());
+  TORCH_CHECK(!cx.defined() || cx.sizes().equals(cell_size),
+           "Expected cell size ", IntArrayRef{cell_size}, ", got ", cx.sizes());
   TORCH_CHECK(!dhy.defined() || dhy.sizes().equals(hidden_size),
            "Expected d_hidden size ", IntArrayRef{hidden_size}, ", got ", dhy.sizes());
-  TORCH_CHECK(!dcy.defined() || dcy.sizes().equals(hidden_size),
-           "Expected d_cell size ", IntArrayRef{hidden_size}, ", got ", dcy.sizes());
+  TORCH_CHECK(!dcy.defined() || dcy.sizes().equals(cell_size),
+           "Expected d_cell size ", IntArrayRef{cell_size}, ", got ", dcy.sizes());
 
   TORCH_CHECK(dhy.is_cuda() && dy.is_cuda() && (!dcy.defined() || dcy.is_cuda()),
            "Gradients aren't CUDA tensors");
@@ -1031,20 +1171,19 @@ std::vector<Tensor> _cudnn_rnn_backward_weight(
     const Tensor& input_r, TensorList weight_arr, int64_t weight_stride0,
     const Tensor& weight_buf, const Tensor& hx, const Tensor& cx,
     const Tensor& output_r,
-    int64_t fn_mode, int64_t fn_hidden_size,
+    int64_t fn_mode, int64_t fn_hidden_size, int64_t fn_proj_size,
     int64_t fn_num_layers, bool batch_first, double fn_dropout,
     bool fn_train, bool fn_bidirectional, IntArrayRef fn_batch_sizes,
     const Tensor& fn_dropout_state, const Tensor& fn_reserve
     ) {
 
   MatrixRef<Tensor> weight{ weight_arr, static_cast<size_t>(weight_stride0) };
-
   auto input = input_r;
   auto output = output_r;
 
   RNNParams fn;
   auto datatype = getCudnnDataType(input);
-  fn.rnn.set(fn_mode, fn_hidden_size, fn_num_layers, fn_bidirectional, promote_rnn_math_type(datatype), datatype);
+  fn.rnn.set(fn_mode, fn_hidden_size, fn_proj_size, fn_num_layers, fn_bidirectional, promote_rnn_math_type(datatype), datatype);
   fn.dropout.set(fn_train, fn_dropout, fn_dropout_state);
   fn.tensors.set(input.sizes(), fn_batch_sizes, batch_first);
 
@@ -1140,7 +1279,7 @@ std::tuple<Tensor, Tensor, Tensor, std::vector<Tensor>> _cudnn_rnn_backward(
     const Tensor& input, TensorList weight, int64_t weight_stride0, const Tensor& weight_buf, const Tensor& hx, const Tensor& cx,
     const Tensor& output, const Tensor& grad_output_r, const Tensor& grad_hy_r,
     const Tensor& grad_cy_r,
-    int64_t mode, int64_t hidden_size,
+    int64_t mode, int64_t hidden_size, int64_t proj_size,
     int64_t num_layers, bool batch_first, double dropout,
     bool train, bool bidirectional, IntArrayRef batch_sizes,
     const Tensor& dropout_state, const Tensor& reserve,
@@ -1156,10 +1295,10 @@ std::tuple<Tensor, Tensor, Tensor, std::vector<Tensor>> _cudnn_rnn_backward(
 
   Tensor dx, dhx, dcx;
   // NB: unconditionally compute this gradient, because it mutates reserve
-  std::tie(dx, dhx, dcx) = at::native::_cudnn_rnn_backward_input(input, weight_buf, hx, cx, output, grad_output, grad_hy, grad_cy, mode, hidden_size, num_layers, batch_first, dropout, train, bidirectional, batch_sizes, dropout_state, reserve, {output_mask[0], output_mask[1], output_mask[2]});
+  std::tie(dx, dhx, dcx) = at::native::_cudnn_rnn_backward_input(input, weight_buf, hx, cx, output, grad_output, grad_hy, grad_cy, mode, hidden_size, proj_size, num_layers, batch_first, dropout, train, bidirectional, batch_sizes, dropout_state, reserve, {output_mask[0], output_mask[1], output_mask[2]});
   std::vector<Tensor> dw;
   if (output_mask[3]) {
-    dw = at::native::_cudnn_rnn_backward_weight(input, weight, weight_stride0, weight_buf, hx, cx, output, mode, hidden_size, num_layers, batch_first, dropout, train, bidirectional, batch_sizes, dropout_state, reserve);
+    dw = at::native::_cudnn_rnn_backward_weight(input, weight, weight_stride0, weight_buf, hx, cx, output, mode, hidden_size, proj_size, num_layers, batch_first, dropout, train, bidirectional, batch_sizes, dropout_state, reserve);
   }
   return std::tuple<Tensor, Tensor, Tensor, std::vector<Tensor>>{dx, dhx, dcx, dw};
 }
@@ -1266,7 +1405,7 @@ DropoutState& get_dropout_state(double dropout_p, bool train, TensorOptions opti
 
 Tensor try_get_weight_buf(
       const Tensor& input, TensorList parameters, bool has_biases,
-      cudnnRNNMode_t mode, int64_t hidden_size, int64_t num_layers, bool bidirectional) {
+      cudnnRNNMode_t mode, int64_t hidden_size, int64_t proj_size, int64_t num_layers, bool bidirectional) {
 
   // Prepare all relevant descriptors
   auto handle = getCudnnHandle();
@@ -1274,7 +1413,7 @@ Tensor try_get_weight_buf(
   auto datatype = getCudnnDataType(any_param);
 
   RNNDescriptorParams rnn;
-  rnn.set(mode, hidden_size, num_layers, bidirectional, promote_rnn_math_type(datatype), datatype);
+  rnn.set(mode, hidden_size, proj_size, num_layers, bidirectional, promote_rnn_math_type(datatype), datatype);
   RNNDescriptor rnn_desc = rnn.descriptor(handle);
 
   TensorGeometry x_geom ({1, input.size(-1)});
@@ -1301,13 +1440,34 @@ Tensor try_get_weight_buf(
 
   int64_t num_parameters = parameters.size();
   int64_t num_ptrs = expected_data_ptrs.size();
-  AT_ASSERT(num_ptrs == (num_parameters * (has_biases ? 1 : 2)));
-  AT_ASSERT(num_ptrs % (has_biases ? 4 : 2) == 0);
-  for (int64_t param_i = 0, ptr_i = 0;
-       ptr_i < num_ptrs;
-       ptr_i += (has_biases ? 2 : 4), param_i += 2) {
-    if (expected_data_ptrs[ptr_i] != parameters[param_i].data_ptr()) return {};
-    if (expected_data_ptrs[ptr_i + 1] != parameters[param_i + 1].data_ptr()) return {};
+  if (proj_size != 0) {
+    AT_ASSERT(num_parameters % (has_biases ? 5 : 3) == 0);
+    AT_ASSERT(num_ptrs % 5 == 0);
+    if (has_biases) {
+      AT_ASSERT(num_ptrs == num_parameters);
+      for (int64_t i = 0; i < num_parameters; i++) {
+        if (expected_data_ptrs[i] != parameters[i].data_ptr()) return {};
+      }
+    } else {
+      AT_ASSERT(num_parameters % 3 == 0);
+      AT_ASSERT(num_ptrs == num_parameters * 5 / 3);
+      for (int64_t param_i = 0, ptr_i = 0;
+          ptr_i < num_ptrs;
+          ptr_i += 5, param_i += 3) {
+        if (expected_data_ptrs[ptr_i] != parameters[param_i].data_ptr()) return {};
+        if (expected_data_ptrs[ptr_i + 1] != parameters[param_i + 1].data_ptr()) return {};
+        if (expected_data_ptrs[ptr_i + 4] != parameters[param_i + 2].data_ptr()) return {};
+      }
+    }
+  } else {
+    AT_ASSERT(num_ptrs == (num_parameters * (has_biases ? 1 : 2)));
+    AT_ASSERT(num_parameters % (has_biases ? 4 : 2) == 0);
+    for (int64_t param_i = 0, ptr_i = 0;
+        ptr_i < num_ptrs;
+        ptr_i += (has_biases ? 2 : 4), param_i += 2) {
+      if (expected_data_ptrs[ptr_i] != parameters[param_i].data_ptr()) return {};
+      if (expected_data_ptrs[ptr_i + 1] != parameters[param_i + 1].data_ptr()) return {};
+    }
   }
   if (!parameters[num_parameters - 1].is_contiguous()) return {};
   return weight_buf;
@@ -1321,22 +1481,32 @@ std::pair<Tensor, hidden_type> _cudnn_impl(
   Tensor hx, cx;
   std::tie(hx, cx) = unpack_hidden(hidden);
   int64_t hidden_size = hx.size(2);
+  int64_t proj_size = 0;
+  // For LSTM models with projections hidden size could be different
+  if (cx.defined() && cx.size(2) != hx.size(2)) {
+    hidden_size = cx.size(2);
+    proj_size = hx.size(2);
+  }
 
   // TODO:  try_get_weight_buf returns a Tensor, but _cudnn_rnn below takes a c10::optional<Tensor>
   // in weight_buf's slot.  Do we want try_get_weight_buf to return a c10::optional<Tensor>
   // instead of a defined or undefined Tensor?
   auto weight_buf = try_get_weight_buf(
-      input, params, has_biases, mode, hidden_size, num_layers, bidirectional);
+      input, params, has_biases, mode, hidden_size, proj_size, num_layers, bidirectional);
 
   TORCH_CHECK(_batch_sizes.dim() == 1, "batch_sizes tensor should be 1D");
   IntArrayRef batch_sizes { _batch_sizes.data_ptr<int64_t>(), static_cast<size_t>(_batch_sizes.size(0)) };
 
   auto & dropout_state = get_dropout_state(dropout_p, train, input.options());
   std::unique_lock<DropoutState> lock { dropout_state };
+  int64_t num_params = has_biases ? 4 : 2;
+  if (proj_size != 0) {
+    ++num_params;
+  }
   // cudnn_output = std::tuple<output, hy, cy, reserve, new_weight_buf>
   auto cudnn_output = at::_cudnn_rnn(
-      input, params, has_biases ? 4 : 2, weight_buf,
-      hx, cx, static_cast<int>(mode), hidden_size, num_layers, /*batch_first=*/false,
+      input, params, num_params, weight_buf,
+      hx, cx, static_cast<int>(mode), hidden_size, proj_size, num_layers, /*batch_first=*/false,
       dropout_p, train, bidirectional, batch_sizes, dropout_state.buffer);
 
   return {std::get<0>(cudnn_output),
@@ -1351,16 +1521,24 @@ std::pair<Tensor, hidden_type> _cudnn_impl(
   Tensor hx, cx;
   std::tie(hx, cx) = unpack_hidden(hidden);
   int64_t hidden_size = hx.size(2);
-
+  int64_t proj_size = 0;
+  // For LSTM models with projections hidden size could be different
+  if (cx.defined() && cx.size(2) != hx.size(2)) {
+    hidden_size = cx.size(2);
+    proj_size = hx.size(2);
+  }
   auto weight_buf = try_get_weight_buf(
-      input, params, has_biases, mode, hidden_size, num_layers, bidirectional);
-
+      input, params, has_biases, mode, hidden_size, proj_size, num_layers, bidirectional);
   auto & dropout_state = get_dropout_state(dropout_p, train, input.options());
   std::unique_lock<DropoutState> lock { dropout_state };
+  int64_t num_params = has_biases ? 4 : 2;
+  if (proj_size != 0) {
+    ++num_params;
+  }
   // cudnn_output = std::tuple<output, hy, cy, reserve, new_weight_buf>
   auto cudnn_output = at::_cudnn_rnn(
-      input, params, has_biases ? 4 : 2, weight_buf,
-      hx, cx, static_cast<int>(mode), hidden_size, num_layers, batch_first, dropout_p,
+      input, params, num_params, weight_buf,
+      hx, cx, static_cast<int>(mode), hidden_size, proj_size, num_layers, batch_first, dropout_p,
       train, bidirectional, /*batch_sizes=*/{}, dropout_state.buffer);
 
   return {std::get<0>(cudnn_output),
