@@ -246,7 +246,7 @@ Tensor permute_wei_for_miopen(Tensor wei, int64_t mode)
 }
 
 void _viewOrCopyParams(MatrixRef<Tensor> params_from, MatrixRef<Tensor> params_to, bool copy) {
-    AT_ASSERTM(params_from.size(0) == params_to.size(0), "number of layers mismatch");
+    TORCH_CHECK(params_from.size(0) == params_to.size(0), "number of layers mismatch");
     for (size_t i = 0; i < params_from.size(0); i++) {
         auto layer_params_from = params_from[i];
         auto layer_params_to = params_to[i];
@@ -257,7 +257,7 @@ void _viewOrCopyParams(MatrixRef<Tensor> params_from, MatrixRef<Tensor> params_t
                 a != layer_params_from.end() && b != layer_params_to.end();
                 ++a, ++b) {
             auto param_from = *a, param_to = *b;
-            AT_ASSERTM(param_from.type() == param_to.type(), "parameter types mismatch");
+            TORCH_CHECK(param_from.type() == param_to.type(), "parameter types mismatch");
             if (copy) {
                 param_to.copy_(param_from.view_as(param_to));
             } else {
@@ -268,7 +268,7 @@ void _viewOrCopyParams(MatrixRef<Tensor> params_from, MatrixRef<Tensor> params_t
 }
 
 void _copyParams_and_permute(MatrixRef<Tensor> params_from, MatrixRef<Tensor> params_to, int64_t mode) {
-    AT_ASSERTM(params_from.size(0) == params_to.size(0), "number of layers mismatch");
+    TORCH_CHECK(params_from.size(0) == params_to.size(0), "number of layers mismatch");
     for (size_t i = 0; i < params_from.size(0); i++) {
         auto layer_params_from = params_from[i];
         auto layer_params_to = params_to[i];
@@ -276,7 +276,7 @@ void _copyParams_and_permute(MatrixRef<Tensor> params_from, MatrixRef<Tensor> pa
                 a != layer_params_from.end() && b != layer_params_to.end();
                 ++a, ++b) {
             auto param_from = *a, param_to = *b;
-            AT_ASSERTM(param_from.type() == param_to.type(), "parameter types mismatch");
+            TORCH_CHECK(param_from.type() == param_to.type(), "parameter types mismatch");
             auto tmp = permute_wei_for_miopen(param_from, mode);
             param_to.copy_(tmp.view_as(param_to));
         }
@@ -297,7 +297,7 @@ int64_t get_num_weights(miopenHandle_t handle, const RNNDescriptor& rnn_desc,
     size_t weight_size;
     MIOPEN_CHECK(miopenGetRNNParamsSize(handle, rnn_desc.desc(), x_desc.desc(), &weight_size, datatype));
     auto element_size = dataSize(datatype);
-    AT_ASSERTM(weight_size % element_size == 0, "miopenGetRNNParamsSize returned nonsensical weight_size.");
+    TORCH_CHECK(weight_size % element_size == 0, "miopenGetRNNParamsSize returned nonsensical weight_size.");
     return weight_size / element_size;
 }
 
@@ -359,7 +359,8 @@ std::pair<std::vector<Tensor>, size_t> get_parameters(miopenHandle_t handle, con
                 params.emplace_back(std::move(param));
                 layer_params_count++;
             } else {
-                AT_ASSERTM(cur_offset == offset, "cur_offset = ", cur_offset, " ; offset = ", offset);
+                TORCH_INTERNAL_ASSERT(cur_offset == offset,
+                                      "cur_offset = ", cur_offset, " ; offset = ", offset);
             }
             cur_offset = offset + param_size;
         }
@@ -392,7 +393,8 @@ std::pair<std::vector<Tensor>, size_t> get_parameters(miopenHandle_t handle, con
                     params.emplace_back(std::move(param));
                     layer_params_count++;
                 } else {
-                    AT_ASSERTM(cur_offset == offset, "cur_offset = ", cur_offset, " ; offset = ", offset);
+                    TORCH_INTERNAL_ASSERT(cur_offset == offset,
+                                          "cur_offset = ", cur_offset, " ; offset = ", offset);
                 }
                 cur_offset = offset + bias_size;
             }
@@ -401,9 +403,9 @@ std::pair<std::vector<Tensor>, size_t> get_parameters(miopenHandle_t handle, con
         if (layer == 0) {
             global_layer_params_count = layer_params_count;
         } else {
-            AT_ASSERTM(global_layer_params_count == layer_params_count,
-                "global_layer_params_count = ", global_layer_params_count,
-                "; layer_params_count = ", layer_params_count);
+            TORCH_INTERNAL_ASSERT(global_layer_params_count == layer_params_count,
+                                  "global_layer_params_count = ", global_layer_params_count,
+                                  "; layer_params_count = ", layer_params_count);
         }
     } // layer
     return std::make_pair(params, global_layer_params_count);
@@ -509,7 +511,6 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> miopen_rnn(
         size_t reserver_size;
         MIOPEN_CHECK(miopenGetRNNTrainingReserveSize(handle, descs.rnn_desc.desc(), fn.tensors.seq_length, x_descs_arr.data(), &reserver_size));
         reserve = at::empty(reserver_size, input.options().dtype(kByte));
-        setMIOpenStreamToCurrent();
         MIOPEN_CHECK(miopenRNNForwardTraining(handle, descs.rnn_desc.desc(), fn.tensors.seq_length,
                 x_descs_arr.data(), x.data_ptr(),
                 descs.hx_desc.desc(), hx.data_ptr(),
@@ -521,7 +522,6 @@ std::tuple<Tensor, Tensor, Tensor, Tensor, Tensor> miopen_rnn(
                 workspace.data_ptr(), workspace_size, reserve.data_ptr(), reserver_size ));
     } else { //Inference.
         reserve = at::empty({0}, input.options().dtype(kByte));
-        setMIOpenStreamToCurrent();
         MIOPEN_CHECK(miopenRNNForwardInference(handle, descs.rnn_desc.desc(), fn.tensors.seq_length,
                 x_descs_arr.data(), x.data_ptr(),
                 descs.hx_desc.desc(), hx.data_ptr(),
@@ -588,7 +588,8 @@ std::tuple<Tensor, Tensor, Tensor, Tensor> miopen_rnn_backward_input(
     auto dhy = grad_hy.contiguous().view(hidden_size);
     auto dcy = grad_cy.defined() ? grad_cy.contiguous().view(hidden_size) : Tensor();
     auto dhx = at::empty(hidden_size, hx.options());
-    AT_ASSERTM(cx.defined() || !output_mask[2], "illegally required grad of cx for non-LSTM RNN");
+    TORCH_INTERNAL_ASSERT(cx.defined() || !output_mask[2],
+                          "illegally required grad of cx for non-LSTM RNN");
     auto dcx = cx.defined() ? at::empty(hidden_size, cx.options()) : Tensor();
 
     TORCH_CHECK(fn_train, "miopen RNN backward can only be called in training mode");
@@ -630,7 +631,6 @@ std::tuple<Tensor, Tensor, Tensor, Tensor> miopen_rnn_backward_input(
         ));
     auto workspace = at::empty(workspace_size, input.options().dtype(kByte));
 
-    setMIOpenStreamToCurrent();
     MIOPEN_CHECK(miopenRNNBackwardData(
         handle,
         descs.rnn_desc.desc(),
@@ -715,7 +715,6 @@ std::vector<Tensor> miopen_rnn_backward_weight(
     auto x_descs_arr = descs.get_x_descs();
     auto y_descs_arr = descs.get_y_descs();
 
-    setMIOpenStreamToCurrent();
     MIOPEN_CHECK(miopenRNNBackwardWeights(
         handle,
         descs.rnn_desc.desc(),
