@@ -3,6 +3,10 @@
 #include <ATen/Tensor.h>
 #include <ATen/vulkan/Context.h>
 
+#ifdef USE_VULKAN_API
+#include <ATen/native/vulkan/api/Context.h>
+#endif /* USE_VULKAN_API */
+
 namespace at {
 namespace vulkan {
 
@@ -23,8 +27,12 @@ at::Tensor& vulkan_copy_(at::Tensor& self, const at::Tensor& src) {
 
 namespace native {
 bool is_vulkan_available() {
+#ifdef USE_VULKAN_API
+  return native::vulkan::api::available();
+#else
   auto p = at::vulkan::g_vulkan_impl_registry.load();
   return p ? p->is_vulkan_available() : false;
+#endif
 }
 } // namespace native
 
