@@ -8,9 +8,18 @@
 // NOTE: functions exist here only to support dispatch via Declarations.cwrap.  You probably don't want to put
 // new functions in here, they should probably be un-genericized.
 
-THC_API void THCTensor_(setStorage)(THCState *state, THCTensor *self, THCStorage *storage_, ptrdiff_t storageOffset_,
-                                    at::IntArrayRef size_, at::IntArrayRef stride_);
+TORCH_CUDA_CU_API void THCTensor_(setStorage)(
+    THCState* state,
+    THCTensor* self,
+    THCStorage* storage_,
+    ptrdiff_t storageOffset_,
+    at::IntArrayRef size_,
+    at::IntArrayRef stride_);
 
-THC_API void THCTensor_(resize)(THCState *state, THCTensor *self, at::IntArrayRef size, at::IntArrayRef stride);
+TORCH_CUDA_CU_API void THCTensor_(resize)(
+    THCState* state,
+    THCTensor* self,
+    at::IntArrayRef size,
+    at::IntArrayRef stride);
 
 #endif
