@@ -1,7 +1,7 @@
 import operator_benchmark as op_bench
 import torch
 import numpy
-from . import configs
+from pt import configs
 
 """EmbeddingBag Operator Benchmark"""
 
@@ -14,13 +14,16 @@ class EmbeddingBagBenchmark(op_bench.TorchBenchmarkBase):
             include_last_offset=include_last_offset,
             sparse=sparse).to(device=device)
         numpy.random.seed((1 << 32) - 1)
-        self.input = torch.tensor(numpy.random.randint(0, embeddingbags, input_size), device=device).long()
         offsets = torch.LongTensor([offset], device=device)
-        self.offset = torch.cat((offsets, torch.tensor([self.input.size(0)], dtype=torch.long)), 0)
+        input = torch.tensor(numpy.random.randint(0, embeddingbags, input_size), device=device).long()
+        self.inputs = {
+            "input": input,
+            "offset": torch.cat((offsets, torch.tensor([input.size(0)], dtype=torch.long)), 0)
+        }
         self.set_module_name('embeddingbag')
 
-    def forward(self):
-        return self.embedding(self.input, self.offset)
+    def forward(self, input, offset):
+        return self.embedding(input, offset)
 
 op_bench.generate_pt_test(configs.embeddingbag_short_configs, EmbeddingBagBenchmark)
 op_bench.generate_pt_gradient_test(configs.embeddingbag_short_configs, EmbeddingBagBenchmark)
