@@ -28,11 +28,13 @@ fill_long_configs = op_bench.cross_product_configs(
 
 class Fill_Benchmark(op_bench.TorchBenchmarkBase):
     def init(self, N, device, dtype):
-        self.input_one = torch.zeros(N, device=device).type(dtype)
+        self.inputs = {
+            "input_one": torch.zeros(N, device=device).type(dtype)
+        }
         self.set_module_name("fill_")
 
-    def forward(self):
-        return self.input_one.fill_(10)
+    def forward(self, input_one):
+        return input_one.fill_(10)
 
 
 op_bench.generate_pt_test(fill_short_configs + fill_long_configs,
