@@ -1,9 +1,9 @@
 
 import operator_benchmark as op_bench
 import torch
-import torch.nn.quantized.dynamic as nnqd
+import torch.nn.quantized as nnq
 import numpy
-from . import configs
+from pt import configs
 
 """
 Microbenchmarks for qEmbeddingBag operators.
@@ -11,7 +11,7 @@ Microbenchmarks for qEmbeddingBag operators.
 
 class QEmbeddingBagBenchmark(op_bench.TorchBenchmarkBase):
     def init(self, embeddingbags, dim, mode, input_size, offset, sparse, include_last_offset, device):
-        self.embedding = nnqd.EmbeddingBag(
+        self.embedding = nnq.EmbeddingBag(
             num_embeddings=embeddingbags,
             embedding_dim=dim,
             mode=mode,
@@ -20,10 +20,14 @@ class QEmbeddingBagBenchmark(op_bench.TorchBenchmarkBase):
         self.input = torch.tensor(numpy.random.randint(0, embeddingbags, input_size), device=device).long()
         offset = torch.LongTensor([offset], device=device)
         self.offset = torch.cat((offset, torch.tensor([self.input.size(0)], dtype=torch.long)), 0)
+        self.inputs = {
+            "input": self.input,
+            "offset": self.offset
+        }
         self.set_module_name('qEmbeddingBag')
 
-    def forward(self):
-        return self.embedding(self.input, self.offset)
+    def forward(self, input, offset):
+        return self.embedding(input, offset)
 
 
 op_bench.generate_pt_test(configs.embeddingbag_short_configs, QEmbeddingBagBenchmark)
