@@ -9,7 +9,7 @@ namespace caffe2 {
 
 class RunCountNetObserver;
 
-class CAFFE2_API RunCountOperatorObserver final
+class TORCH_API RunCountOperatorObserver final
     : public ObserverBase<OperatorBase> {
  public:
   explicit RunCountOperatorObserver(OperatorBase* op) = delete;
@@ -27,7 +27,7 @@ class CAFFE2_API RunCountOperatorObserver final
   RunCountNetObserver* netObserver_;
 };
 
-class CAFFE2_API RunCountNetObserver final
+class TORCH_API RunCountNetObserver final
     : public OperatorAttachingNetObserver<
           RunCountOperatorObserver,
           RunCountNetObserver> {
