@@ -234,7 +234,8 @@ TensorShape& BoundShapeInferencer::CheckAndSetTensorBoundShape(
     bool is_quantized,
     bool allow_existing_shape,
     float scale,
-    int offset) {
+    int offset,
+    bool in_place_op) {
   auto rt = shape_info_.emplace(name, ShapeInfo());
   ShapeInfo& shape_info = rt.first->second;
   TensorShape& shape = shape_info.shape;
@@ -246,8 +247,8 @@ TensorShape& BoundShapeInferencer::CheckAndSetTensorBoundShape(
     shape_info.q_info.offset.push_back(offset);
     shape_info.q_info.axis = 1;
   }
-  // If the shape information exists in shape_info_ already
-  if (!rt.second) {
+  // If the shape information exists in shape_info_ already and we want to compare old/new shapes
+  if (!rt.second && !in_place_op) {
     // Check dim size consistency
     CAFFE_ENFORCE_EQ(
         shape.dims_size(),
@@ -290,13 +291,19 @@ TensorShape& BoundShapeInferencer::CheckAndSetTensorBoundShape(
     return shape;
   }
   // If shape information does not exist in shape_info_,
+  // or shape info is not final,
   // set shape info according to inputs.
-  shape_info.setDimType(t);
-  shape.mutable_dims()->Clear();
-  for (const auto d : bound_dims) {
-    shape.add_dims(d);
+  if (!shape_info.getShapeIsFinal()) {
+    shape_info.setDimType(t);
+    shape.mutable_dims()->Clear();
+    for (const auto d : bound_dims) {
+      shape.add_dims(d);
+    }
+    shape.set_data_type(type);
+    if (in_place_op) {
+      shape_info.setShapeIsFinal(true);
+    }
   }
-  shape.set_data_type(type);
   return shape;
 }
 
@@ -315,6 +322,12 @@ void BoundShapeInferencer::InferGivenTensorFill(const OperatorDef& op) {
   if (it != shape_info_.end()) {
     it->second.setDimType(std::vector<TensorBoundShape::DimType>(
         it->second.shape.dims_size(), TensorBoundShape_DimType_CONSTANT));
+    if (op.type() == "ConstantFill" && op.input_size() >= 1) {
+        auto it_input = shape_info_.find(op.input(0));
+        if (it_input != shape_info_.end()) {
+          it->second.setDimType(it_input->second.getDimType());
+        }
+    }
   }
 }
 
@@ -851,7 +864,12 @@ void BoundShapeInferencer::InferTile(const OperatorDef& op) {
       false);
 }
 
-void BoundShapeInferencer::InferCommonOp(const OperatorDef& op) {
+void BoundShapeInferencer::InferCommonOp(
+  const OperatorDef& op,
+  const OpSchema* schema,
+  bool bypass_input_check,
+  bool in_place_op
+) {
   // First, we need to check that all the input shape/types are already
   // presented
   try {
@@ -859,25 +877,30 @@ void BoundShapeInferencer::InferCommonOp(const OperatorDef& op) {
         types_with_independent_output_shape = {"Int8GenQuantParams",
                                                "Int8QuantSchemeBlobFill",
                                                "ComputeEqualizationScale"};
+    const static std::unordered_set<std::string>
+        pruning_ops = {"RowwisePruneI64", "RowwisePruneI32"};
     std::vector<TensorShape> input_shapes;
     for (const auto& input : op.input()) {
       const auto it = shape_info_.find(input);
       if (it == shape_info_.end() &&
-          !types_with_independent_output_shape.count(op.type())) {
+          !types_with_independent_output_shape.count(op.type()) && !bypass_input_check) {
         LOG(WARNING) << "Cannot find shape info for " << input << ". Skipping "
                      << op.type();
         return;
       }
-      if (types_with_independent_output_shape.count(op.type())) {
+      if (types_with_independent_output_shape.count(op.type()) || (bypass_input_check && it == shape_info_.end())) {
         TensorShape input_shape;
         input_shapes.emplace_back(std::move(input_shape));
-
       } else {
         input_shapes.emplace_back(it->second.shape);
       }
     }
 
-    const OpSchema* schema = OpSchemaRegistry::Schema(op.type());
+    // Schema can be pre-defined.
+    // If not predefined, get the schema for the op.
+    if (schema == nullptr) {
+      schema = OpSchemaRegistry::Schema(op.type());
+    }
     CAFFE_ENFORCE(schema);
     std::vector<TensorShape> output_shapes;
     output_shapes = schema->InferTensor(op, input_shapes);
@@ -923,7 +946,7 @@ void BoundShapeInferencer::InferCommonOp(const OperatorDef& op) {
 
     for (int i = 0; i < output_shapes.size(); i++) {
       const auto& shape = output_shapes[i];
-      if (infered_data_type == TensorProto::UNDEFINED) {
+      if (infered_data_type == TensorProto::UNDEFINED || pruning_ops.find(op.type()) != pruning_ops.end()) {
         infered_data_type = shape.data_type();
       }
       if (shape.unknown_shape()) {
@@ -937,7 +960,8 @@ void BoundShapeInferencer::InferCommonOp(const OperatorDef& op) {
           is_quantized,
           false,
           scale,
-          offset);
+          offset,
+          in_place_op);
     }
   } catch (const caffe2::EnforceNotMet& e) {
     LOG(ERROR) << "Enforce not met while inferring shapes for " << op.type()
