@@ -22,8 +22,7 @@ logger = logging.getLogger(__name__)
 
 
 
-@context.define_context()
-class Job(object):
+class Job(context.Managed):
     """
     A Job defines three TaskGroups: the `init_group`, the `epoch_group` and the
     `exit_group` which will be run by a JobRunner.
@@ -97,11 +96,13 @@ class Job(object):
         self.exit_group = session_class.compile(self.exit_group)
 
     def __enter__(self):
+        super(Job, self).__enter__()
         self.epoch_group.__enter__()
         return self
 
     def __exit__(self, *args):
         self.epoch_group.__exit__()
+        super(Job, self).__exit__(*args)
 
     def add_stop_condition(self, output):
         if isinstance(output, core.BlobReference):
