@@ -211,7 +211,7 @@ void QuantizeConvBias(
     if (use_fp16) {
       bdata_local.resize(bias.numel());
       fbgemm::RoundToFloat16(
-              bdata, bdata_local.data(), bias.numel(), 1 /* FLAGS_caffe2_fbgemm_fake_fp16_clamp */);
+              bdata, bdata_local.data(), bias.numel(), false /* FLAGS_caffe2_fbgemm_fake_fp16_clamp */);
       bdata = bdata_local.data();
     }
     b_quantized.resize(bias.numel());
@@ -856,6 +856,8 @@ REGISTER_EXTERNAL_TENSOR_FUNCTIONS(
     (TypeMeta::Id<Int8ConvDNNLowPPackedWeightBlob>()),
     Int8ConvDNNLowpPackedWeightBlobShapeFunctions);
 
+REGISTER_CPU_OPERATOR(Int8FCPackWeight, FullyConnectedDNNLowPPackWeightOp);
+
 REGISTER_CPU_OPERATOR_WITH_ENGINE(
     Int8FCPackWeight,
     DNNLOWP,
