@@ -11,7 +11,7 @@
 
 namespace caffe2 {
 
-class CAFFE2_API SignalHandler {
+class TORCH_API SignalHandler {
  public:
   enum class Action {
     NONE,
@@ -38,8 +38,8 @@ class CAFFE2_API SignalHandler {
 // This works by setting up certain fatal signal handlers. Previous fatal
 // signal handlers will still be called when the signal is raised. Defaults
 // to being off.
-CAFFE2_API void setPrintStackTracesOnFatalSignal(bool print);
-CAFFE2_API bool printStackTracesOnFatalSignal();
+TORCH_API void setPrintStackTracesOnFatalSignal(bool print);
+TORCH_API bool printStackTracesOnFatalSignal();
 #endif // defined(CAFFE2_SUPPORTS_SIGNAL_HANDLER)
 
 }  // namespace caffe2
