@@ -9,17 +9,17 @@
 
 namespace caffe2 {
 
-CAFFE2_API std::vector<std::string>
+TORCH_API std::vector<std::string>
 split(char separator, const std::string& string, bool ignore_empty = false);
 
-CAFFE2_API std::string trim(const std::string& str);
+TORCH_API std::string trim(const std::string& str);
 
-CAFFE2_API size_t editDistance(
+TORCH_API size_t editDistance(
     const std::string& s1,
     const std::string& s2,
     size_t max_distance = 0);
 
-CAFFE2_API inline bool StartsWith(
+TORCH_API inline bool StartsWith(
     const std::string& str,
     const std::string& prefix) {
   return str.length() >= prefix.length() &&
@@ -27,7 +27,7 @@ CAFFE2_API inline bool StartsWith(
       prefix.end();
 }
 
-CAFFE2_API inline bool EndsWith(
+TORCH_API inline bool EndsWith(
     const std::string& full,
     const std::string& ending) {
   if (full.length() >= ending.length()) {
@@ -39,7 +39,7 @@ CAFFE2_API inline bool EndsWith(
   }
 }
 
-CAFFE2_API int32_t editDistanceHelper(
+TORCH_API int32_t editDistanceHelper(
     const char* s1,
     size_t s1_len,
     const char* s2,
