@@ -1,10 +1,12 @@
 #ifdef USE_CUDA
 
+#include <cmath>
 #include <sstream>
 #include <stdexcept>
-#include "test/cpp/tensorexpr/test_base.h"
 
-#include <cmath>
+#include <gtest/gtest.h>
+
+#include "test/cpp/tensorexpr/test_base.h"
 
 #include <torch/csrc/jit/testing/file_check.h>
 #include "test/cpp/tensorexpr/padded_buffer.h"
@@ -24,7 +26,7 @@ using namespace torch::jit::tensorexpr;
 using namespace torch::jit::tensorexpr;
 
 template <typename ctype>
-void testCudaTestVectorAdd01_impl() {
+static void testCudaTestVectorAdd01_impl() {
   KernelScope kernel_scope;
   const int num_iter = 3;
   const int block_count = 16;
@@ -90,7 +92,7 @@ float sigmoid(float x) {
   return 1.0f / (1.0f + expf(-0.0f - x));
 }
 
-void testCudaSigmoid() {
+TEST(Cuda, Sigmoid_CUDA) {
   KernelScope kernel_scope;
   const int num_iter = 3;
   const int block_count = 16;
@@ -145,7 +147,7 @@ void testCudaSigmoid() {
   cudaFree(c_dev);
 }
 
-void testCudaTestVectorAdd01() {
+TEST(Cuda, TestVectorAdd01_CUDA) {
   // floating types.
   testCudaTestVectorAdd01_impl<float>();
   testCudaTestVectorAdd01_impl<at::Half>();
@@ -215,12 +217,12 @@ static void testCudaTestVectorAdd02_impl(int N, int block_size) {
   cudaFree(c_dev);
 }
 
-void testCudaTestVectorAdd02() {
+TEST(Cuda, TestVectorAdd02_CUDA) {
   testCudaTestVectorAdd02_impl(1024, 128);
   testCudaTestVectorAdd02_impl(1030, 128);
 }
 
-void testCudaHalfCast() {
+TEST(Cuda, HalfCast_CUDA) {
   KernelScope ks;
   auto half = ToDtype<at::Half>();
   Placeholder a("a", half, {4});
@@ -259,7 +261,7 @@ void testCudaHalfCast() {
   cudaFree(bDev);
 }
 
-void testCudaDynamicShape2D() {
+TEST(Cuda, DynamicShape2D_CUDA) {
   KernelScope kernel_scope;
   auto testWithSize = [](int32_t M, int32_t N) {
     VarHandle m("m", kInt);
@@ -322,7 +324,7 @@ void testCudaDynamicShape2D() {
   testWithSize(27, 13);
 }
 
-void testCudaTestRand01() {
+TEST(Cuda, TestRand01_CUDA) {
   KernelScope kernel_scope;
   const int num_iter = 3;
   const int block_count = 16;
@@ -381,7 +383,7 @@ void testCudaTestRand01() {
   cudaFree(c_dev);
 }
 
-void testCudaDynamicShapeSplit() {
+TEST(Cuda, DynamicShapeSplit_CUDA) {
   KernelScope ks;
   constexpr int N = 4096;
   VarHandle n("n", kInt);
@@ -432,7 +434,7 @@ void testCudaDynamicShapeSplit() {
   cudaFree(bDev);
 }
 
-void testCudaOneBlockOneThreadGlobalReduce1() {
+TEST(Cuda, OneBlockOneThreadGlobalReduce1_CUDA) {
   const static int N = 1024;
   KernelScope kernel_scope;
   Placeholder data_buf("data", kFloat, {N});
@@ -499,7 +501,7 @@ void testCudaOneBlockOneThreadGlobalReduce1() {
   cudaFree(output_dev);
 }
 
-void testCudaOneBlockMultiThreadGlobalReduce1() {
+TEST(Cuda, OneBlockMultiThreadGlobalReduce1_CUDA) {
   const static int N = 1024;
   KernelScope kernel_scope;
 
@@ -577,7 +579,7 @@ void testCudaOneBlockMultiThreadGlobalReduce1() {
   cudaFree(b_dev);
 }
 
-void testCudaNoThreadIdxWrite_1() {
+TEST(Cuda, NoThreadIdxWrite_1_CUDA) {
   KernelScope kernel_scope;
 
   // This test does the following reduction:
@@ -674,7 +676,7 @@ void testCudaNoThreadIdxWrite_1() {
   cudaFree(b_dev);
 }
 
-void testCudaSharedMemReduce_1() {
+TEST(Cuda, SharedMemReduce_1_CUDA) {
   // FIXME: this test is flaky in CI.
   KernelScope kernel_scope;
   // This test does the following:
@@ -769,14 +771,14 @@ void testCudaSharedMemReduce_1() {
   // Check the c write is not masked, but the d write is.
   const std::string& verification_pattern =
       R"IR(
-# CHECK: c_ = 0
+# CHECK: c_1 = 0
 # CHECK: for (int m = 0; m < 128
-# CHECK:   c_ = c_ +
+# CHECK:   c_1 = c_1 +
 # CHECK: __syncthreads();
 # CHECK: if (threadIdx.x<1
 # CHECK:   b[blockIdx.x] =
 # CHECK: __syncthreads();
-# CHECK: atomicAdd(&b[blockIdx.x], c_)
+# CHECK: atomicAdd(&b[blockIdx.x], c_1)
 )IR";
 
   torch::jit::testing::FileCheck().run(verification_pattern, oss.str());
@@ -814,7 +816,7 @@ void testCudaSharedMemReduce_1() {
   cudaFree(b_dev);
 }
 
-void testCudaLocalMemReduce_1() {
+TEST(Cuda, LocalMemReduce_1_CUDA) {
   KernelScope kernel_scope;
   // This test does the following:
   //  for k in 0..1:  // block-idx
@@ -927,7 +929,7 @@ void testCudaLocalMemReduce_1() {
   cudaFree(b_dev);
 }
 
-void testCudaHalfSupport() {
+TEST(Cuda, HalfSupport_CUDA) {
   KernelScope ks;
   auto half = ToDtype<at::Half>();
   Placeholder a("a", half, {4});
@@ -985,7 +987,7 @@ void testCudaHalfSupport() {
   cudaFree(dDev);
 }
 
-void testCudaHalfPropagation() {
+TEST(Cuda, HalfPropagation_CUDA) {
   KernelScope kernel_scope;
   auto half = ToDtype<at::Half>();
   Placeholder a("a", half, {4});
@@ -1034,7 +1036,64 @@ void testCudaHalfPropagation() {
   cudaFree(reluDev);
 }
 
-void testCudaPrioritizeDependents() {
+TEST(Cuda, UnusedHalfArgument_CUDA) {
+  KernelScope kernel_scope;
+  Placeholder a("a", kFloat, {4});
+  auto half = ToDtype<at::Half>();
+  Placeholder b("b", half, {4});
+  Tensor* relu = Compute("relu", {{4, "n"}}, [&](const VarHandle& i) {
+    return Max::make(a.load(i), ExprHandle(new FloatImm(0)), true);
+  });
+
+  LoopNest l({relu});
+  l.prepareForCodegen();
+  Stmt* s = l.root_stmt();
+  CudaCodeGen cg(s, {a, b, relu});
+
+  std::ostringstream oss;
+  oss << *cg.stmt();
+
+  // Check the types used by the Max are Float.
+  const std::string& verification_pattern =
+      R"IR(
+# CHECK: for (
+# CHECK:  float v = a[n];
+# CHECK:  relu[n] = Max(v, 0.f
+# CHECK: })IR";
+
+  torch::jit::testing::FileCheck().run(verification_pattern, oss.str());
+
+  // Sanity Cbeck;
+  std::vector<float> aData(4, 2.0f);
+  std::vector<at::Half> bData(4, 2.0f);
+  std::vector<float> reluData(4, 0.0f);
+  at::Half* aDev = nullptr;
+  at::Half* bDev = nullptr;
+  at::Half* reluDev = nullptr;
+  auto aSize = aData.size() * sizeof(aData[0]);
+  auto bSize = bData.size() * sizeof(bData[0]);
+  auto reluSize = reluData.size() * sizeof(reluData[0]);
+
+  cudaMalloc(&aDev, aSize);
+  cudaMalloc(&bDev, bSize);
+  cudaMalloc(&reluDev, reluSize);
+  cudaMemcpy(aDev, aData.data(), aSize, cudaMemcpyHostToDevice);
+  cudaMemcpy(bDev, bData.data(), bSize, cudaMemcpyHostToDevice);
+  cudaMemcpy(reluDev, reluData.data(), reluSize, cudaMemcpyHostToDevice);
+  cudaDeviceSynchronize();
+
+  cg.call({aDev, bDev, reluDev});
+  cudaMemcpy(reluData.data(), reluDev, reluSize, cudaMemcpyDeviceToHost);
+  cudaDeviceSynchronize();
+
+  assertAllEqual(aData, reluData);
+
+  cudaFree(aDev);
+  cudaFree(bDev);
+  cudaFree(reluDev);
+}
+
+TEST(Cuda, PrioritizeDependents_CUDA) {
   KernelScope kernel_scope;
   Placeholder a("a", kFloat, {10});
   Placeholder b("b", kFloat, {12});
@@ -1108,7 +1167,7 @@ void testCudaPrioritizeDependents() {
 
 /// Tests the case where there are two loops which have different extents bound
 /// to the same block dimension. We must mask the smaller extent loop body.
-void testCudaMaskBlockDim() {
+TEST(Cuda, MaskBlockDim_CUDA) {
   KernelScope kernel_scope;
   int A_SIZE = 100;
   int B_SIZE = 50;
@@ -1201,7 +1260,7 @@ void testCudaMaskBlockDim() {
 /// Tests the case with two loops, which have different extents that are bound
 /// to the same thread dimension. This is the same as the above - the smaller
 /// rank write should be masked. But this time we also need to syncthreads.
-void testCudaMaskThreadDim() {
+TEST(Cuda, MaskThreadDim_CUDA) {
   KernelScope kernel_scope;
   int A_SIZE = 50;
   int B_SIZE = 100;
@@ -1296,7 +1355,7 @@ void testCudaMaskThreadDim() {
 /// in distinct dimensions.
 // Note: this is an extremely dumb pattern which we should never see, but is a
 // useful edge case to make sure we've got things covered.
-void testCudaMaskMultiBlockDim() {
+TEST(Cuda, MaskMultiBlockDim_CUDA) {
   KernelScope kernel_scope;
   int A_SIZE = 100;
   int B_SIZE = 50;
@@ -1390,7 +1449,7 @@ void testCudaMaskMultiBlockDim() {
 /// distinct.
 // Note: this is an extremely dumb pattern which we should never see, but is a
 // useful edge case to make sure we've got things covered.
-void testCudaMaskBlockAndThreadDim() {
+TEST(Cuda, MaskBlockAndThreadDim_CUDA) {
   KernelScope kernel_scope;
   int A_SIZE = 100;
   int B_SIZE = 50;
@@ -1482,7 +1541,7 @@ void testCudaMaskBlockAndThreadDim() {
 /// Tests the case where the loopnest has two loops of depth two: each with the
 /// outer loop bound to blockDim.x and the inner loop bound to threadDim.x. In
 /// this case all writes with a rank smaller than the max should be masked.
-void testCudaMaskMultiDim() {
+TEST(Cuda, MaskMultiDim_CUDA) {
   KernelScope kernel_scope;
   int OUTER_SIZE = 10;
   int A_SIZE = 100;
@@ -1612,7 +1671,7 @@ void testCudaMaskMultiDim() {
 // Tests the case where loop extents are symbolic and not known at compile time.
 // In this case both stores must be masked against the extent of the other loop,
 // incase it is larger.
-void testCudaMaskMultiDimSymbolic() {
+TEST(Cuda, MaskMultiDimSymbolic_CUDA) {
   KernelScope kernel_scope;
   VarHandle OUTER_SIZE("OUTER_SIZE", kInt);
   VarHandle A_SIZE("A_SIZE", kInt);
@@ -1748,7 +1807,7 @@ void testCudaMaskMultiDimSymbolic() {
 // bound to the block dimension. Internally the inner loops have different
 // extents but are bound to the same thread dimension. The smaller loop should
 // be masked.
-void testCudaMaskCompoundInnerLoop() {
+TEST(Cuda, MaskCompoundInnerLoop_CUDA) {
   KernelScope kernel_scope;
   int OUTER_SIZE = 10;
   int A_SIZE = 100;
@@ -1885,9 +1944,9 @@ void testCudaMaskCompoundInnerLoop() {
 
 // Tests the case with two loops fused into a common parent, which is not bound
 // to any block or thread dimension - however it's two inner loops are bound to
-// the first thread dimenions. This should work just like the MaskThreadDim test
-// where the bigger loop is unmasked but the smaller is masked.
-void testCudaMaskInnerLoopOneBlock() {
+// the first thread dimensions. This should work just like the MaskThreadDim
+// test where the bigger loop is unmasked but the smaller is masked.
+TEST(Cuda, MaskInnerLoopOneBlock_CUDA) {
   KernelScope kernel_scope;
   int OUTER_SIZE = 10;
   int A_SIZE = 100;
@@ -2026,7 +2085,7 @@ void testCudaMaskInnerLoopOneBlock() {
 // size, but with internal loops bound to different thread rank (ie x and y). In
 // this case both bodies must be masked against the other dimension being > 0.
 // Note: this is a bit degenerate no one would actually write this for perf.
-void testCudaMaskMultiDimMultiAxis() {
+TEST(Cuda, MaskMultiDimMultiAxis_CUDA) {
   KernelScope kernel_scope;
   int OUTER_SIZE = 10;
   int A_SIZE = 30;
@@ -2156,7 +2215,7 @@ void testCudaMaskMultiDimMultiAxis() {
 // Tests the case with two loop nests, each bound to both Block and Thread but
 // the second loop is smaller in both cases - the second store must be masked
 // for both the block and thread dimension.
-void testCudaMaskMultiDimMultiLevel() {
+TEST(Cuda, MaskMultiDimMultiLevel_CUDA) {
   KernelScope kernel_scope;
   int OUTER_A_SIZE = 10;
   int OUTER_B_SIZE = 5;
