@@ -1,12 +1,14 @@
-#include "test/cpp/tensorexpr/test_base.h"
+#include <gtest/gtest.h>
 
-#include "test/cpp/tensorexpr/padded_buffer.h"
-#include "test/cpp/tensorexpr/test_utils.h"
-#include "torch/csrc/jit/tensorexpr/eval.h"
-#include "torch/csrc/jit/tensorexpr/ir.h"
-#include "torch/csrc/jit/tensorexpr/ir_printer.h"
-#include "torch/csrc/jit/tensorexpr/loopnest.h"
-#include "torch/csrc/jit/tensorexpr/tensor.h"
+#include <test/cpp/tensorexpr/test_base.h>
+
+#include <test/cpp/tensorexpr/padded_buffer.h>
+#include <test/cpp/tensorexpr/test_utils.h>
+#include <torch/csrc/jit/tensorexpr/eval.h>
+#include <torch/csrc/jit/tensorexpr/ir.h>
+#include <torch/csrc/jit/tensorexpr/ir_printer.h>
+#include <torch/csrc/jit/tensorexpr/loopnest.h>
+#include <torch/csrc/jit/tensorexpr/tensor.h>
 
 #include <cmath>
 #include <sstream>
@@ -20,7 +22,7 @@ using namespace torch::jit::tensorexpr;
 
 using SimpleIRExprEval = ExprEval<SimpleIREvaluator>;
 
-void testExprBasicValueTest() {
+TEST(Expr, BasicValueTest) {
   KernelScope kernel_scope;
   ExprHandle a = IntImm::make(2), b = IntImm::make(3);
   ExprHandle c = Add::make(a, b);
@@ -28,7 +30,7 @@ void testExprBasicValueTest() {
   ASSERT_EQ(eval.value<int>(), 5);
 }
 
-void testExprBasicValueTest02() {
+TEST(Expr, BasicValueTest02) {
   KernelScope kernel_scope;
   ExprHandle a(2.0f);
   ExprHandle b(3.0f);
@@ -39,7 +41,7 @@ void testExprBasicValueTest02() {
   ASSERT_EQ(eval.value<float>(), -4.0f);
 }
 
-void testExprLetTest01() {
+TEST(Expr, LetTest01) {
   KernelScope kernel_scope;
   VarHandle x("x", kFloat);
   ExprHandle body = ExprHandle(2.f) + (x * ExprHandle(3.f) + ExprHandle(4.f));
@@ -48,7 +50,7 @@ void testExprLetTest01() {
   ASSERT_EQ(eval.value<float>(), 2 + (3 * 3 + 4));
 }
 
-void testExprLetTest02() {
+TEST(Expr, LetTest02) {
   KernelScope kernel_scope;
   VarHandle x("x", kFloat);
   VarHandle y("y", kFloat);
@@ -60,7 +62,7 @@ void testExprLetTest02() {
   ASSERT_EQ(eval.value<float>(), 2 + (3 * 3 + 4 * 6));
 }
 
-void testExprLetStmtTest01() {
+TEST(Expr, LetStmtTest01) {
   KernelScope kernel_scope;
   Placeholder a_buf("a", kFloat, {1});
   Placeholder b_buf("b", kFloat, {1});
@@ -71,7 +73,7 @@ void testExprLetStmtTest01() {
   Stmt* store_b = b_buf.store({0}, var);
   Block* block = Block::make({let_store, store_b});
 
-  SimpleIREvaluator eval(block, a_buf, b_buf);
+  SimpleIREvaluator eval(block, {a_buf, b_buf});
 
   PaddedBuffer<float> a_v(1);
   PaddedBuffer<float> b_v(1);
@@ -84,7 +86,7 @@ void testExprLetStmtTest01() {
   ExpectAllNear(b_v, b_ref, 1e-5);
 }
 
-void testExprIntTest() {
+TEST(Expr, IntTest) {
   KernelScope kernel_scope;
   VarHandle x("x", kInt);
   ExprHandle body = ExprHandle(2) + (x * ExprHandle(3) + ExprHandle(4));
@@ -93,7 +95,7 @@ void testExprIntTest() {
   ASSERT_EQ(eval.value<int>(), 2 + (3 * 3 + 4));
 }
 
-void testExprFloatTest() {
+TEST(Expr, FloatTest) {
   KernelScope kernel_scope;
   VarHandle x("x", kFloat);
   ExprHandle body = ExprHandle(2.f) + (x * ExprHandle(3.f) + ExprHandle(4.f));
@@ -102,7 +104,7 @@ void testExprFloatTest() {
   ASSERT_EQ(eval.value<float>(), 2 + (3 * 3 + 4));
 }
 
-void testExprByteTest() {
+TEST(Expr, ByteTest) {
   KernelScope kernel_scope;
   VarHandle x("x", kByte);
   ExprHandle body = ExprHandle((uint8_t)2) +
@@ -112,7 +114,7 @@ void testExprByteTest() {
   ASSERT_EQ(eval.value<uint8_t>(), 2 + (3 * 3 + 4));
 }
 
-void testExprCharTest() {
+TEST(Expr, CharTest) {
   KernelScope kernel_scope;
   VarHandle x("x", kChar);
   ExprHandle body = ExprHandle((int8_t)2) +
@@ -122,7 +124,7 @@ void testExprCharTest() {
   ASSERT_EQ(eval.value<int8_t>(), 2 + (3 * 3 + 4));
 }
 
-void testExprShortTest() {
+TEST(Expr, ShortTest) {
   KernelScope kernel_scope;
   VarHandle x("x", kShort);
   ExprHandle body = ExprHandle((int16_t)2) +
@@ -132,7 +134,7 @@ void testExprShortTest() {
   ASSERT_EQ(eval.value<int16_t>(), 2 + (3 * 3 + 4));
 }
 
-void testExprLongTest() {
+TEST(Expr, LongTest) {
   KernelScope kernel_scope;
   VarHandle x("x", kLong);
   ExprHandle body = ExprHandle((int64_t)2) +
@@ -142,7 +144,7 @@ void testExprLongTest() {
   ASSERT_EQ(eval.value<int64_t>(), 2 + (3 * 3 + 4));
 }
 
-void testExprHalfTest() {
+TEST(Expr, HalfTest) {
   KernelScope kernel_scope;
   VarHandle x("x", kHalf);
   ExprHandle body = ExprHandle((at::Half)2) +
@@ -152,7 +154,7 @@ void testExprHalfTest() {
   ASSERT_EQ(eval.value<at::Half>(), 2 + (3 * 3 + 4));
 }
 
-void testExprDoubleTest() {
+TEST(Expr, DoubleTest) {
   KernelScope kernel_scope;
   VarHandle x("x", kDouble);
   ExprHandle body = ExprHandle((double)2) +
@@ -162,25 +164,7 @@ void testExprDoubleTest() {
   ASSERT_EQ(eval.value<double>(), 2 + (3 * 3 + 4));
 }
 
-void testExprDisallowBoolArithmetic() {
-  KernelScope kernel_scope;
-  VarHandle x("x", kBool);
-  VarHandle y("y", kBool);
-  std::string error{"arithmetic binary operations on Bool not supported"};
-  ASSERT_THROWS_WITH((x + y), error);
-  ASSERT_THROWS_WITH((x - y), error);
-  ASSERT_THROWS_WITH((x * y), error);
-  ASSERT_THROWS_WITH((x / y), error);
-  ASSERT_THROWS_WITH((x & y), error);
-  ASSERT_THROWS_WITH((x | y), error);
-  ASSERT_THROWS_WITH((x ^ y), error);
-  ASSERT_THROWS_WITH((x << y), error);
-  ASSERT_THROWS_WITH((x >> y), error);
-  ASSERT_THROWS_WITH(Max::make(x, y, /*propagate_nans=*/true), error);
-  ASSERT_THROWS_WITH(Min::make(x, y, /*propagate_nans=*/true), error);
-}
-
-void testExprVectorAdd01() {
+TEST(Expr, VectorAdd01) {
   KernelScope kernel_scope;
   const int kVectorSize = 8;
   const int kVectorCount = 128;
@@ -225,12 +209,12 @@ void testExprVectorAdd01() {
     b_v(i) = i * i * 4;
     c_ref(i) = a_v(i) + b_v(i);
   }
-  SimpleIREvaluator ir_eval(stmt, a_buf, b_buf, c_buf);
+  SimpleIREvaluator ir_eval(stmt, {a_buf, b_buf, c_buf});
   ir_eval(a_v, b_v, c_v);
   ExpectAllNear(c_v, c_ref, 1e-5);
 }
 
-void testExprCompareSelectEQ() {
+TEST(Expr, CompareSelectEQ) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kInt));
@@ -251,7 +235,7 @@ void testExprCompareSelectEQ() {
           CompareSelect::make(
               a.load(i), b.load(i), CompareSelectOperation::kEQ)));
 
-  SimpleIREvaluator ir_eval(memcpy_expr, a, b, c);
+  SimpleIREvaluator ir_eval(memcpy_expr, {a, b, c});
   ir_eval(a_buffer, b_buffer, c_buffer);
 
   ASSERT_EQ(a_buffer.size(), N);
@@ -263,7 +247,7 @@ void testExprCompareSelectEQ() {
   assertAllEqual(c_buffer, 1);
 }
 
-void testExprCompareSelectDtypes() {
+TEST(Expr, CompareSelectDtypes) {
   // LHS and RHS expressions should have the same dtype, but this dtype could
   // differ from the dtype of the return values (but dtypes of true and false
   // return values should be the same).
@@ -296,7 +280,7 @@ void testExprCompareSelectDtypes() {
               FloatImm::make(2.78f),
               CompareSelectOperation::kEQ)));
 
-  SimpleIREvaluator ir_eval(select_expr, a, b, c);
+  SimpleIREvaluator ir_eval(select_expr, {a, b, c});
   ir_eval(a_buffer, b_buffer, c_buffer);
 
   ASSERT_EQ(a_buffer.size(), N);
@@ -308,7 +292,7 @@ void testExprCompareSelectDtypes() {
   ExpectAllNear(c_buffer, c_ref, 1e-7);
 }
 
-void testExprIntrinsicsDtypes() {
+TEST(Expr, IntrinsicsDtypes) {
   KernelScope kernel_scope;
   constexpr int N = 256;
   Placeholder a(BufHandle("A", {N}, kDouble));
@@ -318,9 +302,9 @@ void testExprIntrinsicsDtypes() {
   std::vector<double> b_ref(N, 10.0);
 
   VarHandle i("i", kInt);
-  auto fabs_expr = For::make(i, 0, N, b.store({i}, fabs(a.load(i))));
+  auto abs_expr = For::make(i, 0, N, b.store({i}, tensorexpr::abs(a.load(i))));
 
-  SimpleIREvaluator ir_eval(fabs_expr, a, b);
+  SimpleIREvaluator ir_eval(abs_expr, {a, b});
   ir_eval(a_buffer, b_buffer);
 
   ASSERT_EQ(a_buffer.size(), N);
@@ -330,7 +314,7 @@ void testExprIntrinsicsDtypes() {
   ExpectAllNear(b_buffer, b_ref, 1e-7);
 }
 
-void testExprSubstitute01() {
+TEST(Expr, Substitute01) {
   KernelScope kernel_scope;
   const Var* x = new Var("x", kFloat);
   const Var* y = new Var("y", kFloat);
@@ -351,7 +335,7 @@ void testExprSubstitute01() {
   ASSERT_EQ(e2_str, e2_ref_str);
 }
 
-void testExprMath01() {
+TEST(Expr, Math01) {
   KernelScope kernel_scope;
   ExprHandle v = sin(ExprHandle(1.0f));
 
@@ -365,7 +349,7 @@ void testExprMath01() {
   ASSERT_NEAR(res, v_ref, 1e-6);
 }
 
-void testExprUnaryMath01() {
+TEST(Expr, UnaryMath01) {
   KernelScope kernel_scope;
   struct TestConfig {
     std::function<ExprHandle(const ExprHandle&)> func;
@@ -393,7 +377,7 @@ void testExprUnaryMath01() {
        [](float v) { return std::tanh(v); }},
       {[](const ExprHandle& v) { return exp(v); },
        [](float v) { return std::exp(v); }},
-      {[](const ExprHandle& v) { return fabs(v); },
+      {[](const ExprHandle& v) { return tensorexpr::abs(v); },
        [](float v) { return std::fabs(v); }},
       {[](const ExprHandle& v) { return log(v); },
        [](float v) { return std::log(v); }},
@@ -424,9 +408,15 @@ void testExprUnaryMath01() {
     SimpleIRExprEval eval(v);
     ASSERT_NEAR(eval.value<float>(), v_ref, 1e-6);
   }
+
+  for (float input_v : {std::nan("1"), 0., .5}) {
+    ExprHandle v = FloatImm::make(input_v);
+    SimpleIRExprEval eval(Intrinsics::make(kIsNan, v));
+    ASSERT_NEAR(eval.value<int>(), std::isnan(input_v), 0);
+  }
 }
 
-void testExprBinaryMath01() {
+TEST(Expr, BinaryMath01) {
   KernelScope kernel_scope;
   struct TestConfig {
     std::function<ExprHandle(const ExprHandle&, const ExprHandle&)> func;
@@ -450,7 +440,7 @@ void testExprBinaryMath01() {
   }
 }
 
-void testExprBitwiseOps() {
+TEST(Expr, BitwiseOps) {
   KernelScope kernel_scope;
   ExprHandle a(59);
   ExprHandle b(11);
@@ -462,7 +452,7 @@ void testExprBitwiseOps() {
   ASSERT_EQ(eval.value<int>(), 11);
 }
 
-void testExprDynamicShapeAdd() {
+TEST(Expr, DynamicShapeAdd) {
   KernelScope kernel_scope;
   auto testWithSize = [](int32_t size) {
     VarHandle n("n", kInt);
@@ -474,7 +464,7 @@ void testExprDynamicShapeAdd() {
     std::vector<float> aData(size, 1.0f);
     std::vector<float> bData(size, 2.0f);
     std::vector<float> cData(size, 0.0f);
-    SimpleIREvaluator(s, a, b, c, n)(aData, bData, cData, size);
+    SimpleIREvaluator(s, {a, b, c, n})(aData, bData, cData, size);
     ExpectAllNear(cData, std::vector<float>(size, 3.0f), 1e-7);
   };
   testWithSize(1);
@@ -493,7 +483,7 @@ void testCond01() {
   ExprHandle even_cond = CompareSelect::make(Mod::make(index, 2), 0, kEQ);
   Stmt* assign = Cond::make(even_cond, assign_x2, assign_x3);
   Stmt* for_stmt = For::make(index, 0, N, assign);
-  SimpleIREvaluator(for_stmt, a_buf)(a_v);
+  SimpleIREvaluator(for_stmt, {a_buf})(a_v);
 
   PaddedBuffer<float> a_ref(N);
   for (int i = 0; i < N; i++) {
@@ -555,8 +545,8 @@ void testStmtClone() {
   Stmt* cloned_loop = Stmt::clone(loop);
   std::vector<int> orig_loop_results(N);
   std::vector<int> cloned_loop_results(N);
-  SimpleIREvaluator(loop, a_buf)(orig_loop_results);
-  SimpleIREvaluator(cloned_loop, a_buf)(cloned_loop_results);
+  SimpleIREvaluator(loop, {a_buf})(orig_loop_results);
+  SimpleIREvaluator(cloned_loop, {a_buf})(cloned_loop_results);
 
   assertAllEqual(orig_loop_results, 5);
   assertAllEqual(cloned_loop_results, 5);
@@ -570,8 +560,8 @@ void testStmtClone() {
 
   std::vector<int> orig_loop_results_after_mutation(N);
   std::vector<int> cloned_loop_results_after_mutation(N);
-  SimpleIREvaluator(loop, a_buf)(orig_loop_results_after_mutation);
-  SimpleIREvaluator(cloned_loop, a_buf)(cloned_loop_results_after_mutation);
+  SimpleIREvaluator(loop, {a_buf})(orig_loop_results_after_mutation);
+  SimpleIREvaluator(cloned_loop, {a_buf})(cloned_loop_results_after_mutation);
 
   assertAllEqual(orig_loop_results_after_mutation, 5);
   assertAllEqual(cloned_loop_results_after_mutation, 33);
