@@ -1,15 +1,17 @@
 #ifdef TORCH_ENABLE_LLVM
-#include "test/cpp/tensorexpr/test_base.h"
+#include <gtest/gtest.h>
 
-#include "test/cpp/tensorexpr/padded_buffer.h"
-#include "test/cpp/tensorexpr/test_utils.h"
-#include "torch/csrc/jit/tensorexpr/eval.h"
-#include "torch/csrc/jit/tensorexpr/ir.h"
-#include "torch/csrc/jit/tensorexpr/ir_printer.h"
-#include "torch/csrc/jit/tensorexpr/ir_simplifier.h"
-#include "torch/csrc/jit/tensorexpr/llvm_codegen.h"
-#include "torch/csrc/jit/tensorexpr/loopnest.h"
-#include "torch/csrc/jit/tensorexpr/tensor.h"
+#include <test/cpp/tensorexpr/test_base.h>
+
+#include <test/cpp/tensorexpr/padded_buffer.h>
+#include <test/cpp/tensorexpr/test_utils.h>
+#include <torch/csrc/jit/tensorexpr/eval.h>
+#include <torch/csrc/jit/tensorexpr/ir.h>
+#include <torch/csrc/jit/tensorexpr/ir_printer.h>
+#include <torch/csrc/jit/tensorexpr/ir_simplifier.h>
+#include <torch/csrc/jit/tensorexpr/llvm_codegen.h>
+#include <torch/csrc/jit/tensorexpr/loopnest.h>
+#include <torch/csrc/jit/tensorexpr/tensor.h>
 
 #include <cmath>
 #include <numeric>
@@ -17,7 +19,6 @@
 namespace torch {
 namespace jit {
 using namespace torch::jit::tensorexpr;
-using namespace torch::jit::tensorexpr;
 
 using LLVMExprEval = ExprEval<LLVMCodeGen>;
 
@@ -33,7 +34,7 @@ using LLVMExprEval = ExprEval<LLVMCodeGen>;
   _(at::Half, Half, 0.128f)
 
 #define IMM_TEST(Type, Name, Val)                  \
-  void testLLVM##Name##ImmTest() {                 \
+  TEST(LLVM, Name##ImmTest) {                      \
     KernelScope kernel_scope;                      \
     auto a = Name##Imm::make(Val);                 \
     LLVMExprEval cg(a);                            \
@@ -47,7 +48,7 @@ TEST_LLVM_SCALAR_TYPES(IMM_TEST)
 #undef IMM_TEST
 
 #define ADD_TEST(Type, Name, Val)                  \
-  void testLLVM##Name##AddTest() {                 \
+  TEST(LLVM, Name##AddTest) {                      \
     KernelScope kernel_scope;                      \
     auto a = Name##Imm::make(Val);                 \
     auto b = Name##Imm::make(Val * 2);             \
@@ -63,7 +64,7 @@ TEST_LLVM_SCALAR_TYPES(ADD_TEST)
 #undef ADD_TEST
 
 #define SUB_TEST(Type, Name, Val)                  \
-  void testLLVM##Name##SubTest() {                 \
+  TEST(LLVM, Name##SubTest) {                      \
     KernelScope kernel_scope;                      \
     auto a = Name##Imm::make(Val * 2);             \
     auto b = Name##Imm::make(Val);                 \
@@ -79,7 +80,7 @@ TEST_LLVM_SCALAR_TYPES(SUB_TEST)
 #undef SUB_TEST
 
 #define MUL_TEST(Type, Name, Val)                  \
-  void testLLVM##Name##MulTest() {                 \
+  TEST(LLVM, Name##MulTest) {                      \
     KernelScope kernel_scope;                      \
     auto a = Name##Imm::make(Val);                 \
     auto b = Name##Imm::make((Type)4);             \
@@ -95,7 +96,7 @@ TEST_LLVM_SCALAR_TYPES(MUL_TEST)
 #undef MUL_TEST
 
 #define DIV_TEST(Type, Name, Val)                  \
-  void testLLVM##Name##DivTest() {                 \
+  TEST(LLVM, Name##DivTest) {                      \
     KernelScope kernel_scope;                      \
     auto a = Name##Imm::make((Type)6);             \
     auto b = Name##Imm::make((Type)3);             \
@@ -110,7 +111,7 @@ TEST_LLVM_SCALAR_TYPES(MUL_TEST)
 TEST_LLVM_SCALAR_TYPES(DIV_TEST)
 #undef DIV_TEST
 
-void testLLVMIntToFloatCastTest() {
+TEST(LLVM, IntToFloatCastTest) {
   KernelScope kernel_scope;
   auto a = IntImm::make(2);
   auto b = Cast::make(kFloat, a);
@@ -118,7 +119,7 @@ void testLLVMIntToFloatCastTest() {
   ASSERT_EQ(cg.value<float>(), 2.0);
 }
 
-void testLLVMFloatToIntCastTest() {
+TEST(LLVM, FloatToIntCastTest) {
   KernelScope kernel_scope;
   auto a = FloatImm::make(2.0);
   auto b = Cast::make(kInt, a);
@@ -126,7 +127,7 @@ void testLLVMFloatToIntCastTest() {
   ASSERT_EQ(cg.value<int>(), 2);
 }
 
-void testLLVMIntToLongCastTest() {
+TEST(LLVM, IntToLongCastTest) {
   KernelScope kernel_scope;
   auto a = IntImm::make(12345);
   auto b = Cast::make(kLong, a);
@@ -134,7 +135,7 @@ void testLLVMIntToLongCastTest() {
   ASSERT_EQ(cg.value<int64_t>(), 12345);
 }
 
-void testLLVMByteToCharCastTest() {
+TEST(LLVM, ByteToCharCastTest) {
   KernelScope kernel_scope;
   auto a = ByteImm::make(250);
   auto b = Cast::make(kChar, a);
@@ -142,7 +143,7 @@ void testLLVMByteToCharCastTest() {
   ASSERT_EQ(cg.value<int8_t>(), (int8_t)250);
 }
 
-void testLLVMHalfToLongCastTest() {
+TEST(LLVM, HalfToLongCastTest) {
   KernelScope kernel_scope;
   auto a = HalfImm::make(2.0);
   auto b = Cast::make(kLong, a);
@@ -150,7 +151,7 @@ void testLLVMHalfToLongCastTest() {
   ASSERT_EQ(cg.value<int64_t>(), 2);
 }
 
-void testLLVMByteToDoubleCastTest() {
+TEST(LLVM, ByteToDoubleCastTest) {
   KernelScope kernel_scope;
   auto a = ByteImm::make(2);
   auto b = Cast::make(kDouble, a);
@@ -158,7 +159,96 @@ void testLLVMByteToDoubleCastTest() {
   ASSERT_EQ(cg.value<double>(), 2);
 }
 
-void testLLVMLetTest01() {
+TEST(LLVM, BitCast) {
+  constexpr int16_t ref16 = 1337;
+  constexpr int32_t ref32 = 1337;
+  constexpr int64_t ref64 = 1337;
+  at::Half reff16 = 1337.0f;
+  constexpr float reff32 = 1337.0f;
+  constexpr double reff64 = 1337.0f;
+
+  // this is broken
+  /*{
+    KernelScope kernel_scope;
+    at::Half k_;
+    at::Half* k = &k_;
+    *reinterpret_cast<int16_t*>(k) = ref16;
+    auto a = HalfImm::make(k);
+    auto b = BitCast::make(kShort, a);
+    LLVMExprEval cg(b);
+    ASSERT_EQ(cg.value<int16_t>(), ref16);
+  }*/
+
+  {
+    KernelScope kernel_scope;
+    float k = raw_bitcast<float>(ref32);
+    auto a = FloatImm::make(k);
+    auto b = BitCast::make(kInt, a);
+    LLVMExprEval cg(b);
+    ASSERT_EQ(cg.value<int32_t>(), ref32);
+  }
+
+  {
+    KernelScope kernel_scope;
+    double k = raw_bitcast<double>(ref64);
+    auto a = DoubleImm::make(k);
+    auto b = BitCast::make(kLong, a);
+    LLVMExprEval cg(b);
+    ASSERT_EQ(cg.value<int64_t>(), ref64);
+  }
+
+  {
+    KernelScope kernel_scope;
+    int64_t k = raw_bitcast<int64_t>(reff64);
+    auto a = LongImm::make(k);
+    auto b = BitCast::make(kDouble, a);
+    LLVMExprEval cg(b);
+    ASSERT_EQ(cg.value<double>(), reff64);
+  }
+
+  {
+    KernelScope kernel_scope;
+    int32_t k = raw_bitcast<int32_t>(reff32);
+    auto a = IntImm::make(k);
+    auto b = BitCast::make(kFloat, a);
+    LLVMExprEval cg(b);
+    ASSERT_EQ(cg.value<float>(), reff32);
+  }
+}
+
+TEST(LLVM, fastLogFloat) {
+  KernelScope kernel_scope;
+  const int kTotalSize = 128 * 128;
+  Placeholder a_buf(BufHandle("A", {ExprHandle(kTotalSize)}, kFloat));
+  Placeholder b_buf(BufHandle("B", {ExprHandle(kTotalSize)}, kFloat));
+
+  VarHandle index = VarHandle("index", kInt);
+  ExprHandle load_a = a_buf.load(index);
+  Stmt* store_b = b_buf.store({index}, fast_log(load_a));
+  Stmt* stmt = For::make(index, 0, kTotalSize, store_b);
+
+  PaddedBuffer<float> a_v(kTotalSize);
+  PaddedBuffer<float> b_v(kTotalSize);
+
+  for (int i = 0; i < kTotalSize; ++i) {
+    a_v(i) = at::randn({1}).item().to<float>();
+  }
+
+  LLVMCodeGen ir_eval(stmt, {a_buf, b_buf});
+  ir_eval.call({a_v, b_v});
+
+  for (int i = 0; i < kTotalSize; ++i) {
+    auto test = b_v(i);
+    auto ref = std::log(a_v(i));
+    if (std::isnan(ref)) {
+      ASSERT_EQ(std::isnan(test), true);
+    } else {
+      ASSERT_FLOAT_EQ(test, ref);
+    }
+  }
+}
+
+TEST(LLVM, LetTest01) {
   KernelScope kernel_scope;
 
   Placeholder a(BufHandle("A", {1}, kFloat));
@@ -175,7 +265,7 @@ void testLLVMLetTest01() {
   ASSERT_EQ(v[0], 2.f + 3.f * 3.f + 4.f);
 }
 
-void testLLVMLetTest02() {
+TEST(LLVM, LetTest02) {
   KernelScope kernel_scope;
 
   Placeholder a(BufHandle("A", {1}, kFloat));
@@ -195,7 +285,7 @@ void testLLVMLetTest02() {
   ASSERT_EQ(v[0], 2.f + 3.f * 3.f + 6.f * 4.f);
 }
 
-void testLLVMLetTestMultitype() {
+TEST(LLVM, LetTestMultitype) {
   KernelScope kernel_scope;
 
   Placeholder a(BufHandle("A", {1}, kDouble));
@@ -203,22 +293,22 @@ void testLLVMLetTestMultitype() {
   std::vector<void*> args({v.data()});
   VarHandle x("x", kByte);
   VarHandle y("y", kHalf);
-  auto block =
-      Block::make({Let::make(x, 3),
-                   Let::make(y, 6.f),
-                   a.store(
-                       {0},
-                       Cast::make(
-                           kDouble,
-                           ExprHandle(2.f) +
-                               (x * ExprHandle(3.f) + y * ExprHandle(4.f))))});
+  auto block = Block::make(
+      {Let::make(x, 3),
+       Let::make(y, 6.f),
+       a.store(
+           {0},
+           Cast::make(
+               kDouble,
+               ExprHandle(2.f) +
+                   (x * ExprHandle(3.f) + y * ExprHandle(4.f))))});
 
   LLVMCodeGen cg(block, {a});
   ASSERT_EQ(cg.value<int>(args), 0);
   ASSERT_EQ(v[0], 2.f + 3 * 3.f + 6.f * 4.f);
 }
 
-void testLLVMBufferTest() {
+TEST(LLVM, BufferTest) {
   KernelScope kernel_scope;
   Placeholder a(BufHandle("A", {32}, kFloat));
   std::vector<int32_t> v(5);
@@ -228,7 +318,7 @@ void testLLVMBufferTest() {
   ASSERT_EQ(cg.value<int>(args), 0);
 }
 
-void testLLVMBlockTest() {
+TEST(LLVM, BlockTest) {
   KernelScope kernel_scope;
   Placeholder a(BufHandle("A", {32}, kInt));
   std::vector<int32_t> v = {1, 2};
@@ -246,7 +336,7 @@ void testLLVMBlockTest() {
   ASSERT_EQ(v[1], 4);
 }
 
-void testLLVMLoadStoreTest() {
+TEST(LLVM, LoadStoreTest) {
   KernelScope kernel_scope;
   Placeholder a(BufHandle("A", {1}, kInt));
   Placeholder b(BufHandle("B", {1}, kInt));
@@ -261,7 +351,7 @@ void testLLVMLoadStoreTest() {
   ASSERT_EQ(b_buffer[0], 42);
 }
 
-void testLLVMIfThenElseTest() {
+TEST(LLVM, IfThenElseTest) {
   KernelScope kernel_scope;
   Placeholder a(BufHandle("A", {1}, kInt));
   Placeholder b(BufHandle("B", {1}, kInt));
@@ -278,7 +368,107 @@ void testLLVMIfThenElseTest() {
   ASSERT_EQ(b_buffer[0], 42);
 }
 
-void testLLVMVecLoadStoreTest() {
+// if (x < 10) x = x + 1
+TEST(LLVM, CondNoFalseBlockTest) {
+  KernelScope kernel_scope;
+
+  Placeholder x(BufHandle("X", {1}, kInt));
+  auto cmp = CompareSelect::make(x.load(0), 10, CompareSelectOperation::kLT);
+  auto cond = Cond::make(cmp, x.store({0}, x.load(0) + 1), nullptr);
+
+  for (int32_t x_value : {0, 10, 20}) {
+    std::vector<int32_t> x_buffer = {x_value};
+    std::vector<void*> args({x_buffer.data()});
+    LLVMCodeGen cg(cond, {x});
+    ASSERT_EQ(cg.value<int>(args), 0);
+    if (x_value < 10) {
+      ASSERT_EQ(x_buffer[0], x_value + 1);
+    } else {
+      ASSERT_EQ(x_buffer[0], x_value);
+    }
+  }
+}
+
+// if (x < 10) {
+//   x = x + 1;
+// } else {
+//   x = x - 1;
+// }
+TEST(LLVM, CondTest) {
+  KernelScope kernel_scope;
+
+  Placeholder x(BufHandle("X", {1}, kInt));
+  auto cmp = CompareSelect::make(x.load(0), 10, CompareSelectOperation::kLT);
+  auto cond =
+      Cond::make(cmp, x.store({0}, x.load(0) + 1), x.store({0}, x.load(0) - 1));
+  auto block = Block::make({
+      cond,
+      x.store({0}, x.load(0) * 2),
+  });
+
+  for (int32_t x_value : {0, 10, 20}) {
+    std::vector<int32_t> x_buffer = {x_value};
+    std::vector<void*> args({x_buffer.data()});
+    LLVMCodeGen cg(block, {x});
+    ASSERT_EQ(cg.value<int>(args), 0);
+    if (x_value < 10) {
+      ASSERT_EQ(x_buffer[0], (x_value + 1) * 2);
+    } else {
+      ASSERT_EQ(x_buffer[0], (x_value - 1) * 2);
+    }
+  }
+}
+
+// if (x < 10) {
+//   if (x > 5) {
+//     x = x + 1;
+//   } else {
+//     x = x - 1;
+//   }
+// } else {
+//   if (x <= 15) {
+//     x = x + 2;
+//   } else {
+//     x = x - 2;
+//   }
+// }
+TEST(LLVM, CondNestedTest) {
+  KernelScope kernel_scope;
+
+  Placeholder x(BufHandle("X", {1}, kInt));
+  auto true_cmp =
+      CompareSelect::make(x.load(0), 5, CompareSelectOperation::kGT);
+  auto true_cond = Cond::make(
+      true_cmp, x.store({0}, x.load(0) + 1), x.store({0}, x.load(0) - 1));
+  auto false_cmp =
+      CompareSelect::make(x.load(0), 15, CompareSelectOperation::kLE);
+  auto false_cond = Cond::make(
+      false_cmp, x.store({0}, x.load(0) + 2), x.store({0}, x.load(0) - 2));
+  auto cmp = CompareSelect::make(x.load(0), 10, CompareSelectOperation::kLT);
+  auto cond = Cond::make(cmp, true_cond, false_cond);
+
+  for (int32_t x_value : {0, 8, 15, 20}) {
+    std::vector<int32_t> x_buffer = {x_value};
+    std::vector<void*> args({x_buffer.data()});
+    LLVMCodeGen cg(cond, {x});
+    ASSERT_EQ(cg.value<int>(args), 0);
+    if (x_value < 10) {
+      if (x_value > 5) {
+        ASSERT_EQ(x_buffer[0], x_value + 1);
+      } else {
+        ASSERT_EQ(x_buffer[0], x_value - 1);
+      }
+    } else {
+      if (x_value <= 15) {
+        ASSERT_EQ(x_buffer[0], x_value + 2);
+      } else {
+        ASSERT_EQ(x_buffer[0], x_value - 2);
+      }
+    }
+  }
+}
+
+TEST(LLVM, VecLoadStoreTest) {
   KernelScope kernel_scope;
   Placeholder a(BufHandle("A", {1}, kInt));
   Placeholder b(BufHandle("B", {1}, kInt));
@@ -304,7 +494,7 @@ void testLLVMVecLoadStoreTest() {
 }
 
 #define FLOAT_INTRINSICS_TEST(Name, Lanes)                       \
-  void testLLVMVecFloat_##Name##Lane##Lanes##Test() {            \
+  TEST(LLVM, VecFloat_##Name##Lane##Lanes##Test) {               \
     KernelScope kernel_scope;                                    \
     Placeholder a(BufHandle("A", {1}, kFloat));                  \
     Placeholder b(BufHandle("B", {1}, kFloat));                  \
@@ -347,7 +537,7 @@ FLOAT_INTRINSICS_TEST(lgamma, 8)
 #undef FLOAT_INTRINSICS_TEST
 
 #define DOUBLE_INTRINSICS_TEST(Name, Lanes)                      \
-  void testLLVMVecDouble_##Name##Lane##Lanes##Test() {           \
+  TEST(LLVM, VecDouble_##Name##Lane##Lanes##Test) {              \
     KernelScope kernel_scope;                                    \
     Placeholder a(BufHandle("A", {1}, kDouble));                 \
     Placeholder b(BufHandle("B", {1}, kDouble));                 \
@@ -389,7 +579,7 @@ DOUBLE_INTRINSICS_TEST(expm1, 4)
 DOUBLE_INTRINSICS_TEST(lgamma, 4)
 #undef DOUBLE_INTRINSICS_TEST
 
-void testLLVMVectorizerLoadStoreTest() {
+TEST(LLVM, VectorizerLoadStoreTest) {
   KernelScope kernel_scope;
   Placeholder a(BufHandle("A", {1}, kInt));
 
@@ -399,7 +589,7 @@ void testLLVMVectorizerLoadStoreTest() {
   Placeholder c_buf(BufHandle(c->buf()));
   LoopNest l({c});
   Stmt* s = l.root_stmt();
-  l.vectorize(dynamic_cast<Block*>(s)->front());
+  l.vectorize(dynamic_cast<For*>(dynamic_cast<Block*>(s)->front()));
 
   ASSERT_TRUE(dynamic_cast<For*>(dynamic_cast<Block*>(s)->front()) == nullptr);
 
@@ -412,7 +602,33 @@ void testLLVMVectorizerLoadStoreTest() {
   assertAllEqual(c_vec, 21);
 }
 
-void testLLVMMemcpyTest() {
+TEST(LLVM, VectorizeBitCast) {
+  KernelScope kernel_scope;
+  Placeholder a(BufHandle("A", {128}, kInt));
+
+  Tensor* c = Compute("c", {{128, "i"}}, [&](const VarHandle& i) {
+    return bitcast<float>(a.load(i));
+  });
+
+  Placeholder c_buf(BufHandle(c->buf()));
+  LoopNest l({c});
+  Stmt* s = l.root_stmt();
+  l.vectorize(dynamic_cast<For*>(dynamic_cast<Block*>(s)->front()));
+  ASSERT_TRUE(dynamic_cast<For*>(dynamic_cast<Block*>(s)->front()) == nullptr);
+
+  LLVMCodeGen cg(s, {a, c_buf});
+
+  std::vector<int> a_vec(128);
+  std::vector<float> c_vec(128);
+  for (auto i = 0; i < 128; ++i) {
+    a_vec[i] = raw_bitcast<int>(1337.f);
+  }
+  std::vector<void*> args({a_vec.data(), c_vec.data()});
+  ASSERT_EQ(cg.value<int>(args), 0);
+  assertAllEqual(c_vec, 1337.f);
+}
+
+TEST(LLVM, MemcpyTest) {
   KernelScope kernel_scope;
   constexpr int N = 32;
   Placeholder a(BufHandle("A", {N}, kInt));
@@ -434,7 +650,7 @@ void testLLVMMemcpyTest() {
   assertAllEqual(b_buffer, 42);
 }
 
-void testLLVMBzeroTest() {
+TEST(LLVM, BzeroTest) {
   KernelScope kernel_scope;
   constexpr int N = 32;
   Placeholder b(BufHandle("B", {N}, kInt));
@@ -452,7 +668,7 @@ void testLLVMBzeroTest() {
   assertAllEqual(b_buffer, 0);
 }
 
-void testLLVMElemwiseAdd() {
+TEST(LLVM, ElemwiseAdd) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kInt));
@@ -478,7 +694,7 @@ void testLLVMElemwiseAdd() {
   assertAllEqual(c_buffer, 42);
 }
 
-void testLLVMElemwiseAddFloat() {
+TEST(LLVM, ElemwiseAddFloat) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kFloat));
@@ -504,7 +720,7 @@ void testLLVMElemwiseAddFloat() {
   assertAllEqual(c_buffer, 42.0f);
 }
 
-void testLLVMElemwiseLog10Float() {
+TEST(LLVM, ElemwiseLog10Float) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kFloat));
@@ -534,7 +750,7 @@ void testLLVMElemwiseLog10Float() {
   assertAllEqual(b_buffer, 1.0f);
 }
 
-void testLLVMElemwiseLog1pFloat() {
+TEST(LLVM, ElemwiseLog1pFloat) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kFloat));
@@ -564,7 +780,7 @@ void testLLVMElemwiseLog1pFloat() {
   ExpectAllNear(b_buffer, 3.0f, 1e-5f);
 }
 
-void testLLVMElemwiseMaxInt() {
+TEST(LLVM, ElemwiseMaxInt) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kInt));
@@ -591,7 +807,7 @@ void testLLVMElemwiseMaxInt() {
   assertAllEqual(c_buffer, 41);
 }
 
-void testLLVMElemwiseMinInt() {
+TEST(LLVM, ElemwiseMinInt) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kInt));
@@ -618,7 +834,7 @@ void testLLVMElemwiseMinInt() {
   assertAllEqual(c_buffer, 1);
 }
 
-void testLLVMElemwiseMaxFloat() {
+TEST(LLVM, ElemwiseMaxFloat) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kFloat));
@@ -645,7 +861,7 @@ void testLLVMElemwiseMaxFloat() {
   assertAllEqual(c_buffer, 41.0f);
 }
 
-void testLLVMElemwiseMaxNaNFloat() {
+TEST(LLVM, ElemwiseMaxNaNFloat) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kFloat));
@@ -673,7 +889,7 @@ void testLLVMElemwiseMaxNaNFloat() {
   }
 }
 
-void testLLVMElemwiseMinFloat() {
+TEST(LLVM, ElemwiseMinFloat) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kFloat));
@@ -700,7 +916,7 @@ void testLLVMElemwiseMinFloat() {
   assertAllEqual(c_buffer, 1.0f);
 }
 
-void testLLVMElemwiseMinNaNFloat() {
+TEST(LLVM, ElemwiseMinNaNFloat) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kFloat));
@@ -728,7 +944,7 @@ void testLLVMElemwiseMinNaNFloat() {
   }
 }
 
-void testLLVMElemwiseMod() {
+TEST(LLVM, ElemwiseMod) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kInt));
@@ -754,7 +970,7 @@ void testLLVMElemwiseMod() {
   assertAllEqual(c_buffer, 18);
 }
 
-void testLLVMCompareSelectIntEQ() {
+TEST(LLVM, CompareSelectIntEQ) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kInt));
@@ -795,7 +1011,7 @@ void testLLVMCompareSelectIntEQ() {
   }
 }
 
-void testLLVMCompareSelectFloatEQ() {
+TEST(LLVM, CompareSelectFloatEQ) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kFloat));
@@ -829,7 +1045,7 @@ void testLLVMCompareSelectFloatEQ() {
   assertAllEqual(c_buffer, 1);
 }
 
-void testLLVMCompareSelectByteGT() {
+TEST(LLVM, CompareSelectByteGT) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kByte));
@@ -870,7 +1086,7 @@ void testLLVMCompareSelectByteGT() {
   }
 }
 
-void testLLVMCompareSelectByteGE() {
+TEST(LLVM, CompareSelectByteGE) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kByte));
@@ -906,7 +1122,7 @@ void testLLVMCompareSelectByteGE() {
   }
 }
 
-void testLLVMCompareSelectByteLT() {
+TEST(LLVM, CompareSelectByteLT) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kByte));
@@ -947,7 +1163,7 @@ void testLLVMCompareSelectByteLT() {
   }
 }
 
-void testLLVMCompareSelectByteLE() {
+TEST(LLVM, CompareSelectByteLE) {
   KernelScope kernel_scope;
   constexpr int N = 1024;
   Placeholder a(BufHandle("A", {N}, kByte));
@@ -983,7 +1199,7 @@ void testLLVMCompareSelectByteLE() {
   }
 }
 
-void testLLVMStoreFloat() {
+TEST(LLVM, StoreFloat) {
   KernelScope kernel_scope;
   Placeholder result(BufHandle("result", {1}, kFloat));
   std::vector<float> result_buffer = {0.0f};
@@ -994,7 +1210,7 @@ void testLLVMStoreFloat() {
   ASSERT_EQ(result_buffer[0], 3.14f);
 }
 
-void testLLVMSimpleMath01() {
+TEST(LLVM, SimpleMath01) {
   KernelScope kernel_scope;
   const int N = 1024;
   Tensor* tensor = Compute("f", {{N, "i"}}, [](const VarHandle& i) {
@@ -1016,7 +1232,7 @@ void testLLVMSimpleMath01() {
   ExpectAllNear(f_v, f_ref, 1e-5);
 }
 
-void testLLVMComputeMul() {
+TEST(LLVM, ComputeMul) {
   KernelScope kernel_scope;
   const int N = 1024;
   Placeholder a(BufHandle("a", {N}, kFloat));
@@ -1039,7 +1255,7 @@ void testLLVMComputeMul() {
   assertAllEqual(c_vec, 42.0f);
 }
 
-void testLLVMBroadcastAdd() {
+TEST(LLVM, BroadcastAdd) {
   KernelScope kernel_scope;
   const int M = 32;
   const int N = 1024;
@@ -1072,7 +1288,7 @@ void testLLVMBroadcastAdd() {
   }
 }
 
-void testLLVMBitwiseOps() {
+TEST(LLVM, BitwiseOps) {
   KernelScope kernel_scope;
   auto a = IntImm::make(59);
   auto b = IntImm::make(11);
@@ -1085,7 +1301,7 @@ void testLLVMBitwiseOps() {
   ASSERT_EQ(cg.value<int>(), 11);
 }
 
-void testLLVMDynamicShapeAdd() {
+TEST(LLVM, DynamicShapeAdd) {
   KernelScope kernel_scope;
   auto testWithSize = [](int32_t size) {
     VarHandle n("n", kInt);
@@ -1107,7 +1323,7 @@ void testLLVMDynamicShapeAdd() {
   testWithSize(37);
 }
 
-void testLLVMBindDynamicShapeAdd() {
+TEST(LLVM, BindDynamicShapeAdd) {
   KernelScope kernel_scope;
   auto testWithSize = [](int32_t size) {
     VarHandle n("n", kInt);
@@ -1128,7 +1344,7 @@ void testLLVMBindDynamicShapeAdd() {
   testWithSize(37);
 }
 
-void testLLVMTensorDynamicShapeAdd() {
+TEST(LLVM, TensorDynamicShapeAdd) {
   KernelScope kernel_scope;
   auto testWithSize = [](int32_t size) {
     VarHandle n("n", kInt);
@@ -1151,7 +1367,7 @@ void testLLVMTensorDynamicShapeAdd() {
   testWithSize(37);
 }
 
-void testLLVMDynamicShape2D() {
+TEST(LLVM, DynamicShape2D) {
   KernelScope kernel_scope;
   auto testWithSize = [](int32_t M, int32_t N) {
     VarHandle m("m", kInt);
@@ -1177,7 +1393,7 @@ void testLLVMDynamicShape2D() {
   testWithSize(37, 11);
 }
 
-void testLLVMEmptyStmt() {
+TEST(LLVM, EmptyStmt) {
   KernelScope kernel_scope;
   Stmt* s = new Block({});
 
@@ -1186,7 +1402,7 @@ void testLLVMEmptyStmt() {
   // Just don't crash.
 }
 
-void testLLVMEliminatedStmt() {
+TEST(LLVM, EliminatedStmt) {
   KernelScope kernel_scope;
   Placeholder a(BufHandle("a", {1}, kFloat));
 
@@ -1202,7 +1418,7 @@ void testLLVMEliminatedStmt() {
   cg.call({aData, cData});
 }
 
-void testLLVMSimpleReduction() {
+TEST(LLVM, SimpleReduction) {
   KernelScope kernel_scope;
 
   int M = 128;
@@ -1241,7 +1457,7 @@ void testLLVMSimpleReduction() {
   ExpectAllNear(b_v, b_ref, 1e-5);
 }
 
-void testLLVMRFactorReduction() {
+TEST(LLVM, RFactorReduction) {
   KernelScope kernel_scope;
 
   int M = 128;
@@ -1264,7 +1480,8 @@ void testLLVMRFactorReduction() {
   loops = loop.getLoopStmtsFor(b);
   loop_m = loops.at(2);
   loop_n = loops.at(1);
-  loop.rfactor(b->body(), loop_n->var(), loop_n->body());
+  auto b_body = NodeFinder<ReduceOp>::find(loop.root_stmt())[0];
+  loop.rfactor(b_body, loop_n->var(), loop_n->body());
 
   loop.prepareForCodegen();
   Stmt* s = loop.root_stmt();
@@ -1290,8 +1507,7 @@ void testLLVMRFactorReduction() {
   ExpectAllNear(b_v, b_ref, 1e-5);
 }
 
-// TODO: disabled since this doesn't work.
-void DISABLED_testLLVMRFactorVectorizedReduction() {
+TEST(LLVM, RFactorVectorizedReduction) {
   KernelScope kernel_scope;
 
   int M = 128;
@@ -1300,34 +1516,32 @@ void DISABLED_testLLVMRFactorVectorizedReduction() {
 
   Placeholder a("a", kFloat, {1, M, N});
 
-  // TODO: why doesn't implicit vector<DimArg> work?
-  std::vector<DimArg> axis = {DimArg(1)};
-  std::vector<DimArg> reduce_axis = {DimArg(M), DimArg(N)};
-  Tensor* b = Reduce("sum", axis, Sum(), a, reduce_axis);
+  Tensor* b = Reduce("sum", {{1, "K"}}, Sum(), a, {{M, "M"}, {N, "N"}});
   LoopNest loopnest({b});
   std::vector<For*> loops = loopnest.getLoopStmtsFor(b);
   For* loop_k = loops.at(0);
   For* loop_m = loops.at(1);
   For* loop_n = loops.at(2);
-  loopnest.reorderAxis(loop_n, loop_m);
-  loops = loopnest.getLoopStmtsFor(b);
+  auto b_body = NodeFinder<ReduceOp>::find(loopnest.root_stmt())[0];
+  loopnest.rfactor(b_body, loop_n->var());
+
+  loops = NodeFinder<For>::find(loopnest.root_stmt());
   loop_k = loops.at(0);
-  loop_n = loops.at(1);
+  // loop 1 is the initializer of tmp_buf
   loop_m = loops.at(2);
+  loop_n = loops.at(3);
+  loopnest.reorderAxis(loop_n, loop_m);
+
   // Case-III reductions
-  loopnest.rfactor(b->body(), loop_n->var());
+  loops = NodeFinder<For>::find(loopnest.root_stmt());
+  // Vectorize initializer of tmp_buf
+  loopnest.vectorize(loops[1]);
+  // Vectorize producer of tmp_buf
+  loopnest.vectorize(loops[2]);
+
   loopnest.prepareForCodegen();
-  Stmt* s = loopnest.root_stmt();
-  s = IRSimplifier::simplify(s);
 
-  Block* root_block = dynamic_cast<Block*>(s);
-  auto I = root_block->begin();
-  ++I;
-
-  For* outer_loop = dynamic_cast<For*>(*I);
-  loopnest.vectorize(outer_loop);
-
-  s = IRSimplifier::simplify(s);
+  Stmt* s = IRSimplifier::simplify(loopnest.root_stmt());
   LLVMCodeGen cg(s, {a, b});
 
   PaddedBuffer<float> a_v(1, M, N, "a_v");
@@ -1348,6 +1562,94 @@ void DISABLED_testLLVMRFactorVectorizedReduction() {
   ExpectAllNear(b_v, b_ref, 1e-5);
 }
 
+TEST(LLVM, VectorizedGEMM) {
+  KernelScope ks;
+
+  int M = 32;
+  int N = 32;
+  int K = 48;
+
+  Placeholder AP(BufHandle("A", {M, K}, kFloat));
+  Placeholder BP(BufHandle("B", {K, N}, kFloat));
+  Tensor* CT = Reduce(
+      "gemm",
+      {{M, "M"}, {N, "N"}},
+      Sum(),
+      [&](const ExprHandle& m, const ExprHandle& n, const ExprHandle& k) {
+        return AP.load(m, k) * BP.load(k, n);
+      },
+      {{K, "K"}});
+  LoopNest loop({CT});
+
+  {
+    auto const& loops = loop.getLoopStmtsFor(CT);
+    For* m = loops[0];
+    For* mo;
+    For* mi;
+    loop.splitWithMask(m, 16, &mo, &mi);
+  }
+  {
+    auto const& loops = loop.getLoopStmtsFor(CT);
+    For* n = loops[2];
+    For* no;
+    For* ni;
+    loop.splitWithMask(n, 16, &no, &ni);
+  }
+  // mo, mi, no, ni, k ->
+  // mo, no, mi, ni, k
+  {
+    auto const& loops = loop.getLoopStmtsFor(CT);
+    For* mi = loops[1];
+    For* no = loops[2];
+    loop.reorderAxis(mi, no);
+  }
+  // mo, no, mi, ni, k ->
+  // mo, no, mi, k, ni
+  {
+    auto const& loops = loop.getLoopStmtsFor(CT);
+    For* ni = loops[3];
+    For* k = loops[4];
+    loop.reorderAxis(ni, k);
+  }
+  // mo, no, mi, k, ni ->
+  // mo, no, k, mi, ni
+  {
+    auto const& loops = loop.getLoopStmtsFor(CT);
+    For* mi = loops[2];
+    For* k = loops[3];
+    loop.reorderAxis(mi, k);
+  }
+  {
+    auto loops = NodeFinder<For>::find(loop.root_stmt());
+    loop.vectorize(loops[3]);
+    loop.vectorize(loops.back());
+  }
+
+  loop.prepareForCodegen();
+
+  Stmt* s = loop.root_stmt();
+  s = IRSimplifier::simplify(s);
+  LLVMCodeGen cg(s, {AP, BP, CT});
+
+  PaddedBuffer<float> a_v(M, K, "a_v");
+  PaddedBuffer<float> b_v(K, N, "b_v");
+  PaddedBuffer<float> c_v(M, N, "c_v");
+  PaddedBuffer<float> c_ref(M, N, "c_ref");
+
+  for (int m = 0; m < M; m++) {
+    for (int n = 0; n < N; n++) {
+      c_ref(m, n) = 0.f;
+      for (int k = 0; k < K; k++) {
+        c_ref(m, n) += a_v(m, k) * b_v(k, n);
+      }
+    }
+  }
+
+  cg.call({a_v, b_v, c_v});
+
+  ExpectAllNear(c_v, c_ref, 1e-5);
+}
+
 } // namespace jit
 } // namespace torch
 
