@@ -7,6 +7,8 @@ from torch.testing._internal.common_quantized import override_quantized_engine
 from torch.testing._internal.common_quantization import skipIfNoFBGEMM
 
 from torch.jit._recursive import wrap_cpp_module
+from typing import Any
+from itertools import product
 
 import io
 
@@ -625,7 +627,7 @@ class TestFreezing(JitTestCase):
         self.assertFalse(mf.hasattr('sub'))
         self.assertFalse(mf.hasattr('a'))
         self.assertTrue(mf.hasattr('b'))
-        with self.assertRaisesRegex(RuntimeError, "TestModule does not have a field with name '_forward'"):
+        with self.assertRaisesRegex(AttributeError, "TestModule \(.*\) does not have a field with name '_forward'"):  # noqa: W605
             mf._forward(x)
 
     def test_freeze_module_with_inplace_mutable(self):
@@ -1034,10 +1036,78 @@ class TestFreezing(JitTestCase):
 
         model = torch.jit.script(Net())
         model.train()
-
-        with self.assertRaisesRegex(RuntimeError, 'Freezing module in training mode is not yet supported'):
-            mTrain_freezed = torch._C._freeze_module(model._c)
-
+        mTrain_freezed = torch._C._freeze_module(model._c)
+        # verify mTrain_freezed looks exactly as:
+        # module {
+        #   attributes {
+        #     conv1 = ...
+        #     conv2 = ...
+        #     dropout1 = ...
+        #     dropout2 = ...
+        #     fc1 = ...
+        #     fc2 = ...
+        #   }
+        #   ...
+        #   submodules {
+        #     module conv1 {
+        #       attributes {
+        #          weight = ...
+        #          bias = ...
+        #       }
+        #       ...
+        #     }
+        #     module conv2 {
+        #       attributes {
+        #          weight = ...
+        #          bias = ...
+        #       }
+        #       ...
+        #     }
+        #     module dropout1 {
+        #       attributes {
+        #          training = ...
+        #       }
+        #       ...
+        #     }
+        #     module dropout2 {
+        #       attributes {
+        #          training = ...
+        #       }
+        #       ...
+        #     }
+        #     module fc1 {
+        #       attributes {
+        #          weight = ...
+        #          bias = ...
+        #       }
+        #       ...
+        #     }
+        #     module fc2 {
+        #       attributes {
+        #          weight = ...
+        #          bias = ...
+        #       }
+        #       ...
+        #     }
+        self.assertFalse(mTrain_freezed.hasattr('training'))
+        self.assertTrue(mTrain_freezed.hasattr('conv1'))
+        self.assertFalse(mTrain_freezed.conv1.hasattr('training'))
+        self.assertTrue(mTrain_freezed.conv1.hasattr('weight'))
+        self.assertTrue(mTrain_freezed.conv1.hasattr('bias'))
+        self.assertTrue(mTrain_freezed.hasattr('conv2'))
+        self.assertFalse(mTrain_freezed.conv2.hasattr('training'))
+        self.assertTrue(mTrain_freezed.conv2.hasattr('weight'))
+        self.assertTrue(mTrain_freezed.conv2.hasattr('bias'))
+        self.assertTrue(mTrain_freezed.hasattr('dropout1'))
+        self.assertTrue(mTrain_freezed.dropout1.hasattr('training'))
+        self.assertTrue(mTrain_freezed.hasattr('dropout2'))
+        self.assertTrue(mTrain_freezed.dropout2.hasattr('training'))
+        self.assertTrue(mTrain_freezed.hasattr('fc1'))
+        self.assertTrue(mTrain_freezed.fc1.hasattr('weight'))
+        self.assertTrue(mTrain_freezed.fc1.hasattr('bias'))
+        self.assertTrue(mTrain_freezed.hasattr('fc2'))
+        self.assertTrue(mTrain_freezed.fc2.hasattr('weight'))
+        self.assertTrue(mTrain_freezed.fc2.hasattr('bias'))
         model.eval()
         mEval_freezed = torch._C._freeze_module(model._c)
         self.assertFalse(mEval_freezed.hasattr('conv1'))
@@ -1047,7 +1117,7 @@ class TestFreezing(JitTestCase):
         self.assertFalse(mEval_freezed.hasattr('fc1'))
         self.assertFalse(mEval_freezed.hasattr('dropout2'))
         self.assertFalse(mEval_freezed.hasattr('fc2'))
-        with self.assertRaisesRegex(RuntimeError, "does not have a field with name 'state_dict'"):
+        with self.assertRaisesRegex(AttributeError, "does not have a field with name 'state_dict'"):
             print(mEval_freezed.state_dict())
         buffer = io.BytesIO()
         torch.jit.save(mEval_freezed, buffer)
@@ -1055,6 +1125,14 @@ class TestFreezing(JitTestCase):
         m = torch.jit.load(buffer)
         FileCheck().check_not('GetAttr[name=') \
                    .run(m._c._get_method('forward').graph)
+        m2 = torch._C._freeze_module(model._c, preserveParameters=True)
+        self.assertTrue(m2.hasattr('conv1'))
+        self.assertTrue(m2.hasattr('conv2'))
+        self.assertFalse(m2.hasattr('dropout1'))
+        self.assertFalse(m2.hasattr('training'))
+        self.assertTrue(m2.hasattr('fc1'))
+        self.assertFalse(m2.hasattr('dropout2'))
+        self.assertTrue(m2.hasattr('fc2'))
 
     def test_freeze_module_detach_gradient(self):
         mod = nn.Conv2d(8, 3, 4, 2, 1)
@@ -1068,7 +1146,8 @@ class TestFreezing(JitTestCase):
         inp = torch.ones(1, 8, 32, 32)
         out1 = fmod.forward(inp)
         # FIXME: frozen module mutated from outside (original module).
-        smod.weight[0, 0, 0, 0] += 100.0
+        with torch.no_grad():
+            smod.weight[0, 0, 0, 0] += 100.0
         out2 = fmod.forward(inp)
         out3 = smod(inp)
         self.assertNotEqual(out1, out2)
@@ -1195,3 +1274,246 @@ class TestFreezing(JitTestCase):
             # It used to segfault while running frozen module.
             m_frozen_res = m_frozen(data)
             self.assertEqual(m_res, m_frozen_res)
+
+    def test_module_getattr_indirection(self):
+        @torch.jit.script
+        class ValHolder(object):
+            def __init__(self, val: int):
+                self.val: int = val
+
+        class Mod(nn.Module):
+            def __init__(self):
+                super(Mod, self).__init__()
+                self.mod1 = ValHolder(1)
+                self.mod2 = ValHolder(2)
+
+            def forward(self, cond: bool):
+                if cond:
+                    mod = self.mod1
+                else:
+                    mod = self.mod2
+                return mod.val
+
+        mod = Mod()
+        mod.eval()
+        frozen_mod = torch.jit.freeze(torch.jit.script(mod))
+        mod_eager = Mod()
+        self.assertEqual(mod_eager(True), frozen_mod(True))
+        self.assertEqual(mod_eager(False), frozen_mod(False))
+
+    def test_freeze_module_with_non_static_module_dict_index(self):
+        """
+        Test that a Module contained a non-static ModuleDict index
+        cannot be frozen.
+        """
+        @torch.jit.interface
+        class ModuleInterface(torch.nn.Module):
+            def forward(self, inp: Any) -> Any:
+                pass
+
+        class ImplementsInterface(torch.nn.Module):
+            def forward(self, inp: Any) -> Any:
+                if isinstance(inp, torch.Tensor):
+                    return torch.max(inp, dim=0)
+
+                return inp
+
+        # Test annotation of submodule.
+        class Mod(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.d = torch.nn.ModuleDict({"module": ImplementsInterface()})
+
+            def forward(self, x: torch.Tensor, key: str) -> Any:
+                value: ModuleInterface = self.d[key]
+                return value.forward(x)
+
+        m = torch.jit.script(Mod())
+        m.eval()
+        with self.assertRaisesRegex(RuntimeError, "Freezing modules containing prim::ModuleDictIndex is not supported"):
+            mf = torch._C._freeze_module(m._c)
+
+    def test_freeze_non_module_class_getattr(self):
+        class BoxCoder(object):
+            def __init__(self, bbox_xform_clip):
+                # type: (float) -> None
+                self.bbox_xform_clip = bbox_xform_clip
+
+            def decode(self, input):
+                return input * self.bbox_xform_clip
+
+        class MyModule(torch.nn.Module):
+            __annotations__ = {
+                'box_coder': BoxCoder,
+            }
+
+            def __init__(self):
+                super(MyModule, self).__init__()
+                self.box_coder = BoxCoder(50.)
+
+            def forward(self, input):
+                return self.box_coder.decode(input)
+
+        model = MyModule()
+        model.eval()
+        script_model = torch.jit.freeze(torch.jit.script(model))
+        inp = torch.randn([4, 4])
+        output_eager = model(inp)
+        self.assertEqual(model(inp), script_model(inp))
+        FileCheck().check_not("GetAttr").run(script_model.graph)
+
+class TestFrozenOptimizations(JitTestCase):
+    def setUp(self):
+        self.default_dtype = torch.get_default_dtype()
+        torch.set_default_dtype(torch.double)
+
+    def tearDown(self):
+        torch.set_default_dtype(self.default_dtype)
+
+    def test_conv_bn_folding(self):
+        conv_bias = [True, False]
+        module_pairs = [(nn.Conv1d, nn.BatchNorm1d), (nn.Conv2d, nn.BatchNorm2d), (nn.Conv3d, nn.BatchNorm3d)]
+        use_tracing = [True, False]
+
+        for use_bias, modules, tracing in product(conv_bias, module_pairs, use_tracing):
+            class ConvBN(torch.nn.Module):
+                def __init__(self, in_channels, out_channels, **kwargs):
+                    super(ConvBN, self).__init__()
+                    self.conv = modules[0](in_channels, out_channels, bias=use_bias, **kwargs)
+                    self.bn = modules[1](out_channels, eps=0.001)
+
+                def forward(self, x):
+                    x = self.conv(x)
+                    return self.bn(x)
+
+            mod_eager = ConvBN(3, 32, kernel_size=3, stride=2).eval()
+            inps = [4, 3, 4]
+            if modules[0] == nn.Conv2d:
+                inps.append(inps[-1])
+            if modules[0] == nn.Conv3d:
+                inps.append(inps[-1])
+                inps.append(inps[-1])
+
+            inp = torch.rand(inps)
+
+            if tracing:
+                scripted_mod = torch.jit.trace(mod_eager, (inp))
+            else:
+                scripted_mod = torch.jit.script(mod_eager)
+
+            self.run_pass("inline", scripted_mod.graph)
+            self.run_pass("peephole", scripted_mod.graph)
+            self.run_pass("constant_propagation", scripted_mod.graph)
+
+            FileCheck().check("conv").check("batch").run(scripted_mod.graph)
+            # successfully no-ops with non-const inputs
+            self.run_pass("fold_frozen_conv_bn", scripted_mod.graph)
+            FileCheck().check("conv").check("aten::batch_norm").run(scripted_mod.graph)
+
+            scripted_mod = torch.jit.freeze(scripted_mod)
+            self.run_pass("fold_frozen_conv_bn", scripted_mod.graph)
+            FileCheck().check("conv").check_not("aten::batch_norm").run(scripted_mod.graph)
+
+            self.assertEqual(mod_eager(inp), scripted_mod(inp))
+            self.assertEqual(mod_eager(inp), scripted_mod(inp))
+
+
+    def test_conv_add_folding(self):
+
+        @torch.no_grad()
+        def test_conv_fusion(use_bias, module, tracing, op, scalar, add_tensor, expect_success):
+
+            class ConvOp(torch.nn.Module):
+                __constants__ = ['use_scalar']
+
+                def __init__(self, in_channels, out_channels, tensor=None, **kwargs):
+                    super(ConvOp, self).__init__()
+                    self.conv = module(in_channels, out_channels, bias=use_bias, **kwargs)
+                    self.conv2 = module(in_channels, out_channels, bias=use_bias, **kwargs)
+                    self.use_scalar = scalar
+                    tensor_size = [1 for _ in range(self.conv.weight.ndim)]
+                    tensor_size[1] = self.conv.weight.size(0)
+                    self.tensor = add_tensor if add_tensor is not None else torch.rand(tensor_size)
+                    self.op = op
+
+                def forward(self, x):
+                    x = self.conv(x)
+                    if self.use_scalar:
+                        return self.op(x, 2.)
+                    else:
+                        return self.op(x, self.tensor)
+
+            mod_eager = ConvOp(3, 32, kernel_size=3, stride=2).eval()
+
+            inps = [4, 3, 4]
+            if module == nn.Conv2d:
+                inps.append(inps[-1])
+            if module == nn.Conv3d:
+                inps.append(inps[-1])
+                inps.append(inps[-1])
+
+
+            inp = torch.rand(inps)
+
+            if tracing:
+                scripted_mod = torch.jit.trace(mod_eager, (inp,))
+            else:
+                scripted_mod = torch.jit.script(mod_eager)
+
+            self.run_pass("inline", scripted_mod.graph)
+            op_str = "aten::" + op.__name__
+
+            FileCheck().check("conv").check(op_str).run(scripted_mod.graph)
+            # successively no-ops with non-const inputs
+            self.run_pass("fold_frozen_conv_mul_or_div", scripted_mod.graph)
+            self.run_pass("fold_frozen_conv_add_or_sub", scripted_mod.graph)
+            FileCheck().check("conv").check(op_str).run(scripted_mod.graph)
+            scripted_mod = torch.jit.freeze(scripted_mod)
+            self.run_pass("fold_frozen_conv_mul_or_div", scripted_mod.graph)
+            self.run_pass("fold_frozen_conv_add_or_sub", scripted_mod.graph)
+
+            if expect_success:
+                FileCheck().check("conv").check_not(op_str).run(scripted_mod.graph)
+            else:
+                FileCheck().check("conv").check(op_str).run(scripted_mod.graph)
+
+            self.assertEqual(mod_eager(inp), scripted_mod(inp))
+            self.assertEqual(mod_eager(inp), scripted_mod(inp))
+
+        conv_bias = [True, False]
+        modules = [nn.Conv1d, nn.Conv2d, nn.Conv3d]
+        use_tracing = [False, True]
+        use_scalar = [False, True]
+        ops = [torch.add, torch.sub, torch.mul, torch.div]
+
+        for use_bias, module, tracing, pytorch_op, scalar in product(conv_bias, modules, use_tracing, ops, use_scalar):
+            test_conv_fusion(use_bias, module, tracing, pytorch_op, scalar, add_tensor=None, expect_success=True)
+
+
+        for use_bias, pytorch_op in product(conv_bias, ops):
+            # broadcasting add
+            test_conv_fusion(use_bias, nn.Conv2d, False, pytorch_op, False,
+                             add_tensor=torch.rand(32, 1, 32), expect_success=False)
+
+            # broadcasting add
+            test_conv_fusion(use_bias, nn.Conv2d, False, pytorch_op, False, add_tensor=torch.rand(1, 1), expect_success=True)
+
+            # add with different dtype
+            test_conv_fusion(use_bias, nn.Conv2d, False, pytorch_op, False,
+                             add_tensor=torch.rand(1).to(torch.int), expect_success=False)
+
+    def test_optimize_freeze_module(self):
+        in_channels, out_channels = 3, 32
+        conv = torch.nn.Conv2d(in_channels, out_channels, kernel_size=3, stride=2, bias=True)
+        bn = torch.nn.BatchNorm2d(out_channels, eps=.001)
+        mod = torch.nn.Sequential(conv, bn)
+        # set optimize to False here, by default freezing runs optimize_frozen_module
+        frozen_mod = torch.jit.freeze(torch.jit.script(mod.eval()), optimize_numerics=False)
+        # inspect frozen mod
+        FileCheck().check("batch_norm").run(frozen_mod.graph)
+        torch.jit.optimize_frozen_module(frozen_mod)
+        FileCheck().check_not("batch_norm").run(frozen_mod.graph)
+
+        # optimize_frozen_module should be run
+        frozen_mod = torch.jit.freeze(torch.jit.script(mod.eval()))
+        FileCheck().check_not("batch_norm").run(frozen_mod.graph)
