@@ -1,13 +1,13 @@
 import os
 import sys
 import inspect
-from typing import Dict, List, Optional, Tuple, Any
+from typing import Any, Dict, List, Optional, Tuple
 from textwrap import dedent
 from collections import OrderedDict
 
+from torch import Tensor
 import torch
 from torch.testing import FileCheck
-from torch import Tensor
 
 # Make the helper files in test/ importable
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
@@ -20,23 +20,49 @@ if __name__ == '__main__':
                        "instead.")
 
 class TestList(JitTestCase):
+    def test_list_bool_conversion(self):
+        def if_predicate(l: List[int]):
+            if l:
+                s = 0
+                for n in l:
+                    s += n
+
+                return s
+            else:
+                return -1
+
+        self.checkScript(if_predicate, ([1, 2, 3],))
+        self.checkScript(if_predicate, ([],))
+
+        def while_predicate(l: List[int]):
+            s = 0
+
+            while l:
+                s += l.pop()
+
+        self.checkScript(while_predicate, ([1, 2, 3],))
+        self.checkScript(while_predicate, ([],))
+
+        def ternary_predicate(l: List[int]):
+            return "non-empty" if l else "empty"
+
+        self.checkScript(ternary_predicate, ([1, 2, 3],))
+        self.checkScript(ternary_predicate, ([],))
+
     def test_in_check(self):
-        def int_in(x):
-            # type: (List[int]) -> bool
+        def int_in(x: List[int]) -> bool:
             return 2 in x
 
         self.checkScript(int_in, ([1, 2, 3],))
         self.checkScript(int_in, ([1, 3, 3],))
 
-        def float_in(x):
-            # type: (List[float]) -> bool
+        def float_in(x: List[float]) -> bool:
             return 2. in x
 
         self.checkScript(float_in, ([1., 2., 3.],))
         self.checkScript(float_in, ([1., 3., 3.],))
 
-        def str_in(x):
-            # type: (List[str]) -> bool
+        def str_in(x: List[str]) -> bool:
             return 'hi' in x
 
         self.checkScript(str_in, (['not', 'here'],))
@@ -46,21 +72,21 @@ class TestList(JitTestCase):
     def test_list_literal(self):
         def reassign():
             x = [1]
-            if True:
+            if 1 == 1:
                 x = [2, 3]
             return
         self.checkScript(reassign, (), optimize=False)
 
         def reassign_arity_change():
             x = [1]
-            if True:
+            if 1 == 1:
                 x = [1, 2, 3]
             return
         self.checkScript(reassign_arity_change, (), optimize=False)
 
         def reassign_from_empty_literal():
             x = []
-            if True:
+            if 1 == 1:
                 x = [1, 2, 3]
             return
         with self.assertRaisesRegex(RuntimeError, r"previously has type List\[Tensor\]"):
@@ -68,20 +94,20 @@ class TestList(JitTestCase):
 
         def reassign_from_empty_builtin():
             x = torch.jit.annotate(List[int], [])
-            if True:
+            if 1 == 1:
                 x = [1, 2, 3]
             y = torch.jit.annotate(List[float], [])
-            if True:
+            if 1 == 1:
                 y = [1.0, 2.0, 3.0]
             z = []
-            if True:
+            if 1 == 1:
                 z = [torch.randn([1])]
             return
         self.checkScript(reassign_from_empty_builtin, (), optimize=False)
 
         def reassign_bad_type():
             x = [1]
-            if True:
+            if 1 == 1:
                 x = [1.0]
             return
         with self.assertRaisesRegex(RuntimeError, "previously has type"):
@@ -89,9 +115,9 @@ class TestList(JitTestCase):
 
         def reassign_nested():
             x = torch.jit.annotate(List[int], [])
-            if True:
+            if 1 == 1:
                 x = [1, 2, 3]
-                if True:
+                if 1 == 1:
                     x = [1.0]
             return
         with self.assertRaisesRegex(RuntimeError, "previously has type"):
@@ -101,8 +127,7 @@ class TestList(JitTestCase):
         def inputs():
             return [1, 2, 3, 4]
 
-        def fn(x):
-            # type: (List[int]) -> List[int]
+        def fn(x: List[int]) -> List[int]:
             del x[1]
             return x
 
@@ -115,8 +140,7 @@ class TestList(JitTestCase):
         self.assertEqual(torch.jit.script(fn)(inputs()), python_out)
 
         @torch.jit.script
-        def fn2(x):
-            # type: (List[int]) -> List[int]
+        def fn2(x: List[int]) -> List[int]:
             del x[100]
             return x
 
@@ -125,29 +149,43 @@ class TestList(JitTestCase):
 
         with self.assertRaisesRegex(RuntimeError, "deletion at a single index"):
             @torch.jit.script
-            def fn(x):
-                # type: (List[int]) -> List[int]
+            def fn(x: List[int]) -> List[int]:
                 del x[1:3]
                 return x
 
+    def test_list_keyword(self):
+        def foo():
+            return list([1, 2, 3]), list(("a", "b")), list(range(5)), list("abcdefg")  # noqa: C410
+
+        self.checkScript(foo, ())
+
+        def foo2():
+            x: List[int] = list()
+            x.append(1)
+            return x,
+
+        self.checkScript(foo2, ())
+
+        def foo3():
+            return list(list("abc"))
+
+        self.checkScript(foo3, ())
+        FileCheck().check_count("aten::list", 2, exactly=True).run(torch.jit.script(foo3).graph)
+
     def test_min_bool_list(self):
-        def jit_min_list(a, b):
-            # type: (List[bool], List[bool]) -> List[bool]
+        def jit_min_list(a: List[bool], b: List[bool]) -> List[bool]:
             return min(a, b)
 
         self.checkScript(jit_min_list, ([True, False], [False, True]))
 
     def test_min_max_list(self):
-        def jit_min_list(a, b):
-            # type: (List[int], List[int]) -> List[int]
+        def jit_min_list(a: List[int], b: List[int]) -> List[int]:
             return min(a, b)
 
-        def jit_min_list_float(a, b):
-            # type: (List[float], List[float]) -> List[float]
+        def jit_min_list_float(a: List[float], b: List[float]) -> List[float]:
             return min(a, b)
 
-        def jit_min_list_bool(a, b):
-            # type: (List[bool], List[bool]) -> List[bool]
+        def jit_min_list_bool(a: List[bool], b: List[bool]) -> List[bool]:
             return min(a, b)
 
         def run_tests(func, a, b):
@@ -168,16 +206,13 @@ class TestList(JitTestCase):
                            [False, True], [False, False, True], [False, False, False]]
         run_tests(jit_min_list_bool, args_left_bool, args_right_bool)
 
-        def jit_max_list(a, b):
-            # type: (List[int], List[int]) -> List[int]
+        def jit_max_list(a: List[int], b: List[int]) -> List[int]:
             return max(a, b)
 
-        def jit_max_list_float(a, b):
-            # type: (List[float], List[float]) -> List[float]
+        def jit_max_list_float(a: List[float], b: List[float]) -> List[float]:
             return max(a, b)
 
-        def jit_max_list_bool(a, b):
-            # type: (List[bool], List[bool]) -> List[bool]
+        def jit_max_list_bool(a: List[bool], b: List[bool]) -> List[bool]:
             return max(a, b)
 
         args_left_int = [[1, 8, 8], [8, 1, 1], [], [1], [], [1, 2]]
@@ -347,8 +382,7 @@ class TestList(JitTestCase):
             t2 = scope['func']()
             self.assertEqual(t1, t2)
 
-        def test_fail(x):
-            # type: (List[Tensor]) -> List[Tensor]
+        def test_fail(x: List[Tensor]) -> List[Tensor]:
             x.sort()
             return x
 
@@ -408,6 +442,43 @@ class TestList(JitTestCase):
             return a[3:10] == [3, 4]
         self.checkScript(test_backward_slice, ())
 
+    def test_slice_index(self):
+        a = torch.tensor(
+            [
+                [[1, 11], [2, 22]],
+                [[3, 33], [4, 44]],
+                [[5, 55], [6, 66]],
+            ]
+        )
+
+        def test_index_slice1(x):
+            x = x[:, :, [0, 1]]
+            return x
+        self.checkScript(test_index_slice1, (a,))
+
+        def test_index_slice2(x):
+            x = x[[2, 1, 0], :, :]
+            return x
+        self.checkScript(test_index_slice2, (a,))
+
+        def test_index_slice3(x):
+            x = x[[0, 1], :, [1]]
+            return x
+        self.checkScript(test_index_slice3, (a,))
+
+        def test_index_slice_empty_list(x):
+            empty_list: List[int] = []
+            x = x[empty_list, :, :]
+            return x
+        self.checkScript(test_index_slice_empty_list, (a,))
+
+        def test_index_slice_out_of_bounds_index(x):
+            x = x[[4], :, :]
+            return x
+        with self.assertRaisesRegex(RuntimeError, "index 4 is out of bounds for dimension 0 with size 3"):
+            self.checkScript(test_index_slice_out_of_bounds_index, (a,))
+
+
     def test_mutable_list_append(self):
         def test_append():
             a = [0, 1]
@@ -417,8 +488,7 @@ class TestList(JitTestCase):
         self.checkScript(test_append, ())
 
     def test_comprehensions_basic(self):
-        def comp(l):
-            # type: (List[int]) -> List[int]
+        def comp(l: List[int]) -> List[int]:
 
             n = [x * 3 for x in l]
             return n
@@ -427,8 +497,7 @@ class TestList(JitTestCase):
         self.checkScript(comp, ([1, 2, 3],))
 
     def test_comprehensions_basic_float(self):
-        def comp(l):
-            # type: (List[float]) -> List[float]
+        def comp(l: List[float]) -> List[float]:
 
             n = [x * 3 for x in l]
             return n
@@ -437,8 +506,7 @@ class TestList(JitTestCase):
 
     def test_comprehensions_two_comps(self):
         @torch.jit.script
-        def comp(l1, l2):
-            # type: (List[int], List[int]) -> List[int]
+        def comp(l1: List[int], l2: List[int]) -> List[int]:
 
             n = [x * 3 for x in l1]
             n2 = [x + 2 for x in l2]
@@ -447,8 +515,7 @@ class TestList(JitTestCase):
         self.assertEqual(comp([1, 2, 3], [4, 5]), [3, 6, 9, 6, 7])
 
     def test_comprehension_out_type_not_in_type(self):
-        def list_cast():
-            # type: () -> int
+        def list_cast() -> int:
             li = [int(i) for i in [torch.tensor(0), torch.tensor(1), torch.tensor(2)]]
             return li[0] + li[1] + li[2]
 
@@ -458,15 +525,13 @@ class TestList(JitTestCase):
         def test_func(fn, inputs):
             self.assertEqual(fn(*inputs), torch.jit.script(fn)(*inputs))
 
-        def foo(names, results):
-            # type: (List[int], List[int]) -> List[Tuple[int, int]]
+        def foo(names: List[int], results: List[int]) -> List[Tuple[int, int]]:
             return [(k + 5, v - 2) for k, v in zip(names, results)]
 
         test_func(foo, ([1, 2, 4], [4, 7, 9]))
         test_func(foo, ([5], [4, 7, 9]))
 
-        def fn(x):
-            # type: (int) -> List[int]
+        def fn(x: int) -> List[int]:
             return [i for i in range(x)]  # noqa: C416
 
         test_func(fn, (9,))
@@ -498,7 +563,7 @@ class TestList(JitTestCase):
     def test_mutable_list_append_if(self):
         def test_append_if():
             a = [1]
-            if True:
+            if 1 == 1:
                 a.append(4)
             return a == [1, 4]
         self.checkScript(test_append_if, ())
@@ -506,7 +571,7 @@ class TestList(JitTestCase):
     def test_mutable_list_append_if_else(self):
         def test_append_if_else():
             a = [1]
-            if False:
+            if 1 == 2:
                 a.append(4)
             else:
                 a.append(10)
@@ -546,8 +611,7 @@ class TestList(JitTestCase):
 
     def test_mutable_list_function_inline(self):
         @torch.jit.script
-        def bar(y):
-            # type: (List[int]) -> None
+        def bar(y: List[int]) -> None:
             y.append(4)
 
         @torch.jit.script
@@ -833,8 +897,7 @@ class TestList(JitTestCase):
 
     def test_extend_list_mutable(self):
         @torch.jit.script
-        def extend_list(a, b):
-            # type: (List[Tensor], List[Tensor]) -> List[Tensor]
+        def extend_list(a: List[Tensor], b: List[Tensor]) -> List[Tensor]:
 
             a.extend(b)
             return a
@@ -845,8 +908,7 @@ class TestList(JitTestCase):
 
     def test_extend_list_immutable(self):
         @torch.jit.script
-        def extend_list(a, b):
-            # type: (List[int], List[int]) -> List[int]
+        def extend_list(a: List[int], b: List[int]) -> List[int]:
 
             a.extend(b)
             return a
@@ -857,8 +919,7 @@ class TestList(JitTestCase):
 
     def test_copy_list_mutable(self):
         @torch.jit.script
-        def copy_list(a):
-            # type: (List[Tensor]) -> List[Tensor]
+        def copy_list(a: List[Tensor]) -> List[Tensor]:
             return a.copy()
 
         for l in [[], [torch.rand(2)], [torch.rand(2), torch.rand(2), torch.rand(2)]]:
@@ -866,36 +927,29 @@ class TestList(JitTestCase):
 
     def test_copy_list_immutable(self):
         @torch.jit.script
-        def copy_list(a):
-            # type: (List[int]) -> List[int]
+        def copy_list(a: List[int]) -> List[int]:
             return a.copy()
 
         for l in [[], [1], [1, 2, 3]]:
             self.assertEqual(copy_list(l), l)
 
     def test_min_max_single_list(self):
-        def min_intlist(li):
-            # type: (List[int]) -> int
+        def min_intlist(li: List[int]) -> int:
             return min(li)
 
-        def max_intlist(li):
-            # type: (List[int]) -> int
+        def max_intlist(li: List[int]) -> int:
             return max(li)
 
-        def min_boollist(li):
-            # type: (List[bool]) -> bool
+        def min_boollist(li: List[bool]) -> bool:
             return min(li)
 
-        def max_boollist(li):
-            # type: (List[bool]) -> bool
+        def max_boollist(li: List[bool]) -> bool:
             return max(li)
 
-        def min_floatlist(li):
-            # type: (List[float]) -> float
+        def min_floatlist(li: List[float]) -> float:
             return min(li)
 
-        def max_floatlist(li):
-            # type: (List[float]) -> float
+        def max_floatlist(li: List[float]) -> float:
             return max(li)
 
 
@@ -911,11 +965,11 @@ class TestList(JitTestCase):
             check_list(min_intlist, int_list)
             check_list(max_intlist, int_list)
 
-            bool_li = list(map(lambda x: bool(x), int_list))
+            bool_li = [bool(x) for x in int_list]
             check_list(min_boollist, bool_li)
             check_list(max_boollist, bool_li)
 
-            float_li = list(map(lambda x: float(x), int_list))
+            float_li = [float(x) for x in int_list]
             check_list(min_floatlist, float_li)
             check_list(max_floatlist, float_li)
 
@@ -925,23 +979,19 @@ class TestList(JitTestCase):
         """
         Boolean dtype unit tests.
         """
-        def to_list_bool_0D(x):
-            # type: (torch.Tensor) -> bool
+        def to_list_bool_0D(x: torch.Tensor) -> bool:
             li = torch.jit.annotate(bool, x.tolist())
             return li
 
-        def to_list_bool_1D(x):
-            # type: (torch.Tensor) -> List[bool]
+        def to_list_bool_1D(x: torch.Tensor) -> List[bool]:
             li = torch.jit.annotate(List[bool], x.tolist())
             return li
 
-        def to_list_bool_2D(x):
-            # type: (torch.Tensor) -> List[List[bool]]
+        def to_list_bool_2D(x: torch.Tensor) -> List[List[bool]]:
             li = torch.jit.annotate(List[List[bool]], x.tolist())
             return li
 
-        def to_list_bool_3D(x):
-            # type: (torch.Tensor) -> List[List[List[bool]]]
+        def to_list_bool_3D(x: torch.Tensor) -> List[List[List[bool]]]:
             li = torch.jit.annotate(List[List[List[bool]]], x.tolist())
             return li
 
@@ -966,23 +1016,19 @@ class TestList(JitTestCase):
         """
         Int dtype unit tests.
         """
-        def to_list_int_0D(x):
-            # type: (torch.Tensor) -> int
+        def to_list_int_0D(x: torch.Tensor) -> int:
             li = torch.jit.annotate(int, x.tolist())
             return li
 
-        def to_list_int_1D(x):
-            # type: (torch.Tensor) -> List[int]
+        def to_list_int_1D(x: torch.Tensor) -> List[int]:
             li = torch.jit.annotate(List[int], x.tolist())
             return li
 
-        def to_list_int_2D(x):
-            # type: (torch.Tensor) -> List[List[int]]
+        def to_list_int_2D(x: torch.Tensor) -> List[List[int]]:
             li = torch.jit.annotate(List[List[int]], x.tolist())
             return li
 
-        def to_list_int_3D(x):
-            # type: (torch.Tensor) -> List[List[List[int]]]
+        def to_list_int_3D(x: torch.Tensor) -> List[List[List[int]]]:
             li = torch.jit.annotate(List[List[List[int]]], x.tolist())
             return li
 
@@ -1003,23 +1049,19 @@ class TestList(JitTestCase):
         """
         Float dtype unit tests.
         """
-        def to_list_float_0D(x):
-            # type: (torch.Tensor) -> float
+        def to_list_float_0D(x: torch.Tensor) -> float:
             li = torch.jit.annotate(float, x.tolist())
             return li
 
-        def to_list_float_1D(x):
-            # type: (torch.Tensor) -> List[float]
+        def to_list_float_1D(x: torch.Tensor) -> List[float]:
             li = torch.jit.annotate(List[float], x.tolist())
             return li
 
-        def to_list_float_2D(x):
-            # type: (torch.Tensor) -> List[List[float]]
+        def to_list_float_2D(x: torch.Tensor) -> List[List[float]]:
             li = torch.jit.annotate(List[List[float]], x.tolist())
             return li
 
-        def to_list_float_3D(x):
-            # type: (torch.Tensor) -> List[List[List[float]]]
+        def to_list_float_3D(x: torch.Tensor) -> List[List[List[float]]]:
             li = torch.jit.annotate(List[List[List[float]]], x.tolist())
             return li
 
@@ -1044,28 +1086,23 @@ class TestList(JitTestCase):
             - type annotation with the wrong dimension
             - type annotation with scalar type that doesn't match the input scalar type
         """
-        def to_list_missing_type_annotation(x):
-            # type: (torch.Tensor) -> List[float]
+        def to_list_missing_type_annotation(x: torch.Tensor) -> List[float]:
             li = x.tolist()
             return li
 
-        def to_list_incorrect_type_annotation(x):
-            # type: (torch.Tensor) -> List[float]
+        def to_list_incorrect_type_annotation(x: torch.Tensor) -> List[float]:
             li = torch.jit.annotate(float, x.tolist())
             return li
 
-        def to_list_unsupported_type_annotation(x):
-            # type: (torch.Tensor) -> List[float]
+        def to_list_unsupported_type_annotation(x: torch.Tensor) -> List[float]:
             li = torch.jit.annotate(List[str], x.tolist())
             return li
 
-        def to_list_type_annotation_wrong_dim(x):
-            # type: (torch.Tensor) -> List[List[float]]
+        def to_list_type_annotation_wrong_dim(x: torch.Tensor) -> List[List[float]]:
             li = torch.jit.annotate(List[List[float]], x.tolist())
             return li
 
-        def to_list_type_annotation_incorrect_scalar_type(x):
-            # type: (torch.Tensor) -> List[float]
+        def to_list_type_annotation_incorrect_scalar_type(x: torch.Tensor) -> List[float]:
             li = torch.jit.annotate(List[float], x.tolist())
             return li
 
@@ -1109,18 +1146,15 @@ class TestList(JitTestCase):
         if not torch.cuda.is_available() or torch.cuda.device_count() == 0:
             self.skipTest("CUDA is not available")
 
-        def to_list_bool_1D(x):
-            # type: (torch.Tensor) -> List[bool]
+        def to_list_bool_1D(x: torch.Tensor) -> List[bool]:
             li = torch.jit.annotate(List[bool], x.tolist())
             return li
 
-        def to_list_int_1D(x):
-            # type: (torch.Tensor) -> List[int]
+        def to_list_int_1D(x: torch.Tensor) -> List[int]:
             li = torch.jit.annotate(List[int], x.tolist())
             return li
 
-        def to_list_float_1D(x):
-            # type: (torch.Tensor) -> List[float]
+        def to_list_float_1D(x: torch.Tensor) -> List[float]:
             li = torch.jit.annotate(List[float], x.tolist())
             return li
 
@@ -1132,8 +1166,7 @@ class TestList(JitTestCase):
             5, dtype=torch.double).cuda(),))
 
     def test_no_element_type_annotation(self):
-        def fn_with_comment(x):
-            # type: (torch.Tensor) -> List
+        def fn_with_comment(x: torch.Tensor) -> List:
             a: List = x.tolist()
             return a
 
@@ -1171,12 +1204,39 @@ class TestDict(JitTestCase):
     def dict_bool(self):
         return {True: 1}
 
+    def test_dict_bool_conversion(self):
+        def if_predicate(d: Dict[int, int]):
+            if d:
+                s, t = 0, 0
+                for k, v in d.items():
+                    s += k
+                    t += v
+
+                return s, t
+            else:
+                return -1, -1
+
+        self.checkScript(if_predicate, ({1: 2, 3: 5},))
+        self.checkScript(if_predicate, ({},))
+
+        def while_predicate(d: Dict[int, int]):
+            while d:
+                d.clear()
+
+        self.checkScript(while_predicate, ({1: 2, 3: 5},))
+        self.checkScript(while_predicate, ({},))
+
+        def ternary_predicate(d: Dict[int, int]):
+            return "non-empty" if d else "empty"
+
+        self.checkScript(ternary_predicate, ({1: 2, 3: 5},))
+        self.checkScript(ternary_predicate, ({},))
+
     def test_del(self):
         def inputs():
             return {'hi': 2, 'bye': 3}
 
-        def fn(x):
-            # type: (Dict[str, int]) -> Dict[str, int]
+        def fn(x: Dict[str, int]) -> Dict[str, int]:
             del x['hi']
             return x
 
@@ -1192,8 +1252,7 @@ class TestDict(JitTestCase):
 
     def test_keys(self):
         @torch.jit.script
-        def keys(x):
-            # type: (Dict[str, Tensor]) -> List[str]
+        def keys(x: Dict[str, Tensor]) -> List[str]:
             return list(x.keys())
 
         self.assertEqual(set(keys(self.dict())), set(self.dict().keys()))
@@ -1208,30 +1267,26 @@ class TestDict(JitTestCase):
 
     def test_values(self):
         @torch.jit.script
-        def values(x):
-            # type: (Dict[str, Tensor]) -> List[Tensor]
+        def values(x: Dict[str, Tensor]) -> List[Tensor]:
             return list(x.values())
 
         the_dict = self.dict()
         self.assertEqual(set(values(the_dict)), set(the_dict.values()))
 
     def test_len(self):
-        def length(x):
-            # type: (Dict[str, Tensor]) -> int
+        def length(x: Dict[str, Tensor]) -> int:
             return len(x)
 
         self.checkScript(length, (self.dict(),))
 
     def test_copy(self):
-        def func(x):
-            # type: (Dict[str, Tensor]) -> Dict[str, Tensor]
+        def func(x: Dict[str, Tensor]) -> Dict[str, Tensor]:
             return x.copy()
 
         self.checkScript(func, (self.dict(),))
 
     def test_items(self):
-        def func(x):
-            # type: (Dict[str, Tensor]) -> List[Tuple[str, Tensor]]
+        def func(x: Dict[str, Tensor]) -> List[Tuple[str, Tensor]]:
             return x.items()
 
         # The value returned by Python is in arbitrary order, so we can't use
@@ -1246,8 +1301,7 @@ class TestDict(JitTestCase):
             self.assertTrue(item in script_out)
 
     def test_pop(self):
-        def pop(x, key):
-            # type: (Dict[str, Tensor], str) -> Tuple[Tensor, Dict[str, Tensor]]
+        def pop(x: Dict[str, Tensor], key: str) -> Tuple[Tensor, Dict[str, Tensor]]:
             return x.pop(key), x
 
         # checkScript doesn't copy the inputs, so we can't use it since this mutates
@@ -1263,16 +1317,14 @@ class TestDict(JitTestCase):
             torch.jit.script(pop)(self.dict(), 'x')
 
 
-        def default_pop(x, key, default):
-            # type: (Dict[str, Tensor], str, Tensor) -> Tuple[Tensor, Dict[str, Tensor]]
+        def default_pop(x: Dict[str, Tensor], key: str, default: Tensor) -> Tuple[Tensor, Dict[str, Tensor]]:
             return x.pop(key, default), x
 
         tester(default_pop, 'a', torch.randn(2, 2))
         tester(default_pop, 'x', torch.randn(2, 2))
 
     def test_setdefault(self):
-        def setdefault(x, key, default):
-            # type: (Dict[str, Tensor], str, Tensor) -> Dict[str, Tensor]
+        def setdefault(x: Dict[str, Tensor], key: str, default: Tensor) -> Dict[str, Tensor]:
             x.setdefault(key, default)
             return x
 
@@ -1280,8 +1332,7 @@ class TestDict(JitTestCase):
         self.checkScript(setdefault, (self.dict(), 'nonexistant', torch.randn(2, 2)))
 
     def test_update(self):
-        def update(a, b):
-            # type: (Dict[str, Tensor], Dict[str, Tensor]) -> Tuple[Dict[str, Tensor], Dict[str, Tensor]]
+        def update(a: Dict[str, Tensor], b: Dict[str, Tensor]) -> Tuple[Dict[str, Tensor], Dict[str, Tensor]]:
             a.update(b)
             return a, b
 
@@ -1298,8 +1349,7 @@ class TestDict(JitTestCase):
         self.checkScript(foo, ())
 
     def test_aug_assign(self):
-        def aug_assign_dict_tensor(a):
-            # type: (Dict[str, Tensor]) -> Dict[str, Tensor]
+        def aug_assign_dict_tensor(a: Dict[str, Tensor]) -> Dict[str, Tensor]:
             a['a'] += 1
             a['b'] -= 12
             a['c'] *= 122
@@ -1307,8 +1357,7 @@ class TestDict(JitTestCase):
             a['c'] %= 2
             return a
 
-        def aug_assign_dict_prim(a):
-            # type: (Dict[str, float]) -> Dict[str, float]
+        def aug_assign_dict_prim(a: Dict[str, float]) -> Dict[str, float]:
             a['a'] += 3.4
             a['b'] -= 2.4
             a['c'] *= 3.0
@@ -1321,8 +1370,7 @@ class TestDict(JitTestCase):
 
     def test_popitem(self):
         @torch.jit.script
-        def popitem(x):
-            # type: (Dict[str, Tensor]) -> Tuple[Tuple[str, Tensor], Dict[str, Tensor]]
+        def popitem(x: Dict[str, Tensor]) -> Tuple[Tuple[str, Tensor], Dict[str, Tensor]]:
             item = x.popitem()
             return item, x
 
@@ -1340,65 +1388,56 @@ class TestDict(JitTestCase):
         self.assertTrue(isinstance(script_out[0][1], torch.Tensor))
 
     def test_clear(self):
-        def clear(x):
-            # type: (Dict[str, Tensor]) -> Dict[str, Tensor]
+        def clear(x: Dict[str, Tensor]) -> Dict[str, Tensor]:
             x.clear()
             return x
 
         self.checkScript(clear, (self.dict(),))
 
     def test_get(self):
-        def get(x, key):
-            # type: (Dict[str, Tensor], str) -> Optional[Tensor]
+        def get(x: Dict[str, Tensor], key: str) -> Optional[Tensor]:
             return x.get(key)
 
         self.checkScript(get, (self.dict(), 'a'))
         self.checkScript(get, (self.dict(), "doesn't exist"))
 
-        def get_default(x, key):
-            # type: (Dict[str, Tensor], str) -> Optional[Tensor]
+        def get_default(x: Dict[str, Tensor], key: str) -> Optional[Tensor]:
             return x.get(key, torch.randn(2, 2))
 
         self.checkScript(get, (self.dict(), 'a'))
         self.checkScript(get, (self.dict(), "doesn't exist"))
 
     def test_get_boolkey(self):
-        def get(x, key):
-            # type: (Dict[bool, int], bool) -> Optional[int]
+        def get(x: Dict[bool, int], key: bool) -> Optional[int]:
             return x.get(key)
 
         self.checkScript(get, (self.dict_bool(), True))
         self.checkScript(get, (self.dict_bool(), False))
 
-        def get_default(x, key):
-            # type: (Dict[bool, int], bool) -> int
+        def get_default(x: Dict[bool, int], key: bool) -> int:
             return x.get(key, 42)
 
         self.checkScript(get_default, (self.dict_bool(), True))
         self.checkScript(get_default, (self.dict_bool(), False))
 
     def test_basic(self):
-        def simple(x):
-            # type: (Dict[str, int]) -> Dict[str, int]
+        def simple(x: Dict[str, int]) -> Dict[str, int]:
             return x
 
         self.checkScript(simple, ({'item': 20, 'other_item': 120},))
 
-        def index(x):
-            # type: (Dict[str, int]) -> int
+        def index(x: Dict[str, int]) -> int:
             return x['item']
 
         self.checkScript(index, ({'item': 20, 'other_item': 120},))
 
-        def type_default():
-            # type: () -> Dict[str, Tensor]
+        def type_default() -> Dict[str, Tensor]:
             return {}
 
         self.checkScript(type_default, ())
 
         @torch.jit.script
-        def missing_index(x):
-            # type: (Dict[str, int]) -> int
+        def missing_index(x: Dict[str, int]) -> int:
             return x['dne']
 
         with self.assertRaisesRegex(RuntimeError, "KeyError"):
@@ -1420,16 +1459,14 @@ class TestDict(JitTestCase):
         '''))
         self.assertEqual({10: 1.2, 11: 1.3}, cu.literal3())
 
-        def list_of_dicts():
-            # type: () -> List[Dict[str, Tensor]]
+        def list_of_dicts() -> List[Dict[str, Tensor]]:
             return [{'word': torch.ones(2) + 3}, {'other word': torch.ones(1) + 2}]
 
         self.checkScript(list_of_dicts, ())
 
     def test_mutability(self):
         @torch.jit.script
-        def fn():
-            # type: () -> Dict[str, int]
+        def fn() -> Dict[str, int]:
             a = torch.jit.annotate(Dict[str, int], {})
             a['ok'] = 10
             return a
@@ -1439,14 +1476,12 @@ class TestDict(JitTestCase):
     def test_key_type(self):
         with self.assertRaisesRegex(RuntimeError, "but instead found type"):
             @torch.jit.script
-            def fn(a):
-                # type: (Dict[str, int]) -> int
+            def fn(a: Dict[str, int]) -> int:
                 return a[None]
 
     def test_loop(self):
         @torch.jit.script
-        def fn(x):
-            # type: (int) -> Dict[str, int]
+        def fn(x: int) -> Dict[str, int]:
             a = torch.jit.annotate(Dict[str, int], {})
             for i in range(x):
                 a['ok'] = i
@@ -1465,16 +1500,14 @@ class TestDict(JitTestCase):
         self.checkScript(fn, (torch.rand(2, 3), torch.rand(2, 3)))
 
     def test_membership(self):
-        def fn(x, y):
-            # type: (Dict[int, int], int) -> int
+        def fn(x: Dict[int, int], y: int) -> int:
             return x.get(y, 3)
 
         d = {1: 2, 3: 4}
         self.checkScript(fn, (d, 3))
         self.checkScript(fn, (d, 2))
 
-        def optional(x, y):
-            # type: (Dict[int, int], int) -> bool
+        def optional(x: Dict[int, int], y: int) -> bool:
             res = x.get(y)
             return res is None
 
@@ -1483,18 +1516,15 @@ class TestDict(JitTestCase):
 
         with self.assertRaisesRegex(RuntimeError, "is actually of type Optional"):
             @torch.jit.script
-            def bad_types(x, y):
-                # type: (Dict[int, int], int) -> int
+            def bad_types(x: Dict[int, int], y: int) -> int:
                 return x.get(y)  # noqa: T484
 
     def test_dict_to_python(self):
         @torch.jit.ignore
-        def python_lookup(my_dict, keys):
-            # type: (Dict[str, int], List[str]) -> List[int]
+        def python_lookup(my_dict: Dict[str, int], keys: List[str]) -> List[int]:
             return [my_dict[k] for k in keys]
 
-        def fn(my_dict, keys):
-            # type: (Dict[str, int], List[str]) -> List[int]
+        def fn(my_dict: Dict[str, int], keys: List[str]) -> List[int]:
             return python_lookup(my_dict, keys)
 
         a_dict = {'a': torch.ones(1), 'b': torch.ones(1) + 1, 'c': torch.ones(1) + 2}
@@ -1546,8 +1576,7 @@ class TestDict(JitTestCase):
         key and value types produces an error.
         """
         # This function uses a type comment.
-        def fn_with_comment(input):
-            # type: (Dict) -> Any
+        def fn_with_comment(input: Dict) -> Any:
             return input
 
         # This function uses Python3 style type annotations.
