@@ -1,7 +1,7 @@
 import os
 import sys
 
-from typing import List
+from typing import Any, List, Tuple
 from collections import OrderedDict
 import torch
 import torch.nn as nn
@@ -405,6 +405,46 @@ class TestModuleContainers(JitTestCase):
             b = AnotherBadModule()
             torch.jit.script(b)
 
+    def test_normal_list_attribute_with_modules_error(self):
+        """
+        Test that an attempt to script a module with a regular list attribute
+        containing other modules fails with a relevant error message.
+        """
+        class Mod(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.a = [torch.nn.ReLU(), torch.nn.ReLU()]
+
+            def forward(self):
+                return len(self.a)
+
+        error_msg = "Could not infer type of list element: Cannot infer concrete type of torch.nn.Module"
+        with self.assertRaisesRegex(RuntimeError, error_msg):
+            torch.jit.script(Mod())
+
+    def test_unannotated_module_attribute_error(self):
+        """
+        Test that an attempt to script a module with an unannotated attribute
+        whose type cannot be inferred fails with a relevant error message
+        suggesting annotating the attribute.
+        """
+        class JitClass:  # noqa: B903
+            def __init__(self, v: int):
+                self.v = v
+
+        class Mod(torch.nn.Module):
+            # Adding following line makes the case pass
+            # v: JitClass
+            def __init__(self):
+                super().__init__()
+                self.v = JitClass(2)
+
+            def forward(self):
+                return self.v
+
+        with self.assertRaisesRegex(RuntimeError, "try adding a type annotation for the attribute"):
+            torch.jit.script(Mod())
+
     def test_empty_dict_override_contains(self):
         class CustomModuleInterface(torch.nn.Module):
             def __init__(self):
@@ -428,3 +468,64 @@ class TestModuleContainers(JitTestCase):
 
         m = MyModule()
         self.checkModule(m, [torch.randn(2, 2)])
+
+    def test_typed_module_dict(self):
+        """
+        Test that a type annotation can be provided for a ModuleDict that allows
+        non-static indexing.
+        """
+        @torch.jit.interface
+        class ModuleInterface(torch.nn.Module):
+            def forward(self, inp: Any) -> Any:
+                pass
+
+        class ImplementsInterface(torch.nn.Module):
+            def forward(self, inp: Any) -> Any:
+                if isinstance(inp, torch.Tensor):
+                    return torch.max(inp, dim=0)
+
+                return inp
+
+        class DoesNotImplementInterface(torch.nn.Module):
+            def forward(self, inp: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor]:
+                return torch.max(inp, dim=0)
+
+        # Test annotation of submodule.
+
+        class Mod(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.d = torch.nn.ModuleDict({"module": ImplementsInterface()})
+
+            def forward(self, x: torch.Tensor, key: str) -> Any:
+                value: ModuleInterface = self.d[key]
+                return value.forward(x)
+
+        m = Mod()
+        self.checkModule(m, (torch.randn(2, 2), "module"))
+
+        # Test annotation of self.
+        class ModDict(torch.nn.ModuleDict):
+            def __init__(self):
+                super().__init__({"module": ImplementsInterface()})
+
+            def forward(self, x: torch.Tensor, key: str) -> Any:
+                submodule: ModuleInterface = self[key]
+                return submodule.forward(x)
+
+        m = ModDict()
+        self.checkModule(m, (torch.randn(2, 2), "module"))
+
+        # Test error message thrown when annotated attribute does not comply with the
+        # annotation.
+        class ModWithWrongAnnotation(torch.nn.ModuleDict):
+            def __init__(self):
+                super().__init__()
+                self.d = torch.nn.ModuleDict({"module": DoesNotImplementInterface()})
+
+            def forward(self, x: torch.Tensor, key: str) -> Any:
+                submodule: ModuleInterface = self.d[key]
+                return submodule.forward(x)
+
+        with self.assertRaisesRegex(RuntimeError, r"Attribute module is not of annotated type"):
+            torch.jit.script(ModWithWrongAnnotation())
