@@ -1,12 +1,13 @@
 # flake8: noqa
 # TODO: enable linting check for this file
 
-from typing import List
+from typing import List, Any
 import torch
 import torch.nn as nn
 import os
 import sys
-from torch.testing._internal.jit_utils import JitTestCase
+from torch import Tensor
+from torch.testing._internal.jit_utils import JitTestCase, make_global
 
 # Make the helper files in test/ importable
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
@@ -22,36 +23,30 @@ class OrigModule(nn.Module):
     def __init__(self):
         super(OrigModule, self).__init__()
 
-    def one(self, inp1, inp2):
-        # type: (Tensor, Tensor) -> Tensor
+    def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
         return inp1 + inp2 + 1
 
-    def two(self, input):
-        # type: (Tensor) -> Tensor
+    def two(self, input: Tensor) -> Tensor:
         return input + 2
 
-    def forward(self, input):
-        # type: (Tensor) -> Tensor
+    def forward(self, input: Tensor) -> Tensor:
         return input + self.one(input, input) + 1
 
 class NewModule(nn.Module):
     def __init__(self):
         super(NewModule, self).__init__()
 
-    def one(self, inp1, inp2):
-        # type: (Tensor, Tensor) -> Tensor
+    def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
         return inp1 * inp2 + 1
 
-    def forward(self, input):
-        # type: (Tensor) -> Tensor
+    def forward(self, input: Tensor) -> Tensor:
         return self.one(input, input + 1)
 
 class TestModuleInterface(JitTestCase):
     def test_not_submodule_interface_call(self):
         @torch.jit.interface
         class ModuleInterface(nn.Module):
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 pass
 
         class TestNotModuleInterfaceCall(nn.Module):
@@ -61,75 +56,61 @@ class TestModuleInterface(JitTestCase):
                 super(TestNotModuleInterfaceCall, self).__init__()
                 self.proxy_mod = OrigModule()
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return self.proxy_mod.two(input)
 
         with self.assertRaisesRegex(RuntimeError, "Tried to access nonexistent attribute"):
             torch.jit.script(TestNotModuleInterfaceCall())
 
     def test_module_interface(self):
-        global OneTwoModule, OneTwoClass
         @torch.jit.interface
         class OneTwoModule(nn.Module):
-            def one(self, x, y):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, x: Tensor, y: Tensor) -> Tensor:
                 pass
 
-            def two(self, x):
-                # type: (Tensor) -> Tensor
+            def two(self, x: Tensor) -> Tensor:
                 pass
 
-            def forward(self, x):
-                # type: (Tensor) -> Tensor
+            def forward(self, x: Tensor) -> Tensor:
                 pass
 
         @torch.jit.interface
         class OneTwoClass(object):
-            def one(self, x, y):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, x: Tensor, y: Tensor) -> Tensor:
                 pass
 
-            def two(self, x):
-                # type: (Tensor) -> Tensor
+            def two(self, x: Tensor) -> Tensor:
                 pass
 
         class FooMod(nn.Module):
-            def one(self, x, y):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, x: Tensor, y: Tensor) -> Tensor:
                 return x + y
 
-            def two(self, x):
-                # type: (Tensor) -> Tensor
+            def two(self, x: Tensor) -> Tensor:
                 return 2 * x
 
-            def forward(self, x):
-                # type: (Tensor) -> Tensor
+            def forward(self, x: Tensor) -> Tensor:
                 return self.one(self.two(x), x)
 
         class BarMod(nn.Module):
-            def one(self, x, y):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, x: Tensor, y: Tensor) -> Tensor:
                 return x * y
 
-            def two(self, x):
-                # type: (Tensor) -> Tensor
+            def two(self, x: Tensor) -> Tensor:
                 return 2 / x
 
-            def forward(self, x):
-                # type: (Tensor) -> Tensor
+            def forward(self, x: Tensor) -> Tensor:
                 return self.two(self.one(x, x))
 
             @torch.jit.export
-            def forward2(self, x):
-                # type: (Tensor) -> Tensor
+            def forward2(self, x: Tensor) -> Tensor:
                 return self.two(self.one(x, x)) + 1
 
+        make_global(OneTwoModule, OneTwoClass)
         def use_module_interface(mod_list: List[OneTwoModule], x: torch.Tensor):
             return mod_list[0].forward(x) + mod_list[1].forward(x)
 
-        def use_class_interface(mod_list, x):
-            # type: (List[OneTwoClass], Tensor) -> Tensor
+        def use_class_interface(mod_list: List[OneTwoClass], x: Tensor) -> Tensor:
             return mod_list[0].two(x) + mod_list[1].one(x, x)
 
         scripted_foo_mod = torch.jit.script(FooMod())
@@ -139,48 +120,66 @@ class TestModuleInterface(JitTestCase):
         self.checkScript(use_class_interface,
                          ([scripted_foo_mod, scripted_bar_mod], torch.rand(3, 4),))
 
-        def call_module_interface_on_other_method(mod_interface, x):
-            # type: (OneTwoModule, Tensor) -> Tensor
+        def call_module_interface_on_other_method(mod_interface: OneTwoModule, x: Tensor) -> Tensor:
             return mod_interface.forward2(x)
 
         # ensure error out when we call the module on the method other than the interface specified.
         with self.assertRaisesRegex(RuntimeError, "Tried to access nonexistent attribute or method"):
             self.checkScript(call_module_interface_on_other_method, (scripted_bar_mod, torch.rand(3, 4),))
 
-
-    def test_module_interface_subtype(self):
-        global OneTwoModule
+    def test_module_doc_string(self):
         @torch.jit.interface
-        class OneTwoModule(nn.Module):
-            def one(self, x, y):
+        class TestInterface(nn.Module):
+            def one(self, inp1, inp2):
                 # type: (Tensor, Tensor) -> Tensor
                 pass
-
-            def two(self, x):
+            def forward(self, input):
                 # type: (Tensor) -> Tensor
+                r"""stuff 1"""
+                r"""stuff 2"""
+                pass
+                r"""stuff 3"""
+
+        class TestModule(nn.Module):
+            proxy_mod : TestInterface
+
+            def __init__(self):
+                super(TestModule, self).__init__()
+                self.proxy_mod = OrigModule()
+
+            def forward(self, input):
+                # type: (Tensor) -> Tensor
+                return self.proxy_mod.forward(input)
+
+        input = torch.randn(3, 4)
+        self.checkModule(TestModule(), (input,))
+
+    def test_module_interface_subtype(self):
+        @torch.jit.interface
+        class OneTwoModule(nn.Module):
+            def one(self, x: Tensor, y: Tensor) -> Tensor:
                 pass
 
-            def forward(self, x):
-                # type: (Tensor) -> Tensor
+            def two(self, x: Tensor) -> Tensor:
                 pass
 
+            def forward(self, x: Tensor) -> Tensor:
+                pass
+
+        make_global(OneTwoModule)
         @torch.jit.script
-        def as_module_interface(x):
-            # type: (OneTwoModule) -> OneTwoModule
+        def as_module_interface(x: OneTwoModule) -> OneTwoModule:
             return x
 
         @torch.jit.script
         class Foo(object):
-            def one(self, x, y):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, x: Tensor, y: Tensor) -> Tensor:
                 return x + y
 
-            def two(self, x):
-                # type: (Tensor) -> Tensor
+            def two(self, x: Tensor) -> Tensor:
                 return 2 * x
 
-            def forward(self, x):
-                # type: (Tensor) -> Tensor
+            def forward(self, x: Tensor) -> Tensor:
                 return self.one(self.two(x), x)
 
         # check class object is not a subtype of module interface
@@ -188,12 +187,10 @@ class TestModuleInterface(JitTestCase):
             as_module_interface(Foo())
 
         class WrongMod(nn.Module):
-            def two(self, x):
-                # type: (int) -> int
+            def two(self, x: int) -> int:
                 return 2 * x
 
-            def forward(self, x):
-                # type: (Tensor) -> Tensor
+            def forward(self, x: Tensor) -> Tensor:
                 return x + torch.randn(3, self.two(3))
 
         scripted_wrong_mod = torch.jit.script(WrongMod())
@@ -202,23 +199,58 @@ class TestModuleInterface(JitTestCase):
         with self.assertRaisesRegex(RuntimeError, "is not compatible with interface"):
             as_module_interface(scripted_wrong_mod)
 
+        # Check that interface implementations can be contravariant in argument types and covariant in return type.
+        @torch.jit.interface
+        class TensorToAny(nn.Module):
+            def forward(self, input: torch.Tensor) -> Any:
+                pass
+
+        make_global(TensorToAny)
+        @torch.jit.script
+        def as_tensor_to_any(x: TensorToAny) -> TensorToAny:
+            return x
+
+        @torch.jit.interface
+        class AnyToAny(nn.Module):
+            def forward(self, input: Any) -> Any:
+                pass
+
+        make_global(AnyToAny)
+        @torch.jit.script
+        def as_any_to_any(x: AnyToAny) -> AnyToAny:
+            return x
+
+        class TensorToAnyImplA(nn.Module):
+            def forward(self, input: Any) -> Any:
+                return input
+
+        class TensorToAnyImplB(nn.Module):
+            def forward(self, input: Any) -> torch.Tensor:
+                return torch.tensor([1])
+
+        class AnyToAnyImpl(nn.Module):
+            def forward(self, input: Any) -> torch.Tensor:
+                return torch.tensor([1])
+
+        as_tensor_to_any(torch.jit.script(TensorToAnyImplA()))
+        as_tensor_to_any(torch.jit.script(TensorToAnyImplB()))
+        as_any_to_any(torch.jit.script(AnyToAnyImpl()))
+
+
     def test_module_interface_inheritance(self):
         with self.assertRaisesRegex(RuntimeError, "does not support inheritance yet. Please directly"):
             @torch.jit.interface
             class InheritMod(nn.ReLU):
-                def three(self, x):
-                    # type: (Tensor) -> Tensor
+                def three(self, x: Tensor) -> Tensor:
                     return 3 * x
 
     def test_module_swap(self):
         @torch.jit.interface
         class ModuleInterface(nn.Module):
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 pass
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 pass
 
         class TestModule(nn.Module):
@@ -228,8 +260,7 @@ class TestModuleInterface(JitTestCase):
                 super(TestModule, self).__init__()
                 self.proxy_mod = OrigModule()
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return self.proxy_mod.forward(input)
 
         scripted_mod = torch.jit.script(TestModule())
@@ -247,20 +278,17 @@ class TestModuleInterface(JitTestCase):
     def test_module_swap_wrong_module(self):
         @torch.jit.interface
         class ModuleInterface(nn.Module):
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 pass
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 pass
 
         class NewModuleWrong(nn.Module):
             def __init__(self):
                 super(NewModuleWrong, self).__init__()
 
-            def forward(self, input):
-                # type: (int) -> int
+            def forward(self, input: int) -> int:
                 return input + 1
 
         class TestModule(nn.Module):
@@ -270,8 +298,7 @@ class TestModuleInterface(JitTestCase):
                 super(TestModule, self).__init__()
                 self.proxy_mod = OrigModule()
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return self.proxy_mod.forward(input)
 
         scripted_mod = torch.jit.script(TestModule())
@@ -282,12 +309,10 @@ class TestModuleInterface(JitTestCase):
     def test_module_swap_no_lazy_compile(self):
         @torch.jit.interface
         class ModuleInterface(nn.Module):
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 pass
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 pass
 
         class TestModule(nn.Module):
@@ -297,20 +322,17 @@ class TestModuleInterface(JitTestCase):
                 super(TestModule, self).__init__()
                 self.proxy_mod = OrigModule()
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return self.proxy_mod.forward(input)
 
         class NewModuleMethodNotLazyCompile(nn.Module):
             def __init__(self):
                 super(NewModuleMethodNotLazyCompile, self).__init__()
 
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 return inp1 * inp2 + 1
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return input + 1
 
         scripted_mod = torch.jit.script(TestModule())
@@ -324,12 +346,10 @@ class TestModuleInterface(JitTestCase):
                 super(NewModuleMethodManualExport, self).__init__()
 
             @torch.jit.export
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 return inp1 * inp2 + 1
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return input + 1
 
         scripted_mod.proxy_mod = torch.jit.script(NewModuleMethodManualExport())
@@ -343,8 +363,7 @@ class TestModuleInterface(JitTestCase):
                 super(TestNoModuleInterface, self).__init__()
                 self.proxy_mod = OrigModule()
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return self.proxy_mod(input)
 
         scripted_no_module_interface = torch.jit.script(TestNoModuleInterface())
@@ -352,19 +371,17 @@ class TestModuleInterface(JitTestCase):
         scripted_no_module_interface.proxy_mod = torch.jit.script(OrigModule())
         # proxy_mod is neither a module interface or have the same JIT type, should fail
         with self.assertRaisesRegex(RuntimeError,
-                                    "Expected a value of type '__torch__.jit.test_module_interface.OrigModule' " +
-                                    "for field 'proxy_mod', but found '__torch__.jit.test_module_interface.NewModule'"):
+                                    "Expected a value of type '__torch__.jit.test_module_interface.OrigModule \(.*\)' " +
+                                    "for field 'proxy_mod', but found '__torch__.jit.test_module_interface.NewModule \(.*\)'"):
             scripted_no_module_interface.proxy_mod = torch.jit.script(NewModule())
 
     def test_script_module_as_interface_swap(self):
         @torch.jit.interface
         class ModuleInterface(nn.Module):
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 pass
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 pass
 
         class OrigScriptModule(torch.jit.ScriptModule):
@@ -372,13 +389,11 @@ class TestModuleInterface(JitTestCase):
                 super(OrigScriptModule, self).__init__()
 
             @torch.jit.script_method
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 return inp1 + inp2 + 1
 
             @torch.jit.script_method
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return input + self.one(input, input) + 1
 
         class NewScriptModule(torch.jit.ScriptModule):
@@ -386,13 +401,11 @@ class TestModuleInterface(JitTestCase):
                 super(NewScriptModule, self).__init__()
 
             @torch.jit.script_method
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 return inp1 * inp2 + 1
 
             @torch.jit.script_method
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return self.one(input, input + 1)
 
         class TestNNModuleWithScriptModule(nn.Module):
@@ -402,8 +415,7 @@ class TestModuleInterface(JitTestCase):
                 super(TestNNModuleWithScriptModule, self).__init__()
                 self.proxy_mod = OrigScriptModule()
 
-            def forward(self, input):
-                # type: (Tensor) -> Tensor
+            def forward(self, input: Tensor) -> Tensor:
                 return self.proxy_mod.forward(input)
 
         input = torch.randn(3, 4)
@@ -434,8 +446,7 @@ class TestModuleInterface(JitTestCase):
 
         @torch.jit.interface
         class ModInterface(torch.nn.Module):
-            def forward(self, x):
-                # type:  (Tensor) -> int
+            def forward(self, x: Tensor) -> int:
                 pass
 
         class TestModule(torch.nn.Module):
@@ -482,8 +493,7 @@ class TestModuleInterface(JitTestCase):
 
         @torch.jit.interface
         class ModInterface(torch.nn.Module):
-            def forward(self, x):
-                # type:  (Tensor) -> int
+            def forward(self, x: Tensor) -> int:
                 pass
 
         class TestModule(torch.nn.Module):
@@ -526,8 +536,7 @@ class TestModuleInterface(JitTestCase):
 
         @torch.jit.interface
         class ModInterface(torch.nn.Module):
-            def forward(self, x):
-                # type:  (Tensor) -> Tensor
+            def forward(self, x: Tensor) -> Tensor:
                 pass
 
         class TestModule(torch.nn.Module):
@@ -572,8 +581,7 @@ class TestModuleInterface(JitTestCase):
 
         @torch.jit.interface
         class ModInterface(torch.nn.Module):
-            def forward(self, x):
-                # type:  (Tensor) -> Tensor
+            def forward(self, x: Tensor) -> Tensor:
                 pass
 
         class TestModule(torch.nn.Module):
@@ -615,8 +623,7 @@ class TestModuleInterface(JitTestCase):
 
         @torch.jit.interface
         class ModInterface(torch.nn.Module):
-            def forward(self, x):
-                # type:  (Tensor) -> Tensor
+            def forward(self, x: Tensor) -> Tensor:
                 pass
 
         class TestModule(torch.nn.Module):
@@ -650,8 +657,7 @@ class TestModuleInterface(JitTestCase):
     def test_module_apis_interface(self):
         @torch.jit.interface
         class ModuleInterface(nn.Module):
-            def one(self, inp1, inp2):
-                # type: (Tensor, Tensor) -> Tensor
+            def one(self, inp1: Tensor, inp2: Tensor) -> Tensor:
                 pass
 
         class TestModule(nn.Module):
