@@ -5,3 +5,6 @@ class PackageAObject:
 
     def __init__(self, obj):
         self.obj = obj
+
+    def return_result(self):
+        return result
