@@ -11,7 +11,9 @@ from torch.quantization import (
     quantize_dynamic,
 )
 from torch.quantization._numeric_suite import (
+    OutputLogger,
     Shadow,
+    ShadowLogger,
     compare_model_outputs,
     compare_model_stub,
     compare_weights,
@@ -21,6 +23,7 @@ from torch.testing._internal.common_quantization import (
     AnnotatedConvModel,
     AnnotatedSingleLayerLinearModel,
     LSTMwithHiddenDynamicModel,
+    AnnotatedTwoLayerLinearModel,
     QuantizationTestCase,
     SingleLayerLinearDynamicModel,
     test_only_eval_fn,
@@ -31,32 +34,32 @@ from torch.testing._internal.common_quantized import override_qengines
 class SubModule(torch.nn.Module):
     def __init__(self):
         super(SubModule, self).__init__()
-        self.mod1 = nn.Identity()
-        self.mod2 = nn.ReLU()
-
-    def forward(self, x):
-        x = self.mod1(x)
-        x = self.mod2(x)
-        return x
-
-
-class ModelWithSubModules(torch.nn.Module):
-    def __init__(self):
-        super(ModelWithSubModules, self).__init__()
         self.qconfig = default_qconfig
-        self.mod1 = SubModule()
-        self.conv = torch.nn.Conv2d(3, 5, 3, bias=False).to(dtype=torch.float)
+        self.mod1 = torch.nn.Conv2d(3, 3, 3, bias=False).to(dtype=torch.float)
+        self.mod2 = nn.ReLU()
         self.quant = QuantStub()
         self.dequant = DeQuantStub()
 
     def forward(self, x):
         x = self.quant(x)
         x = self.mod1(x)
-        x = self.conv(x)
+        x = self.mod2(x)
         x = self.dequant(x)
         return x
 
 
+class ModelWithSubModules(torch.nn.Module):
+    def __init__(self):
+        super(ModelWithSubModules, self).__init__()
+        self.mod1 = SubModule()
+        self.conv = torch.nn.Conv2d(3, 5, 3, bias=False).to(dtype=torch.float)
+
+    def forward(self, x):
+        x = self.mod1(x)
+        x = self.conv(x)
+        return x
+
+
 class ModelWithFunctionals(torch.nn.Module):
     def __init__(self):
         super(ModelWithFunctionals, self).__init__()
@@ -85,8 +88,7 @@ class ModelWithFunctionals(torch.nn.Module):
 class TestEagerModeNumericSuite(QuantizationTestCase):
     @override_qengines
     def test_compare_weights_conv_static(self):
-        r"""Compare the weights of float and static quantized conv layer
-        """
+        r"""Compare the weights of float and static quantized conv layer"""
 
         qengine = torch.backends.quantized.engine
 
@@ -103,13 +105,12 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             model.eval()
             if hasattr(model, "fuse_model"):
                 model.fuse_model()
-            q_model = quantize(model, test_only_eval_fn, self.img_data_2d)
+            q_model = quantize(model, test_only_eval_fn, [self.img_data_2d])
             compare_and_validate_results(model, q_model)
 
     @override_qengines
     def test_compare_weights_linear_static(self):
-        r"""Compare the weights of float and static quantized linear layer
-        """
+        r"""Compare the weights of float and static quantized linear layer"""
 
         qengine = torch.backends.quantized.engine
 
@@ -126,13 +127,12 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             model.eval()
             if hasattr(model, "fuse_model"):
                 model.fuse_model()
-            q_model = quantize(model, test_only_eval_fn, self.calib_data)
+            q_model = quantize(model, test_only_eval_fn, [self.calib_data])
             compare_and_validate_results(model, q_model)
 
     @override_qengines
     def test_compare_weights_linear_dynamic(self):
-        r"""Compare the weights of float and dynamic quantized linear layer
-        """
+        r"""Compare the weights of float and dynamic quantized linear layer"""
 
         qengine = torch.backends.quantized.engine
 
@@ -142,7 +142,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             )
             self.assertEqual(len(weight_dict), 1)
             for k, v in weight_dict.items():
-                self.assertTrue(v["float"].shape == v["quantized"].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
         model_list = [SingleLayerLinearDynamicModel(qengine)]
         for model in model_list:
@@ -154,8 +156,7 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
 
     @override_qengines
     def test_compare_weights_lstm_dynamic(self):
-        r"""Compare the weights of float and dynamic quantized LSTM layer
-        """
+        r"""Compare the weights of float and dynamic quantized LSTM layer"""
 
         qengine = torch.backends.quantized.engine
 
@@ -165,7 +166,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             )
             self.assertEqual(len(weight_dict), 1)
             for k, v in weight_dict.items():
-                self.assertTrue(v["float"].shape == v["quantized"].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
         model_list = [LSTMwithHiddenDynamicModel(qengine)]
         for model in model_list:
@@ -177,8 +180,7 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
 
     @override_qengines
     def test_compare_model_stub_conv_static(self):
-        r"""Compare the output of static quantized conv layer and its float shadow module
-        """
+        r"""Compare the output of static quantized conv layer and its float shadow module"""
 
         qengine = torch.backends.quantized.engine
 
@@ -186,7 +188,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             ob_dict = compare_model_stub(float_model, q_model, module_swap_list, data)
             self.assertEqual(len(ob_dict), 1)
             for k, v in ob_dict.items():
-                self.assertTrue(v["float"].shape == v["quantized"].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
         model_list = [AnnotatedConvModel(qengine), AnnotatedConvBnReLUModel(qengine)]
         module_swap_list = [nn.Conv2d, nn.intrinsic.modules.fused.ConvReLU2d]
@@ -194,15 +198,14 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             model.eval()
             if hasattr(model, "fuse_model"):
                 model.fuse_model()
-            q_model = quantize(model, test_only_eval_fn, self.img_data_2d)
+            q_model = quantize(model, test_only_eval_fn, [self.img_data_2d])
             compare_and_validate_results(
                 model, q_model, module_swap_list, self.img_data_2d[0][0]
             )
 
     @override_qengines
     def test_compare_model_stub_linear_static(self):
-        r"""Compare the output of static quantized linear layer and its float shadow module
-        """
+        r"""Compare the output of static quantized linear layer and its float shadow module"""
 
         qengine = torch.backends.quantized.engine
 
@@ -210,7 +213,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             ob_dict = compare_model_stub(float_model, q_model, module_swap_list, data)
             self.assertEqual(len(ob_dict), 1)
             for k, v in ob_dict.items():
-                self.assertTrue(v["float"].shape == v["quantized"].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
         linear_data = self.calib_data[0][0]
         module_swap_list = [nn.Linear]
@@ -219,31 +224,55 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             model.eval()
             if hasattr(model, "fuse_model"):
                 model.fuse_model()
-            q_model = quantize(model, test_only_eval_fn, self.calib_data)
+            q_model = quantize(model, test_only_eval_fn, [self.calib_data])
+            compare_and_validate_results(model, q_model, module_swap_list, linear_data)
+
+    @override_qengines
+    def test_compare_model_stub_partial(self):
+        r"""Compare the output of static quantized linear layer and its float shadow module"""
+
+        qengine = torch.backends.quantized.engine
+        # TODO: Rebase on top of PR to remove compare and validate results here
+
+        def compare_and_validate_results(float_model, q_model, module_swap_list, data):
+            ob_dict = compare_model_stub(float_model, q_model, module_swap_list, data)
+            self.assertEqual(len(ob_dict), 1)
+            for k, v in ob_dict.items():
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
+
+        linear_data = self.calib_data[0][0]
+        module_swap_list = [nn.Linear]
+        model_list = [AnnotatedTwoLayerLinearModel()]
+        for model in model_list:
+            model.eval()
+            if hasattr(model, "fuse_model"):
+                model.fuse_model()
+            q_model = quantize(model, test_only_eval_fn, [self.calib_data])
             compare_and_validate_results(model, q_model, module_swap_list, linear_data)
 
     @override_qengines
     def test_compare_model_stub_submodule_static(self):
-        r"""Compare the output of static quantized submodule and its float shadow module
-        """
+        r"""Compare the output of static quantized submodule and its float shadow module"""
 
         qengine = torch.backends.quantized.engine
 
         model = ModelWithSubModules().eval()
-        q_model = quantize(model, test_only_eval_fn, self.img_data_2d)
-        module_swap_list = [SubModule]
+        q_model = quantize(model, test_only_eval_fn, [self.img_data_2d])
+        module_swap_list = [SubModule, nn.Conv2d]
         ob_dict = compare_model_stub(
             model, q_model, module_swap_list, self.img_data_2d[0][0]
         )
+        # Since conv is not quantized, we do not insert a shadow module
+        # mod1 contains a linear that is quantized, so we insert a shadow module
         self.assertTrue(isinstance(q_model.mod1, Shadow))
         self.assertFalse(isinstance(q_model.conv, Shadow))
-        for k, v in ob_dict.items():
-            torch.testing.assert_allclose(v["float"], v["quantized"].dequantize())
+
 
     @override_qengines
     def test_compare_model_stub_functional_static(self):
-        r"""Compare the output of static quantized functional layer and its float shadow module
-        """
+        r"""Compare the output of static quantized functional layer and its float shadow module"""
 
         qengine = torch.backends.quantized.engine
 
@@ -264,12 +293,13 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
         self.assertTrue(isinstance(q_model.my_scalar_add, Shadow))
         self.assertTrue(isinstance(q_model.my_scalar_mul, Shadow))
         for k, v in ob_dict.items():
-            self.assertTrue(v["float"].shape == v["quantized"].shape)
+            self.assertTrue(len(v["float"]) == len(v["quantized"]))
+            for i, val in enumerate(v["quantized"]):
+                self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
     @override_qengines
     def test_compare_model_stub_linear_dynamic(self):
-        r"""Compare the output of dynamic quantized linear layer and its float shadow module
-        """
+        r"""Compare the output of dynamic quantized linear layer and its float shadow module"""
 
         qengine = torch.backends.quantized.engine
 
@@ -277,7 +307,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             ob_dict = compare_model_stub(float_model, q_model, module_swap_list, data)
             self.assertEqual(len(ob_dict), 1)
             for k, v in ob_dict.items():
-                self.assertTrue(v["float"].shape == v["quantized"].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
         linear_data = self.calib_data[0][0]
 
@@ -292,8 +324,7 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
 
     @override_qengines
     def test_compare_model_stub_lstm_dynamic(self):
-        r"""Compare the output of dynamic quantized LSTM layer and its float shadow module
-        """
+        r"""Compare the output of dynamic quantized LSTM layer and its float shadow module"""
 
         qengine = torch.backends.quantized.engine
 
@@ -305,7 +336,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             )
             self.assertEqual(len(ob_dict), 1)
             for k, v in ob_dict.items():
-                self.assertTrue(v["float"].shape == v["quantized"].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
         lstm_input = torch.rand((1, 1, 2))
         lstm_hidden = (torch.rand(1, 1, 2), torch.rand(1, 1, 2))
@@ -341,7 +374,7 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             model.eval()
             if hasattr(model, "fuse_model"):
                 model.fuse_model()
-            q_model = quantize(model, test_only_eval_fn, self.img_data_2d)
+            q_model = quantize(model, test_only_eval_fn, [self.img_data_2d])
             compare_and_validate_results(model, q_model, self.img_data_2d[0][0])
 
     @override_qengines
@@ -357,7 +390,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
 
             self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
             for k, v in act_compare_dict.items():
-                self.assertTrue(v["float"].shape == v["quantized"].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
         linear_data = self.calib_data[0][0]
         model_list = [AnnotatedSingleLayerLinearModel(qengine)]
@@ -365,7 +400,7 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
             model.eval()
             if hasattr(model, "fuse_model"):
                 model.fuse_model()
-            q_model = quantize(model, test_only_eval_fn, self.calib_data)
+            q_model = quantize(model, test_only_eval_fn, [self.calib_data])
             compare_and_validate_results(model, q_model, linear_data)
 
     @override_qengines
@@ -393,7 +428,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
         }
         self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
         for k, v in act_compare_dict.items():
-            self.assertTrue(v["float"].shape == v["quantized"].shape)
+            self.assertTrue(len(v["float"]) == len(v["quantized"]))
+            for i, val in enumerate(v["quantized"]):
+                self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
     @override_qengines
     def test_compare_model_outputs_linear_dynamic(self):
@@ -408,7 +445,9 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
 
             self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
             for k, v in act_compare_dict.items():
-                self.assertTrue(v["float"].shape == v["quantized"].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
 
         linear_data = self.calib_data[0][0]
 
@@ -435,7 +474,18 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
 
             self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
             for k, v in act_compare_dict.items():
-                self.assertTrue(v["float"][0].shape == v["quantized"][0].shape)
+                self.assertTrue(len(v["float"]) == len(v["quantized"]))
+                for i, val in enumerate(v["quantized"]):
+                    self.assertTrue(len(v["float"][i]) == len(v["quantized"][i]))
+                    if i == 0:
+                        self.assertTrue(v["float"][i][0].shape == v["quantized"][i][0].shape)
+                    else:
+                        self.assertTrue(
+                            v["float"][i][0].shape == v["quantized"][i][0].shape
+                        )
+                        self.assertTrue(
+                            v["float"][i][1].shape == v["quantized"][i][1].shape
+                        )
 
         lstm_input = torch.rand((1, 1, 2))
         lstm_hidden = (torch.rand(1, 1, 2), torch.rand(1, 1, 2))
@@ -447,3 +497,35 @@ class TestEagerModeNumericSuite(QuantizationTestCase):
                 model.fuse_model()
             q_model = quantize_dynamic(model)
             compare_and_validate_results(model, q_model, lstm_input, lstm_hidden)
+
+    @override_qengines
+    def test_output_logger(self):
+        r"""Compare output from OutputLogger with the expected results"""
+        x = torch.rand(2, 2)
+        y = torch.rand(2, 1)
+
+        l = []
+        l.append(x)
+        l.append(y)
+
+        logger = OutputLogger()
+        logger.forward(x)
+        logger.forward(y)
+
+        self.assertEqual(l, logger.stats["tensor_val"])
+
+    @override_qengines
+    def test_shadow_logger(self):
+        r"""Compare output from ShawdowLogger with the expected results"""
+        a_float = torch.rand(2, 2)
+        a_quantized = torch.rand(2, 2)
+
+        b_float = torch.rand(3, 2, 2)
+        b_quantized = torch.rand(3, 2, 2)
+
+        logger = ShadowLogger()
+        logger.forward(a_float, a_quantized)
+        logger.forward(b_float, b_quantized)
+
+        self.assertEqual(len(logger.stats["float"]), 2)
+        self.assertEqual(len(logger.stats["quantized"]), 2)
