@@ -1,7 +1,10 @@
 import torch
 import unittest
-from torch.testing._internal.common_utils import TestCase, run_tests
+from torch.testing._internal.common_utils import TestCase, run_tests, TEST_WITH_ROCM, TEST_WITH_SLOW
 from torch.testing._internal.common_device_type import instantiate_device_type_tests, dtypes, skipCUDAIfRocm
+from torch._six import inf, nan
+
+N_values = [20] if not TEST_WITH_SLOW else [30, 300]
 
 class TestForeach(TestCase):
     foreach_bin_ops = [
@@ -18,20 +21,6 @@ class TestForeach(TestCase):
         torch._foreach_div_,
     ]
 
-    foreach_bin_ops_sl = [
-        torch._foreach_add_scalar_list,
-        torch._foreach_sub_scalar_list,
-        torch._foreach_mul_scalar_list,
-        torch._foreach_div_scalar_list,
-    ]
-
-    foreach_bin_ops_sl_ = [
-        torch._foreach_add_scalar_list_,
-        torch._foreach_sub_scalar_list_,
-        torch._foreach_mul_scalar_list_,
-        torch._foreach_div_scalar_list_,
-    ]
-
     torch_bin_ops = [
         torch.add,
         torch.sub,
@@ -39,6 +28,39 @@ class TestForeach(TestCase):
         torch.div,
     ]
 
+    unary_ops = [
+        # foreach_op, foreach_op_, torch_op, bf16, complex64/128
+        (torch._foreach_sqrt, torch._foreach_sqrt_, torch.sqrt, True , True),
+        (torch._foreach_exp, torch._foreach_exp_, torch.exp, True, True),
+        (torch._foreach_acos, torch._foreach_acos_, torch.acos, False, True),
+        (torch._foreach_asin, torch._foreach_asin_, torch.asin, False, True),
+        (torch._foreach_atan, torch._foreach_atan_, torch.atan, False, True),
+        (torch._foreach_cos, torch._foreach_cos_, torch.cos, True, True),
+        (torch._foreach_cosh, torch._foreach_cosh_, torch.cosh, False, True),
+        (torch._foreach_log, torch._foreach_log_, torch.log, True, True),
+        (torch._foreach_log10, torch._foreach_log10_, torch.log10, True, True),
+        (torch._foreach_log2, torch._foreach_log2_, torch.log2, True, True),
+        (torch._foreach_neg, torch._foreach_neg_, torch.neg, True, True),
+        (torch._foreach_tan, torch._foreach_tan_, torch.tan, False, True),
+        (torch._foreach_tanh, torch._foreach_tanh_, torch.tanh, True, True),
+        (torch._foreach_sin, torch._foreach_sin_, torch.sin, False, True),
+        (torch._foreach_sinh, torch._foreach_sinh_, torch.sinh, False, True),
+        (torch._foreach_ceil, torch._foreach_ceil_, torch.ceil, False, False),
+        (torch._foreach_erf, torch._foreach_erf_, torch.erf, True, False),
+        (torch._foreach_erfc, torch._foreach_erfc_, torch.erfc, False, False),
+        (torch._foreach_expm1, torch._foreach_expm1_, torch.expm1, False, False),
+        (torch._foreach_floor, torch._foreach_floor_, torch.floor, False, False),
+        (torch._foreach_log1p, torch._foreach_log1p_, torch.log1p, True, False),
+        (torch._foreach_round, torch._foreach_round_, torch.round, False, False),
+        (torch._foreach_frac, torch._foreach_frac_, torch.frac, False, False),
+        (torch._foreach_reciprocal, torch._foreach_reciprocal_, torch.reciprocal, True, True),
+        (torch._foreach_sigmoid, torch._foreach_sigmoid_, torch.sigmoid, True, False),
+        (torch._foreach_trunc, torch._foreach_trunc_, torch.trunc, False, False),
+
+        # See test_abs
+        # (torch._foreach_abs, torch._foreach_abs_, torch.abs, True, True),
+    ]
+
     def _get_test_data(self, device, dtype, N):
         if dtype in [torch.bfloat16, torch.bool, torch.float16]:
             tensors = [torch.randn(N, N, device=device).to(dtype) for _ in range(N)]
@@ -50,68 +72,181 @@ class TestForeach(TestCase):
         return tensors
 
     def _test_bin_op_list(self, device, dtype, foreach_op, foreach_op_, torch_op):
-        for N in [30, 300]:
+        for N in N_values:
             tensors1 = self._get_test_data(device, dtype, N)
             tensors2 = self._get_test_data(device, dtype, N)
 
-            expected = [torch_op(tensors1[i], tensors2[i]) for i in range(N)]
+            # Mimics cuda kernel dtype flow.  With fp16/bf16 input, runs in fp32 and casts output back to fp16/bf16.
+            control_dtype = torch.float32 if (self.device_type == 'cuda' and
+                                              (dtype is torch.float16 or dtype is torch.bfloat16)) else dtype
+            expected = [torch_op(tensors1[i].to(dtype=control_dtype),
+                                 tensors2[i].to(dtype=control_dtype)).to(dtype=dtype) for i in range(N)]
             res = foreach_op(tensors1, tensors2)
             foreach_op_(tensors1, tensors2)
             self.assertEqual(res, tensors1)
-            self.assertEqual(tensors1, res)
-
-    def _test_unary_op(self, device, dtype, foreach_op, foreach_op_, torch_op):
-        for N in [30, 300]:
-            tensors1 = self._get_test_data(device, dtype, N)
-            expected = [torch_op(tensors1[i]) for i in range(N)]
-            res = foreach_op(tensors1)
-            foreach_op_(tensors1)
-            self.assertEqual(res, tensors1)
-            self.assertEqual(tensors1, expected)
+            if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                self.assertEqual(tensors1, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+            else:
+                self.assertEqual(tensors1, expected)
 
     def _test_pointwise_op(self, device, dtype, foreach_op, foreach_op_, torch_op):
+        for N in N_values:
+            values = [2 + i for i in range(N)]
+            for vals in [values[0], values]:
+                tensors = self._get_test_data(device, dtype, N)
+                tensors1 = self._get_test_data(device, dtype, N)
+                tensors2 = self._get_test_data(device, dtype, N)
+
+                # Mimics cuda kernel dtype flow.  With fp16/bf16 input, runs in fp32 and casts output back to fp16/bf16.
+                control_dtype = torch.float32 if (self.device_type == 'cuda' and 
+                                                  (dtype is torch.float16 or dtype is torch.bfloat16)) else dtype
+
+                if not isinstance(vals, list):
+                    expected = [torch_op(tensors[i].to(dtype=control_dtype),
+                                         tensors1[i].to(dtype=control_dtype),
+                                         tensors2[i].to(dtype=control_dtype),
+                                         value=values[0]).to(dtype=dtype) for i in range(N)]
+                else:
+                    expected = [torch_op(tensors[i].to(dtype=control_dtype),
+                                         tensors1[i].to(dtype=control_dtype),
+                                         tensors2[i].to(dtype=control_dtype),
+                                         value=values[i]).to(dtype=dtype) for i in range(N)]
+
+                res = foreach_op(tensors, tensors1, tensors2, vals)
+                foreach_op_(tensors, tensors1, tensors2, vals)
+                self.assertEqual(res, tensors)
+
+                if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                    self.assertEqual(tensors, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+                else:
+                    self.assertEqual(tensors, expected)
+
+                # test error cases
+                for op in [torch._foreach_addcmul, torch._foreach_addcmul_, torch._foreach_addcdiv, torch._foreach_addcdiv_]:
+                    tensors = self._get_test_data(device, dtype, N)
+                    tensors1 = self._get_test_data(device, dtype, N)
+                    tensors2 = self._get_test_data(device, dtype, N)
+
+                    with self.assertRaisesRegex(RuntimeError, "Tensor list must have same number of elements as scalar list."):
+                        op(tensors, tensors1, tensors2, [2 for _ in range(N + 1)])
+
+                    with self.assertRaisesRegex(RuntimeError, "Tensor list must have same number of elements as scalar list."):
+                        op(tensors, tensors1, tensors2, [2 for _ in range(N - 1)])
+
+                    tensors = self._get_test_data(device, dtype, N + 1)
+                    with self.assertRaisesRegex(RuntimeError, "Tensor lists must have the same number of tensors, got 21 and 20"):
+                        op(tensors, tensors1, tensors2, [2 for _ in range(N)])
+
+                    tensors1 = self._get_test_data(device, dtype, N + 1)
+                    with self.assertRaisesRegex(RuntimeError, "Tensor lists must have the same number of tensors, got 21 and 20"):
+                        op(tensors, tensors1, tensors2, [2 for _ in range(N)])
+
+    def _test_bin_op_list_alpha(self, device, dtype, foreach_op, foreach_op_, torch_op):
         for N in [30, 300]:
-            tensors = self._get_test_data(device, dtype, N)
             tensors1 = self._get_test_data(device, dtype, N)
             tensors2 = self._get_test_data(device, dtype, N)
-            value = 2
+            alpha = 2
 
-            expected = [torch_op(tensors[i], tensors1[i], tensors2[i], value=value) for i in range(N)]
+            # Mimics cuda kernel dtype flow.  With fp16/bf16 input, runs in fp32 and casts output back to fp16/bf16.
+            control_dtype = torch.float32 if (self.device_type == 'cuda' and
+                                              (dtype is torch.float16 or dtype is torch.bfloat16)) else dtype
+            expected = [torch_op(tensors1[i].to(dtype=control_dtype),
+                                 torch.mul(tensors2[i].to(dtype=control_dtype),
+                                 alpha)).to(dtype=dtype) for i in range(N)]
+            res = foreach_op(tensors1, tensors2, alpha=alpha)
+            foreach_op_(tensors1, tensors2, alpha=alpha)
+            self.assertEqual(res, tensors1)
 
-            res = foreach_op(tensors, tensors1, tensors2, value)
-            foreach_op_(tensors, tensors1, tensors2, value)
-            self.assertEqual(res, tensors)
-            self.assertEqual(tensors, expected)
-
-    def _test_bin_op_list_alpha(self, device, dtype, foreach_op, foreach_op_, torch_op, N=20):
-        tensors1 = self._get_test_data(device, dtype, N)
-        tensors2 = self._get_test_data(device, dtype, N)
-        alpha = 2
-
-        expected = [torch_op(tensors1[i], torch.mul(tensors2[i], alpha)) for i in range(N)]
-        res = foreach_op(tensors1, tensors2, alpha=alpha)
-        foreach_op_(tensors1, tensors2, alpha=alpha)
-        self.assertEqual(res, tensors1)
-
-        if dtype == torch.bool:
-            expected = [e.to(torch.bool) for e in expected]
-        self.assertEqual(tensors1, expected)
+            if dtype == torch.bool:
+                expected = [e.to(torch.bool) for e in expected]
+            if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                self.assertEqual(tensors1, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+            else:
+                self.assertEqual(tensors1, expected)
 
     #
     # Unary ops
     #
-    @dtypes(*[torch.float, torch.double, torch.complex64, torch.complex128])
-    def test_sqrt(self, device, dtype):
-        self._test_unary_op(device, dtype, torch._foreach_sqrt, torch._foreach_sqrt_, torch.sqrt)
+    @dtypes(*(torch.testing.floating_and_complex_types_and(torch.bfloat16, torch.half)))
+    def test_unary_ops(self, device, dtype):
+        for fe_op, fe_op_, torch_op, support_bfloat16, support_complex in self.unary_ops:
+            for N in N_values:
+                tensors1 = self._get_test_data(device, dtype, N)
+                # Mimics cuda kernel dtype flow.  With fp16/bf16 input, runs in fp32 and casts output back to fp16/bf16.
+                control_dtype = torch.float32 if (self.device_type == 'cuda' and
+                                                  (dtype is torch.float16 or dtype is torch.bfloat16)) else dtype
 
-    @dtypes(*[torch.float, torch.double, torch.complex64, torch.complex128])
-    def test_exp(self, device, dtype):
-        self._test_unary_op(device, dtype, torch._foreach_exp, torch._foreach_exp_, torch.exp)
+                if self.device_type == 'cpu' and dtype == torch.half and torch_op not in [torch.neg, torch.frac, torch.reciprocal]:
+                    with self.assertRaisesRegex(RuntimeError, r"not implemented for \'Half\'"):
+                        expected = [torch_op(tensors1[i]) for i in range(N)]
+
+                    with self.assertRaisesRegex(RuntimeError, r"not implemented for \'Half\'"):
+                        res = fe_op(tensors1)
+                    break
+
+                if dtype == torch.bfloat16 and not support_bfloat16:
+                    if self.device_type == 'cuda' or torch_op in [torch.sinh, torch.cosh]:
+                        with self.assertRaisesRegex(RuntimeError, r"not implemented for \'BFloat16\'"):
+                            expected = [torch_op(tensors1[i]) for i in range(N)]
+
+                        with self.assertRaisesRegex(RuntimeError, r"not implemented for \'BFloat16\'"):
+                            res = fe_op(tensors1)
+                        break
+
+                if dtype in [torch.complex64, torch.complex128] and not support_complex:
+                    if not (self.device_type == 'cpu' and torch_op in [torch.sigmoid]):
+                        # not using assertRaisesRegex due to different error messages
+                        with self.assertRaises(RuntimeError):
+                            expected = [torch_op(tensors1[i]) for i in range(N)]
+
+                        with self.assertRaises(RuntimeError):
+                            res = fe_op(tensors1)
+                        break
+
+                expected = [torch_op(tensors1[i].to(dtype=control_dtype)).to(dtype=dtype) for i in range(N)]
+                res = fe_op(tensors1)
+                if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                    self.assertEqual(res, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+
+                    fe_op_(tensors1)
+                    self.assertEqual(res, tensors1)
+                else:
+                    self.assertEqual(res, expected)
+
+                    fe_op_(tensors1)
+                    self.assertEqual(res, tensors1)
+
+    # Separate test for abs due to a lot of special cases
+    # Absolute value of a complex number a + bj is defined as sqrt(a^2 + b^2), i.e. a floating point
+    @dtypes(*(torch.testing.floating_and_complex_types_and(torch.bfloat16, torch.half)))
+    def test_abs(self, device, dtype):
+        for N in N_values:
+            tensors1 = self._get_test_data(device, dtype, N)
+            # Mimics cuda kernel dtype flow.  With fp16/bf16 input, runs in fp32 and casts output back to fp16/bf16.
+            control_dtype = torch.float32 if (self.device_type == 'cuda' and
+                                              (dtype is torch.float16 or dtype is torch.bfloat16)) else dtype
+
+            expected = [torch.abs(tensors1[i].to(dtype=control_dtype)).to(dtype=dtype) for i in range(N)]
+            res = torch._foreach_abs(tensors1)
+            if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                self.assertEqual(res, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+
+                torch._foreach_abs_(tensors1)
+                self.assertEqual(res, tensors1)
+            else:
+                expected = [torch.abs(tensors1[i]) for i in range(N)]
+                self.assertEqual(res, expected)
+
+                if dtype in [torch.complex64, torch.complex128]:
+                    with self.assertRaisesRegex(RuntimeError, r"In-place abs is not supported for complex tensors."):
+                        torch._foreach_abs_(tensors1)
+                else:
+                    torch._foreach_abs_(tensors1)
+                    self.assertEqual(res, tensors1)
 
     #
     # Pointwise ops
     #
-    @skipCUDAIfRocm
     @dtypes(*torch.testing.get_all_dtypes(include_bfloat16=False, include_bool=False, include_complex=False))
     def test_addcmul(self, device, dtype):
         if self.device_type == 'cpu':
@@ -139,12 +274,84 @@ class TestForeach(TestCase):
                 return
         self._test_pointwise_op(device, dtype, torch._foreach_addcdiv, torch._foreach_addcdiv_, torch.addcdiv)
 
+    @dtypes(*torch.testing.get_all_dtypes(include_bfloat16=False, include_bool=False, include_complex=False))
+    def test_min_max(self, device, dtype):
+        for N in N_values:
+            tensors1 = self._get_test_data(device, dtype, N)
+            tensors2 = self._get_test_data(device, dtype, N)
+
+            # Mimics cuda kernel dtype flow.  With fp16/bf16 input, runs in fp32 and casts output back to fp16/bf16.
+            control_dtype = torch.float32 if (self.device_type == 'cuda' and
+                                              (dtype is torch.float16 or dtype is torch.bfloat16)) else dtype
+
+            expected_max = [torch.max(tensors1[i].to(dtype=control_dtype),
+                                      tensors2[i].to(dtype=control_dtype)).to(dtype=dtype) for i in range(N)]
+
+            expected_min = [torch.min(tensors1[i].to(dtype=control_dtype),
+                                      tensors2[i].to(dtype=control_dtype)).to(dtype=dtype) for i in range(N)]
+
+            res_max = torch._foreach_maximum(tensors1, tensors2)
+            self.assertEqual(res_max, expected_max)
+
+            res_min = torch._foreach_minimum(tensors1, tensors2)
+            self.assertEqual(res_min, expected_min)
+
+
+    @dtypes(*(torch.testing.get_all_fp_dtypes(include_half=True, include_bfloat16=False)))
+    def test_max_min_float_inf_nan(self, device, dtype):
+        a = [
+            torch.tensor([float('inf')], device=device, dtype=dtype),
+            torch.tensor([-float('inf')], device=device, dtype=dtype),
+            torch.tensor([float('nan')], device=device, dtype=dtype),
+            torch.tensor([float('nan')], device=device, dtype=dtype)
+        ]
+
+        b = [
+            torch.tensor([-float('inf')], device=device, dtype=dtype),
+            torch.tensor([float('inf')], device=device, dtype=dtype),
+            torch.tensor([float('inf')], device=device, dtype=dtype),
+            torch.tensor([float('nan')], device=device, dtype=dtype)
+        ]
+
+        expected = [torch.max(a1, b1) for a1, b1 in zip(a, b)]
+        res = torch._foreach_maximum(a, b)
+        self.assertEqual(expected, res)
+
+        expected = [torch.min(a1, b1) for a1, b1 in zip(a, b)]
+        res = torch._foreach_minimum(a, b)
+        self.assertEqual(expected, res)
+
+    @dtypes(*(torch.testing.get_all_fp_dtypes(include_half=True, include_bfloat16=False)))
+    def test_max_min_inf_nan(self, device, dtype):
+        a = [
+            torch.tensor([inf], device=device, dtype=dtype),
+            torch.tensor([-inf], device=device, dtype=dtype),
+            torch.tensor([nan], device=device, dtype=dtype),
+            torch.tensor([nan], device=device, dtype=dtype)
+        ]
+
+        b = [
+            torch.tensor([-inf], device=device, dtype=dtype),
+            torch.tensor([inf], device=device, dtype=dtype),
+            torch.tensor([inf], device=device, dtype=dtype),
+            torch.tensor([nan], device=device, dtype=dtype)
+        ]
+
+        expected_max = [torch.max(a1, b1) for a1, b1 in zip(a, b)]
+        res_max = torch._foreach_maximum(a, b)
+        self.assertEqual(expected_max, res_max)
+
+        expected_min = [torch.min(a1, b1) for a1, b1 in zip(a, b)]
+        res_min = torch._foreach_minimum(a, b)
+        self.assertEqual(expected_min, res_min)
+
     #
     # Ops with scalar
     #
+    @skipCUDAIfRocm
     @dtypes(*torch.testing.get_all_dtypes())
     def test_int_scalar(self, device, dtype):
-        for N in [30, 300]:
+        for N in N_values:
             for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops,
                                                                      self.foreach_bin_ops_,
                                                                      self.torch_bin_ops):
@@ -179,15 +386,16 @@ class TestForeach(TestCase):
                     foreach_bin_op_(tensors, scalar)
                     self.assertEqual(tensors, expected)
 
-    # TODO[Fix scalar list]: 
+    # TODO[Fix scalar list]:
     # We need to update codegen to correctly handle function overloads with float[] and int[].
     # As optimizers work with float tensors, the result will always be torch.float32 for now.
     # Current schema is using 'float[]' as scalar list type.
+    @skipCUDAIfRocm
     @dtypes(*torch.testing.get_all_dtypes())
     def test_int_scalarlist(self, device, dtype):
-        for N in [30, 300]:
-            for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops_sl,
-                                                                     self.foreach_bin_ops_sl_,
+        for N in N_values:
+            for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops,
+                                                                     self.foreach_bin_ops_,
                                                                      self.torch_bin_ops):
                 tensors = self._get_test_data(device, dtype, N)
                 scalars = [1 for _ in range(N)]
@@ -217,7 +425,7 @@ class TestForeach(TestCase):
                     else:
                         # TODO[type promotion]: Fix once type promotion is enabled.
                         self.assertEqual(res, [e.to(dtype) for e in expected])
-                else: 
+                else:
                     self.assertEqual(res, expected)
 
                 if dtype in torch.testing.integral_types() and self.device_type == 'cpu':
@@ -228,15 +436,23 @@ class TestForeach(TestCase):
                     foreach_bin_op_(tensors, scalars)
                     self.assertEqual(res, tensors)
 
+    @skipCUDAIfRocm
     @dtypes(*torch.testing.get_all_dtypes())
     def test_float_scalar(self, device, dtype):
-        for N in [30, 300]:
+        for N in N_values:
             for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops,
                                                                      self.foreach_bin_ops_,
                                                                      self.torch_bin_ops):
                 tensors = self._get_test_data(device, dtype, N)
                 scalar = 3.3
-                expected = [torch_bin_op(t, scalar) for t in tensors]
+
+                # Mimics cuda kernel dtype flow.  With fp16/bf16 input, runs in fp32 and casts output back to fp16/bf16.
+                control_dtype = torch.float32 if (self.device_type == 'cuda' and
+                                                  (dtype is torch.float16 or dtype is torch.bfloat16)) else dtype
+                expected = [torch_bin_op(t.to(dtype=control_dtype),
+                                         scalar) for t in tensors]
+                if (dtype is torch.float16 or dtype is torch.bfloat16):
+                    expected = [e.to(dtype=dtype) for e in expected]
 
                 if dtype == torch.bool:
                     if foreach_bin_op == torch._foreach_sub:
@@ -248,7 +464,10 @@ class TestForeach(TestCase):
                     return
 
                 res = foreach_bin_op(tensors, scalar)
-                self.assertEqual(res, expected)
+                if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                    self.assertEqual(res, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+                else:
+                    self.assertEqual(res, expected)
 
                 if dtype in torch.testing.integral_types():
                     with self.assertRaisesRegex(RuntimeError, "result type Float can't be cast to the desired output type"):
@@ -256,17 +475,28 @@ class TestForeach(TestCase):
                     return
 
                 foreach_bin_op_(tensors, scalar)
-                self.assertEqual(tensors, expected)
+                if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                    self.assertEqual(tensors, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+                else:
+                    self.assertEqual(tensors, expected)
 
+    @skipCUDAIfRocm
     @dtypes(*torch.testing.get_all_dtypes())
     def test_float_scalarlist(self, device, dtype):
-        for N in [30, 300]:
-            for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops_sl,
-                                                                     self.foreach_bin_ops_sl_,
+        for N in N_values:
+            for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops,
+                                                                     self.foreach_bin_ops_,
                                                                      self.torch_bin_ops):
                 tensors = self._get_test_data(device, dtype, N)
                 scalars = [1.1 for _ in range(N)]
-                expected = [torch_bin_op(t, s) for t, s in zip(tensors, scalars)]
+
+                # If incoming dtype is float16 or bfloat16, runs in float32 and casts output back to dtype.
+                control_dtype = torch.float32 if (self.device_type == 'cuda' and
+                                                  (dtype is torch.float16 or dtype is torch.bfloat16)) else dtype
+                expected = [torch_bin_op(t.to(dtype=control_dtype),
+                                         s) for t, s in zip(tensors, scalars)]
+                if (dtype is torch.float16 or dtype is torch.bfloat16):
+                    expected = [e.to(dtype=dtype) for e in expected]
 
                 # we dont support bool and complex types on CUDA for now
                 if (dtype in torch.testing.get_all_complex_dtypes() or dtype == torch.bool) and self.device_type == 'cuda':
@@ -295,7 +525,10 @@ class TestForeach(TestCase):
                     self.assertEqual(tensors, res)
                     return
                 else:
-                    self.assertEqual(res, expected)
+                    if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                        self.assertEqual(res, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+                    else:
+                        self.assertEqual(res, expected)
 
                 if dtype in torch.testing.integral_types() and self.device_type == "cpu":
                     with self.assertRaisesRegex(RuntimeError, "result type Float can't be cast to the desired output type"):
@@ -303,11 +536,15 @@ class TestForeach(TestCase):
                     return
 
                 foreach_bin_op_(tensors, scalars)
-                self.assertEqual(tensors, expected)
+                if (dtype is torch.float16 or dtype is torch.bfloat16) and TEST_WITH_ROCM:
+                    self.assertEqual(tensors, expected, atol=1.e-3, rtol=self.dtype_precisions[dtype][0])
+                else:
+                    self.assertEqual(tensors, expected)
 
+    @skipCUDAIfRocm
     @dtypes(*torch.testing.get_all_dtypes())
     def test_complex_scalar(self, device, dtype):
-        for N in [30, 300]:
+        for N in N_values:
             for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops,
                                                                      self.foreach_bin_ops_,
                                                                      self.torch_bin_ops):
@@ -345,9 +582,9 @@ class TestForeach(TestCase):
 
     @dtypes(*torch.testing.get_all_dtypes())
     def test_complex_scalarlist(self, device, dtype):
-        for N in [30, 300]:
-            for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops_sl,
-                                                                     self.foreach_bin_ops_sl_,
+        for N in N_values:
+            for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops,
+                                                                     self.foreach_bin_ops_,
                                                                      self.torch_bin_ops):
                 tensors = self._get_test_data(device, dtype, N)
                 scalars = [3 + 5j for _ in range(N)]
@@ -368,9 +605,10 @@ class TestForeach(TestCase):
                 with self.assertRaisesRegex(TypeError, "argument 'scalars' must be tuple of floats"):
                     foreach_bin_op_(tensors, scalars)
 
+    @skipCUDAIfRocm
     @dtypes(*torch.testing.get_all_dtypes())
     def test_bool_scalar(self, device, dtype):
-        for N in [30, 300]:
+        for N in N_values:
             for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops,
                                                                      self.foreach_bin_ops_,
                                                                      self.torch_bin_ops):
@@ -418,11 +656,12 @@ class TestForeach(TestCase):
                         foreach_bin_op_(tensors, scalar)
                         self.assertEqual(tensors, expected)
 
+    @skipCUDAIfRocm
     @dtypes(*torch.testing.get_all_dtypes())
     def test_bool_scalarlist(self, device, dtype):
-        for N in [30, 300]:
-            for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops_sl,
-                                                                     self.foreach_bin_ops_sl_,
+        for N in N_values:
+            for foreach_bin_op, foreach_bin_op_, torch_bin_op in zip(self.foreach_bin_ops,
+                                                                     self.foreach_bin_ops_,
                                                                      self.torch_bin_ops):
                 tensors = self._get_test_data(device, dtype, N)
                 scalars = [True for _ in range(N)]
@@ -436,7 +675,7 @@ class TestForeach(TestCase):
                             foreach_bin_op_(tensors, scalars)
                         return
                     else:
-                        if foreach_bin_op == torch._foreach_sub_scalar_list:
+                        if foreach_bin_op == torch._foreach_sub:
                             with self.assertRaisesRegex(RuntimeError, "Subtraction, the `-` operator, with a bool tensor"):
                                 foreach_bin_op_(tensors, scalars)
 
@@ -459,7 +698,7 @@ class TestForeach(TestCase):
                             foreach_bin_op(tensors, scalars)
                         return
 
-                    if foreach_bin_op == torch._foreach_sub_scalar_list:
+                    if foreach_bin_op == torch._foreach_sub:
                         if self.device_type == "cpu":
                             # see TODO[Fix scalar list]
                             res = foreach_bin_op(tensors, scalars)
@@ -551,56 +790,57 @@ class TestForeach(TestCase):
     #
     # Ops with list
     #
-    def test_add_list_error_cases(self, device):
-        tensors1 = []
-        tensors2 = []
+    def test_bin_op_list_error_cases(self, device):
+        for bin_op, bin_op_ in zip(self.foreach_bin_ops, self.foreach_bin_ops_):
+            tensors1 = []
+            tensors2 = []
 
-        # Empty lists
-        with self.assertRaises(RuntimeError):
-            torch._foreach_add(tensors1, tensors2)
-        with self.assertRaises(RuntimeError):
-            torch._foreach_add_(tensors1, tensors2)
+            # Empty lists
+            with self.assertRaisesRegex(RuntimeError, "There were no tensor arguments to this function"):
+                bin_op(tensors1, tensors2)
+            with self.assertRaisesRegex(RuntimeError, "There were no tensor arguments to this function"):
+                bin_op_(tensors1, tensors2)
 
-        # One empty list
-        tensors1.append(torch.tensor([1], device=device))
-        with self.assertRaisesRegex(RuntimeError, "Tensor list must have at least one tensor."):
-            torch._foreach_add(tensors1, tensors2)
-        with self.assertRaisesRegex(RuntimeError, "Tensor list must have at least one tensor."):
-            torch._foreach_add_(tensors1, tensors2)
+            # One empty list
+            tensors1.append(torch.tensor([1], device=device))
+            with self.assertRaisesRegex(RuntimeError, "Tensor list must have same number of elements as scalar list."):
+                bin_op(tensors1, tensors2)
+            with self.assertRaisesRegex(RuntimeError, "Tensor list must have same number of elements as scalar list."):
+                bin_op_(tensors1, tensors2)
 
-        # Lists have different amount of tensors
-        tensors2.append(torch.tensor([1], device=device))
-        tensors2.append(torch.tensor([1], device=device))
-        with self.assertRaisesRegex(RuntimeError, "Tensor lists must have the same number of tensors, got 1 and 2"):
-            torch._foreach_add(tensors1, tensors2)
-        with self.assertRaisesRegex(RuntimeError, "Tensor lists must have the same number of tensors, got 1 and 2"):
-            torch._foreach_add_(tensors1, tensors2)
+            # Lists have different amount of tensors
+            tensors2.append(torch.tensor([1], device=device))
+            tensors2.append(torch.tensor([1], device=device))
+            with self.assertRaisesRegex(RuntimeError, "Tensor lists must have the same number of tensors, got 1 and 2"):
+                bin_op(tensors1, tensors2)
+            with self.assertRaisesRegex(RuntimeError, "Tensor lists must have the same number of tensors, got 1 and 2"):
+                bin_op_(tensors1, tensors2)
 
-        # Different dtypes
-        tensors1 = [torch.zeros(10, 10, device=device, dtype=torch.float) for _ in range(10)]
-        tensors2 = [torch.ones(10, 10, device=device, dtype=torch.int) for _ in range(10)]
+            # Different dtypes
+            tensors1 = [torch.zeros(10, 10, device=device, dtype=torch.float) for _ in range(10)]
+            tensors2 = [torch.ones(10, 10, device=device, dtype=torch.int) for _ in range(10)]
 
-        with self.assertRaisesRegex(RuntimeError, "All tensors in the tensor list must have the same dtype."):
-            torch._foreach_add(tensors1, tensors2)
-        with self.assertRaisesRegex(RuntimeError, "All tensors in the tensor list must have the same dtype."):
-            torch._foreach_add_(tensors1, tensors2)
+            with self.assertRaisesRegex(RuntimeError, "All tensors in the tensor list must have the same dtype."):
+                bin_op(tensors1, tensors2)
+            with self.assertRaisesRegex(RuntimeError, "All tensors in the tensor list must have the same dtype."):
+                bin_op_(tensors1, tensors2)
 
-        # different devices
-        if torch.cuda.is_available() and torch.cuda.device_count() > 1:
-            tensor1 = torch.zeros(10, 10, device="cuda:0")
-            tensor2 = torch.ones(10, 10, device="cuda:1")
-            with self.assertRaisesRegex(RuntimeError, "Expected all tensors to be on the same device"):
-                torch._foreach_add([tensor1], [tensor2])
-            with self.assertRaisesRegex(RuntimeError, "Expected all tensors to be on the same device"):
-                torch._foreach_add_([tensor1], [tensor2])
+            # different devices
+            if torch.cuda.is_available() and torch.cuda.device_count() > 1:
+                tensor1 = torch.zeros(10, 10, device="cuda:0")
+                tensor2 = torch.ones(10, 10, device="cuda:1")
+                with self.assertRaisesRegex(RuntimeError, "Expected all tensors to be on the same device"):
+                    bin_op([tensor1], [tensor2])
+                with self.assertRaisesRegex(RuntimeError, "Expected all tensors to be on the same device"):
+                    bin_op_([tensor1], [tensor2])
 
-        # Coresponding tensors with different sizes
-        tensors1 = [torch.zeros(10, 10, device=device) for _ in range(10)]
-        tensors2 = [torch.ones(11, 11, device=device) for _ in range(10)]
-        with self.assertRaisesRegex(RuntimeError, "Corresponding tensors in lists must have the same size"):
-            torch._foreach_add(tensors1, tensors2)
-        with self.assertRaisesRegex(RuntimeError, r", got \[10, 10\] and \[11, 11\]"):
-            torch._foreach_add_(tensors1, tensors2)
+            # Corresponding tensors with different sizes
+            tensors1 = [torch.zeros(10, 10, device=device) for _ in range(10)]
+            tensors2 = [torch.ones(11, 11, device=device) for _ in range(10)]
+            with self.assertRaisesRegex(RuntimeError, "Corresponding tensors in lists must have the same size"):
+                bin_op(tensors1, tensors2)
+            with self.assertRaisesRegex(RuntimeError, r", got \[10, 10\] and \[11, 11\]"):
+                bin_op_(tensors1, tensors2)
 
     @dtypes(*torch.testing.get_all_dtypes())
     def test_add_list(self, device, dtype):
@@ -633,7 +873,7 @@ class TestForeach(TestCase):
                 self.skipTest("Skipped! See https://github.com/pytorch/pytorch/issues/44489")
             return
 
-        for N in [30, 300]:
+        for N in N_values:
             tensors1 = self._get_test_data(device, dtype, N)
 
             if dtype in [torch.bfloat16, torch.bool, torch.float16]:
@@ -647,33 +887,6 @@ class TestForeach(TestCase):
             self.assertEqual(res, tensors1)
             self.assertEqual(tensors1, res)
 
-    def test_bin_op_list_error_cases(self, device):
-        tensors1 = []
-        tensors2 = []
-
-        for bin_op in self.foreach_bin_ops + self.foreach_bin_ops_:
-            # Empty lists
-            with self.assertRaises(RuntimeError):
-                bin_op(tensors1, tensors2)
-
-            # One empty list
-            tensors1.append(torch.tensor([1], device=device))
-            with self.assertRaises(RuntimeError):
-                bin_op(tensors1, tensors2)
-
-            # Lists have different amount of tensors
-            tensors2.append(torch.tensor([1], device=device))
-            tensors2.append(torch.tensor([1], device=device))
-            with self.assertRaises(RuntimeError):
-                bin_op(tensors1, tensors2)
-
-            # Different dtypes
-            tensors1 = [torch.zeros(2, 2, device=device, dtype=torch.float) for _ in range(2)]
-            tensors2 = [torch.ones(2, 2, device=device, dtype=torch.int) for _ in range(2)]
-
-            with self.assertRaises(RuntimeError):
-                bin_op(tensors1, tensors2)
-
     @dtypes(*torch.testing.get_all_dtypes())
     def test_add_list_different_sizes(self, device, dtype):
         tensors1 = [torch.zeros(10 + n, 10 + n, device=device, dtype=dtype) for n in range(10)]
