@@ -1,5 +1,6 @@
 import copy
 import itertools
+import functools
 import unittest
 
 try:
@@ -15,10 +16,13 @@ import torch.nn.functional as F
 import torch.jit
 import torch.backends.mkldnn
 from torch.utils import mkldnn as mkldnn_utils
-from torch.testing._internal.common_utils import TestCase, run_tests, TemporaryFileName
+from torch.testing._internal.common_utils import TestCase, run_tests, TemporaryFileName, gradcheck, gradgradcheck
 
-from torch.autograd.gradcheck import gradgradcheck, gradcheck
+# batched grad doesn't support mkldnn
+gradcheck = functools.partial(gradcheck, check_batched_grad=False)
+gradgradcheck = functools.partial(gradgradcheck, check_batched_grad=False)
 
+types = [torch.float, torch.bfloat16]
 
 # Comment the line below to find out the CI machines having MKL-DNN build disabled
 @unittest.skipIf(not torch._C.has_mkldnn, "MKL-DNN build is disabled")
@@ -29,17 +33,55 @@ class TestMkldnn(TestCase):
                            torch.randn((1, 2, 3, 4, 5),
                                        dtype=torch.float, device=torch.device('cpu'))[:, :, :, :, 1]]:
             cpu_tensor.requires_grad_()
-            mkldnn_tensor = cpu_tensor.to_mkldnn()
-            cpu_tensor_1 = mkldnn_tensor.to_dense()
-            self.assertEqual(cpu_tensor, cpu_tensor_1)
-            self.assertEqual(mkldnn_tensor.dtype, torch.float)
-            self.assertEqual(mkldnn_tensor.device, torch.device('cpu'))
-            self.assertEqual(mkldnn_tensor.size(), torch.Size([1, 2, 3, 4]))
-            self.assertEqual(mkldnn_tensor.numel(), cpu_tensor.numel())
-            self.assertEqual(mkldnn_tensor.element_size(), cpu_tensor.element_size())
-            self.assertRaisesRegex(RuntimeError,
-                                   "Cannot access data pointer of Tensor that doesn't have storage",
-                                   lambda: mkldnn_tensor.data_ptr() != 0)
+            # float cpu tensor to mkldnn float tensor or bfloat tensor.
+            for dtype1 in types:
+                mkldnn_tensor = cpu_tensor.to_mkldnn(dtype1)
+                self.assertEqual(mkldnn_tensor.dtype, dtype1)
+                cpu_tensor_1 = mkldnn_tensor.to_dense()
+                # not given dtype for to_dense, mkldnn tensor has same dtype with cpu tensor
+                self.assertEqual(mkldnn_tensor.dtype, cpu_tensor_1.dtype)
+                # mkldnn float/bfloat tensor to cpu float or bfloat tensor
+                for dtype2 in types:
+                    cpu_tensor_2 = mkldnn_tensor.to_dense(dtype2)
+                    self.assertEqual(cpu_tensor_2.dtype, dtype2)
+                    atol = 1e-5 if dtype1 == torch.float and dtype2 == torch.float else 1e-2
+                    self.assertEqual(cpu_tensor, cpu_tensor_2.float(), atol=atol, rtol=0)
+
+                self.assertEqual(mkldnn_tensor.device, torch.device('cpu'))
+                self.assertEqual(mkldnn_tensor.size(), torch.Size([1, 2, 3, 4]))
+                self.assertEqual(mkldnn_tensor.numel(), cpu_tensor.numel())
+                if dtype1 == torch.float:
+                    self.assertEqual(mkldnn_tensor.element_size(), cpu_tensor.element_size())
+                else:
+                    self.assertEqual(mkldnn_tensor.element_size(), cpu_tensor.element_size() / 2)
+                self.assertRaisesRegex(RuntimeError,
+                                       "Cannot access data pointer of Tensor that doesn't have storage",
+                                       lambda: mkldnn_tensor.data_ptr() != 0)
+
+            # bfloat cpu tensor to mkldnn float tensor or bfloat tensor.
+            cpu_tensor_bf16 = cpu_tensor.bfloat16()
+            for dtype1 in types:
+                mkldnn_tensor = cpu_tensor_bf16.to_mkldnn(dtype1)
+                self.assertEqual(mkldnn_tensor.dtype, dtype1)
+                cpu_tensor_1 = mkldnn_tensor.to_dense()
+                # not given dtype for to_dense, mkldnn tensor has same dtype with cpu tensor
+                self.assertEqual(mkldnn_tensor.dtype, cpu_tensor_1.dtype)
+                # mkldnn float/bfloat tensor to cpu float or bfloat tensor
+                for dtype2 in types:
+                    cpu_tensor_2 = mkldnn_tensor.to_dense(dtype2)
+                    self.assertEqual(cpu_tensor_2.dtype, dtype2)
+                    self.assertEqual(cpu_tensor_bf16, cpu_tensor_2.bfloat16(), atol=1e-5, rtol=0)
+
+                self.assertEqual(mkldnn_tensor.device, torch.device('cpu'))
+                self.assertEqual(mkldnn_tensor.size(), torch.Size([1, 2, 3, 4]))
+                self.assertEqual(mkldnn_tensor.numel(), cpu_tensor.numel())
+                if dtype1 == torch.bfloat16:
+                    self.assertEqual(mkldnn_tensor.element_size(), cpu_tensor_bf16.element_size())
+                else:
+                    self.assertEqual(mkldnn_tensor.element_size(), cpu_tensor_bf16.element_size() * 2)
+                self.assertRaisesRegex(RuntimeError,
+                                       "Cannot access data pointer of Tensor that doesn't have storage",
+                                       lambda: mkldnn_tensor.data_ptr() != 0)
 
     def test_unsupported(self):
         # unsupported types and unsupported types with gpu
@@ -551,6 +593,25 @@ class TestMkldnn(TestCase):
             self._test_serialization(mkldnn_linear, (x.to_mkldnn(),))
             self._test_tracing(mkldnn_linear, (x.to_mkldnn(),))
 
+    def test_linear_backward(self):
+        in_features = torch.randint(3, 10, (1,)).item()
+        out_features = torch.randint(3, 100, (1,)).item()
+        x = torch.randn(3, in_features, dtype=torch.float32) * 10
+        # Only works for bias case by dispatch onednn linear to addmm.
+        for bias in [True]:
+            x1 = x.clone().requires_grad_()
+            x2 = x.clone().to_mkldnn().requires_grad_()
+            linear = torch.nn.Linear(in_features, out_features).float()
+            mkldnn_linear = copy.deepcopy(linear)
+            y1 = linear(x1).sum()
+            y2 = mkldnn_linear(x2).to_dense().sum()
+            y1.backward()
+            y2.backward()
+            self.assertEqual(x1.grad, x2.grad.to_dense())
+            self.assertEqual(linear.weight.grad, mkldnn_linear.weight.grad)
+            if bias:
+                self.assertEqual(linear.bias.grad, mkldnn_linear.bias.grad)
+
     def test_softmax(self):
         x = torch.randn(3, 4, 5, dtype=torch.float32) * 10
         for dim in range(x.ndim):
