@@ -919,9 +919,9 @@ class TestTypePromotion(TestCase):
         t = torch.tensor((1), dtype=dtypes[0], device=device)
         out = torch.empty(0, dtype=dtypes[1], device=device)
 
-        ops = (torch.neg, torch.floor, torch.ceil, torch.cos, torch.erf, torch.log)
-        float_only_ops = {torch.floor, torch.ceil, torch.cos, torch.erf, torch.log}
-        real_only_ops = {torch.floor, torch.ceil, torch.erf}
+        ops = (torch.neg, torch.floor, torch.ceil)
+        float_only_ops = {torch.floor, torch.ceil}
+        real_only_ops = {torch.floor, torch.ceil}
         for op in ops:
             if dtypes[0] is not dtypes[1]:
                 with self.assertRaises(RuntimeError):
