@@ -9,8 +9,11 @@ def show():
     return torch._C._show_config()
 
 # TODO: In principle, we could provide more structured version/config
-# information here.  We're not for now; considering doing so if someone
-# asks for it.
+# information here. For now only CXX_FLAGS is exposed, as Timer
+# uses them.
+def _cxx_flags():
+    """Returns the CXX_FLAGS used when building PyTorch."""
+    return torch._C._cxx_flags()
 
 def parallel_info():
     r"""Returns detailed string with parallelization settings"""
