@@ -1,6 +1,6 @@
 #include <torch/csrc/python_headers.h>
 #ifdef _MSC_VER
-#include <Windows.h>
+#include <c10/util/win32-headers.h>
 #endif
 #include <structmember.h>
 
