@@ -7,6 +7,7 @@
 #include <torch/cuda.h>
 #include <torch/data.h>
 #include <torch/enum.h>
+#include <torch/fft.h>
 #include <torch/jit.h>
 #include <torch/linalg.h>
 #include <torch/nn.h>
@@ -15,3 +16,4 @@
 #include <torch/types.h>
 #include <torch/utils.h>
 #include <torch/autograd.h>
+#include <torch/version.h>
