@@ -25,7 +25,7 @@ inline at::Tensor clone_obey_contract(const at::Tensor& new_grad, const at::Tens
     // (1)
     // Does this dicey-looking sequence attach the result to new_grad's
     // history if GradMode::is_enabled()?  Yes, and @alband says it should.
-    return std::move(at::empty_strided(variable.sizes(), variable.strides(),
+    return std::move(new_grad.new_empty_strided(variable.sizes(), variable.strides(),
                                        variable.options().memory_format(c10::nullopt))
                      .copy_(new_grad));
   } else {
