@@ -12,7 +12,7 @@ namespace rpc {
 class TORCH_API RequestCallback {
  public:
   // Invoke the callback.
-  std::shared_ptr<FutureMessage> operator()(Message& request) const;
+  std::shared_ptr<JitFuture> operator()(Message& request) const;
 
   virtual ~RequestCallback() {}
 
@@ -24,8 +24,7 @@ class TORCH_API RequestCallback {
   // message containing an exception. Different rpc agent implementations are
   // expected to ensure delivery of the response/exception based on their
   // implementation specific mechanisms.
-  virtual std::shared_ptr<FutureMessage> processMessage(
-      Message& request) const = 0;
+  virtual std::shared_ptr<JitFuture> processMessage(Message& request) const = 0;
 };
 
 } // namespace rpc
