@@ -1,11 +1,12 @@
-
 #include <torch/csrc/jit/codegen/cuda/index_compute.h>
+
 #include <c10/util/Exception.h>
 #include <torch/csrc/jit/codegen/cuda/arith.h>
 #include <torch/csrc/jit/codegen/cuda/instrumentation.h>
 #include <torch/csrc/jit/codegen/cuda/ir_all_nodes.h>
 #include <torch/csrc/jit/codegen/cuda/ir_iostream.h>
 #include <torch/csrc/jit/codegen/cuda/kernel_ir_builder.h>
+#include <torch/csrc/jit/codegen/cuda/kernel_ir_printer.h>
 #include <torch/csrc/jit/codegen/cuda/lower2device.h>
 #include <torch/csrc/jit/codegen/cuda/lower_utils.h>
 #include <torch/csrc/jit/codegen/cuda/transform_iter.h>
@@ -14,6 +15,7 @@
 namespace torch {
 namespace jit {
 namespace fuser {
+namespace cuda {
 
 namespace {
 
@@ -810,7 +812,7 @@ kir::TensorIndex* Index::getGlobalProducerIndex(
         " dim: ",
         i,
         " id: ",
-        kir_root_dom_i);
+        kir::toString(kir_root_dom_i));
 
     auto root_ind = index_map.at(kir_root_dom_i);
     TORCH_INTERNAL_ASSERT(kir::isLoweredScalar(root_ind));
@@ -928,7 +930,7 @@ kir::TensorIndex* Index::getProducerIndex_impl(
         " dim: ",
         i,
         " id: ",
-        kir_root_dom_i);
+        kir::toString(kir_root_dom_i));
 
     auto root_ind_i = index_map.at(kir_root_dom_i);
     TORCH_INTERNAL_ASSERT(kir::isLoweredScalar(root_ind_i));
@@ -1037,7 +1039,7 @@ kir::TensorIndex* Index::getGlobalConsumerIndex(
         " dim: ",
         i,
         " id: ",
-        kir_root_dom_i);
+        kir::toString(kir_root_dom_i));
     auto ind = index_map.at(kir_root_dom_i);
 
     if (i == root_dom.size() - 1 && inner_most_dim_contig) {
@@ -1099,7 +1101,7 @@ kir::TensorIndex* Index::getConsumerIndex_impl(
         " dim: ",
         i,
         " id: ",
-        kir_root_dom_i);
+        kir::toString(kir_root_dom_i));
     auto root_ind_i = index_map.at(kir_root_dom_i);
     TORCH_INTERNAL_ASSERT(kir::isLoweredScalar(root_ind_i));
 
@@ -1278,6 +1280,7 @@ std::pair<std::vector<Val*>, bool> Index::getConsumerRootPredIndices(
   return std::make_pair(root_inds, use_rfactor);
 }
 
+} // namespace cuda
 } // namespace fuser
 } // namespace jit
 } // namespace torch
