@@ -15,6 +15,7 @@
 namespace torch {
 namespace jit {
 namespace fuser {
+namespace cuda {
 
 // Returns true if both v1 and v2 are scalars, are the same type of scalars, and
 // dispatches to the inherited Val type's `->sameAs` call. e.g. if both vals are
@@ -34,7 +35,7 @@ bool areEqualScalars(Val* v1, Val* v2);
  *   3) Reduction across a dimension i.e. val.sum(axis=2)
  *   4) split/merge
  */
-class TORCH_CUDA_API UnaryOp : public Expr {
+class TORCH_CUDA_CU_API UnaryOp : public Expr {
  public:
   ~UnaryOp() = default;
   UnaryOp(UnaryOpType _type, Val* _out, Val* _in);
@@ -72,7 +73,7 @@ class TORCH_CUDA_API UnaryOp : public Expr {
  *  1) Add/mul/div/mod/sub (A * B)
  *  2) LT (A < B)
  */
-class TORCH_CUDA_API BinaryOp : public Expr {
+class TORCH_CUDA_CU_API BinaryOp : public Expr {
  public:
   ~BinaryOp() = default;
   BinaryOp(BinaryOpType _type, Val* _out, Val* _lhs, Val* _rhs);
@@ -112,7 +113,7 @@ class TORCH_CUDA_API BinaryOp : public Expr {
  * Broadcast _in to match _out. broadcast_dims are relative to out. Where
  * broadcast_dims.size() + _in->nDims() == _out->nDims().
  */
-class TORCH_CUDA_API BroadcastOp : public Expr {
+class TORCH_CUDA_CU_API BroadcastOp : public Expr {
  public:
   ~BroadcastOp() = default;
   BroadcastOp(Val* _out, Val* _in);
@@ -146,7 +147,7 @@ class TORCH_CUDA_API BroadcastOp : public Expr {
  * tensor. The output tensors size will be the size of all
  * non-reduction/non-broadcast dimensions.
  */
-class TORCH_CUDA_API ReductionOp : public Expr {
+class TORCH_CUDA_CU_API ReductionOp : public Expr {
  public:
   ~ReductionOp() = default;
   ReductionOp(BinaryOpType _reduction_op_type, Val* _init, Val* _out, Val* _in);
@@ -182,7 +183,7 @@ class TORCH_CUDA_API ReductionOp : public Expr {
   Val* const in_ = nullptr;
 };
 
-class TORCH_CUDA_API TernaryOp : public Expr {
+class TORCH_CUDA_CU_API TernaryOp : public Expr {
  public:
   ~TernaryOp() = default;
   TernaryOp(TernaryOpType _type, Val* _out, Val* _in1, Val* _in2, Val* _in3);
@@ -227,7 +228,7 @@ class TORCH_CUDA_API TernaryOp : public Expr {
 // TensorDomains which represent how to iterate over a tensor is made up of
 // IterDomains to form an ND iterable. We directly set parallization strategies
 // on IterDomains.
-class TORCH_CUDA_API IterDomain : public Val {
+class TORCH_CUDA_CU_API IterDomain : public Val {
  public:
   IterDomain(
       Val* _start,
@@ -362,7 +363,7 @@ class TORCH_CUDA_API IterDomain : public Val {
  * operations that take in a TensorDomain, applies a transformation and outputs
  * a tensor domain.
  */
-class TORCH_CUDA_API TensorDomain : public Val {
+class TORCH_CUDA_CU_API TensorDomain : public Val {
  public:
   TensorDomain() = delete;
   ~TensorDomain() = default;
@@ -555,7 +556,7 @@ class TORCH_CUDA_API TensorDomain : public Val {
  * Representation a split on an IterDomain by "factor"
  * TODO: Implement split by nparts
  */
-class TORCH_CUDA_API Split : public Expr {
+class TORCH_CUDA_CU_API Split : public Expr {
  public:
   ~Split() = default;
 
@@ -597,7 +598,7 @@ class TORCH_CUDA_API Split : public Expr {
  * if there is one.
  * TODO: Should this be a unary op type?
  */
-class TORCH_CUDA_API Merge : public Expr {
+class TORCH_CUDA_CU_API Merge : public Expr {
  public:
   ~Merge() = default;
   Merge(IterDomain* _out, IterDomain* _outer, IterDomain* _inner);
@@ -635,7 +636,7 @@ class TORCH_CUDA_API Merge : public Expr {
  * - blockDim.z
  * - T3.stride[2]
  */
-class TORCH_CUDA_API NamedScalar : public Val {
+class TORCH_CUDA_CU_API NamedScalar : public Val {
  public:
   ~NamedScalar() = default;
   NamedScalar() = delete;
@@ -677,6 +678,7 @@ class TORCH_CUDA_API NamedScalar : public Val {
   std::string name_;
 };
 
+} // namespace cuda
 } // namespace fuser
 } // namespace jit
 } // namespace torch
