@@ -12,6 +12,7 @@
 namespace torch {
 namespace jit {
 namespace fuser {
+namespace cuda {
 
 namespace {
 
@@ -1197,8 +1198,8 @@ class ConcretizeDomain : private BackwardVisitor {
     bcast_domain_map_[id] = concretized(To);
   }
 
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Woverloaded-virtual"
+  using BackwardVisitor::handle;
+
   void handle(ReductionOp* rop) override {
     concretizePwOp(rop);
   }
@@ -1214,7 +1215,6 @@ class ConcretizeDomain : private BackwardVisitor {
   void handle(TernaryOp* top) override {
     concretizePwOp(top);
   };
-#pragma clang diagnostic pop
 
  private:
   using MapType = std::unordered_map<IterDomain*, IterDomain*>;
@@ -1222,7 +1222,12 @@ class ConcretizeDomain : private BackwardVisitor {
 };
 
 void ConcretizeDomain::concretizePwOp(Expr* e) {
-  TensorView* tv = *ir_utils::filterByType<TensorView>(e->outputs()).begin();
+  if (e->output(0)->getValType() != ValType::TensorView) {
+    return;
+  }
+
+  TORCH_INTERNAL_ASSERT(e->outputs().size() == 1);
+  TensorView* tv = e->output(0)->as<TensorView>();
 
   std::vector<IterDomain*> io = tv->getRootDomain();
 
@@ -1316,8 +1321,13 @@ class ProveValEqual : private IterVisitor {
 
   // Inspect a pointwise op and record the identified equality
   void provePwOp(Expr* e) {
-    TensorView* tv = *ir_utils::filterByType<TensorView>(e->outputs()).begin();
-    std::vector<IterDomain*> io = tv->getRootDomain();
+    if (e->output(0)->getValType() != ValType::TensorView) {
+      return;
+    }
+
+    TORCH_INTERNAL_ASSERT(e->outputs().size() == 1);
+    TensorView* tv = e->output(0)->as<TensorView>();
+    const std::vector<IterDomain*>& io = tv->getRootDomain();
 
     // Record equalities from output to all the inputs
     // ignores un-concretizable broadcasts
@@ -1331,8 +1341,8 @@ class ProveValEqual : private IterVisitor {
     }
   }
 
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Woverloaded-virtual"
+  using IterVisitor::handle;
+
   void handle(ReductionOp* rop) override {
     provePwOp(rop);
   }
@@ -1348,7 +1358,6 @@ class ProveValEqual : private IterVisitor {
   void handle(TernaryOp* top) override {
     provePwOp(top);
   }
-#pragma clang diagnostic pop
 
  private:
   ConcretizeDomain cd_;
@@ -1470,6 +1479,7 @@ c10::optional<ParallelType> NamedScalar::getParallelIndex() const {
   return c10::nullopt;
 }
 
+} // namespace cuda
 } // namespace fuser
 } // namespace jit
 } // namespace torch
