@@ -22,15 +22,17 @@ namespace cuda {
 // Get fusion_node ready for execution.
 // find or compile `CudaKernel` for graph stored in `attr::Subgraph`
 // this function assigns `attr::cache_id` to `fusion_node`
-TORCH_CUDA_API void compileCudaFusionGroup(Node* fusion_node);
+TORCH_CUDA_CU_API void compileCudaFusionGroup(Node* fusion_node);
 
 // Execute fusion_node.
 // Current protocol is that the function allocates output tensor append them to
 // `stack` after execution.
 // TODO: support shape inferencing. Right now we only handles static shape
-TORCH_CUDA_API void runCudaFusionGroup(const Node* fusion_node, Stack& stack);
+TORCH_CUDA_CU_API void runCudaFusionGroup(
+    const Node* fusion_node,
+    Stack& stack);
 
-TORCH_CUDA_API void CudaFuseGraph(std::shared_ptr<Graph>& graph);
+TORCH_CUDA_CU_API void CudaFuseGraph(std::shared_ptr<Graph>& graph);
 
 } // namespace cuda
 } // namespace fuser
