@@ -474,7 +474,7 @@ class IrParser {
             value_map.emplace(node->output()->unique(), out);
           },
           [](const Node* node) -> bool {
-            // we don't support cast of output types yet;
+            // TODO: support cast of output types yet;
             if (!node->inputs()[3]->type()->isSubtypeOf(
                     static_cast<c10::TypePtr>(NoneType::get()))) {
               // We can only handle output as half and float;
@@ -638,7 +638,7 @@ bool isNodeParsible(const Node* node) {
   return IrParser::canParseNode(node);
 }
 
-std::unique_ptr<Fusion> parseJitIR(std::shared_ptr<Graph>& graph) {
+std::unique_ptr<Fusion> parseJitIR(const std::shared_ptr<Graph>& graph) {
   FUSER_PERF_SCOPE("parseJitIR");
 
   IrParser parser(graph);
