@@ -112,9 +112,7 @@ bool scheduleFusion(Fusion* fusion, const at::ArrayRef<c10::IValue> inputs) {
     out_tv->split(0, kPwThreadX);
     // Split by another 4 which will be our unroll factor
     auto ur_factor = disable_unroll ? 1 : kUnrollFactor;
-    if (!disable_unroll) {
-      out_tv->split(0, ur_factor);
-    }
+    out_tv->split(0, ur_factor);
   }
 
   for (auto output : fusion->outputs()) {
@@ -290,7 +288,7 @@ ReductionParams reductionHeuristic(
 }
 } // anonymous namespace
 
-TORCH_CUDA_API c10::optional<ReductionParams> getReductionHeuristics(
+TORCH_CUDA_CU_API c10::optional<ReductionParams> getReductionHeuristics(
     Fusion* fusion,
     const at::ArrayRef<c10::IValue>& fusion_inputs,
     TensorView* red_tv) {
