@@ -1,6 +1,6 @@
 #pragma once
 
-#include <aten/src/ATen/Context.h>
+#include <ATen/Context.h>
 #include <torch/csrc/jit/ir/ir.h>
 
 namespace torch {
@@ -8,7 +8,7 @@ namespace jit {
 namespace fuser {
 namespace cuda {
 
-TORCH_CUDA_API void TypePropagate(std::shared_ptr<Graph>& graph);
+TORCH_CUDA_CU_API void TypePropagate(std::shared_ptr<Graph>& graph);
 
 } // namespace cuda
 } // namespace fuser
