@@ -1,4 +1,5 @@
 #include <torch/csrc/jit/codegen/cuda/transform_replay.h>
+
 #include <torch/csrc/jit/codegen/cuda/arith.h>
 #include <torch/csrc/jit/codegen/cuda/fusion.h>
 #include <torch/csrc/jit/codegen/cuda/instrumentation.h>
@@ -11,6 +12,7 @@
 namespace torch {
 namespace jit {
 namespace fuser {
+namespace cuda {
 
 using id_map = std::unordered_map<IterDomain*, IterDomain*>;
 
@@ -317,12 +319,16 @@ std::pair<TensorDomain*, unsigned int> TransformReplay::replayPasC(
 
   unsigned int producer_compute_at_axis = new_IDs.size();
   // Add axes in (2)
-  std::unordered_set<IterDomain*> consumer_CA_ids_set(
-      consumer_CA_ids.begin(), consumer_CA_ids.end());
   for (auto c_id : consumer->domain()) {
     auto it = replay_PasC.getReplay().find(c_id);
     if (it != replay_PasC.getReplay().end()) {
       auto id = it->second;
+      // If the leaf id from ReplayTransformations is used to move
+      // forward in BestEffortReplay, it is not a final ID.
+      if (producer_replayed_leaves.getUnorderedLeafIDs().find(id) ==
+          producer_replayed_leaves.getUnorderedLeafIDs().end()) {
+        continue;
+      }
       if (used_IDs.find(id) == used_IDs.end()) {
         new_IDs.push_back(id);
         used_IDs.emplace(id);
@@ -491,12 +497,16 @@ std::pair<TensorDomain*, unsigned int> TransformReplay::replayCasP(
   }
 
   // Add axes in (2)
-  std::unordered_set<IterDomain*> consumer_CA_ids_set(
-      producer_CA_ids.begin(), producer_CA_ids.end());
   for (auto p_id : producer->domain()) {
     auto it = replay_CasP.getReplay().find(p_id);
     if (it != replay_CasP.getReplay().end()) {
       auto id = it->second;
+      // If the leaf id from ReplayTransformations is used to move
+      // forward in BestEffortReplay, it is not a final ID.
+      if (consumer_replayed_leaves.getUnorderedLeafIDs().find(id) ==
+          consumer_replayed_leaves.getUnorderedLeafIDs().end()) {
+        continue;
+      }
       if (used_IDs.find(id) == used_IDs.end()) {
         new_IDs.push_back(id);
         used_IDs.emplace(id);
@@ -560,6 +570,7 @@ std::pair<TensorView*, unsigned int> TransformReplay::replayCasP(
   return {consumer, replay.second};
 }
 
+} // namespace cuda
 } // namespace fuser
 } // namespace jit
 } // namespace torch
