@@ -91,7 +91,7 @@ static std::string variableType(const std::shared_ptr<c10::Type>& t) {
     return "double";
   } else if (t->kind() == TypeKind::BoolType) {
     return "bool";
-  } else if (auto scalar_type = t->expect<TensorType>()->scalarType()) {
+  } else if (auto scalar_type = t->expectRef<TensorType>().scalarType()) {
     return calcScalarTypeName(*scalar_type);
   }
   // something went wrong with the type analysis during shape propagation
@@ -118,7 +118,7 @@ static std::string typeCastedValueName(
   } else if (t->kind() == TypeKind::NoneType) {
     // Support None value for optional arguments like memory format
     return vn;
-  } else if (auto scalar_type = t->expect<TensorType>()->scalarType()) {
+  } else if (auto scalar_type = t->expectRef<TensorType>().scalarType()) {
     if (*scalar_type != outtype) {
       return std::string("((") + calcScalarTypeName(outtype) + ") " + vn + ")";
     }
@@ -261,7 +261,7 @@ static std::string encodeRHS(const Node* n) {
   } else {
     size_t i = 0;
 
-    auto outtype = n->output()->type()->expect<TensorType>()->scalarType();
+    auto outtype = n->output()->type()->expectRef<TensorType>().scalarType();
     TORCH_INTERNAL_ASSERT(outtype);
 
     for (auto in : n->inputs()) {
@@ -518,7 +518,9 @@ std::string generateKernel(
   // Note: Random number generation is only supported for CUDA kernels.
   // Note: Constant None node is ignored and we will handle it in the
   //       places where the constant None node is used
-  for (const auto& n : graph.nodes()) {
+  // Note: No need to iterate over reference as n is a pointer
+  for (const auto n : graph.nodes()) {
+    static_assert(std::is_pointer<decltype(n)>::value, "n must be a pointer");
     // Note: FusedConcat nodes work by narrowing the output Tensors before the
     // kernel runs
     if (n->kind() == prim::FusedConcat)
