@@ -1,6 +1,7 @@
 #pragma once
+
+#include <ATen/core/function_schema.h>
 #include <ATen/core/ivalue.h>
-//#include <aten/src/Aten/core/operator_name.h>
 #include <vector>
 
 namespace torch {
@@ -15,7 +16,7 @@ class Function {
  public:
   Function(c10::QualifiedName name);
   bool run(Stack& stack) const;
-  c10::IValue operator()(Stack& stack);
+  c10::IValue operator()(Stack& stack) const;
   const std::string& name() const;
   const c10::QualifiedName& qualname() const;
   void append_instruction(OpCode op, int X, int N);
@@ -31,10 +32,15 @@ class Function {
   void set_register_size(size_t size);
 
   std::string get_module_debug_info(size_t pc) const;
+  const std::shared_ptr<Code> get_code() const;
+
+  void setSchema(c10::FunctionSchema schema);
+  const at::optional<c10::FunctionSchema>& getSchema() const;
 
  private:
   c10::QualifiedName name_;
   std::shared_ptr<Code> code_;
+  at::optional<c10::FunctionSchema> schema_; // (byte-code version 4+)
   std::vector<std::string> pc_to_module_debug_info_;
 };
 
