@@ -1,4 +1,5 @@
 #include <torch/csrc/jit/mobile/import.h>
+
 #include <ATen/core/ivalue.h>
 #include <caffe2/serialize/inline_container.h>
 #include <torch/csrc/jit/api/compilation_unit.h>
@@ -15,29 +16,47 @@
 
 // The import process to serialize the bytecode package.
 // An example for bytecode.pkl of a small mobile_module looks like:
-// (3,
-//   ('__torch__.m.forward',
-//     (('instructions',
-//       (('STOREN', 1, 2),
-//        ('DROPR', 1, 0),
-//        ('MOVE', 2, 0),
-//        ('OP', 0, 0),
-//        ('RET', 0, 0))),
-//      ('operators', (('aten::Int', 'Tensor'),)),
-//      ('constants', ()),
-//      ('types', ()),
-//      ('register_size', 2))))
+// (4,  # model version number (caffe2::serialize::kProducedBytecodeVersion)
+//  # first method
+//  (
+//   # function name
+//   '__torch__.m.forward',
+//   # code
+//   (('instructions',
+//     (('STOREN', 1, 2),
+//      ('DROPR', 1, 0),
+//      ('MOVE', 2, 0),
+//      ('OP', 0, 0),
+//      ('RET', 0, 0))),
+//    ('operators', (('aten::Int', 'Tensor'),)),
+//    ('constants', ()),
+//    ('types', ()),
+//    ('register_size', 2)),
+//   # schema -- optional (forward-compatible addition to version 4)
+//   (('arguments',
+//     ((('name', 'x'), ('type', 'Tensor'), ('default_value', 13)),
+//      ...)),  # more args follow here
+//    ('returns',
+//     ((('name', ''), ('type', 'Tensor'), ('default_value', None)),
+//      ...)),  # more return values follow here
+//   )),
+//  # more methods follow here
+//  ...)
 
 // In addition, the module debugging information can be saved
 // in mobile_debug.pkl. An example for it looks like:
-// (3,
-//   ('__torch__.m.forward',
-//     (('module_debug_info', (top(A).foo(B).forward)))))
+// (4,
+//  ('__torch__.m.forward',
+//   (('module_debug_info', (top(A).foo(B).forward)))))
 
 // Note that currently the backward compatibility is not supported by bytecode.
-// This format and process need to be revisted and redesigned if we want to
+// This format and process need to be revisited and redesigned if we want to
 // support backward compatibility in future.
 
+// Note that the following function-schema fields are not supported:
+//  - Argument::{known_length_,kwarg_only_}
+//  - FunctionSchema::{overload_name_, is_vararg_, is_varret_}
+
 namespace c10 {
 // std::string serializeType(const Type &t);
 TypePtr parseType(const std::string& pythonStr);
@@ -86,12 +105,65 @@ void print_unsupported_ops_and_throw(
   TORCH_CHECK(
       false,
       "Following ops cannot be found. ",
-      "May need to add them explicitly to the selective build operator whitelist, ",
-      "or re-run the export_opnames to update the whitelist:",
+      "Check fburl.com/missing_ops for the fix.",
       error_message);
 }
 
-void parseMethods(
+// The deserializer class which loads the bytecode package from bc files.
+class BytecodeDeserializer final {
+ public:
+  explicit BytecodeDeserializer(std::unique_ptr<PyTorchStreamReader> reader);
+  mobile::Module deserialize(c10::optional<at::Device> device);
+  mobile::Module deserialize(
+      c10::optional<at::Device> device,
+      ExtraFilesMap& extra_files);
+  std::unordered_map<std::string, std::string> deserializeMetadata(
+      c10::optional<at::Device> device);
+  void deserialize_only_extra(
+      c10::optional<at::Device> device,
+      ExtraFilesMap& extra_files);
+
+ private:
+  TypePtr resolveTypeName(const c10::QualifiedName& qn);
+  void parseMethods(
+      const std::vector<IValue>& vals,
+      const c10::optional<std::vector<IValue>>& debug_info_vals,
+      mobile::CompilationUnit& mcu);
+  c10::IValue readArchive(
+      const std::string& archive_name,
+      std::shared_ptr<mobile::CompilationUnit> mcu);
+  std::unordered_map<std::string, std::string> readMobileMetadata(
+      std::shared_ptr<mobile::CompilationUnit> mcu);
+  std::shared_ptr<CompilationUnit> compilation_unit_;
+  std::unordered_set<std::string> imported_libs_;
+  std::unique_ptr<PyTorchStreamReader> reader_{};
+  c10::optional<at::Device> device_;
+};
+
+BytecodeDeserializer::BytecodeDeserializer(
+    std::unique_ptr<PyTorchStreamReader> reader)
+    : compilation_unit_(std::make_shared<CompilationUnit>()),
+      reader_(std::move(reader)) {}
+
+TypePtr BytecodeDeserializer::resolveTypeName(const c10::QualifiedName& qn) {
+  static const c10::QualifiedName torchPrefix = "__torch__";
+  // HACK: first we check whether the name starts with `__torch__` to
+  // tell if it's "supposed" to be a class type. This is a reliable
+  // check today, but there is no guarantee that this is the case. The
+  // real solution is to merge type parsers so we can share class
+  // resolution logic.
+  if (torchPrefix.isPrefixOf(qn)) {
+    if (compilation_unit_->get_class(qn) == nullptr) {
+      auto typeptr = ClassType::create(qn, compilation_unit_, true);
+      compilation_unit_->register_type(typeptr);
+    }
+    return compilation_unit_->get_class(qn);
+  } else {
+    return c10::parseType(qn.qualifiedName());
+  }
+}
+
+void BytecodeDeserializer::parseMethods(
     const std::vector<IValue>& vals,
     const c10::optional<std::vector<IValue>>& debug_info_vals,
     mobile::CompilationUnit& mcu) {
@@ -123,31 +195,38 @@ void parseMethods(
         "The numbers of bytecode values and debug info values do not match.");
   }
 
+  // Process all methods in this mobile module.
   for (size_t i = method_i_start; i < vals.size(); ++i) {
     const auto& element = vals[i];
     const auto& m_tuple = element.toTuple()->elements();
     const std::string& function_name = m_tuple[0].toStringRef();
-    IValue table = m_tuple[1];
+    IValue codeTable = m_tuple[1];
+    auto schemaTable = // older files do not store function schema
+        (model_version > 0x4L || (model_version == 0x4L && m_tuple.size() >= 3))
+        ? at::optional<IValue>{m_tuple[2]}
+        : at::nullopt;
 
     auto function = std::unique_ptr<mobile::Function>(
         new mobile::Function(c10::QualifiedName(function_name)));
 
     const auto& ins_list =
-        expect_field(table, "instructions", BYTECODE_INDEX_INSTRUCTION)
+        expect_field(codeTable, "instructions", BYTECODE_INDEX_INSTRUCTION)
             .toTuple()
             ->elements();
     const auto& ops_list =
-        expect_field(table, "operators", BYTECODE_INDEX_OPERATOR)
+        expect_field(codeTable, "operators", BYTECODE_INDEX_OPERATOR)
             .toTuple()
             ->elements();
     const auto& consts_list =
-        expect_field(table, "constants", BYTECODE_INDEX_CONSTANT)
+        expect_field(codeTable, "constants", BYTECODE_INDEX_CONSTANT)
             .toTuple()
             ->elements();
     const auto& types_list =
-        expect_field(table, "types", BYTECODE_INDEX_TYPE).toTuple()->elements();
+        expect_field(codeTable, "types", BYTECODE_INDEX_TYPE)
+            .toTuple()
+            ->elements();
     const auto& register_size =
-        expect_field(table, "register_size", BYTECODE_INDEX_REGISTER_SIZE)
+        expect_field(codeTable, "register_size", BYTECODE_INDEX_REGISTER_SIZE)
             .toInt();
 
     std::vector<IValue> module_debug_info_list;
@@ -191,6 +270,8 @@ void parseMethods(
     }
 
     std::unordered_set<std::string> unsupported_op_names;
+    // ops_list is the list of operator names that were read in from
+    // bytecode.plk for the method that is currently being processed.
     for (const auto& op : ops_list) {
       auto op_item = op.toTuple()->elements();
       TORCH_CHECK(
@@ -213,41 +294,69 @@ void parseMethods(
       function->append_constant(constant);
     }
 
+    static const c10::QualifiedName classPrefix = "__torch__.torch.classes";
     for (const auto& t : types_list) {
-      function->append_type(c10::parseType(t.toStringRef()));
+      c10::QualifiedName qn(t.toStringRef());
+      if (classPrefix.isPrefixOf(qn)) {
+        auto classType = getCustomClass(qn.qualifiedName());
+        TORCH_CHECK(
+            classType,
+            "The implementation of class ",
+            qn.qualifiedName(),
+            " cannot be found.");
+        function->append_type(classType);
+      } else {
+        function->append_type(c10::parseType(t.toStringRef()));
+      }
     }
 
     function->set_register_size(register_size);
 
+    // function schema
+    if (schemaTable) { // (schema is optional for back compat)
+      auto parseArgList = [this](const std::vector<IValue>& argTables) {
+        std::vector<c10::Argument> args;
+        for (auto&& argTable : argTables) {
+          auto name =
+              expect_field(argTable, "name", BYTECODE_INDEX_ARGUMENT_NAME)
+                  .toStringRef();
+          const auto& type = resolveTypeName(
+              (expect_field(argTable, "type", BYTECODE_INDEX_ARGUMENT_TYPE))
+                  .toStringRef());
+          auto default_value = expect_field(
+                                   argTable,
+                                   "default_value",
+                                   BYTECODE_INDEX_ARGUMENT_DEFAULT_VALUE)
+                                   .toIValue();
+          auto arg =
+              c10::Argument(name, type, c10::nullopt /*N*/, default_value);
+          args.emplace_back(std::move(arg));
+        }
+        return args;
+      };
+      const auto& arg_list =
+          expect_field(
+              *schemaTable, "arguments", BYTECODE_INDEX_SCHEMA_ARGUMENTS)
+              .toTuple()
+              ->elements();
+      const auto& ret_list =
+          expect_field(*schemaTable, "returns", BYTECODE_INDEX_SCHEMA_RETURNS)
+              .toTuple()
+              ->elements();
+      c10::FunctionSchema schema(
+          function_name,
+          "" /*overload_name*/,
+          parseArgList(arg_list),
+          parseArgList(ret_list),
+          false /*is_varargs*/,
+          false /*is_varret*/);
+      function->setSchema(std::move(schema));
+    }
+
     mcu.register_function(std::move(function));
   }
 }
 
-// The deserializer class which loads the bytecode package from bc files.
-class BytecodeDeserializer final {
- public:
-  explicit BytecodeDeserializer(std::unique_ptr<PyTorchStreamReader> reader);
-  mobile::Module deserialize(c10::optional<at::Device> device);
-  std::unordered_map<std::string, std::string> deserializeMetadata(
-      c10::optional<at::Device> device);
-
- private:
-  c10::IValue readArchive(
-      const std::string& archive_name,
-      std::shared_ptr<mobile::CompilationUnit> mcu);
-  std::unordered_map<std::string, std::string> readMobileMetadata(
-      std::shared_ptr<mobile::CompilationUnit> mcu);
-  std::shared_ptr<CompilationUnit> compilation_unit_;
-  std::unordered_set<std::string> imported_libs_;
-  std::unique_ptr<PyTorchStreamReader> reader_;
-  c10::optional<at::Device> device_;
-};
-
-BytecodeDeserializer::BytecodeDeserializer(
-    std::unique_ptr<PyTorchStreamReader> reader)
-    : compilation_unit_(std::make_shared<CompilationUnit>()),
-      reader_(std::move(reader)) {}
-
 std::unordered_map<std::string, std::string> BytecodeDeserializer::
     deserializeMetadata(c10::optional<at::Device> device) {
   device_ = device;
@@ -255,10 +364,43 @@ std::unordered_map<std::string, std::string> BytecodeDeserializer::
   return readMobileMetadata(mcu);
 }
 
+void BytecodeDeserializer::deserialize_only_extra(
+    c10::optional<at::Device> device,
+    ExtraFilesMap& extra_files) {
+  device_ = device;
+  for (const auto& kv : extra_files) {
+    const std::string& key = "extra/" + kv.first;
+    if (reader_->hasRecord(key)) {
+      at::DataPtr meta_ptr;
+      size_t meta_size = 0;
+      std::tie(meta_ptr, meta_size) = reader_->getRecord(key);
+      extra_files[kv.first] =
+          std::string(static_cast<char*>(meta_ptr.get()), meta_size);
+    }
+  }
+}
+
+mobile::Module BytecodeDeserializer::deserialize(
+    c10::optional<at::Device> device,
+    ExtraFilesMap& extra_files) {
+  deserialize_only_extra(device, extra_files);
+  return deserialize(device);
+}
+
 mobile::Module BytecodeDeserializer::deserialize(
     c10::optional<at::Device> device) {
   device_ = device;
   auto mcu = std::make_shared<mobile::CompilationUnit>();
+
+  // bvals can have 2 possible formats:
+  //
+  // 1. Old format: bvals is an array (Tuple) of N elements, each element being
+  // itself a Tuple(method_name, method_table).
+  //
+  // 2. New format: bvals is an array (Tuple) of 1+N elements. The first element
+  // being a Tuple (int, table), and the integer stands for the bytecode version
+  // number. The rest of the elements are the same as before.
+  //
   auto bvals = readArchive("bytecode", mcu).toTuple()->elements();
 
   c10::optional<std::vector<IValue>> debug_info_bvals;
@@ -308,23 +450,8 @@ c10::IValue BytecodeDeserializer::readArchive(
     return len;
   };
 
-  static const c10::QualifiedName torchPrefix = "__torch__";
-  auto type_resolver = [&](const c10::QualifiedName& qn) {
-    TypePtr type;
-    // HACK: first we check whether the name starts with `__torch__` to tell if
-    // it's "supposed" to be a class type. This is a reliable check today, but
-    // there is no guarantee that this is the case. The real solution is to
-    // merge type parsers so we can share class resolution logic.
-    if (torchPrefix.isPrefixOf(qn)) {
-      if (compilation_unit_->get_class(qn) == nullptr) {
-        auto typeptr = ClassType::create(qn, compilation_unit_, true);
-        compilation_unit_->register_type(typeptr);
-      }
-      type = compilation_unit_->get_class(qn);
-    } else {
-      type = c10::parseType(qn.qualifiedName());
-    }
-    return c10::StrongTypePtr(compilation_unit_, type);
+  auto type_resolver = [this](const c10::QualifiedName& qn) {
+    return c10::StrongTypePtr(compilation_unit_, resolveTypeName(qn));
   };
 
   auto obj_loader = [&](at::StrongTypePtr type, IValue input) {
@@ -386,43 +513,70 @@ c10::IValue BytecodeDeserializer::readArchive(
 mobile::Module _load_for_mobile(
     std::istream& in,
     c10::optional<at::Device> device) {
-  std::unique_ptr<IStreamAdapter> rai = std::make_unique<IStreamAdapter>(&in);
-  auto module = _load_for_mobile(std::move(rai), device);
-  return module;
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(in, device, extra_files);
 }
 
 mobile::Module _load_for_mobile(
     const std::string& filename,
     c10::optional<at::Device> device) {
-  std::unique_ptr<FileAdapter> rai = std::make_unique<FileAdapter>(filename);
-  auto module = _load_for_mobile(std::move(rai), device);
-  return module;
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(filename, device, extra_files);
 }
 
 mobile::Module _load_for_mobile(
     std::unique_ptr<ReadAdapterInterface> rai,
     c10::optional<c10::Device> device) {
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(std::move(rai), device, extra_files);
+}
+
+mobile::Module _load_for_mobile(
+    std::istream& in,
+    c10::optional<at::Device> device,
+    ExtraFilesMap& extra_files) {
+  std::unique_ptr<IStreamAdapter> rai = std::make_unique<IStreamAdapter>(&in);
+  auto module = _load_for_mobile(std::move(rai), device, extra_files);
+  return module;
+}
+
+mobile::Module _load_for_mobile(
+    const std::string& filename,
+    c10::optional<at::Device> device,
+    ExtraFilesMap& extra_files) {
+  std::unique_ptr<FileAdapter> rai = std::make_unique<FileAdapter>(filename);
+  auto module = _load_for_mobile(std::move(rai), device, extra_files);
+  return module;
+}
+
+mobile::Module _load_for_mobile(
+    std::unique_ptr<ReadAdapterInterface> rai,
+    c10::optional<c10::Device> device,
+    ExtraFilesMap& extra_files) {
   auto observer = torch::observerConfig().getModuleObserver();
+  auto instance_key = std::rand();
   if (observer) {
-    observer->onEnterLoadModel();
+    observer->onEnterLoadModel(instance_key);
   }
   auto reader = torch::make_unique<PyTorchStreamReader>(std::move(rai));
   BytecodeDeserializer deserializer(std::move(reader));
   try {
-    mobile::Module result = deserializer.deserialize(std::move(device));
+    mobile::Module result = deserializer.deserialize(device, extra_files);
     std::unordered_map<std::string, std::string> copied_metadata =
         result.metadata();
     if (result.metadata().find("model_name") == result.metadata().end()) {
       copied_metadata["model_name"] = result.name();
     }
     if (observer) {
-      observer->onExitLoadModel(copied_metadata);
+      observer->onExitLoadModel(instance_key, copied_metadata);
     }
     return result;
   } catch (c10::Error& error) {
     if (observer) {
       observer->onFailLoadModel(
-          error.what(), deserializer.deserializeMetadata(std::move(device)));
+          instance_key,
+          error.what(),
+          deserializer.deserializeMetadata(std::move(device)));
     }
     TORCH_RETHROW(error);
   } catch (...) {
@@ -440,12 +594,29 @@ mobile::Module _load_for_mobile(
     } catch (c10::Error& error) {
       if (observer) {
         observer->onFailLoadModel(
-            error.what(), deserializer.deserializeMetadata(std::move(device)));
+            instance_key,
+            error.what(),
+            deserializer.deserializeMetadata(std::move(device)));
       }
       TORCH_RETHROW(error);
     }
   }
 }
 
+void _load_extra_only_for_mobile(
+    const std::string& filename,
+    c10::optional<at::Device> device,
+    ExtraFilesMap& extra_files) {
+  std::unique_ptr<FileAdapter> rai = std::make_unique<FileAdapter>(filename);
+  auto observer = torch::observerConfig().getModuleObserver();
+  auto instance_key = std::rand();
+  if (observer) {
+    observer->onEnterLoadModel(instance_key);
+  }
+  auto reader = torch::make_unique<PyTorchStreamReader>(std::move(rai));
+  BytecodeDeserializer deserializer(std::move(reader));
+  deserializer.deserialize_only_extra(device, extra_files);
+}
+
 } // namespace jit
 } // namespace torch
