@@ -5,7 +5,9 @@
 namespace torch {
 namespace jit {
 
-TORCH_API void RemoveInplaceOpsForONNX(const std::shared_ptr<Graph>& graph);
+TORCH_API void RemoveInplaceOpsForONNX(
+    const std::shared_ptr<Graph>& graph,
+    Module* model);
 
 TORCH_API void PrepareInplaceOpsForONNX(const std::shared_ptr<Graph>& graph);
 
