@@ -4,19 +4,20 @@
 #include <ATen/core/jit_type.h>
 #include <ATen/core/qualified_name.h>
 #include <ATen/core/stack.h>
+#include <pybind11/complex.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/pytypes.h>
 #include <torch/csrc/Device.h>
 #include <torch/csrc/Dtype.h>
 #include <torch/csrc/Layout.h>
 #include <torch/csrc/QScheme.h>
+#include <torch/csrc/Stream.h>
 #include <torch/csrc/WindowsTorchApiMacro.h>
 #include <torch/csrc/jit/api/module.h>
 #include <torch/csrc/jit/frontend/schema_matching.h>
 #include <torch/csrc/jit/frontend/tracer.h>
 #include <torch/csrc/jit/python/module_python.h>
 #include <torch/csrc/jit/python/python_custom_class.h>
-#include <torch/csrc/jit/python/python_ivalue.h>
 #include <torch/csrc/jit/python/python_tracer.h>
 #include <torch/csrc/jit/resource_guard.h>
 #include <torch/csrc/jit/runtime/operator.h>
@@ -29,6 +30,11 @@
 #endif
 
 #include <ATen/core/function_schema.h>
+#include <c10/core/Stream.h>
+#ifdef USE_C10D_NCCL
+#include <c10/cuda/CUDACachingAllocator.h>
+#include <c10/cuda/CUDAStream.h>
+#endif
 #include <c10/util/Exception.h>
 
 #include <algorithm>
@@ -48,13 +54,34 @@
 namespace torch {
 namespace jit {
 
-inline IValue toIValue(
+void clear_registered_instances(void* ptr);
+
+IValue toIValue(
     py::handle obj,
     const TypePtr& type,
     c10::optional<int32_t> N = c10::nullopt);
 
 py::object toPyObject(IValue ivalue);
 
+// Wrap Python function to guard deref
+// NB: Need VISIBILITY_HIDDEN for silencing compiler error,
+// 'torch::jit::PythonFunctionGuard' declared with greater visibility than the
+// type of its field 'torch::jit::PythonFunctionGuard::func_'
+struct VISIBILITY_HIDDEN PythonFunctionGuard {
+  explicit PythonFunctionGuard(py::function func) : func_(std::move(func)) {}
+
+  ~PythonFunctionGuard() {
+    pybind11::gil_scoped_acquire ag;
+    func_.dec_ref();
+    // explicitly setting PyObject* to nullptr to prevent py::object's dtor to
+    // decref on the PyObject again.
+    // See Note [Destructing py::object] in python_ivalue.h
+    func_.ptr() = nullptr;
+  }
+
+  py::function func_;
+};
+
 // The PythonFutureWrapper for ivalue::Future
 //
 // NB: VISIBILITY_HIDDEN is for silencing compiling error,
@@ -85,6 +112,9 @@ struct VISIBILITY_HIDDEN PythonFutureWrapper
     // without grabbing the GIL.
     py::gil_scoped_acquire acquire;
     py::object py_obj = toPyObject(fut->value());
+    // unwrap_func is a general compositional function that takes in a
+    // py::object and executes some python function. It is currently mostly used
+    // to throw python exceptions.
     if (unwrap_func) {
       (*unwrap_func)(py_obj);
     }
@@ -112,11 +142,12 @@ struct VISIBILITY_HIDDEN PythonFutureWrapper
     // Future owns a reference to the py::function in its callback
     // vector, but Future does not acquire GIL on destruction.
     auto pf = std::make_shared<PythonFunctionGuard>(std::move(cb));
+
     return std::make_shared<jit::PythonFutureWrapper>(fut->then(
         // Capture a copy of the ivalue::Future instead of the `this` pointer
         // because the PythonFutureWrapper object could have been deleted
         // when the callbacks are fired. For example, RPC only captures the
-        // ivalue::Future instead of PythonFutureWrapper in FutureMessage's
+        // ivalue::Future instead of PythonFutureWrapper in JitFuture's
         // callback functions. Hence, if user code does not hold a reference to
         // this PythonFutureWrapper object, there is no guarantee that the
         // PythonFutureWrapper is still valid when running the callback.
@@ -144,6 +175,36 @@ struct VISIBILITY_HIDDEN PythonFutureWrapper
         PyObjectType::get()));
   }
 
+  void add_done_callback(py::function cb) {
+    auto pf = std::make_shared<PythonFunctionGuard>(std::move(cb));
+    fut->addCallback(std::bind(
+        [pyFut(this->getPtr())](std::shared_ptr<PythonFunctionGuard> pf) {
+          try {
+            pybind11::gil_scoped_acquire ag;
+            pf->func_(pyFut);
+          } catch (py::error_already_set& e) {
+            {
+              pybind11::gil_scoped_acquire ag;
+              // Release ownership on py::objects and also restore Python
+              // Error Indicator.
+              e.restore();
+              // Clear the Python Error Indicator as we has recorded the
+              // exception in the response message.
+              PyErr_Clear();
+            }
+            // Log and ignore exceptions raised through the callback
+            VLOG(1) << "Got the following error when running the callback: "
+                    << e.what();
+
+          } catch (std::exception& e) {
+            // Log and ignore exceptions raised through the callback
+            VLOG(1) << "Got the following error when running the callback: "
+                    << e.what();
+          }
+        },
+        std::move(pf)));
+  }
+
   void markCompleted(const py::object& pyValue) {
     DCHECK(PyGILState_Check());
     IValue value = toIValue(pyValue, PyObjectType::get());
@@ -158,22 +219,6 @@ struct VISIBILITY_HIDDEN PythonFutureWrapper
   c10::optional<UnwrapFunc> unwrap_func;
 
  private:
-  // Wrap Python function to guard deref
-  struct PythonFunctionGuard {
-    explicit PythonFunctionGuard(py::function func) : func_(std::move(func)) {}
-
-    ~PythonFunctionGuard() {
-      pybind11::gil_scoped_acquire ag;
-      func_.dec_ref();
-      // explicitly setting PyObject* to nullptr to prevent py::object's dtor to
-      // decref on the PyObject again.
-      // See Note [Destructing py::object] in python_ivalue.h
-      func_.ptr() = nullptr;
-    }
-
-    py::function func_;
-  };
-
   std::shared_ptr<PythonFutureWrapper> getPtr() {
     return shared_from_this();
   }
@@ -216,34 +261,15 @@ inline TypedIValue toDictKeyIValue(py::handle key) {
 }
 
 inline c10::optional<TypePtr> unifyOrInitializeType(
-    TypePtr accum,
-    TypePtr unify) {
+    const TypePtr& accum,
+    const TypePtr& unify) {
   if (!accum) {
     return unify;
   }
   return unifyTypes(accum, unify);
 }
 
-struct InferredType {
-  InferredType(TypePtr type) : type_(std::move(type)) {}
-  InferredType(std::string reason)
-      : type_(nullptr), reason_(std::move(reason)) {}
-  TypePtr type() const {
-    TORCH_INTERNAL_ASSERT(type_);
-    return type_;
-  }
-  bool success() const {
-    return type_ != nullptr;
-  }
-  const std::string& reason() const {
-    TORCH_INTERNAL_ASSERT(!type_);
-    return reason_;
-  }
-
- private:
-  TypePtr type_;
-  std::string reason_;
-};
+using InferredType = c10::InferredType;
 
 InferredType tryToInferContainerType(py::handle input);
 
@@ -256,8 +282,7 @@ InferredType tryToInferContainerType(py::handle input);
 inline InferredType tryToInferType(py::handle input) {
   // Try tensor types
   if (THPVariable_Check(input.ptr())) {
-    auto tensor = py::cast<at::Tensor>(input);
-    return InferredType(TensorType::create(tensor));
+    return InferredType(TensorType::get());
   }
 
   if (input.is(py::none())) {
@@ -276,12 +301,16 @@ inline InferredType tryToInferType(py::handle input) {
     return InferredType(IntType::get());
   } else if (py::isinstance<py::float_>(input)) {
     return InferredType(FloatType::get());
+  } else if (PyComplex_CheckExact(input.ptr())) {
+    return InferredType(ComplexType::get());
   } else if (py::isinstance<py::str>(input)) {
     return InferredType(StringType::get());
   } else if (THPLayout_Check(input.ptr())) {
     return InferredType(IntType::get());
   } else if (THPDevice_Check(input.ptr())) {
     return InferredType(DeviceObjType::get());
+  } else if (THPStream_Check(input.ptr())) {
+    return InferredType(StreamObjType::get());
   } else if (THPDtype_Check(input.ptr())) {
     return InferredType(IntType::get());
   } else if (THPQScheme_Check(input.ptr())) {
@@ -327,6 +356,16 @@ inline InferredType tryToInferType(py::handle input) {
 #endif
   }
 
+  if (as_module(py::cast<py::object>(input))) {
+    return InferredType("Cannot infer type of ScriptModule");
+  }
+
+  auto module_type = py::module::import("torch.nn").attr("Module");
+  py::bool_ is_module = py::isinstance(input, module_type);
+  if (py::cast<bool>(is_module)) {
+    return InferredType("Cannot infer concrete type of torch.nn.Module");
+  }
+
   // Try container types
   return tryToInferContainerType(input);
 }
@@ -434,7 +473,7 @@ inline InferredType tryToInferContainerType(py::handle input) {
   }
 }
 
-inline bool isTraceableType(TypePtr type) {
+inline bool isTraceableType(const TypePtr& type) {
   if (type->isSubtypeOf(TensorType::get())) {
     return true;
   }
@@ -447,7 +486,9 @@ inline bool isTraceableType(TypePtr type) {
     return std::all_of(
         tuple_type->elements().begin(),
         tuple_type->elements().end(),
-        [](TypePtr element_type) { return isTraceableType(element_type); });
+        [](const TypePtr& element_type) {
+          return isTraceableType(element_type);
+        });
   }
 
   if (auto dict_type = type->cast<DictType>()) {
@@ -480,13 +521,13 @@ inline Stack toTraceableStack(const py::tuple& inputs) {
 inline IValue createGenericList(py::handle obj, const TypePtr& elem_type) {
   auto elems = c10::impl::GenericList(elem_type);
   for (auto elem : obj) {
-    elems.push_back(toIValue(std::move(elem), elem_type));
+    elems.push_back(toIValue(elem, elem_type));
   }
   return IValue(std::move(elems));
 }
 
 inline IValue createGenericDict(
-    py::dict obj,
+    const py::dict& obj,
     const TypePtr& key_type,
     const TypePtr& value_type) {
   c10::impl::GenericDict elems(key_type, value_type);
@@ -506,256 +547,9 @@ inline void guardAgainstNamedTensor(const T& var) {
       "workaround please drop names via `tensor = tensor.rename(None)`.");
 }
 
-inline IValue toIValue(
-    py::handle obj,
-    const TypePtr& type,
-    c10::optional<int32_t> N) {
-  switch (type->kind()) {
-    case TypeKind::TensorType: {
-      auto var = py::cast<autograd::Variable>(obj);
-      if (var.is_sparse()) {
-        TORCH_WARN_ONCE(
-            "Using sparse tensors in TorchScript is experimental. Many optimization "
-            "pathways have not been thoroughly tested with sparse tensors. Please "
-            "include the fact that the network is running sparse tensors in any bug "
-            "reports submitted.");
-      }
-      guardAgainstNamedTensor<autograd::Variable>(var);
-      return var;
-    }
-    case TypeKind::FloatType:
-      return py::cast<double>(obj);
-    case TypeKind::IntType:
-    // TODO(xintchen): Handling LayoutType and ScalarTypeType correctly.
-    case TypeKind::LayoutType:
-    case TypeKind::ScalarTypeType:
-      if (THPDtype_Check(obj.ptr())) {
-        auto dtype = reinterpret_cast<THPDtype*>(obj.ptr());
-        return static_cast<int64_t>(dtype->scalar_type);
-      }
-      if (THPQScheme_Check(obj.ptr())) {
-        auto qscheme = reinterpret_cast<THPQScheme*>(obj.ptr());
-        return static_cast<uint8_t>(qscheme->qscheme);
-      }
-      if (THPLayout_Check(obj.ptr())) {
-        auto layout = reinterpret_cast<THPLayout*>(obj.ptr());
-        return static_cast<int8_t>(layout->layout);
-      }
-      return py::cast<int64_t>(obj);
-    case TypeKind::NoneType:
-      if (!obj.is_none()) {
-        throw py::cast_error(
-            c10::str("Cannot cast ", py::str(obj), " to None"));
-      }
-      return {};
-    case TypeKind::BoolType:
-      return py::cast<bool>(obj);
-    case TypeKind::TupleType: {
-      py::tuple tuple = py::cast<py::tuple>(obj);
-      size_t tuple_size = tuple.size();
-      auto tuple_type = type->cast<TupleType>();
-      const auto& elem_types = tuple_type->elements();
-      if (elem_types.size() != tuple_size) {
-        throw py::cast_error(c10::str(
-            "Object ",
-            py::str(obj),
-            " had a different number of elements than type ",
-            type->repr_str()));
-      }
-      std::vector<IValue> values;
-      values.reserve(tuple_size);
-      for (size_t i = 0; i < tuple_size; ++i) {
-        values.push_back(toIValue(tuple[i], elem_types[i]));
-      }
-      return tuple_type->name()
-          ? c10::ivalue::Tuple::createNamed(std::move(values), tuple_type)
-          : c10::ivalue::Tuple::create(std::move(values));
-    }
-    case TypeKind::StringType:
-      return ConstantString::create(py::cast<std::string>(obj));
-    case TypeKind::DeviceObjType: {
-      auto device = reinterpret_cast<THPDevice*>(obj.ptr());
-      return device->device;
-    }
-    case TypeKind::ListType: {
-      const auto& elem_type = type->expect<ListType>()->getElementType();
-      switch (elem_type->kind()) {
-        // allows single int/float to be broadcasted to a fixed size list
-        case TypeKind::IntType:
-          if (!N || !py::isinstance<py::int_>(obj)) {
-            return IValue(py::cast<std::vector<int64_t>>(obj));
-          } else {
-            int64_t value = py::cast<int64_t>(obj);
-            c10::List<int64_t> repeated;
-            repeated.reserve(*N);
-            for (int i = 0; i < *N; ++i) {
-              repeated.push_back(value);
-            }
-            return repeated;
-          }
-        case TypeKind::FloatType:
-          if (!N || !py::isinstance<py::float_>(obj)) {
-            return IValue(py::cast<std::vector<double>>(obj));
-          } else {
-            double value = py::cast<double>(obj);
-            c10::List<double> repeated;
-            repeated.reserve(*N);
-            for (int i = 0; i < *N; ++i) {
-              repeated.push_back(value);
-            }
-            return repeated;
-          }
-        case TypeKind::BoolType:
-          return IValue(py::cast<std::vector<bool>>(obj));
-        case TypeKind::TensorType:
-          return IValue(py::cast<std::vector<at::Tensor>>(obj));
-        default:
-          return createGenericList(obj, elem_type);
-      }
-    }
-    case TypeKind::DictType: {
-      const auto& dict_type = type->expect<DictType>();
-      return createGenericDict(
-          py::cast<py::dict>(obj),
-          dict_type->getKeyType(),
-          dict_type->getValueType());
-    }
-    case TypeKind::OptionalType: {
-      // check if it's a none obj since optional accepts NoneType
-      if (obj.is_none()) {
-        // check if it's a none obj since optional accepts NoneType
-        // return an IValue() to denote a NoneType
-        return {};
-      }
-      return toIValue(obj, type->expect<OptionalType>()->getElementType());
-    }
-    case TypeKind::ClassType: {
-      auto classType = type->expect<ClassType>();
-      if (auto mod = as_module(py::cast<py::object>(obj))) {
-        // if obj is already a ScriptModule, just return its ivalue
-        return mod.value()._ivalue();
-      }
-      // otherwise is a normal class object, we create a fresh
-      // ivalue::Object to use from the py object.
-      // 1. create a bare ivalue
-      const size_t numAttrs = classType->numAttributes();
-      auto cu = classType->compilation_unit();
-      auto userObj = c10::ivalue::Object::create(
-          c10::StrongTypePtr(cu, classType), numAttrs);
-
-      // 2. copy all the contained types
-      for (size_t slot = 0; slot < numAttrs; slot++) {
-        const auto& attrType = classType->getAttribute(slot);
-        const auto& attrName = classType->getAttributeName(slot);
-
-        const auto& contained = py::getattr(obj, attrName.c_str());
-        userObj->setSlot(slot, toIValue(contained, attrType));
-      }
-      return userObj;
-    }
-    case TypeKind::InterfaceType: {
-      auto interfaceType = type->expect<InterfaceType>();
-      // When converting an pyobj to an interface, we check if rhs
-      // is module or normal torchscript class, get the type and ivalue
-      // from them correspondingly.
-      c10::ClassTypePtr classType = nullptr;
-      IValue res;
-      if (auto mod = as_module(py::cast<py::object>(obj))) {
-        classType = mod.value().type();
-        res = mod.value()._ivalue();
-      } else {
-        // We inspect the value to found the compiled TorchScript class
-        // and then create a ivalue::Object from that class type.
-        py::str qualified_name = py::module::import("torch._jit_internal")
-                                     .attr("_qualified_name")(obj.get_type());
-        auto pyCu = get_python_cu();
-        classType = pyCu->get_class(c10::QualifiedName(qualified_name));
-        if (!classType) {
-          throw std::runtime_error(c10::str(
-              "Assigning the object ",
-              py::str(obj),
-              " to an interface fails because the value is not "
-              "a TorchScript compatible type, did you forget to",
-              "turn it into a user defined TorchScript class?"));
-        }
-        res = toIValue(std::move(obj), classType);
-      }
-      // check if the classType conform with the interface or not
-      std::stringstream why_not;
-      if (!classType->isSubtypeOfExt(interfaceType, &why_not)) {
-        throw py::cast_error(c10::str(
-            "Object ",
-            py::str(obj),
-            " is not compatible with interface ",
-            interfaceType->repr_str(),
-            "\n",
-            why_not.str()));
-      }
-      return res;
-    }
-    case TypeKind::NumberType: {
-      if (THPDtype_Check(obj.ptr())) {
-        auto dtype = reinterpret_cast<THPDtype*>(obj.ptr());
-        return static_cast<int64_t>(dtype->scalar_type);
-      }
-      if (THPQScheme_Check(obj.ptr())) {
-        auto qscheme = reinterpret_cast<THPQScheme*>(obj.ptr());
-        return static_cast<uint8_t>(qscheme->qscheme);
-      }
-      if (THPLayout_Check(obj.ptr())) {
-        auto layout = reinterpret_cast<THPLayout*>(obj.ptr());
-        return static_cast<int8_t>(layout->layout);
-      }
-      if (py::isinstance<py::int_>(obj)) {
-        return py::cast<int64_t>(obj);
-      } else if (py::isinstance<py::float_>(obj)) {
-        return py::cast<double>(obj);
-      } else {
-        throw py::cast_error(
-            c10::str("Cannot cast ", py::str(obj), " to ", type->repr_str()));
-      }
-    }
-    case TypeKind::RRefType: {
-#ifdef USE_RPC
-      return obj.cast<torch::distributed::rpc::PyRRef>().toIValue();
-#else
-      AT_ERROR("RRef is only supported with the distributed package");
-#endif
-    } break;
-    case TypeKind::PyObjectType: {
-      return c10::ivalue::ConcretePyObjectHolder::create(obj);
-    }
-    case TypeKind::CapsuleType: {
-      return IValue::make_capsule(
-          py::cast<c10::intrusive_ptr<CustomClassHolder>>(obj));
-    }
-    case TypeKind::FutureType: {
-      return obj.cast<std::shared_ptr<PythonFutureWrapper>>()->fut;
-    }
-    case TypeKind::AnyType:
-      return toTypeInferredIValue(obj);
-    case TypeKind::FunctionType:
-    case TypeKind::GeneratorType:
-    case TypeKind::QuantizerType:
-    case TypeKind::VarType:
-    case TypeKind::QSchemeType:
-    case TypeKind::AnyListType:
-    case TypeKind::AnyTupleType:
-    case TypeKind::AnyClassType:
-    case TypeKind::AnyEnumType:
-      break;
-    case TypeKind::EnumType:
-      EnumTypePtr enum_type = type->expect<EnumType>();
-      py::object py_obj = py::reinterpret_borrow<py::object>(obj);
-      std::string name = py::cast<std::string>(obj.attr("name"));
-      IValue value = toIValue(obj.attr("value"), enum_type->getValueType(), {});
-      auto enum_holder =
-          c10::make_intrusive<c10::ivalue::EnumHolder>(enum_type, name, value);
-      return IValue(enum_holder);
-  }
-  throw py::cast_error(c10::str(
-      "toIValue() cannot handle converting to type: ", type->repr_str()));
-}
+// Defined in pybind_utils.cpp to break a circular dependency with
+// python_ivalue.h
+IValue toIValue(py::handle obj, const TypePtr& type, c10::optional<int32_t> N);
 
 // Small wrapper around getting the type name string from Python to make
 // types easier to interpret, e.g. give the structural type for a NamedTuple
@@ -805,6 +599,15 @@ inline IValue argumentToIValue(
             py::repr(object)),
         "\nCast error details: ",
         error.what()));
+  } catch (const py::error_already_set& error) {
+    throw schema_match_error(c10::str(
+        schema.formatTypeMismatchMsg(
+            argument,
+            friendlyTypeName(object),
+            argumentPosition,
+            py::repr(object)),
+        "\n Python error details: ",
+        error.what()));
   }
 }
 
@@ -854,6 +657,9 @@ inline py::object toPyObject(IValue ivalue) {
     return py::cast(autograd::Variable(std::move(tensor)));
   } else if (ivalue.isDouble()) {
     return py::cast(std::move(ivalue).toDouble());
+  } else if (ivalue.isComplexDouble()) {
+    return py::cast(
+        static_cast<std::complex<double>>(std::move(ivalue).toComplexDouble()));
   } else if (ivalue.isInt()) {
     return py::cast(std::move(ivalue).toInt());
   } else if (ivalue.isBool()) {
@@ -932,7 +738,7 @@ inline py::object toPyObject(IValue ivalue) {
     // PyObject
     return py::reinterpret_borrow<py::object>(ivalue.toPyObject());
   } else if (ivalue.isCapsule()) {
-    return py::cast(ivalue.toCapsule());
+    return py::cast(c10::Capsule(ivalue.toCapsule()));
   } else if (ivalue.isFuture()) {
     return py::cast(std::make_shared<PythonFutureWrapper>(ivalue.toFuture()));
   } else if (ivalue.isEnum()) {
@@ -1006,9 +812,9 @@ inline Stack createStackForSchema(
     push(stack, std::move(*self));
   }
   // First push all positional args.
-  for (size_t i = 0; i < args.size(); ++i) {
+  for (const auto& arg : args) {
     // Use the type information from the schema to convert the PyObject.
-    push(stack, argumentToIValue(schema, stack.size(), args[i]));
+    push(stack, argumentToIValue(schema, stack.size(), arg));
   }
 
   // Now for every remaining non-positional argument in the schema, look for it
@@ -1085,15 +891,16 @@ inline Stack evilDeprecatedBadCreateStackDoNotUse(
 // tracing graph.
 inline py::object runAndInsertCall(
     Function& callee,
-    tuple_slice args,
-    py::kwargs kwargs,
+    const tuple_slice& args,
+    const py::kwargs& kwargs,
     c10::optional<IValue> self,
     // Lambda that tells this function how to insert `callee` into the graph if
     // we're tracing.
-    std::function<Value*(Graph&, const MatchedSchema& match)> callInserter) {
-  auto stack = createStackForSchema(
-      callee.getSchema(), std::move(args), std::move(kwargs), std::move(self));
-  auto tracing_state = tracer::getTracingState();
+    const std::function<Value*(Graph&, const MatchedSchema& match)>&
+        callInserter) {
+  auto stack =
+      createStackForSchema(callee.getSchema(), args, kwargs, std::move(self));
+  const auto& tracing_state = tracer::getTracingState();
   if (!tracing_state) {
     pybind11::gil_scoped_release no_gil_guard;
     // If we're not tracing, just run the callee as normal.
@@ -1143,8 +950,8 @@ inline py::object runAndInsertCall(
 
 inline py::object invokeScriptFunctionFromPython(
     Function& callee,
-    tuple_slice args,
-    py::kwargs kwargs) {
+    const tuple_slice& args,
+    const py::kwargs& kwargs) {
   return runAndInsertCall(
       callee,
       args,
@@ -1157,8 +964,8 @@ inline py::object invokeScriptFunctionFromPython(
 
 inline py::object invokeScriptMethodFromPython(
     Method& callee,
-    tuple_slice args,
-    py::kwargs kwargs) {
+    const tuple_slice& args,
+    const py::kwargs& kwargs) {
   auto self = callee.owner()._ivalue();
   return runAndInsertCall(
       callee.function(),
@@ -1170,20 +977,18 @@ inline py::object invokeScriptMethodFromPython(
       });
 }
 
-inline py::object invokeOperatorFromPython(
+inline std::pair<std::shared_ptr<Operator>, Stack> getOpWithStack(
     const std::vector<std::shared_ptr<Operator>>& operations,
     py::args args,
-    py::kwargs kwargs) {
+    const py::kwargs& kwargs) {
   Stack stack;
-
   if (operations.size() == 1) {
-    const Operator& op = *operations.at(0);
+    std::shared_ptr<Operator> op = operations.at(0);
     // Create a stack full of the arguments and keyword arguments.
     stack = createStackForSchema(
-        op.schema(), std::move(args), std::move(kwargs), c10::nullopt);
+        op->schema(), std::move(args), kwargs, c10::nullopt);
 
-    pybind11::gil_scoped_release no_gil_guard;
-    op.getOperation()(&stack);
+    return std::make_pair(op, stack);
   } else {
     std::vector<schema_match_error> errors;
     std::shared_ptr<Operator> found_op = nullptr;
@@ -1205,6 +1010,17 @@ inline py::object invokeOperatorFromPython(
       throw std::runtime_error(ss.str());
     }
 
+    return std::make_pair(found_op, stack);
+  }
+}
+inline py::object invokeOperatorFromPython(
+    const std::vector<std::shared_ptr<Operator>>& operations,
+    py::args args,
+    const py::kwargs& kwargs) {
+  auto opWithStack = getOpWithStack(operations, args, kwargs);
+  std::shared_ptr<Operator> found_op = std::get<0>(opWithStack);
+  Stack stack = std::get<1>(opWithStack);
+  {
     pybind11::gil_scoped_release no_gil_guard;
     found_op->getOperation()(&stack);
   }
