@@ -1,4 +1,5 @@
 #include <torch/csrc/jit/runtime/symbolic_script.h>
+
 #include <torch/csrc/jit/frontend/ir_emitter.h>
 #include <torch/csrc/jit/runtime/operator.h>
 
@@ -402,6 +403,23 @@ const std::vector<std::string> functions = {
                 return grad_self, grad_other
 
             return torch.matmul(self, other), backward
+
+        def linear(input : Tensor,
+                   weight : Tensor,
+                   bias : Optional[Tensor]):
+            result = torch.linear(input, weight, bias)
+ 
+            def backward(grad_output):
+                if bias is not None:
+                   grad_bias = grad_output._grad_sum_to_size(bias.size())
+                else:
+                   grad_bias = None
+
+                weight_size = weight.size()
+                grad_input = torch.matmul(grad_output, weight)
+                grad_weight = torch.matmul(grad_output.reshape(-1, weight_size[0]).t(), input.reshape(-1, weight_size[1]))
+                return grad_input, grad_weight, grad_bias
+            return result, backward
     )",
     R"(
         def addcmul(self,
@@ -761,6 +779,31 @@ const std::vector<std::string> functions = {
                 return grad_output / other, None
             return self / other, backward
 
+        def div_2(self, other, *, rounding_mode: str):
+            result = torch.div(self, other, rounding_mode=rounding_mode)
+            self_size, other_size = AD_sizes_if_not_equal_multi_0(self, other, result)
+            def backward(grad_output):
+                if rounding_mode == "true":
+                    grad_self = (grad_output / other)._grad_sum_to_size(self_size)
+                    grad_other = (-grad_output * self / (other * other))._grad_sum_to_size(other_size)
+                else:
+                    grad_self = torch.zeros_like(self)
+                    grad_other = torch.zeros_like(other)
+
+                return grad_self, grad_other, None
+
+            return result, backward
+
+        def div_3(self, other: number, *,  rounding_mode: str):
+            result = torch.div(self, other, rounding_mode=rounding_mode)
+            def backward(grad_output):
+                if rounding_mode == "true":
+                    grad_self = (grad_output / other)
+                else:
+                    grad_self = torch.zeros_like(self, memory_format=1)
+                return grad_self, None, None
+            return result, backward
+
         def max(self, other):
             result = torch.max(self, other)
             self_size, other_size = AD_sizes_if_not_equal_multi_0(self, other, result)
@@ -1369,8 +1412,8 @@ std::pair<std::shared_ptr<Graph>, Value*> extractClosure(Value* closure) {
   Value* context = closure->node()->inputs().at(1);
 
   TORCH_CHECK(
-      fn->node()->kind() == prim::Function,
-      "closure tuple must contain a prim::Function");
+      fn->node()->kind() == prim::Closure,
+      "closure tuple must contain a prim::Closure");
   return std::make_pair(fn->node()->g(attr::Subgraph), context);
 }
 
