@@ -2,6 +2,7 @@
 #include <ATen/core/List.h>
 #include <ATen/core/functional.h>
 #include <ATen/core/ivalue.h>
+#include <ATen/core/jit_type.h>
 #include <ATen/core/stack.h>
 
 namespace torch {
@@ -11,6 +12,8 @@ void tupleUnpack(Stack& stack);
 
 void format(Stack& stack, size_t num_inputs);
 
+void percentFormat(Stack& stack, size_t num_inputs);
+
 void listUnpack(Stack& stack, size_t num_outputs);
 
 void tupleConstruct(Stack& stack, size_t num_inputs);
@@ -20,11 +23,17 @@ void namedTupleConstruct(
     at::TupleTypePtr type,
     size_t num_inputs);
 
-void listConstruct(Stack& stack, at::ListTypePtr list_type, size_t num_inputs);
+void listConstruct(
+    Stack& stack,
+    const at::ListType& list_type,
+    size_t num_inputs);
 
-void dictConstruct(Stack& stack, at::DictTypePtr type, size_t num_inputs);
+void dictConstruct(
+    Stack& stack,
+    const at::DictTypePtr& type,
+    size_t num_inputs);
 
-void createObject(Stack& stack, at::ClassTypePtr type);
+void createObject(Stack& stack, const at::ClassTypePtr& type);
 
 void isinstance(Stack& stack, at::ArrayRef<at::TypePtr> types);
 
