@@ -32,7 +32,7 @@ class SourceRangeUnpickler {
   virtual c10::optional<SourceRange> findSourceRangeThatGenerated(
       const SourceRange& range) = 0;
 
-  virtual ~SourceRangeUnpickler() {}
+  virtual ~SourceRangeUnpickler() = default;
 };
 
 } // namespace jit
