@@ -59,6 +59,8 @@ Backend backendToBackendOfDeviceType(Backend b, DeviceType d) {
     case DeviceType::XLA:
       TORCH_CHECK(!isSparse(b), "Sparse not implemented for XLA");
       return Backend::XLA;
+    case DeviceType::XPU:
+      return backendToXPU(b);
     default:
       AT_ERROR("Unknown device type");
   }
@@ -270,7 +272,7 @@ Tensor internal_new_from_data(
 
   if (PyArray_Check(data)) {
     TORCH_CHECK(!pin_memory, "Can't pin tensor constructed from numpy");
-    auto tensor = tensor_from_numpy(data);
+    auto tensor = tensor_from_numpy(data, /*warn_if_not_writeable=*/!copy_numpy);
     const auto& inferred_scalar_type = type_inference ? tensor.scalar_type() : scalar_type;
     auto device = device_opt.has_value() ? *device_opt : at::Device(computeDeviceType(dispatch_key));
     pybind11::gil_scoped_release no_gil;
@@ -334,24 +336,41 @@ Tensor legacy_new_from_sequence(
 // in x.new(y), 'x' is the base.
 void check_base_legacy_new(c10::DispatchKey dispatch_key, at::Layout expected_layout) {
   if (expected_layout == c10::kStrided) {
-    TORCH_CHECK(dispatch_key == c10::DispatchKey::CPU
-                || dispatch_key == c10::DispatchKey::CUDA
-                || dispatch_key == c10::DispatchKey::HIP
-                || dispatch_key == c10::DispatchKey::XLA,
-                "new(): expected DispatchKey: ", c10::DispatchKey::CPU,
-                " or ", c10::DispatchKey::CUDA,
-                " or ", c10::DispatchKey::HIP,
-                " or ", c10::DispatchKey::XLA,
-                " but got: ", dispatch_key);
+    TORCH_CHECK(
+        dispatch_key == c10::DispatchKey::CPU ||
+            dispatch_key == c10::DispatchKey::CUDA ||
+            dispatch_key == c10::DispatchKey::HIP ||
+            dispatch_key == c10::DispatchKey::XLA ||
+            dispatch_key == c10::DispatchKey::XPU,
+        "new(): expected DispatchKey: ",
+        c10::DispatchKey::CPU,
+        " or ",
+        c10::DispatchKey::CUDA,
+        " or ",
+        c10::DispatchKey::HIP,
+        " or ",
+        c10::DispatchKey::XLA,
+        " or ",
+        c10::DispatchKey::XPU,
+        " but got: ",
+        dispatch_key);
   } else if(expected_layout == c10::kSparse) {
     // NOTE: no sparse XLA
-    TORCH_CHECK(dispatch_key == c10::DispatchKey::SparseCPU
-                || dispatch_key == c10::DispatchKey::SparseCUDA
-                || dispatch_key == c10::DispatchKey::SparseHIP,
-                "new(): expected DispatchKey: ", c10::DispatchKey::SparseCPU,
-                " or ", c10::DispatchKey::SparseCUDA,
-                " or ", c10::DispatchKey::SparseHIP,
-                " but got: ", dispatch_key);
+    TORCH_CHECK(
+        dispatch_key == c10::DispatchKey::SparseCPU ||
+            dispatch_key == c10::DispatchKey::SparseCUDA ||
+            dispatch_key == c10::DispatchKey::SparseHIP ||
+            dispatch_key == c10::DispatchKey::SparseXPU,
+        "new(): expected DispatchKey: ",
+        c10::DispatchKey::SparseCPU,
+        " or ",
+        c10::DispatchKey::SparseCUDA,
+        " or ",
+        c10::DispatchKey::SparseHIP,
+        " or ",
+        c10::DispatchKey::SparseXPU,
+        " but got: ",
+        dispatch_key);
   } else {
     TORCH_INTERNAL_ASSERT(false, "unexpected layout");
   }
@@ -360,9 +379,8 @@ void check_base_legacy_new(c10::DispatchKey dispatch_key, at::Layout expected_la
 void check_legacy_ctor_device(c10::DispatchKey dispatch_key, c10::optional<Device> device) {
   if (device.has_value()) {
     TORCH_CHECK(computeDeviceType(dispatch_key) == device.value().type(),
-             "legacy constructor for device type: ", computeDeviceType(dispatch_key),
-             " was passed device type: ", device.value().type(),
-             ", but device type must be: ", computeDeviceType(dispatch_key));
+             "legacy constructor expects device type: ", computeDeviceType(dispatch_key),
+             "but device type: ", device.value().type(), " was passed");
   }
 }
 
