@@ -19,7 +19,7 @@ int main(int argc, char** argv) {
   }
 
   // Kick off work
-  std::vector<std::shared_ptr<ProcessGroup::Work>> pending;
+  std::vector<c10::intrusive_ptr<ProcessGroup::Work>> pending;
   for (auto i = 0; i < ntensors; i++) {
     std::vector<at::Tensor> tmp = {tensors[i]};
     pending.push_back(pg.allreduce(tmp));
