@@ -1,5 +1,5 @@
 from .modules import *
-from .parameter import Parameter
+from .parameter import Parameter, UninitializedParameter
 from .parallel import DataParallel
 from . import init
 from . import utils
