@@ -1,11 +1,13 @@
-
 from .linear_relu import LinearReLU
-from .conv_fused import ConvBn2d, ConvBnReLU2d, ConvReLU2d, update_bn_stats, freeze_bn_stats
+from .conv_fused import ConvBn1d, ConvBn2d, ConvBnReLU1d, ConvBnReLU2d, ConvReLU2d, \
+    update_bn_stats, freeze_bn_stats
 
 __all__ = [
     'LinearReLU',
     'ConvReLU2d',
+    'ConvBn1d',
     'ConvBn2d',
+    'ConvBnReLU1d',
     'ConvBnReLU2d',
     'update_bn_stats',
     'freeze_bn_stats'
