@@ -1,10 +1,11 @@
 
 from .linear import Linear
-from .rnn import LSTM, LSTMCell, RNNCell, GRUCell
+from .rnn import LSTM, GRU, LSTMCell, RNNCell, GRUCell
 
 __all__ = [
     'Linear',
     'LSTM',
+    'GRU',
     'LSTMCell',
     'RNNCell',
     'GRUCell',
