@@ -197,7 +197,7 @@ class LBFGS(Optimizer):
         ``param_bytes * (history_size + 1)`` bytes). If it doesn't fit in memory
         try reducing the history size, or use a different algorithm.
 
-    Arguments:
+    Args:
         lr (float): learning rate (default: 1)
         max_iter (int): maximal number of iterations per optimization step
             (default: 20)
@@ -283,7 +283,7 @@ class LBFGS(Optimizer):
     def step(self, closure):
         """Performs a single optimization step.
 
-        Arguments:
+        Args:
             closure (callable): A closure that reevaluates the model
                 and returns the loss.
         """
