@@ -1,12 +1,12 @@
-
 import torch
 import torch.nn as nn
 import torch.nn.quantized as nnq
 import torch.nn.quantized.dynamic as nnqd
 from torch.quantization import prepare
+from typing import Dict, List, Optional, Any, Union, Callable, Set
 
 from .quantization_mappings import (
-    get_compare_output_module_list,
+    get_default_compare_output_module_list,
 )
 
 NON_LEAF_MODULE_TO_ADD_OBSERVER_ALLOW_LIST = {
@@ -17,7 +17,10 @@ NON_LEAF_MODULE_TO_ADD_OBSERVER_ALLOW_LIST = {
 }
 
 
-def _find_match(str_list, key_str, postfix):
+def _find_match(
+    str_list: Union[Dict[str, Any], List[str]], key_str: str,
+    postfix: str,
+) -> Optional[str]:
     split_str = key_str.split(".")
     if split_str[-1] == postfix:
         match_string = "".join(key_str.split(".")[0:-1])
@@ -41,11 +44,14 @@ def _find_match(str_list, key_str, postfix):
                     return s2
                 if match_string == pattern2:
                     return s2
+        return None
     else:
         return None
 
 
-def compare_weights(float_dict, quantized_dict):
+def compare_weights(
+    float_dict: Dict[str, Any], quantized_dict: Dict[str, Any]
+) -> Dict[str, Dict[str, torch.Tensor]]:
     r"""Compare the weights of the float module with its corresponding quantized
     module. Return a dict with key corresponding to module names and each entry being
     a dictionary with two keys 'float' and 'quantized', containing the float and
@@ -66,7 +72,8 @@ def compare_weights(float_dict, quantized_dict):
         a dictionary with two keys 'float' and 'quantized', containing the float and
         quantized weights
     """
-    weight_dict = {}
+    torch._C._log_api_usage_once("quantization_api._numeric_suite.compare_weights")
+    weight_dict: Dict[str, Dict] = {}
     for key in quantized_dict:
         match_key = _find_match(float_dict, key, "weight")
         if match_key is not None:
@@ -103,7 +110,10 @@ def compare_weights(float_dict, quantized_dict):
     return weight_dict
 
 
-def _get_logger_dict_helper(mod, target_dict, prefix=""):
+def _get_logger_dict_helper(
+    mod: nn.Module, target_dict: Dict[str, Any],
+    prefix: str = "",
+) -> None:
     r"""This is the helper function for get_logger_dict
 
     Args:
@@ -125,7 +135,7 @@ def _get_logger_dict_helper(mod, target_dict, prefix=""):
         _get_logger_dict_helper(child, target_dict, module_prefix)
 
 
-def get_logger_dict(mod, prefix=""):
+def get_logger_dict(mod: nn.Module, prefix: str = "") -> Dict[str, Dict]:
     r"""Traverse the modules and save all logger stats into target dict.
     This is mainly used for quantization accuracy debug.
 
@@ -141,8 +151,9 @@ def get_logger_dict(mod, prefix=""):
     Return:
         target_dict: the dictionary used to save all logger stats
     """
+    torch._C._log_api_usage_once("quantization_api._numeric_suite.get_logger_dict")
 
-    target_dict = {}
+    target_dict: Dict[str, Dict] = {}
     _get_logger_dict_helper(mod, target_dict, prefix)
     return target_dict
 
@@ -154,6 +165,10 @@ class Logger(nn.Module):
     def __init__(self):
         super(Logger, self).__init__()
         self.stats = {}
+        # We only insert observer if the op is quantized with static quantization,
+        # which is identified by activation_observer.dtype == quint8.  This is needed
+        # when attaching Logger as observer for FX mode
+        self.dtype = torch.quint8
 
     def forward(self, x):
         pass
@@ -166,23 +181,16 @@ class ShadowLogger(Logger):
 
     def __init__(self):
         super(ShadowLogger, self).__init__()
-        self.stats["float"] = None
-        self.stats["quantized"] = None
+        self.stats["float"] = []
+        self.stats["quantized"] = []
 
     def forward(self, x, y):
         if len(x) > 1:
             x = x[0]
         if len(y) > 1:
             y = y[0]
-        if self.stats["quantized"] is None:
-            self.stats["quantized"] = x.detach()
-        else:
-            self.stats["quantized"] = torch.cat((self.stats["quantized"], x.detach()))
-
-        if self.stats["float"] is None:
-            self.stats["float"] = y.detach()
-        else:
-            self.stats["float"] = torch.cat((self.stats["float"], y.detach()))
+        self.stats["quantized"].append(x.detach())
+        self.stats["float"].append(y.detach())
 
 
 class OutputLogger(Logger):
@@ -191,21 +199,19 @@ class OutputLogger(Logger):
 
     def __init__(self):
         super(OutputLogger, self).__init__()
-        self.stats["tensor_val"] = None
+        self.stats["tensor_val"] = []
+
 
     def forward(self, x):
-        if self.stats["tensor_val"] is None:
-            self.stats["tensor_val"] = x
-        else:
-            self.stats["tensor_val"] = torch.cat((self.stats["tensor_val"], x))
+        self.stats["tensor_val"].append(x)
         return x
 
 
-def _convert_tuple_to_list(t):
+def _convert_tuple_to_list(t: Any) -> Any:
     return list(_convert_tuple_to_list(x) for x in t) if type(t) is tuple else t
 
 
-def _dequantize_tensor_list(t):
+def _dequantize_tensor_list(t: Any) -> Any:
     return (
         list(_dequantize_tensor_list(x) for x in t)
         if type(t) is list
@@ -234,7 +240,7 @@ class Shadow(nn.Module):
         self.dequant = nnq.DeQuantize()
         self.logger = Logger()
 
-    def forward(self, *x):
+    def forward(self, *x) -> torch.Tensor:
         xl = _convert_tuple_to_list(x)
         output = self.orig_module(*xl)
         xl_float = _dequantize_tensor_list(xl)
@@ -242,7 +248,7 @@ class Shadow(nn.Module):
         self.logger(output, shadow_output)
         return output
 
-    def add(self, x, y):
+    def add(self, x: torch.Tensor, y: torch.Tensor) -> torch.Tensor:
         output = self.orig_module.add(x, y)
         x = x.dequantize()
         y = y.dequantize()
@@ -250,14 +256,14 @@ class Shadow(nn.Module):
         self.logger(output, shadow_output)
         return output
 
-    def add_scalar(self, x, y):
+    def add_scalar(self, x: torch.Tensor, y: float) -> torch.Tensor:
         output = self.orig_module.add_scalar(x, y)
         x = x.dequantize()
         shadow_output = self.shadow_module.add_scalar(x, y)
         self.logger(output, shadow_output)
         return output
 
-    def mul(self, x, y):
+    def mul(self, x: torch.Tensor, y: torch.Tensor) -> torch.Tensor:
         output = self.orig_module.mul(x, y)
         x = x.dequantize()
         y = y.dequantize()
@@ -265,21 +271,21 @@ class Shadow(nn.Module):
         self.logger(output, shadow_output)
         return output
 
-    def mul_scalar(self, x, y):
+    def mul_scalar(self, x: torch.Tensor, y: float) -> torch.Tensor:
         output = self.orig_module.mul_scalar(x, y)
         x = x.dequantize()
         shadow_output = self.shadow_module.mul_scalar(x, y)
         self.logger(output, shadow_output)
         return output
 
-    def cat(self, x, dim=0):
+    def cat(self, x: List[torch.Tensor], dim: int = 0) -> torch.Tensor:
         output = self.orig_module.cat(x, dim)
         x = [y.dequantize() for y in x]
         shadow_output = self.shadow_module.cat(x, dim)
         self.logger(output, shadow_output)
         return output
 
-    def add_relu(self, x, y):
+    def add_relu(self, x: torch.Tensor, y: torch.Tensor) -> torch.Tensor:
         output = self.orig_module.add_relu(x, y)
         x = x.dequantize()
         y = y.dequantize()
@@ -288,7 +294,10 @@ class Shadow(nn.Module):
         return output
 
 
-def prepare_model_with_stubs(float_module, q_module, module_swap_list, Logger):
+def prepare_model_with_stubs(
+    float_module: nn.Module, q_module: nn.Module,
+    module_swap_list: Set[type], Logger: Callable,
+) -> None:
     r"""Prepare the model by attaching the float module to its matching quantized
     module as the shadow if the float module type is in module_swap_list.
 
@@ -304,6 +313,7 @@ def prepare_model_with_stubs(float_module, q_module, module_swap_list, Logger):
         Logger: type of logger to be used in shadow module to process the outputs of
             quantized module and its float shadow module
     """
+    torch._C._log_api_usage_once("quantization_api._numeric_suite.prepare_model_with_stubs")
 
     float_module_children = {}
     for name, mod in float_module.named_children():
@@ -311,6 +321,7 @@ def prepare_model_with_stubs(float_module, q_module, module_swap_list, Logger):
 
     reassign = {}
     for name, mod in q_module.named_children():
+
         if name not in float_module_children:
             continue
 
@@ -319,16 +330,26 @@ def prepare_model_with_stubs(float_module, q_module, module_swap_list, Logger):
         if type(float_mod) not in module_swap_list:
             prepare_model_with_stubs(float_mod, mod, module_swap_list, Logger)
 
-        if type(float_mod) in module_swap_list:
+        # Insert shadow module only if the module is not of the same type as
+        # the floating point module
+        if type(float_mod) in module_swap_list and not _is_identical_module_type(mod, float_mod):
             reassign[name] = Shadow(mod, float_mod, Logger)
 
     for key, value in reassign.items():
         q_module._modules[key] = value
 
+def _is_identical_module_type(mod1, mod2):
+    # Compare if two modules have the same dtype
+    mod1_module_types = [type(mod) for mod in mod1.modules()]
+    mod2_module_types = [type(mod) for mod in mod2.modules()]
+    return mod1_module_types == mod2_module_types
+
+
 
 def compare_model_stub(
-    float_model, q_model, module_swap_list, *data, Logger=ShadowLogger
-):
+    float_model: nn.Module, q_model: nn.Module, module_swap_list: Set[type],
+    *data, Logger=ShadowLogger
+) -> Dict[str, Dict]:
     r"""Compare quantized module in a model with its floating point counterpart,
     feeding both of them the same input. Return a dict with key corresponding to
     module names and each entry being a dictionary with two keys 'float' and
@@ -359,13 +380,16 @@ def compare_model_stub(
         Logger: type of logger to be used in shadow module to process the outputs of
             quantized module and its float shadow module
     """
+    torch._C._log_api_usage_once("quantization_api._numeric_suite.compare_model_stub")
     prepare_model_with_stubs(float_model, q_model, module_swap_list, Logger)
     q_model(*data)
     ob_dict = get_logger_dict(q_model)
     return ob_dict
 
 
-def get_matching_activations(float_module, q_module):
+def get_matching_activations(
+    float_module: nn.Module, q_module: nn.Module,
+) -> Dict[str, Dict[str, torch.Tensor]]:
     r"""Find the matching activation between float and quantized modules.
 
     Args:
@@ -377,9 +401,10 @@ def get_matching_activations(float_module, q_module):
         entry being a dictionary with two keys 'float' and 'quantized', containing
         the matching float and quantized activations
     """
+    torch._C._log_api_usage_once("quantization_api._numeric_suite.get_matching_activations")
     float_dict = get_logger_dict(float_module)
     quantized_dict = get_logger_dict(q_module)
-    act_dict = {}
+    act_dict: Dict[str, Dict] = {}
     for key in quantized_dict:
         match_key = _find_match(sorted(float_dict, reverse=True), key, "stats")
         if match_key is not None:
@@ -390,11 +415,11 @@ def get_matching_activations(float_module, q_module):
 
 
 def prepare_model_outputs(
-    float_module,
-    q_module,
+    float_module: nn.Module,
+    q_module: nn.Module,
     Logger=OutputLogger,
     allow_list=None
-):
+) -> None:
     r"""Prepare the model by attaching the logger to both float module
     and quantized module if they are in the allow_list.
 
@@ -404,13 +429,14 @@ def prepare_model_outputs(
         Logger: type of logger to be attached to float_module and q_module
         allow_list: list of module types to attach logger
     """
+    torch._C._log_api_usage_once("quantization_api._numeric_suite.prepare_model_outputs")
     if allow_list is None:
-        allow_list = get_compare_output_module_list()
+        allow_list = get_default_compare_output_module_list()
 
     qconfig_debug = torch.quantization.QConfig(activation=Logger, weight=None)
-    float_module.qconfig = qconfig_debug
+    float_module.qconfig = qconfig_debug  # type: ignore
     prepare(float_module, inplace=True, allow_list=allow_list)
-    q_module.qconfig = qconfig_debug
+    q_module.qconfig = qconfig_debug  # type: ignore
     prepare(
         q_module,
         inplace=True,
@@ -420,12 +446,12 @@ def prepare_model_outputs(
 
 
 def compare_model_outputs(
-    float_model,
-    q_model,
+    float_model: nn.Module,
+    q_model: nn.Module,
     *data,
     Logger=OutputLogger,
     allow_list=None
-):
+) -> Dict[str, Dict[str, torch.Tensor]]:
     r"""Compare output activations between float and quantized models at
     corresponding locations for the same input. Return a dict with key corresponding
     to quantized module names and each entry being a dictionary with two keys
@@ -450,8 +476,9 @@ def compare_model_outputs(
         and each entry being a dictionary with two keys 'float' and 'quantized',
         containing the matching float and quantized activations
     """
+    torch._C._log_api_usage_once("quantization_api._numeric_suite.compare_model_outputs")
     if allow_list is None:
-        allow_list = get_compare_output_module_list()
+        allow_list = get_default_compare_output_module_list()
     prepare_model_outputs(float_model, q_model, Logger, allow_list)
     float_model(*data)
     q_model(*data)
