@@ -1,29 +1,28 @@
-
 import copy
 import itertools
 import warnings
 
 import torch
 import torch.nn as nn
-import torch.nn.intrinsic as nni
 import torch.nn.quantized as nnq
-import torch.nn.intrinsic.qat as nniqat
+import torch.nn.quantizable as nnqa
+from torch.nn.intrinsic import _FusedModule
 
-from .quantization_mappings import (get_dynamic_quant_module_mappings,
-                                    get_static_quant_module_mappings,
-                                    get_qat_module_mappings,
-                                    get_qconfig_propagation_list)
-
-from .custom_module_class_mappings import (
-    is_custom_module_class,
-    get_observed_custom_module_class,
-    get_quantized_custom_module_class,
-    mark_observed_custom_module,
-    is_observed_custom_module,
+from .quantization_mappings import (
+    get_default_dynamic_quant_module_mappings,
+    get_default_static_quant_module_mappings,
+    get_default_qat_module_mappings,
+    get_default_qconfig_propagation_list,
+    _has_special_act_post_process,
+    _get_special_act_post_process,
 )
 
 from .stubs import DeQuantStub, QuantWrapper
-from .qconfig import default_dynamic_qconfig, float16_dynamic_qconfig, float_qparams_dynamic_qconfig
+from .qconfig import default_dynamic_qconfig, float16_dynamic_qconfig, float_qparams_weight_only_qconfig
+
+def is_activation_post_process(module):
+    return (isinstance(module, torch.quantization.ObserverBase) or
+            isinstance(module, torch.quantization.FakeQuantizeBase))
 
 def _propagate_qconfig_helper(module, qconfig_dict, allow_list=None,
                               qconfig_parent=None, prefix=''):
@@ -45,12 +44,14 @@ def _propagate_qconfig_helper(module, qconfig_dict, allow_list=None,
     """
     # TODO: Add test
     if allow_list is None:
-        allow_list = get_qconfig_propagation_list()
+        allow_list = get_default_qconfig_propagation_list()
 
     module_qconfig = qconfig_dict.get(type(module), qconfig_parent)
     module_qconfig = qconfig_dict.get(prefix, module_qconfig)
     module_qconfig = getattr(module, 'qconfig', module_qconfig)
 
+    torch.quantization.qconfig.assert_valid_qconfig(module_qconfig, module)
+
     module.qconfig = module_qconfig
     for name, child in module.named_children():
         module_prefix = prefix + '.' + name if prefix else name
@@ -81,19 +82,12 @@ def _observer_forward_hook(self, input, output):
     """
     return self.activation_post_process(output)
 
-def _observer_forward_pre_hook(self, input):
-    ''' Forward pre hook that calls observer on the input (can be a tuple of values)
-    '''
-    self.activation_pre_process(*input)
-    # Returning nothing is Ok, Module._call_impl will intrepret this
-    # as the pre_hook making no changes to the input, as desired
-
 def register_activation_post_process_hook(module):
     assert hasattr(module, 'activation_post_process'), \
         'Expect activation_post_process attribut already attached to the module'
     return module.register_forward_hook(_observer_forward_hook)
 
-def add_observer_(module, qconfig_propagation_list=None, non_leaf_module_list=None, device=None, prehook=None):
+def add_observer_(module, qconfig_propagation_list=None, non_leaf_module_list=None, device=None, custom_module_class_mapping=None):
     r"""Add observer for the leaf child of the module.
 
     This function insert observer module to all leaf child module that
@@ -108,7 +102,10 @@ def add_observer_(module, qconfig_propagation_list=None, non_leaf_module_list=No
         None, module is modified inplace with added observer modules and forward_hooks
     """
     if qconfig_propagation_list is None:
-        qconfig_propagation_list = get_qconfig_propagation_list()
+        qconfig_propagation_list = get_default_qconfig_propagation_list()
+
+    if custom_module_class_mapping is None:
+        custom_module_class_mapping = {}
 
     # respect device affinity when adding observers
     if device is None:
@@ -119,8 +116,8 @@ def add_observer_(module, qconfig_propagation_list=None, non_leaf_module_list=No
         )
         device = next(iter(devices)) if len(devices) > 0 else None
 
-    def get_activation_post_process(qconfig, device):
-        activation = qconfig.activation()
+    def get_activation_post_process(qconfig, device, special_act_post_process=None):
+        activation = qconfig.activation() if special_act_post_process is None else special_act_post_process()
         if device is not None:
             activation.to(device)
         return activation
@@ -128,49 +125,48 @@ def add_observer_(module, qconfig_propagation_list=None, non_leaf_module_list=No
     def needs_observation(m):
         return hasattr(m, 'qconfig') and m.qconfig is not None
 
-    def insert_activation_post_process(m):
+    def insert_activation_post_process(m, special_act_post_process=None):
         """ Adds an activation post process module and register
         a post hook that calls the module
         """
-        if needs_observation(m):
+        # We don't insert observer/fake_quantize for DeQuantStub
+        if needs_observation(m) and not isinstance(m, DeQuantStub):
             # observer and hook will be gone after we swap the module
-            m.add_module('activation_post_process', get_activation_post_process(m.qconfig, device))
+            m.add_module('activation_post_process', get_activation_post_process(m.qconfig, device, special_act_post_process))
             # Register observer as the first entry in the hook list
             # All post forward hooks are preserved and will be executed after the observer before convert
             handle = register_activation_post_process_hook(m)
             m._forward_hooks.move_to_end(handle.id, last=False)
 
     for name, child in module.named_children():
-        if type(child) == nnq.FloatFunctional or type(child) == nnq.QFunctional:
-            if hasattr(child, 'qconfig') and child.qconfig is not None:
-                child.activation_post_process = get_activation_post_process(child.qconfig, device)
-        elif non_leaf_module_list is not None and type(child) in non_leaf_module_list:
-            insert_activation_post_process(child)
-            # TODO: remove
+        if type(child) in [nnq.FloatFunctional, nnq.QFunctional]:
             if needs_observation(child):
-                # Attaching prehook
-                if prehook is not None:
-                    child.add_module('activation_pre_process', prehook())
-                    child.register_forward_pre_hook(_observer_forward_pre_hook)
-        elif needs_observation(child) and is_custom_module_class(type(child)):
-            observed_child = get_observed_custom_module_class(type(child)).from_float(child)
-            mark_observed_custom_module(observed_child, type(child))
+                child.activation_post_process = get_activation_post_process(child.qconfig, device)
+        elif isinstance(child, _FusedModule):
+            # activation_post_process are now added directly to nn.Sequentail/_FusedModule
+            if needs_observation(child):
+                insert_activation_post_process(child)
+        elif _has_special_act_post_process(child):
+            special_act_post_process = _get_special_act_post_process(child)
+            insert_activation_post_process(child, special_act_post_process)
+        elif non_leaf_module_list is not None and type(child) in non_leaf_module_list:
+            if needs_observation(child):
+                insert_activation_post_process(child)
+        elif needs_observation(child) and type(child) in custom_module_class_mapping:
+            observed_child = custom_module_class_mapping[type(child)].from_float(child)
             setattr(module, name, observed_child)
-            insert_activation_post_process(observed_child)
+            # TODO: These are the modules that cannot be observed
+            #       Once there are more, we should move them to a separate list
+            if custom_module_class_mapping[type(child)] != nnqa.LSTM:
+                insert_activation_post_process(observed_child)
         else:
-            add_observer_(child, qconfig_propagation_list, non_leaf_module_list, device, prehook)
+            add_observer_(child, qconfig_propagation_list, non_leaf_module_list, device, custom_module_class_mapping)
 
     # Insert observers only for leaf nodes, note that this observer is for
     # the output of the module, for input QuantStub will observe them
     if len(module._modules) == 0 and not isinstance(module, torch.nn.Sequential) \
        and type(module) in qconfig_propagation_list:
         insert_activation_post_process(module)
-        # TOOD: remove
-        if needs_observation(module):
-            # Attaching prehook
-            if prehook is not None:
-                module.add_module('activation_pre_process', prehook())
-                module.register_forward_pre_hook(_observer_forward_pre_hook)
 
 def get_unique_devices_(module):
     return {p.device for p in module.parameters()} | \
@@ -199,7 +195,8 @@ def add_quant_dequant(module):
     return module
 
 def prepare(model, inplace=False, allow_list=None,
-            observer_non_leaf_module_list=None, prehook=None):
+            observer_non_leaf_module_list=None,
+            prepare_custom_config_dict=None):
     r"""Prepares a copy of the model for quantization calibration or quantization-aware training.
 
     Quantization configuration should be assigned preemptively
@@ -209,18 +206,37 @@ def prepare(model, inplace=False, allow_list=None,
     will be propagated.
 
     Args:
-        model: input model to be modified in-place
-        inplace: carry out model transformations in-place, the original module is mutated
-        allow_list: list of quantizable modules
-        observer_non_leaf_module_list: list of non-leaf modules we want to add observer
-        prehook: observer we want to add to forward_pre_hook
+        `model`: input model to be modified in-place
+        `inplace`: carry out model transformations in-place, the original module is mutated
+        `allow_list`: list of quantizable modules
+        `observer_non_leaf_module_list`: list of non-leaf modules we want to add observer
+        `prepare_custom_config_dict`: customization configuration dictionary for prepare function
+
+    .. code-block:: python
+
+       # Example of prepare_custom_config_dict:
+       prepare_custom_config_dict = {
+           # user will manually define the corresponding observed
+           # module class which has a from_float class method that converts
+           # float custom module to observed custom module
+           "float_to_observed_custom_module_class": {
+               CustomModule: ObservedCustomModule
+           }
+        }
+
     """
+    torch._C._log_api_usage_once("quantization_api.quantize.prepare")
+    if prepare_custom_config_dict is None:
+        prepare_custom_config_dict = {}
+    custom_module_class_mapping = prepare_custom_config_dict.get("float_to_observed_custom_module_class", {})
+
     if not inplace:
         model = copy.deepcopy(model)
 
+    # TODO: remove allow_list
     qconfig_propagation_list = allow_list
     if qconfig_propagation_list is None:
-        qconfig_propagation_list = get_qconfig_propagation_list()
+        qconfig_propagation_list = get_default_qconfig_propagation_list()
     propagate_qconfig_(model, qconfig_dict=None)
 
     # sanity check common API misusage
@@ -229,9 +245,27 @@ def prepare(model, inplace=False, allow_list=None,
                       "passed correct configuration through `qconfig_dict` or "
                       "by assigning the `.qconfig` attribute directly on submodules")
 
-    add_observer_(model, qconfig_propagation_list, observer_non_leaf_module_list, prehook=prehook)
+    add_observer_(
+        model, qconfig_propagation_list, observer_non_leaf_module_list,
+        custom_module_class_mapping=custom_module_class_mapping)
     return model
 
+def _remove_activation_post_process(module):
+    # TODO: maybe we should change activation_post_process to _activation_post_process
+    # to prevent it from being used by user
+    if hasattr(module, 'activation_post_process') and \
+       is_activation_post_process(module.activation_post_process):
+        delattr(module, 'activation_post_process')
+
+    # remove activation_post_proceess hook
+    handle_ids_to_remove = set()
+    for handle_id, hook_fn in module._forward_hooks.items():
+        if hook_fn is _observer_forward_hook:
+            handle_ids_to_remove.add(handle_id)
+    for handle_id in handle_ids_to_remove:
+        module._forward_hooks.pop(handle_id)
+
+# TODO: rename to something more general
 def _remove_qconfig(module):
     r"""Clean up the qconfig left in the module so that new qconfig can be
     propagated.
@@ -245,6 +279,8 @@ def _remove_qconfig(module):
     if hasattr(module, "qconfig"):
         del module.qconfig
 
+    _remove_activation_post_process(module)
+
 def quantize(model, run_fn, run_args, mapping=None, inplace=False):
     r"""Quantize the input float model with post training static quantization.
 
@@ -262,13 +298,14 @@ def quantize(model, run_fn, run_args, mapping=None, inplace=False):
     Return:
         Quantized model.
     """
+    torch._C._log_api_usage_once("quantization_api.quantize.quantize")
     if mapping is None:
-        mapping = get_static_quant_module_mappings()
+        mapping = get_default_static_quant_module_mappings()
     if not inplace:
         model = copy.deepcopy(model)
     model.eval()
     prepare(model, inplace=True)
-    run_fn(model, run_args)
+    run_fn(model, *run_args)
     convert(model, mapping, inplace=True)
     return model
 
@@ -302,6 +339,7 @@ def quantize_dynamic(model, qconfig_spec=None, dtype=torch.qint8,
             with which the submodule needs to be replaced
 
     """
+    torch._C._log_api_usage_once("quantization_api.quantize.quantize_dynamic")
     if qconfig_spec is None:
         if dtype == torch.qint8:
             qconfig_spec = {
@@ -323,7 +361,7 @@ def quantize_dynamic(model, qconfig_spec=None, dtype=torch.qint8,
             }
         elif dtype == torch.quint8:
             qconfig_spec = {
-                nn.EmbeddingBag : float_qparams_dynamic_qconfig,
+                nn.EmbeddingBag : float_qparams_weight_only_qconfig,
             }
         else:
             raise ValueError(
@@ -334,13 +372,13 @@ def quantize_dynamic(model, qconfig_spec=None, dtype=torch.qint8,
         elif dtype is torch.float16:
             default_qconfig = float16_dynamic_qconfig
         elif dtype is torch.quint8:
-            default_qconfig = float_qparams_dynamic_qconfig
+            default_qconfig = float_qparams_weight_only_qconfig
         else:
             raise RuntimeError('Unknown dtype specified for quantize_dynamic: ', str(dtype))
         qconfig_spec = dict(zip(qconfig_spec, itertools.repeat(default_qconfig)))
 
     if mapping is None:
-        mapping = get_dynamic_quant_module_mappings()
+        mapping = get_default_dynamic_quant_module_mappings()
 
     if not inplace:
         model = copy.deepcopy(model)
@@ -364,8 +402,10 @@ def prepare_qat(model, mapping=None, inplace=False):
         inplace: carry out model transformations in-place, the original module
                  is mutated
     """
+    torch._C._log_api_usage_once("quantization_api.quantize.prepare_qat")
     if mapping is None:
-        mapping = get_qat_module_mappings()
+        mapping = get_default_qat_module_mappings()
+
     if not inplace:
         model = copy.deepcopy(model)
 
@@ -387,36 +427,57 @@ def quantize_qat(model, run_fn, run_args, inplace=False):
     Return:
         Quantized model.
     """
+    torch._C._log_api_usage_once("quantization_api.quantize.quantize_qat")
     if not inplace:
         model = copy.deepcopy(model)
     model.train()
     prepare_qat(model, inplace=True)
-    run_fn(model, run_args)
+    run_fn(model, *run_args)
     convert(model, inplace=True)
     return model
 
-def convert(module, mapping=None, inplace=False, remove_qconfig=True):
+def convert(
+        module, mapping=None, inplace=False, remove_qconfig=True,
+        convert_custom_config_dict=None):
     r"""Converts submodules in input module to a different module according to `mapping`
     by calling `from_float` method on the target module class. And remove qconfig at the
     end if remove_qconfig is set to True.
 
     Args:
-        module: input module
-        mapping: a dictionary that maps from source module type to target
-                 module type, can be overwritten to allow swapping user defined
-                 Modules
-        inplace: carry out model transformations in-place, the original module
-                 is mutated
+        `module`: prepared and calibrated module
+        `mapping`: a dictionary that maps from source module type to target
+                   module type, can be overwritten to allow swapping user defined
+                   Modules
+        `inplace`: carry out model transformations in-place, the original module
+                   is mutated
+        `convert_custom_config_dict`: custom configuration dictionary for convert function
+
+    .. code-block:: python
+
+       # Example of convert_custom_config_dict:
+       convert_custom_config_dict = {
+           # user will manually define the corresponding quantized
+           # module class which has a from_observed class method that converts
+           # observed custom module to quantized custom module
+           "observed_to_quantized_custom_module_class": {
+               ObservedCustomModule: QuantizedCustomModule
+           }
+       }
 
     """
+    torch._C._log_api_usage_once("quantization_api.quantize.convert")
     if not inplace:
         module = copy.deepcopy(module)
-    _convert(module, mapping, inplace=True)
+    _convert(
+        module, mapping, inplace=True,
+        convert_custom_config_dict=convert_custom_config_dict)
     if remove_qconfig:
         _remove_qconfig(module)
     return module
 
-def _convert(module, mapping=None, inplace=False):
+def _convert(
+        module, mapping=None, inplace=False,
+        convert_custom_config_dict=None):
     r"""Converts submodules in input module to a different module according to `mapping`
     by calling `from_float` method on the target module class
 
@@ -430,40 +491,29 @@ def _convert(module, mapping=None, inplace=False):
 
     """
     if mapping is None:
-        mapping = get_static_quant_module_mappings()
+        mapping = get_default_static_quant_module_mappings()
+    if convert_custom_config_dict is None:
+        convert_custom_config_dict = {}
+    custom_module_class_mapping = convert_custom_config_dict.get("observed_to_quantized_custom_module_class", {})
+
     if not inplace:
         module = copy.deepcopy(module)
     reassign = {}
-    # TODO(jerryzh): remove after deciding on the impl of intrinsic modules
-    # This is required because intrinsic modules right now are implemented as
-    # nn.Sequential and we don't want to swap their constituents
-    SWAPPABLE_MODULES = (nni.ConvBn2d,
-                         nni.ConvBnReLU2d,
-                         nni.LinearReLU,
-                         nni.BNReLU2d,
-                         nni.BNReLU3d,
-                         nni.ConvBn1d,
-                         nni.ConvReLU1d,
-                         nni.ConvBnReLU1d,
-                         nni.ConvReLU2d,
-                         nni.ConvReLU3d,
-                         nniqat.ConvBn2d,
-                         nniqat.ConvBnReLU2d)
-
     for name, mod in module.named_children():
-        # both swappable modules and observed custom modules are
+        # both fused modules and observed custom modules are
         # swapped as one unit
-        if type(mod) not in SWAPPABLE_MODULES and \
-           not is_observed_custom_module(mod):
-            _convert(mod, mapping, inplace=True)
-        reassign[name] = swap_module(mod, mapping)
+        if not isinstance(mod, _FusedModule) and \
+           type(mod) not in custom_module_class_mapping:
+            _convert(mod, mapping, True,  # inplace
+                     custom_module_class_mapping)
+        reassign[name] = swap_module(mod, mapping, custom_module_class_mapping)
 
     for key, value in reassign.items():
         module._modules[key] = value
 
     return module
 
-def swap_module(mod, mapping):
+def swap_module(mod, mapping, custom_module_class_mapping):
     r"""Swaps the module if it has a quantized counterpart and it has an
     `observer` attached.
 
@@ -475,11 +525,10 @@ def swap_module(mod, mapping):
         The corresponding quantized module of `mod`
     """
     new_mod = mod
-    # Always replace dequantstub with dequantize
-    if hasattr(mod, 'qconfig') and mod.qconfig is not None or type(mod) == DeQuantStub:
+    if hasattr(mod, 'qconfig') and mod.qconfig is not None:
         swapped = False
-        if is_observed_custom_module(mod):
-            new_mod = get_quantized_custom_module_class(mod._FLOAT_MODULE).from_observed(mod)
+        if type(mod) in custom_module_class_mapping:
+            new_mod = custom_module_class_mapping[type(mod)].from_observed(mod)
             swapped = True
         elif type(mod) in mapping:
             new_mod = mapping[type(mod)].from_float(mod)
