@@ -3,7 +3,6 @@ This module converts objects into numpy array.
 """
 import numpy as np
 import torch
-import six
 
 
 def make_np(x):
@@ -16,7 +15,7 @@ def make_np(x):
     """
     if isinstance(x, np.ndarray):
         return x
-    if isinstance(x, six.string_types):  # Caffe2 will pass name of blob(s) to fetch
+    if isinstance(x, str):  # Caffe2 will pass name of blob(s) to fetch
         return _prepare_caffe2(x)
     if np.isscalar(x):
         return np.array([x])
